@@ -1,7 +1,7 @@
 
 
 /*
- * This event handling stuff was based on Tk.
+ * This event handling stuff was inspired on Tk.
  */
 
 #include "WINGsP.h"
@@ -403,38 +403,27 @@ void
 WMCreateEventHandler(WMView *view, unsigned long mask, WMEventProc *eventProc,
 		     void *clientData)
 {
-    W_EventHandler *handler;
-    W_EventHandler *ptr = view->handlerList;
+    W_EventHandler *handler, *ptr;
     unsigned long eventMask;
-        
-    if (ptr==NULL) {
+    WMBagIterator iter;
+
+    
+    handler = NULL;
+    eventMask = mask;
+
+    WM_ITERATE_BAG(view->eventHandlers, ptr, iter) {
+	if (ptr->clientData == clientData && ptr->proc == eventProc) {
+	    handler = ptr;
+	    eventMask |= ptr->eventMask;
+	}
+    }
+    if (!handler) {
 	handler = wmalloc(sizeof(W_EventHandler));
 
-	handler->nextHandler = NULL;
-	
-	view->handlerList = handler;
-	
-	eventMask = mask;
-    } else {
-	handler = NULL;
-	eventMask = mask;
-	while (ptr != NULL) {
-	    if (ptr->clientData == clientData && ptr->proc == eventProc) {
-		handler = ptr;
-	    }	    
-	    eventMask |= ptr->eventMask;
-	    
-	    ptr = ptr->nextHandler;
-	}
-	if (!handler) {
-	    handler = wmalloc(sizeof(W_EventHandler));
-	    handler->nextHandler = view->handlerList;
-	    view->handlerList = handler;
-	}	
+	WMPutInBag(view->eventHandlers, handler);
     }
-
     /* select events for window */
-    handler->eventMask = mask;
+    handler->eventMask = eventMask;
     handler->proc = eventProc;
     handler->clientData = clientData;
 }
@@ -450,31 +439,24 @@ void
 WMDeleteEventHandler(WMView *view, unsigned long mask, WMEventProc *eventProc, 
 		     void *clientData)
 {
-    W_EventHandler *handler, *ptr, *pptr;
-    
-    ptr = view->handlerList;
-    
+    W_EventHandler *handler, *ptr;
+    WMBagIterator iter;
+        
     handler = NULL;
-    pptr = NULL;
     
-    while (ptr!=NULL) {
+    WM_ITERATE_BAG(view->eventHandlers, ptr, iter) {
 	if (ptr->eventMask == mask && ptr->proc == eventProc 
 	    && ptr->clientData == clientData) {
 	    handler = ptr;
 	    break;
 	}
-	pptr = ptr;
-	ptr = ptr->nextHandler;
     }
     
     if (!handler)
 	return;
     
-    if (!pptr) {
-	view->handlerList = handler->nextHandler;
-    } else {
-	pptr->nextHandler = handler->nextHandler;
-    }
+    WMRemoveFromBag(view->eventHandlers, handler);
+
     wfree(handler);
 }
 
@@ -483,14 +465,11 @@ WMDeleteEventHandler(WMView *view, unsigned long mask, WMEventProc *eventProc,
 void
 W_CleanUpEvents(WMView *view)
 {
-    W_EventHandler *ptr, *nptr;
+    W_EventHandler *ptr;
+    WMBagIterator iter;
     
-    ptr = view->handlerList;
-        
-    while (ptr!=NULL) {
-	nptr = ptr->nextHandler;
+    WM_ITERATE_BAG(view->eventHandlers, ptr, iter) {
 	wfree(ptr);
-	ptr = nptr;
     }
 }
 
@@ -529,18 +508,47 @@ void
 W_CallDestroyHandlers(W_View *view)
 {
     XEvent event;
+    WMBagIterator iter;
     W_EventHandler *hPtr;
     
     event.type = DestroyNotify;
     event.xdestroywindow.window = view->window;
     event.xdestroywindow.event = view->window;
-    hPtr = view->handlerList;
-    while (hPtr!=NULL) {
+
+    WM_ITERATE_BAG(view->eventHandlers, hPtr, iter) {
 	if (hPtr->eventMask & StructureNotifyMask) {
 	    (*hPtr->proc)(&event, hPtr->clientData);
 	}
-	
-	hPtr = hPtr->nextHandler;
+    }
+}
+
+
+
+void
+WMSetViewNextResponder(WMView *view, WMView *responder)
+{
+    /* set the widget to receive keyboard events that aren't handled
+     * by this widget */
+    
+    view->nextResponder = responder;
+}
+
+
+void
+WMRelayToNextResponder(WMView *view, XEvent *event)
+{
+    unsigned long mask = eventMasks[event->xany.type];
+
+    if (view->nextResponder) {
+	WMView *next = view->nextResponder;
+	W_EventHandler *hPtr;
+	WMBagIterator iter;
+
+	WM_ITERATE_BAG(next->eventHandlers, hPtr, iter) {
+	    if ((hPtr->eventMask & mask)) {
+		(*hPtr->proc)(event, hPtr->clientData);
+	    }
+	}
     }
 }
 
@@ -552,6 +560,7 @@ WMHandleEvent(XEvent *event)
     W_View *view, *vPtr, *toplevel;
     unsigned long mask;
     Window window;
+    WMBagIterator iter;
 
     if (event->type == MappingNotify) {
 	XRefreshKeyboardMapping(&event->xmapping);
@@ -570,6 +579,7 @@ WMHandleEvent(XEvent *event)
 	}
     }
     view = W_GetViewForXWindow(event->xany.display, window);
+
     if (!view) {
 	if (extraEventHandler)
 	    (extraEventHandler)(event);
@@ -621,7 +631,7 @@ WMHandleEvent(XEvent *event)
     }
 
 
-    if (view->screen->modal && toplevel!=view->screen->modalView
+    if (view->screen->modalLoop && toplevel!=view->screen->modalView
 	&& !toplevel->flags.worksWhenModal) {
 	if (event->type == KeyPress || event->type == KeyRelease
 	    || event->type == MotionNotify || event->type == ButtonPress
@@ -642,37 +652,26 @@ WMHandleEvent(XEvent *event)
      * might destroy the widget. */
     W_RetainView(toplevel);
 
-    hPtr = view->handlerList;
-
-    while (hPtr!=NULL) {
-	W_EventHandler *tmp;
-
-	tmp = hPtr->nextHandler;
-
+    WM_ITERATE_BAG(view->eventHandlers, hPtr, iter) {
 	if ((hPtr->eventMask & mask)) {
 	    (*hPtr->proc)(event, hPtr->clientData);
 	}
-	
-	hPtr = tmp;
     }
-    
+#if 0
     /* pass the event to the top level window of the widget */
+    /* TODO: change this to a responder chain */
     if (view->parent != NULL) {
 	vPtr = view;
 	while (vPtr->parent != NULL)
 	    vPtr = vPtr->parent;
-	
-	hPtr = vPtr->handlerList;
 
-	while (hPtr != NULL) {
-	
+	WM_ITERATE_BAG(vPtr->eventHandlers, hPtr, iter) {
 	    if (hPtr->eventMask & mask) {
 		(*hPtr->proc)(event, hPtr->clientData);
 	    }
-	    hPtr = hPtr->nextHandler;
 	}
     }
-
+#endif
     /* save button click info to track double-clicks */
     if (view->screen->ignoreNextDoubleClick) {
 	view->screen->ignoreNextDoubleClick = 0;
@@ -682,9 +681,9 @@ WMHandleEvent(XEvent *event)
 	    view->screen->lastClickTime = event->xbutton.time;
 	}
     }
-    
+
     W_ReleaseView(toplevel);
-    
+
     return True;
 }
 
