@@ -12,6 +12,7 @@
 #define PATH_MAX 1024
 #endif
 
+
 typedef struct W_FilePanel {
     WMWindow *win;
     
@@ -28,7 +29,9 @@ typedef struct W_FilePanel {
     
     WMButton *homeButton;
     WMButton *trashcanButton;
+    WMButton *createDirButton;
     WMButton *disketteButton;
+    WMButton *unmountButton;
 
     WMView *accessoryView;
 
@@ -39,7 +42,6 @@ typedef struct W_FilePanel {
     struct {
 	unsigned int canExit:1;
 	unsigned int canceled:1;       /* clicked on cancel */
-	unsigned int done:1;
 	unsigned int filtered:1;
 	unsigned int canChooseFiles:1;
 	unsigned int canChooseDirectories:1;
@@ -56,7 +58,7 @@ typedef struct W_FilePanel {
 #define WP_OPEN         0
 #define WP_SAVE         1
 
-#define PWIDTH		320
+#define PWIDTH		330
 #define PHEIGHT 	360
 
 static void listDirectoryOnColumn(WMFilePanel *panel, int column, char *path);
@@ -68,10 +70,14 @@ static void fillColumn(WMBrowserDelegate *self, WMBrowser *bPtr, int column,
 
 static void deleteFile();
 
+static void createDir();
+
 static void goHome();
 
 static void goFloppy();
 
+static void goUnmount();
+
 static void buttonClick();
 
 static char *getCurrentFileName(WMFilePanel *panel);
@@ -92,22 +98,22 @@ static WMBrowserDelegate browserDelegate = {
 static int
 closestListItem(WMList *list, char *text, Bool exact)
 {
-    WMListItem *item = WMGetListItem(list, 0);
-    int i = 0;
-    int len = strlen(text);
+    WMListItem *item;
+    WMArray *items = WMGetListItems(list);
+    int i, len = strlen(text);
 
     if (len==0)
 	return -1;
 
-    while (item) {
+    for(i=0; i<WMGetArrayItemCount(items); i++) {
+        item = WMGetFromArray(items, i);
         if (strlen(item->text) >= len &&
             ((exact && strcmp(item->text, text)==0) ||
              (!exact && strncmp(item->text, text, len)==0))) {
 	    return i;
 	}
-	item = item->nextPtr;
-	i++;
     }
+
     return -1;
 }
 
@@ -145,14 +151,14 @@ textChangedObserver(void *observerData, WMNotification *notification)
 
 	    WMInsertTextFieldText(panel->fileField, &item->text[textLen],
                                   textLen);
-	    WMSetTextFieldCursorPosition(panel->fileField, itemTextLen);
 	    range.position = textLen;
 	    range.count = itemTextLen - textLen;
 	    WMSelectTextFieldRange(panel->fileField, range);
+            /*WMSetTextFieldCursorPosition(panel->fileField, itemTextLen);*/
 	}
     }
 
-    free(text);
+    wfree(text);
 }
 
 
@@ -207,10 +213,12 @@ makeFilePanel(WMScreen *scrPtr, char *name, char *title)
     WMSetFrameRelief(fPtr->line, WRGroove);
 
     fPtr->browser = WMCreateBrowser(fPtr->win);
+    WMSetBrowserAllowEmptySelection(fPtr->browser, True);
     WMSetBrowserDelegate(fPtr->browser, &browserDelegate);
     WMSetBrowserAction(fPtr->browser, browserClick, fPtr);
     WMSetBrowserDoubleAction(fPtr->browser, browserDClick, fPtr);
     WMMoveWidget(fPtr->browser, 7, 72);
+    WMResizeWidget(fPtr->browser, PWIDTH-14,200);
     WMHangData(fPtr->browser, fPtr);
 
     fPtr->nameLabel = WMCreateLabel(fPtr->win);
@@ -229,8 +237,8 @@ makeFilePanel(WMScreen *scrPtr, char *name, char *title)
 			      fPtr->fileField);
     
     fPtr->okButton = WMCreateCommandButton(fPtr->win);
-    WMMoveWidget(fPtr->okButton, 230, 325);
-    WMResizeWidget(fPtr->okButton, 80, 28);
+    WMMoveWidget(fPtr->okButton, 245, 325);
+    WMResizeWidget(fPtr->okButton, 75, 28);
     WMSetButtonText(fPtr->okButton, "OK");
     WMSetButtonImage(fPtr->okButton, scrPtr->buttonArrow);
     WMSetButtonAltImage(fPtr->okButton, scrPtr->pushedButtonArrow);
@@ -238,36 +246,52 @@ makeFilePanel(WMScreen *scrPtr, char *name, char *title)
     WMSetButtonAction(fPtr->okButton, buttonClick, fPtr);
     
     fPtr->cancelButton = WMCreateCommandButton(fPtr->win);
-    WMMoveWidget(fPtr->cancelButton, 140, 325);
-    WMResizeWidget(fPtr->cancelButton, 80, 28);
+    WMMoveWidget(fPtr->cancelButton, 165, 325);
+    WMResizeWidget(fPtr->cancelButton, 75, 28);
     WMSetButtonText(fPtr->cancelButton, "Cancel");
     WMSetButtonAction(fPtr->cancelButton, buttonClick, fPtr);
 
-    fPtr->homeButton = WMCreateCommandButton(fPtr->win);
-    WMMoveWidget(fPtr->homeButton, 55, 325);
-    WMResizeWidget(fPtr->homeButton, 28, 28);
-    WMSetButtonImagePosition(fPtr->homeButton, WIPImageOnly);
-    WMSetButtonImage(fPtr->homeButton, scrPtr->homeIcon);
-    WMSetButtonAltImage(fPtr->homeButton, scrPtr->altHomeIcon);
-    WMSetButtonAction(fPtr->homeButton, goHome, fPtr);
-
     fPtr->trashcanButton = WMCreateCommandButton(fPtr->win);
-    WMMoveWidget(fPtr->trashcanButton, 25, 325);
+    WMMoveWidget(fPtr->trashcanButton, 7, 325);
     WMResizeWidget(fPtr->trashcanButton, 28, 28);
     WMSetButtonImagePosition(fPtr->trashcanButton, WIPImageOnly);
     WMSetButtonImage(fPtr->trashcanButton, scrPtr->trashcanIcon);
     WMSetButtonAltImage(fPtr->trashcanButton, scrPtr->altTrashcanIcon);
     WMSetButtonAction(fPtr->trashcanButton, deleteFile, fPtr);
 
+    fPtr->createDirButton = WMCreateCommandButton(fPtr->win);
+    WMMoveWidget(fPtr->createDirButton, 37, 325);
+    WMResizeWidget(fPtr->createDirButton, 28, 28);
+    WMSetButtonImagePosition(fPtr->createDirButton, WIPImageOnly);
+    WMSetButtonImage(fPtr->createDirButton, scrPtr->createDirIcon);
+    WMSetButtonAltImage(fPtr->createDirButton, scrPtr->altCreateDirIcon);
+    WMSetButtonAction(fPtr->createDirButton, createDir, fPtr);
+
+    fPtr->homeButton = WMCreateCommandButton(fPtr->win);
+    WMMoveWidget(fPtr->homeButton, 67, 325);
+    WMResizeWidget(fPtr->homeButton, 28, 28);
+    WMSetButtonImagePosition(fPtr->homeButton, WIPImageOnly);
+    WMSetButtonImage(fPtr->homeButton, scrPtr->homeIcon);
+    WMSetButtonAltImage(fPtr->homeButton, scrPtr->altHomeIcon);
+    WMSetButtonAction(fPtr->homeButton, goHome, fPtr);
+
     fPtr->disketteButton = WMCreateCommandButton(fPtr->win);
-    WMMoveWidget(fPtr->disketteButton, 85, 325);
+    WMMoveWidget(fPtr->disketteButton, 97, 325);
     WMResizeWidget(fPtr->disketteButton, 28, 28);
     WMSetButtonImagePosition(fPtr->disketteButton, WIPImageOnly);
     WMSetButtonImage(fPtr->disketteButton, scrPtr->disketteIcon);
     WMSetButtonAltImage(fPtr->disketteButton, scrPtr->altDisketteIcon);
     WMSetButtonAction(fPtr->disketteButton, goFloppy, fPtr);
 
-    /*xxxxxxxxxxxx***/
+    fPtr->unmountButton = WMCreateCommandButton(fPtr->win);
+    WMMoveWidget(fPtr->unmountButton, 127, 325);
+    WMResizeWidget(fPtr->unmountButton, 28, 28);
+    WMSetButtonImagePosition(fPtr->unmountButton, WIPImageOnly);
+    WMSetButtonImage(fPtr->unmountButton, scrPtr->unmountIcon);
+    WMSetButtonAltImage(fPtr->unmountButton, scrPtr->altUnmountIcon);
+    WMSetButtonAction(fPtr->unmountButton, goUnmount, fPtr);
+    WMSetButtonEnabled(fPtr->unmountButton, False);
+
 
     WMRealizeWidget(fPtr->win);
     WMMapSubwidgets(fPtr->win);
@@ -277,6 +301,10 @@ makeFilePanel(WMScreen *scrPtr, char *name, char *title)
 
     WMLoadBrowserColumnZero(fPtr->browser);
     
+    WMSetWindowInitialPosition(fPtr->win,
+	       (scrPtr->rootView->size.width - WMWidgetWidth(fPtr->win))/2,
+               (scrPtr->rootView->size.height - WMWidgetHeight(fPtr->win))/2);
+
     fPtr->flags.canChooseFiles = 1;
     fPtr->flags.canChooseDirectories = 1;
     fPtr->flags.autoCompletion = 1;
@@ -333,7 +361,7 @@ WMFreeFilePanel(WMFilePanel *panel)
     WMRemoveNotificationObserver(panel);
     WMUnmapWidget(panel->win);
     WMDestroyWidget(panel->win);
-    free(panel);
+    wfree(panel);
 }
 
 
@@ -352,7 +380,6 @@ WMRunModalFilePanelForDirectory(WMFilePanel *panel, WMWindow *owner,
 
     WMSetFilePanelDirectory(panel, path);
 
-    panel->flags.done = 0;
     switch(panel->flags.panelType) {
     case WP_OPEN:
         if (fileTypes)
@@ -371,20 +398,11 @@ WMRunModalFilePanelForDirectory(WMFilePanel *panel, WMWindow *owner,
         break;
     }
 
-    WMSetWindowUPosition(panel->win, 
-		 (scr->rootView->size.width - WMWidgetWidth(panel->win))/2,
-		 (scr->rootView->size.height - WMWidgetHeight(panel->win))/2);
     WMSetLabelText(panel->titleLabel, name);
 
-    scr->modalView = W_VIEW(panel->win);
     WMMapWidget(panel->win);
 
-    scr->modal = 1;
-    while (!panel->flags.done) {	
-	WMNextEvent(scr->display, &event);
-	WMHandleEvent(&event);
-    }
-    scr->modal = 0;
+    WMRunModalLoop(scr, W_VIEW(panel->win));
 
     /* Must withdraw window because the next time we map
      * it, it might have a different transient owner. 
@@ -502,6 +520,19 @@ filterFileName(WMFilePanel *panel, char *file, Bool isDirectory)
 }
 
 
+#define CAST(item) (*((WMListItem**)item))
+static int
+comparer(const void *a, const void *b)
+{
+    if (CAST(a)->isBranch == CAST(b)->isBranch)
+      return (strcmp(CAST(a)->text, CAST(b)->text));
+    if (CAST(a)->isBranch)
+      return (-1);
+    return (1);
+}
+#undef CAST
+
+
 static void
 listDirectoryOnColumn(WMFilePanel *panel, int column, char *path)
 {
@@ -548,10 +579,10 @@ listDirectoryOnColumn(WMFilePanel *panel, int column, char *path)
 	    isDirectory = S_ISDIR(stat_buf.st_mode);
 	    
 	    if (filterFileName(panel, dentry->d_name, isDirectory))
-		WMAddSortedBrowserItem(bPtr, column, dentry->d_name, 
-					  isDirectory);
+		WMInsertBrowserItem(bPtr, column, -1, dentry->d_name, isDirectory);
 	}
     }
+    WMSortBrowserColumnWithComparer(bPtr, column, comparer);
 
     closedir(dir);
 }
@@ -571,7 +602,7 @@ fillColumn(WMBrowserDelegate *self, WMBrowser *bPtr, int column, WMList *list)
 
     panel = WMGetHangedData(bPtr);
     listDirectoryOnColumn(panel, column, path);
-    free(path);
+    wfree(path);
 }
 
 
@@ -594,84 +625,211 @@ browserClick(WMBrowser *bPtr, WMFilePanel *panel)
     }
 }
 
-#define ERROR_PANEL(s)  err_str = wmalloc(strlen(file)+strlen(s)); \
-                        sprintf(err_str, s, file);  \
-                        WMRunAlertPanel(WMWidgetScreen(panel->win), panel->win, \
-                        "Error", err_str, "OK", NULL, NULL);
+
+static void
+showError(WMScreen *scr, WMWindow *owner, char *s, char *file)
+{
+    char *errStr;
+
+    if (file) {
+        errStr = wmalloc(strlen(file)+strlen(s));
+        sprintf(errStr, s, file);
+    } else {
+        errStr = wstrdup(s);
+    }
+    WMRunAlertPanel(scr, owner, "Error", errStr, "OK", NULL, NULL);
+    wfree(errStr);
+}
+
+
+static void
+createDir(WMButton *bPre, WMFilePanel *panel)
+{
+    char *dirName, *directory, *file, *s;
+    WMScreen *scr = WMWidgetScreen(panel->win);
+
+    dirName = WMRunInputPanel(scr, panel->win, "Create Directory",
+                              "Enter directory name", "", "OK", "Cancel");
+    if (!dirName)
+        return;
+
+    directory = getCurrentFileName(panel);
+    s = strrchr(directory,'/');
+    if (s) s[1] = 0;
+
+    if (dirName[0] == '/') {
+        directory[0] = 0;
+    } else {
+        while ((s = strstr(directory,"//"))) {
+            int i;
+            for (i = 2;s[i] == '/';i++);
+            strcpy(s, &s[i-1]);
+        }
+        if ((s = strrchr(directory, '/')) && !s[1]) s[0] = 0;
+    }
+    while ((s = strstr(dirName,"//"))) {
+        int i;
+        for (i = 2;s[i] == '/';i++);
+        strcpy(s, &s[i-1]);
+    }
+    if ((s = strrchr(dirName, '/')) && !s[1]) s[0] = 0;
+
+    file = wmalloc(strlen(dirName)+strlen(directory)+1);
+    sprintf(file, "%s/%s", directory, dirName);
+    while ((s = strstr(file,"//"))) {
+        int i;
+        for (i = 2;s[i] == '/';i++);
+        strcpy(s, &s[i-1]);
+    }
+
+    if (mkdir(file,0xfff) != 0) {
+        switch (errno) {
+            case EACCES:
+                showError(scr, panel->win, "Permission denied.", NULL);
+                break;
+            case EEXIST:
+                showError(scr, panel->win, "'%s' already existes.", file);
+                break;
+            case ENOENT:
+                showError(scr, panel->win, "Path does not exist.", NULL);
+        }
+    }
+    else WMSetFilePanelDirectory(panel, file);
+
+    wfree(dirName);
+    wfree(directory);
+    wfree(file);
+}
 
 static void
 deleteFile(WMButton *bPre, WMFilePanel *panel)
 {
     char *file;
-    char *buffer;
-    char *err_str;
-
-    WMFilePanel *deletePanel;
+    char *buffer, *s;
+    struct stat filestat;
+    WMScreen *scr = WMWidgetScreen(panel->win);
 
     file = getCurrentFileName(panel);
-    if (file[strlen(file)-1] == '/') {
-        ERROR_PANEL("%s is a directory.");
-        free(err_str);
-        free(file);
-        return;
+
+    while ((s = strstr(file,"//"))) {
+        int i;
+        for (i = 2;s[i] == '/';i++);
+        strcpy(s, &s[i-1]);
     }
-    buffer = wmalloc(strlen(file)+15);
-    sprintf(buffer,"Delete file %s ?\x0",file);
+    if (strlen(file) > 1 && (s = strrchr(file, '/')) && !s[1]) s[0] = 0;
+
+    if (stat(file,&filestat)) {
+        switch (errno) {
+            case ENOENT:
+                showError(scr, panel->win, "'%s' does not exist.", file);
+                break;
+            case EACCES:
+                showError(scr, panel->win, "Permission denied.", NULL);
+                break;
+            case ENOMEM:
+                showError(scr, panel->win,
+                        "Insufficient memory available.", NULL);
+                break;
+            case EROFS:
+                showError(scr, panel->win,
+                        "'%s' is on a read-only filesystem.", file);
+                break;
+            default:
+                showError(scr, panel->win, "Can not delete '%s'.", file);
+        }
+        wfree(file);
+        return;
+    } else if (S_ISDIR(filestat.st_mode)) {
+        buffer = wmalloc(strlen(file)+20);
+        sprintf(buffer,"Delete directory %s ?",file);
+    } else {
+        buffer = wmalloc(strlen(file)+15);
+        sprintf(buffer,"Delete file %s ?",file);
+    }
+
     if (!WMRunAlertPanel(WMWidgetScreen(panel->win), panel->win,
                 "Warning", buffer, "OK", "Cancel", NULL)) {
-        int rem_stat;
-        if (rem_stat = remove(file)) {
+        if (S_ISDIR(filestat.st_mode)) {
+            if (rmdir(file) != 0) {
+                switch (errno) {
+                    case EACCES:
+                        showError(scr, panel->win, "Permission denied.", NULL);
+                        break;
+                    case ENOENT:
+                        showError(scr, panel->win, "Directory '%s' does not exist.", file);
+                        break;
+                    case ENOTEMPTY:
+                        showError(scr, panel->win, "Directory '%s' is not empty.", file);
+                        break;
+                    case EBUSY:
+                        showError(scr, panel->win, "Directory '%s' is busy.", file);
+                        break;
+                    default:
+                        showError(scr, panel->win, "Can not delete '%s'.", file);
+                }
+            } else {
+                char *s = strrchr(file,'/');
+                if (s) s[0] = 0;
+                WMSetFilePanelDirectory(panel, file);
+            }
+        } else if (remove(file) != 0) {
             switch (errno) {
                 case EISDIR:
-                    ERROR_PANEL("%s is a directory.");
+                    showError(scr, panel->win, "'%s' is a directory.", file);
                     break;
                 case ENOENT:
-                    ERROR_PANEL("%s does not exist.");
+                    showError(scr, panel->win, "'%s' does not exist.", file);
                     break;
                 case EACCES:
-                    ERROR_PANEL("Permission denied.");
+                    showError(scr, panel->win, "Permission denied.", NULL);
                     break;
                 case ENOMEM:
-                    ERROR_PANEL("Insufficient kernel memory was available.");
+                    showError(scr, panel->win,
+                            "Insufficient memory available.", NULL);
                     break;
                 case EROFS:
-                    ERROR_PANEL("%s refers to a file on a read-only filesystem.");
+                    showError(scr, panel->win,
+                            "'%s' is on a read-only filesystem.", file);
                     break;
                 default:
-                    ERROR_PANEL("Can not delete %s.");
+                    showError(scr, panel->win, "Can not delete '%s'.", file);
             }
-            free(err_str);
-        }
-        else {
+        } else {
             char *s = strrchr(file,'/');
             if (s) s[1] = 0;
             WMSetFilePanelDirectory(panel, file);
         }
     }
-    free(buffer);
-    free(file);
+    wfree(buffer);
+    wfree(file);
 }
 
+static void
+goUnmount(WMButton *bPtr, WMFilePanel *panel)
+{
+}
+
+
 static void
 goFloppy(WMButton *bPtr, WMFilePanel *panel)
 {
-    char *file, *err_str;
     struct stat filestat;
+    WMScreen *scr = WMWidgetScreen(panel->win);
 
-    /* home is statically allocated. Don't free it! */
-    if (stat("/floppy",&filestat)) {
-        ERROR_PANEL("An error occured browsing /floppy.");
-        free(err_str);
+    if (stat(WINGsConfiguration.floppyPath, &filestat)) {
+        showError(scr, panel->win, "An error occured browsing '%s'.",
+                  WINGsConfiguration.floppyPath);
         return;
     } else if (!S_ISDIR(filestat.st_mode)) {
-        ERROR_PANEL("/floppy is not a directory.");
-        free(err_str);
+        showError(scr, panel->win, "'%s' is not a directory.",
+                  WINGsConfiguration.floppyPath);
         return;
     }
     
-    WMSetFilePanelDirectory(panel, "/floppy/");
+    WMSetFilePanelDirectory(panel, WINGsConfiguration.floppyPath);
 }
 
+
 static void
 goHome(WMButton *bPtr, WMFilePanel *panel)
 {
@@ -706,12 +864,16 @@ handleEvents(XEvent *event, void *data)
             WMResizeWidget(pPtr->fileField, newWidth-60-10, 24);
             WMMoveWidget(pPtr->nameLabel, 7, newHeight-(PHEIGHT-282));
             WMMoveWidget(pPtr->fileField, 60, newHeight-(PHEIGHT-278));
-            WMMoveWidget(pPtr->okButton, newWidth-(PWIDTH-230),
-                         newHeight-(PHEIGHT-325));
-            WMMoveWidget(pPtr->cancelButton, newWidth-(PWIDTH-140),
-                         newHeight-(PHEIGHT-325));
-            WMMoveWidget(pPtr->homeButton, 55, newHeight-(PHEIGHT-325));
-            WMMoveWidget(pPtr->trashcanButton, 25, newHeight-(PHEIGHT-325));
+            WMMoveWidget(pPtr->okButton, newWidth-(PWIDTH-245),
+                    newHeight-(PHEIGHT-325));
+            WMMoveWidget(pPtr->cancelButton, newWidth-(PWIDTH-165),
+                    newHeight-(PHEIGHT-325));
+
+            WMMoveWidget(pPtr->trashcanButton, 7, newHeight-(PHEIGHT-325));
+            WMMoveWidget(pPtr->createDirButton, 37, newHeight-(PHEIGHT-325));
+            WMMoveWidget(pPtr->homeButton, 67, newHeight-(PHEIGHT-325));
+            WMMoveWidget(pPtr->disketteButton, 97, newHeight-(PHEIGHT-325));
+            WMMoveWidget(pPtr->unmountButton, 127, newHeight-(PHEIGHT-325));
 
 	    newColumnCount = (newWidth - 14) / 140;
 	    WMSetBrowserMaxVisibleColumns(pPtr->browser, newColumnCount);
@@ -740,8 +902,8 @@ getCurrentFileName(WMFilePanel *panel)
         } else
            strcpy(tmp, file);
 
-	free(file);
-	free(path);
+	wfree(file);
+	wfree(path);
 	return tmp;
     } else {
 	return path;
@@ -759,7 +921,7 @@ validOpenFile(WMFilePanel *panel)
 
     if (file[0] != '\0')
         haveFile = 1;
-    free(file);
+    wfree(file);
 
     col = WMGetBrowserSelectedColumn(panel->browser);
     item = WMGetBrowserSelectedItemInColumn(panel->browser, col);
@@ -798,10 +960,10 @@ buttonClick(WMButton *bPtr, WMFilePanel *panel)
 		WMRunAlertPanel(WMWidgetScreen(panel->win), panel->win,
 				"Error", "File does not exist.",
 				"Ok", NULL, NULL);
-		free(file);
+		wfree(file);
 		return;
 	    }
-	    free(file);
+	    wfree(file);
 	}
 	panel->flags.canceled = 0;
     } else
@@ -809,7 +971,7 @@ buttonClick(WMButton *bPtr, WMFilePanel *panel)
 
     range.count = range.position = 0;
     WMSelectTextFieldRange(panel->fileField, range);
-    panel->flags.done = 1;
+    WMBreakModalLoop(WMWidgetScreen(bPtr));
 }
 
 
