@@ -6,7 +6,7 @@
 #include "WUtil.h"
 
 #include <ctype.h>
-#include <strings.h>
+#include <string.h>
 
 
 typedef struct W_FontPanel {
