@@ -3,7 +3,7 @@
 #include "WINGsP.h"
 
 #include <X11/Xutil.h>
-
+#include <X11/keysym.h>
 #include <X11/cursorfont.h>
 
 /********** data ************/
@@ -515,6 +515,20 @@ loadPixmaps(WMScreen *scr)
 }
 
 
+WMScreen*
+WMOpenScreen()
+{
+    Display *dpy = XOpenDisplay("");
+    
+    if (!dpy) {
+	wwarning("WINGs: could not open display %s",
+		 XDisplayName(""));
+	return NULL;
+    }
+    
+    return WMCreateSimpleApplicationScreen(dpy);
+}
+
 
 WMScreen*
 WMCreateSimpleApplicationScreen(Display *display)
@@ -559,10 +573,17 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 	    "XdndDrop",
 	    "XdndFinished",
 	    "XdndTypeList",
+	    "XdndActionCopy",
+	    "XdndActionMove",
+	    "XdndActionLink",
+	    "XdndActionAsk",
+	    "XdndActionPrivate",
 	    "XdndStatus",
+	    "_WINGS_DND_MOUSE_OFFSET",
 	    "WM_STATE"
     };
     Atom atoms[sizeof(atomNames)/sizeof(char*)];
+    int i = 0;
 
     if (!initialized) {
 
@@ -595,6 +616,42 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 
     scrPtr->fontCache = WMCreateHashTable(WMStringPointerHashCallbacks);
 
+    scrPtr->ignoredModifierMask = 0;
+    {
+	int i;
+	XModifierKeymap *modmap;
+	KeyCode nlock, slock;
+	static int mask_table[8] = {
+	    ShiftMask,LockMask,ControlMask,Mod1Mask,
+		Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask
+	};
+	unsigned int numLockMask, scrollLockMask;
+	
+	nlock = XKeysymToKeycode(display, XK_Num_Lock);
+	slock = XKeysymToKeycode(display, XK_Scroll_Lock);
+	
+	/*
+	 * Find out the masks for the NumLock and ScrollLock modifiers,
+	 * so that we can bind the grabs for when they are enabled too.
+	 */
+	modmap = XGetModifierMapping(display);
+	
+	if (modmap!=NULL && modmap->max_keypermod>0) {
+	    for (i=0; i<8*modmap->max_keypermod; i++) {
+		if (modmap->modifiermap[i]==nlock && nlock!=0)
+		    numLockMask = mask_table[i/modmap->max_keypermod];
+		else if (modmap->modifiermap[i]==slock && slock!=0)
+		    scrollLockMask = mask_table[i/modmap->max_keypermod];
+	    }
+	}
+	
+	if (modmap)
+	    XFreeModifiermap(modmap);
+	
+	
+	scrPtr->ignoredModifierMask = numLockMask|scrollLockMask|LockMask;
+    }
+    
     /* initially allocate some colors */
     WMWhiteColor(scrPtr);
     WMBlackColor(scrPtr);
@@ -766,26 +823,35 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 	}
     }
 #endif
-    
-    scrPtr->attribsAtom = atoms[0];
-    
-    scrPtr->deleteWindowAtom = atoms[1];
 
-    scrPtr->protocolsAtom = atoms[2];
+    i = 0;
+    scrPtr->attribsAtom = atoms[i++];
+
+    scrPtr->deleteWindowAtom = atoms[i++];
+
+    scrPtr->protocolsAtom = atoms[i++];
     
-    scrPtr->clipboardAtom = atoms[3];
+    scrPtr->clipboardAtom = atoms[i++];
 
-    scrPtr->xdndAwareAtom = atoms[4];
-    scrPtr->xdndSelectionAtom = atoms[5];
-    scrPtr->xdndEnterAtom = atoms[6];
-    scrPtr->xdndLeaveAtom = atoms[7];
-    scrPtr->xdndPositionAtom = atoms[8];
-    scrPtr->xdndDropAtom = atoms[9];
-    scrPtr->xdndFinishedAtom = atoms[10];
-    scrPtr->xdndTypeListAtom = atoms[11];
-    scrPtr->xdndStatusAtom = atoms[12];
+    scrPtr->xdndAwareAtom = atoms[i++];
+    scrPtr->xdndSelectionAtom = atoms[i++];
+    scrPtr->xdndEnterAtom = atoms[i++];
+    scrPtr->xdndLeaveAtom = atoms[i++];
+    scrPtr->xdndPositionAtom = atoms[i++];
+    scrPtr->xdndDropAtom = atoms[i++];
+    scrPtr->xdndFinishedAtom = atoms[i++];
+    scrPtr->xdndTypeListAtom = atoms[i++];
+    scrPtr->xdndStatusAtom = atoms[i++];
 
-    scrPtr->wmStateAtom = atoms[13];
+    scrPtr->xdndActionCopy = atoms[i++];
+    scrPtr->xdndActionMove = atoms[i++];
+    scrPtr->xdndActionLink = atoms[i++];
+    scrPtr->xdndActionAsk = atoms[i++];
+    scrPtr->xdndActionPrivate = atoms[i++];
+
+    scrPtr->wmIconDragOffsetAtom = atoms[i++];
+
+    scrPtr->wmStateAtom = atoms[i++];
 	
     scrPtr->rootView = W_CreateRootView(scrPtr);
 
@@ -799,6 +865,24 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
 }
 
 
+
+void
+WMSetWidgetDefaultFont(WMScreen *scr, WMFont *font)
+{
+    WMReleaseFont(scr->normalFont);
+    scr->normalFont = WMRetainFont(font);
+}
+
+
+void
+WMSetWidgetDefaultBoldFont(WMScreen *scr, WMFont *font)
+{
+    WMReleaseFont(scr->boldFont);
+    scr->boldFont = WMRetainFont(font);
+}
+
+
+
 void 
 WMHangData(WMWidget *widget, void *data)
 {
@@ -817,6 +901,7 @@ WMGetHangedData(WMWidget *widget)
 void
 WMDestroyWidget(WMWidget *widget)
 {
+    W_UnmapView(W_VIEW(widget));
     W_DestroyView(W_VIEW(widget));
 }
 
@@ -846,6 +931,13 @@ WMMapWidget(WMWidget *w)
 }
 
 
+void
+WMReparentWidget(WMWidget *w, WMWidget *newParent, int x, int y)
+{
+    W_ReparentView(W_VIEW(w), newParent, x, y);
+}
+
+
 static void
 makeChildrenAutomap(W_View *view, int flag)
 {
@@ -892,10 +984,33 @@ WMUnmapWidget(WMWidget *w)
 }
 
 
+Bool
+WMWidgetIsMapped(WMWidget *w)
+{
+    return W_VIEW(w)->flags.mapped;
+}
+
+
 void
 WMSetWidgetBackgroundColor(WMWidget *w, WMColor *color)
 {    
     W_SetViewBackgroundColor(W_VIEW(w), color);
+    if (W_VIEW(w)->flags.mapped)
+	WMRedisplayWidget(w);
+}
+
+
+void
+WMRaiseWidget(WMWidget *w)
+{
+     W_RaiseView(W_VIEW(w));
+}
+
+
+void
+WMLowerWidget(WMWidget *w)
+{
+     W_LowerView(W_VIEW(w));
 }
 
 
@@ -973,6 +1088,35 @@ WMScreenMainLoop(WMScreen *scr)
 }
 
 
+void
+WMBreakModalLoop(WMScreen *scr)
+{
+    scr->modalLoop = 0;
+}
+
+
+void
+WMRunModalLoop(WMScreen *scr, WMView *view)
+{
+    /* why is scr passed if is determined from the view? */
+    /*WMScreen *scr = view->screen;*/
+    int oldModalLoop = scr->modalLoop;
+    WMView *oldModalView = scr->modalView;
+
+    scr->modalView = view;
+
+    scr->modalLoop = 1;
+    while (scr->modalLoop) {
+	XEvent event;
+	
+	WMNextEvent(scr->display, &event);
+	WMHandleEvent(&event);
+    }
+
+    scr->modalView = oldModalView;
+    scr->modalLoop = oldModalLoop;
+}
+
 
 Display*
 WMScreenDisplay(WMScreen *scr)
