@@ -14,6 +14,10 @@
 #define AUTOSCROLL_DELAY 		40
 
 
+char *WMScrollerDidScrollNotification = "WMScrollerDidScrollNotification";
+
+
+
 typedef struct W_Scroller {
     W_Class widgetClass;
     W_View *view;
@@ -672,6 +676,8 @@ handlePush(Scroller *sPtr, int pushX, int pushY, int alternate)
     
     if (doAction && sPtr->action) {
 	(*sPtr->action)(sPtr, sPtr->clientData);
+	
+	WMPostNotificationName(WMScrollerDidScrollNotification, sPtr, NULL);
     }
 }
 
@@ -751,6 +757,8 @@ handleMotion(Scroller *sPtr, int mouseX, int mouseY)
 	WMSetScrollerParameters(sPtr, newFloatValue, sPtr->knobProportion);
 	if (sPtr->action) {
 	    (*sPtr->action)(sPtr, sPtr->clientData);
+	    WMPostNotificationName(WMScrollerDidScrollNotification, sPtr, 
+				   NULL);	    
 	}
     } else {
 	int part;
@@ -780,6 +788,7 @@ autoScroll(void *clientData)
     
     if (sPtr->action) {
 	(*sPtr->action)(sPtr, sPtr->clientData);
+	WMPostNotificationName(WMScrollerDidScrollNotification, sPtr, NULL);
     }
     sPtr->timerID= WMAddTimerHandler(AUTOSCROLL_DELAY, autoScroll, clientData);
 }
@@ -814,15 +823,45 @@ handleActionEvents(XEvent *event, void *data)
 	break;
 	
      case ButtonPress:
-	/* FIXME: change Mod1Mask with something else */
-	handlePush(sPtr, event->xbutton.x, event->xbutton.y,
-		   (event->xbutton.state & Mod1Mask)
-		   ||event->xbutton.button==Button2);
-	/* continue scrolling if pushed on the buttons */
-	if (sPtr->flags.hitPart == WSIncrementLine
-	    || sPtr->flags.hitPart == WSDecrementLine) {
-	    sPtr->timerID = WMAddTimerHandler(AUTOSCROLL_INITIAL_DELAY,
-					      autoScroll, sPtr);
+        /* FIXME: change Mod1Mask with something else */
+        if (sPtr->flags.documentFullyVisible)
+            break;
+	if (event->xbutton.button==WINGsConfiguration.mouseWheelUp) {
+            if (event->xbutton.state & ControlMask) {
+                sPtr->flags.hitPart = WSDecrementPage;
+            } else if (event->xbutton.state & ShiftMask) {
+                sPtr->flags.hitPart = WSDecrementLine;
+            } else {
+                sPtr->flags.hitPart = WSDecrementWheel;
+            }
+	    if (sPtr->action) {
+		(*sPtr->action)(sPtr, sPtr->clientData);
+		WMPostNotificationName(WMScrollerDidScrollNotification, sPtr, 
+				       NULL);
+	    }
+	} else if (event->xbutton.button==WINGsConfiguration.mouseWheelDown) {
+            if (event->xbutton.state & ControlMask) {
+                sPtr->flags.hitPart = WSIncrementPage;
+            } else if (event->xbutton.state & ShiftMask) {
+                sPtr->flags.hitPart = WSIncrementLine;
+            } else {
+                sPtr->flags.hitPart = WSIncrementWheel;
+            }
+	    if (sPtr->action) {
+		(*sPtr->action)(sPtr, sPtr->clientData);
+		WMPostNotificationName(WMScrollerDidScrollNotification, sPtr, 
+				       NULL);
+	    }
+	} else {
+	    handlePush(sPtr, event->xbutton.x, event->xbutton.y,
+			   (event->xbutton.state & Mod1Mask)
+			   ||event->xbutton.button==Button2);
+	    /* continue scrolling if pushed on the buttons */
+	    if (sPtr->flags.hitPart == WSIncrementLine
+		|| sPtr->flags.hitPart == WSDecrementLine) {
+		sPtr->timerID = WMAddTimerHandler(AUTOSCROLL_INITIAL_DELAY,
+						      autoScroll, sPtr);
+	    }
 	}
 	break;
 
@@ -830,6 +869,8 @@ handleActionEvents(XEvent *event, void *data)
 	if (sPtr->flags.draggingKnob) {
 	    if (sPtr->action) {
 		(*sPtr->action)(sPtr, sPtr->clientData);
+		WMPostNotificationName(WMScrollerDidScrollNotification, sPtr, 
+				       NULL);
 	    }
 	}
 	if (sPtr->timerID) {
