@@ -36,7 +36,7 @@ static void paintScrollView(ScrollView *sPtr);
 static void handleEvents(XEvent *event, void *data);
 static void handleViewportEvents(XEvent *event, void *data);
 static void resizeScrollView();
-
+static void updateScrollerProportion();
 
 W_ViewDelegate _ScrollViewViewDelegate = {
     NULL,
@@ -64,13 +64,14 @@ WMCreateScrollView(WMWidget *parent)
 	return NULL;
     }
     sPtr->viewport = W_CreateView(sPtr->view);
-    if (!sPtr->view) {
+    if (!sPtr->viewport) {
 	W_DestroyView(sPtr->view);
 	wfree(sPtr);
 	return NULL;
     }
     sPtr->view->self = sPtr;
-
+    sPtr->viewport->self = sPtr;
+    
     sPtr->view->delegate = &_ScrollViewViewDelegate;
 
     sPtr->viewport->flags.mapWhenRealized = 1;
@@ -88,6 +89,45 @@ WMCreateScrollView(WMWidget *parent)
 }
 
 
+static void
+applyScrollerValues(WMScrollView *sPtr)
+{
+    int x, y;
+
+    if (sPtr->contentView == NULL)
+	return;
+    
+    if (sPtr->flags.hasHScroller) {
+	float v = WMGetScrollerValue(sPtr->hScroller);
+	int size;
+
+	size = sPtr->contentView->size.width-sPtr->viewport->size.width;
+	
+	x = v * size;
+    } else {
+	x = 0;
+    }
+    
+    if (sPtr->flags.hasVScroller) {
+	float v = WMGetScrollerValue(sPtr->vScroller);
+
+	int size;
+
+	size = sPtr->contentView->size.width-sPtr->viewport->size.width;
+	
+	y = v * size;
+    } else {
+	y = 0;
+    }
+    
+    x = WMAX(0, x);
+    y = WMAX(0, y);
+    
+    W_MoveView(sPtr->contentView, -x, -y);
+    
+    W_RaiseView(sPtr->viewport);
+}
+
 
 static void
 reorganizeInterior(WMScrollView *sPtr)
@@ -116,7 +156,7 @@ reorganizeInterior(WMScrollView *sPtr)
     }
     
     if (sPtr->flags.hasHScroller) {
-	int h = W_VIEW(sPtr->hScroller)->size.height;
+	int h = 20;
 
 	ch -= h;
 
@@ -138,7 +178,7 @@ reorganizeInterior(WMScrollView *sPtr)
     }
     
     if (sPtr->flags.hasVScroller) {
-	int w = W_VIEW(sPtr->vScroller)->size.width;
+	int w = 20;
 	cw -= w;
 	cx += w;
 	hx += w - 1;
@@ -163,7 +203,7 @@ reorganizeInterior(WMScrollView *sPtr)
 
     W_ResizeView(sPtr->viewport, cw, ch);
     W_MoveView(sPtr->viewport, cx, cy);
-    
+
     if (sPtr->flags.hasHScroller) {
 	WMResizeWidget(sPtr->hScroller, hw, 20);
 	WMMoveWidget(sPtr->hScroller, hx, hy);
@@ -172,6 +212,8 @@ reorganizeInterior(WMScrollView *sPtr)
 	WMResizeWidget(sPtr->vScroller, 20, vh);
 	WMMoveWidget(sPtr->vScroller, vx, vy);
     }
+    
+    applyScrollerValues(sPtr);
 }
 
 
@@ -179,6 +221,7 @@ static void
 resizeScrollView(W_ViewDelegate *self, WMView *view)
 {
     reorganizeInterior(view->self);
+    updateScrollerProportion(view->self);
 }
 
 
@@ -203,11 +246,11 @@ WMResizeScrollViewContent(WMScrollView *sPtr, unsigned int width,
     }
     
     if (sPtr->flags.hasVScroller) {
-	w -= W_VIEW(sPtr->hScroller)->size.width;
+	width -= W_VIEW(sPtr->hScroller)->size.width;
 	WMResizeWidget(sPtr->vScroller, 20, h);
     }    
     if (sPtr->flags.hasHScroller) {
-	h -= W_VIEW(sPtr->hScroller)->size.height;
+	height -= W_VIEW(sPtr->hScroller)->size.height;
 	WMResizeWidget(sPtr->hScroller, w, 20);
 	WMMoveWidget(sPtr->hScroller, x, h);
     }
@@ -236,6 +279,42 @@ WMSetScrollViewPageScroll(WMScrollView *sPtr, int amount)
 }
 
 
+WMRect 
+WMGetScrollViewVisibleRect(WMScrollView *sPtr)
+{
+    WMRect rect;
+
+    rect.pos.x = -sPtr->contentView->pos.x;
+    rect.pos.y = -sPtr->contentView->pos.y;
+    rect.size = sPtr->viewport->size;
+
+    return rect;
+}
+
+
+void
+WMScrollViewScrollPoint(WMScrollView *sPtr, WMPoint point)
+{
+    float xsize, ysize;
+    float xpos, ypos;
+    
+    xsize = sPtr->contentView->size.width-sPtr->viewport->size.width;
+    ysize = sPtr->contentView->size.height-sPtr->viewport->size.height;
+    
+    xpos = point.x / xsize;
+    ypos = point.y / ysize;
+
+    if (sPtr->hScroller)
+	WMSetScrollerParameters(sPtr->hScroller, xpos,
+				WMGetScrollerKnobProportion(sPtr->hScroller));
+    if (sPtr->vScroller)
+	WMSetScrollerParameters(sPtr->vScroller, ypos,
+				WMGetScrollerKnobProportion(sPtr->vScroller));
+    
+    W_MoveView(sPtr->contentView, -point.x, -point.y);
+}
+
+
 static void
 doScrolling(WMWidget *self, void *data)
 {
@@ -258,7 +337,7 @@ doScrolling(WMWidget *self, void *data)
 	vpsize = 1;
 
     switch (WMGetScrollerHitPart(self)) {
-     case WSDecrementLine:
+    case WSDecrementLine:
 	if (pos > 0) {
 	    pos-=sPtr->lineScroll;
 	    if (pos < 0)
@@ -268,7 +347,7 @@ doScrolling(WMWidget *self, void *data)
 				    WMGetScrollerKnobProportion(self));
 	}
 	break;
-     case WSIncrementLine:
+    case WSIncrementLine:
 	if (pos < size) {
 	    pos+=sPtr->lineScroll;
 	    if (pos > size)
@@ -278,12 +357,13 @@ doScrolling(WMWidget *self, void *data)
 				    WMGetScrollerKnobProportion(self));
 	}
 	break;
-     case WSKnob:
+
+    case WSKnob:
 	value = WMGetScrollerValue(self);
 	pos = value*size;
 	break;
-	
-     case WSDecrementPage:
+
+    case WSDecrementPage:
 	if (pos > 0) {
 	    pos -= vpsize;
 	    if (pos < 0)
@@ -292,9 +372,20 @@ doScrolling(WMWidget *self, void *data)
 	    WMSetScrollerParameters(self, value,
 				    WMGetScrollerKnobProportion(self));
 	}
+        break;
+
+    case WSDecrementWheel:
+	if (pos > 0) {
+	    pos -= vpsize/3;
+	    if (pos < 0)
+		pos = 0;
+	    value = (float)pos / size;
+	    WMSetScrollerParameters(self, value,
+				    WMGetScrollerKnobProportion(self));
+	}
 	break;
-	
-     case WSIncrementPage:
+
+    case WSIncrementPage:
 	if (pos < size) {
 	    pos += vpsize;
 	    if (pos > size)
@@ -304,7 +395,18 @@ doScrolling(WMWidget *self, void *data)
 				    WMGetScrollerKnobProportion(self));
 	}
 	break;
-	
+
+    case WSIncrementWheel:
+	if (pos < size) {
+	    pos += vpsize/3;
+	    if (pos > size)
+		pos = size;
+	    value = (float)pos / size;
+	    WMSetScrollerParameters(self, value,
+				    WMGetScrollerKnobProportion(self));
+	}
+	break;
+
      case WSNoPart:
      case WSKnobSlot:
 	break;
@@ -424,6 +526,8 @@ WMSetScrollViewRelief(WMScrollView *sPtr, WMReliefType type)
 {
     sPtr->flags.relief = type;
     
+    reorganizeInterior(sPtr);
+    
     if (sPtr->view->flags.mapped)
 	paintScrollView(sPtr);
     
@@ -431,6 +535,7 @@ WMSetScrollViewRelief(WMScrollView *sPtr, WMReliefType type)
 }
 
 
+
 static void
 paintScrollView(ScrollView *sPtr)
 {
@@ -483,7 +588,10 @@ handleEvents(XEvent *event, void *data)
      case Expose:
 	if (event->xexpose.count!=0)
 	    break;
-	paintScrollView(sPtr);
+	if (event->xexpose.serial == 0) /* means it's artificial */
+	    W_RedisplayView(sPtr->contentView);
+	else
+	    paintScrollView(sPtr);
 	break;
 	
      case DestroyNotify:
@@ -496,9 +604,7 @@ handleEvents(XEvent *event, void *data)
 
 static void
 destroyScrollView(ScrollView *sPtr)
-{
-    
-   
+{   
     wfree(sPtr);
 }
 
