@@ -40,7 +40,6 @@ typedef struct W_TextField {
     short offsetWidth;		       /* offset of text from border */
 
     WMRange selection;
-    WMRange prevselection;
 
     WMFont *font;
 
@@ -65,6 +64,11 @@ typedef struct W_TextField {
 	unsigned int secure:1;	       /* password entry style */
 
 	unsigned int pointerGrabbed:1;
+	
+	unsigned int ownsSelection:1;
+	
+	unsigned int waitingSelection:1; /* requested selection, but
+					  * didnt get yet */
 
 	/**/
 	unsigned int notIllegalMovement:1;
@@ -117,6 +121,20 @@ struct W_ViewDelegate _TextFieldViewDelegate = {
 };
 
 
+
+static void lostHandler(WMView *view, Atom selection, void *cdata);
+
+static WMData *requestHandler(WMView *view, Atom selection, Atom target,
+			      void *cdata, Atom *type);
+
+
+static WMSelectionProcs selectionHandler = {
+    requestHandler,
+	lostHandler,
+	NULL
+};
+
+
 #define TEXT_WIDTH(tPtr, start)	(WMWidthOfString((tPtr)->font, \
 		   &((tPtr)->text[(start)]), (tPtr)->textLen - (start) + 1))
 
@@ -124,6 +142,40 @@ struct W_ViewDelegate _TextFieldViewDelegate = {
 		   &((tPtr)->text[(start)]), (end) - (start) + 1))
 
 
+static void
+normalizeRange(TextField *tPtr, WMRange *range)
+{
+    if (range->position < 0 && range->count < 0)
+    	range->count = 0;
+
+    if (range->count == 0) {
+    	/*range->position = 0; why is this?*/
+	return;
+    }
+    
+    /* (1,-2) ~> (0,1) ; (1,-1) ~> (0,1) ; (2,-1) ~> (1,1) */
+    if (range->count < 0) { /* && range->position >= 0 */
+	if (range->position + range->count < 0) {
+	    range->count = range->position;
+	    range->position = 0;
+	} else {
+	    range->count = -range->count;
+    	    range->position -= range->count;
+	}
+    /* (-2,1) ~> (0,0) ; (-1,1) ~> (0,0) ; (-1,2) ~> (0,1) */
+    } else if (range->position < 0) { /* && range->count > 0 */
+    	if (range->position + range->count < 0) {
+	    range->position = range->count = 0;
+	} else {
+	    range->count += range->position;
+	    range->position = 0;
+	}
+    }
+    
+    if (range->position + range->count > tPtr->textLen)
+    	range->count = tPtr->textLen - range->position;
+}
+
 static void
 memmv(char *dest, char *src, int size)
 {
@@ -177,75 +229,76 @@ decrToFit(TextField *tPtr)
 #undef TEXT_WIDTH
 #undef TEXT_WIDTH2
 
-static Bool
-requestHandler(WMWidget *w, Atom selection, Atom target, Atom *type,
-	       void **value, unsigned *length, int *format) 
+
+
+static WMData*
+requestHandler(WMView *view, Atom selection, Atom target, void *cdata, 
+	       Atom *type)
 {
-    TextField *tPtr = w;
-    int count,count2;
+    TextField *tPtr = view->self;
+    int count;
     Display *dpy = tPtr->view->screen->display;
     Atom _TARGETS;
-    char *text;
-    text = XGetAtomName(tPtr->view->screen->display,target);
-    XFree(text);
-    text = XGetAtomName(tPtr->view->screen->display,selection);
-    XFree(text);
+    Atom TEXT = XInternAtom(dpy, "TEXT", False);
+    Atom COMPOUND_TEXT = XInternAtom(dpy, "COMPOUND_TEXT", False);
+    WMData *data;
 
-    *format = 32;
-    *length = 0;
-    *value = NULL;
     count = tPtr->selection.count < 0
         ? tPtr->selection.position + tPtr->selection.count
         : tPtr->selection.position;
 
-    if (target == XA_STRING ||
-            target == XInternAtom(dpy, "TEXT", False) ||
-            target == XInternAtom(dpy, "COMPOUND_TEXT", False)) {
-        *value = wstrdup(&(tPtr->text[count]));
-        *length = abs(tPtr->selection.count);
-        *format = 8;
+    if (target == XA_STRING || target == TEXT || target == COMPOUND_TEXT) {
+	
+	data = WMCreateDataWithBytes(&(tPtr->text[count]), 
+				      abs(tPtr->selection.count));
+	WMSetDataFormat(data, 8);
         *type = target;
-        return True;
+	
+        return data;
     }
     
     _TARGETS = XInternAtom(dpy, "TARGETS", False);
     if (target == _TARGETS) {
-        int *ptr;
+        Atom *ptr;
 
-        *length = 4;
-        ptr = *value = (char *) wmalloc(4 * sizeof(Atom));
+        ptr = wmalloc(4 * sizeof(Atom));
         ptr[0] = _TARGETS;
         ptr[1] = XA_STRING;
-        ptr[2] = XInternAtom(dpy, "TEXT", False);
-        ptr[3] = XInternAtom(dpy, "COMPOUND_TEXT", False);
+        ptr[2] = TEXT;
+        ptr[3] = COMPOUND_TEXT;
+	
+	data = WMCreateDataWithBytes(ptr, 4*4);
+	WMSetDataFormat(data, 32);
+	
         *type = target;
-        return True;
+        return data;
     }
-    /*
-    *target = XA_PRIMARY;
-    */
-    return False;
+
+    return NULL;
 
 }
 
 
 static void
-lostHandler(WMWidget *w, Atom selection)
+lostHandler(WMView *view, Atom selection, void *cdata)
 {
-    TextField *tPtr = (WMTextField*)w;
+    TextField *tPtr = (WMTextField*)view->self;
 
+    tPtr->flags.ownsSelection = 0;
     tPtr->selection.count = 0;
     paintTextField(tPtr);
 }
 
+
 static void
 _notification(void *observerData, WMNotification *notification)
 {
     WMTextField *to = (WMTextField*)observerData;
     WMTextField *tw = (WMTextField*)WMGetNotificationClientData(notification);
-    if (to != tw) lostHandler(to, 0);
+    if (to != tw) lostHandler(to->view, XA_PRIMARY, NULL);
 }
 
+
 WMTextField*
 WMCreateTextField(WMWidget *parent)
 {
@@ -258,7 +311,7 @@ WMCreateTextField(WMWidget *parent)
     
     tPtr->view = W_CreateView(W_VIEW(parent));
     if (!tPtr->view) {
-	free(tPtr);
+	wfree(tPtr);
 	return NULL;
     }
     tPtr->view->self = tPtr;
@@ -294,8 +347,6 @@ WMCreateTextField(WMWidget *parent)
 			 |ButtonReleaseMask|ButtonPressMask|KeyPressMask|Button1MotionMask,
 			 handleTextFieldActionEvents, tPtr);
     
-    WMCreateSelectionHandler(tPtr, XA_PRIMARY, CurrentTime, requestHandler,
-             lostHandler, NULL);
     WMAddNotificationObserver(_notification, tPtr, "_lostOwnership", tPtr);
 
 
@@ -308,6 +359,8 @@ WMCreateTextField(WMWidget *parent)
 void
 WMSetTextFieldDelegate(WMTextField *tPtr, WMTextFieldDelegate *delegate)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     tPtr->delegate = delegate;
 }
 
@@ -327,7 +380,7 @@ WMInsertTextFieldText(WMTextField *tPtr, char *text, int position)
     /* check if buffer will hold the text */
     if (len + tPtr->textLen >= tPtr->bufferSize) {
 	tPtr->bufferSize = tPtr->textLen + len + TEXT_BUFFER_INCR;
-	tPtr->text = realloc(tPtr->text, tPtr->bufferSize);
+	tPtr->text = wrealloc(tPtr->text, tPtr->bufferSize);
     }
     
     if (position < 0 || position >= tPtr->textLen) {
@@ -357,35 +410,28 @@ WMInsertTextFieldText(WMTextField *tPtr, char *text, int position)
     paintTextField(tPtr);
 }
 
-
 void
 WMDeleteTextFieldRange(WMTextField *tPtr, WMRange range)
 {
     CHECK_CLASS(tPtr, WC_TextField);
 
-    if (range.position >= tPtr->textLen)
-	return;
-    
-    if (range.count < 1) {
-	if (range.position < 0)
-	    range.position = 0;
-	tPtr->text[range.position] = 0;
-	tPtr->textLen = range.position;
-	
-	tPtr->cursorPosition = 0;
-	tPtr->viewPosition = 0;
-    } else {
-	if (range.position + range.count > tPtr->textLen)
-	    range.count = tPtr->textLen - range.position;
-	memmv(&(tPtr->text[range.position]), &(tPtr->text[range.position+range.count]),
-                tPtr->textLen - (range.position+range.count) + 1);
-	tPtr->textLen -= range.count;
-	
-	if (tPtr->cursorPosition > range.position)
-	    tPtr->cursorPosition -= range.count;
+    normalizeRange(tPtr, &range);
 
-	decrToFit(tPtr);
-    }
+    if (!range.count)
+    	return;
+    
+    memmv(&(tPtr->text[range.position]), &(tPtr->text[range.position+range.count]),
+            tPtr->textLen - (range.position+range.count) + 1);
+
+    tPtr->textLen -= range.count;
+
+    /* try to keep cursorPosition at the same place */
+    tPtr->viewPosition -= range.count;
+    if (tPtr->viewPosition < 0)
+    	tPtr->viewPosition = 0;
+    tPtr->cursorPosition = range.position;
+    
+    decrToFit(tPtr);
 
     paintTextField(tPtr);
 }
@@ -404,6 +450,8 @@ WMGetTextFieldText(WMTextField *tPtr)
 void
 WMSetTextFieldText(WMTextField *tPtr, char *text)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     if ((text && strcmp(tPtr->text, text) == 0) ||
         (!text && tPtr->textLen == 0))
         return;
@@ -416,15 +464,12 @@ WMSetTextFieldText(WMTextField *tPtr, char *text)
 	
 	if (tPtr->textLen >= tPtr->bufferSize) {
 	    tPtr->bufferSize = tPtr->textLen + TEXT_BUFFER_INCR;
-	    tPtr->text = realloc(tPtr->text, tPtr->bufferSize);
+	    tPtr->text = wrealloc(tPtr->text, tPtr->bufferSize);
 	}
 	strcpy(tPtr->text, text);
     }
-    /*
-    if (tPtr->textLen < tPtr->cursorPosition)
-	tPtr->cursorPosition = tPtr->textLen;
-     */
-    tPtr->cursorPosition = tPtr->textLen;
+
+    tPtr->cursorPosition = tPtr->selection.position = tPtr->textLen;
     tPtr->viewPosition = 0;
     tPtr->selection.count = 0;
     
@@ -433,24 +478,18 @@ WMSetTextFieldText(WMTextField *tPtr, char *text)
 }
 
 
-void
-WMSetTextFieldFont(WMTextField *tPtr, WMFont *font)
-{
-    /* TODO: update font change after field is mapped */
-    WMReleaseFont(tPtr->font);
-    tPtr->font = WMRetainFont(font);
-}
-
-
 void
 WMSetTextFieldAlignment(WMTextField *tPtr, WMAlignment alignment)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     tPtr->flags.alignment = alignment;
+	
     if (alignment!=WALeft) {
 	wwarning("only left alignment is supported in textfields");
 	return;
     }
-	
+    
     if (tPtr->view->flags.realized) {
 	paintTextField(tPtr);
     }
@@ -460,6 +499,8 @@ WMSetTextFieldAlignment(WMTextField *tPtr, WMAlignment alignment)
 void
 WMSetTextFieldBordered(WMTextField *tPtr, Bool bordered)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     tPtr->flags.bordered = bordered;
 
     if (tPtr->view->flags.realized) {
@@ -471,6 +512,8 @@ WMSetTextFieldBordered(WMTextField *tPtr, Bool bordered)
 void
 WMSetTextFieldBeveled(WMTextField *tPtr, Bool flag)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     tPtr->flags.beveled = flag;
 
     if (tPtr->view->flags.realized) {
@@ -483,6 +526,8 @@ WMSetTextFieldBeveled(WMTextField *tPtr, Bool flag)
 void
 WMSetTextFieldSecure(WMTextField *tPtr, Bool flag)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     tPtr->flags.secure = flag;
     
     if (tPtr->view->flags.realized) {
@@ -494,6 +539,8 @@ WMSetTextFieldSecure(WMTextField *tPtr, Bool flag)
 Bool
 WMGetTextFieldEditable(WMTextField *tPtr)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     return tPtr->flags.enabled;
 }
 
@@ -501,6 +548,8 @@ WMGetTextFieldEditable(WMTextField *tPtr)
 void
 WMSetTextFieldEditable(WMTextField *tPtr, Bool flag)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     tPtr->flags.enabled = flag;
     
     if (tPtr->view->flags.realized) {
@@ -512,23 +561,15 @@ WMSetTextFieldEditable(WMTextField *tPtr, Bool flag)
 void
 WMSelectTextFieldRange(WMTextField *tPtr, WMRange range)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     if (tPtr->flags.enabled) {
-        if (range.position < 0) {
-            range.count += range.position;
-            range.count = (range.count < 0) ? 0 : range.count;
-            range.position = 0;
-        } else if (range.position > tPtr->textLen) {
-            range.position = tPtr->textLen;
-            range.count = 0;
-        }
-
-        if (range.position + range.count > tPtr->textLen)
-            range.count = tPtr->textLen - range.position;
-
-        tPtr->prevselection = tPtr->selection; /* check if this is needed */
+    	normalizeRange(tPtr, &range);
 
         tPtr->selection = range;
 
+        tPtr->cursorPosition = range.position + range.count;
+
         if (tPtr->view->flags.realized) {
             paintTextField(tPtr);
         }
@@ -539,6 +580,8 @@ WMSelectTextFieldRange(WMTextField *tPtr, WMRange range)
 void
 WMSetTextFieldCursorPosition(WMTextField *tPtr, unsigned int position)
 {
+    CHECK_CLASS(tPtr, WC_TextField);
+    
     if (tPtr->flags.enabled) {
         if (position > tPtr->textLen)
             position = tPtr->textLen;
@@ -597,6 +640,32 @@ WMSetTextFieldPrevTextField(WMTextField *tPtr, WMTextField *prev)
 }
 
 
+void 
+WMSetTextFieldFont(WMTextField *tPtr, WMFont *font)
+{
+    CHECK_CLASS(tPtr, WC_TextField);
+    
+    if (tPtr->font)
+	WMReleaseFont(tPtr->font);
+    tPtr->font = WMRetainFont(font);
+  
+    tPtr->offsetWidth = 
+	WMAX((tPtr->view->size.height - WMFontHeight(tPtr->font))/2, 1);
+
+    if (tPtr->view->flags.realized) {
+	paintTextField(tPtr);
+    }
+}
+
+
+
+WMFont*
+WMGetTextFieldFont(WMTextField *tPtr)
+{
+    return tPtr->font;
+}
+
+
 static void 
 didResizeTextField(W_ViewDelegate *self, WMView *view)
 {
@@ -669,7 +738,7 @@ paintCursor(TextField *tPtr)
 	      tPtr->view->size.height - tPtr->offsetWidth - 1);
 
     if (tPtr->flags.secure)
-        free(text);
+        wfree(text);
 }
 
 
@@ -833,7 +902,7 @@ paintTextField(TextField *tPtr)
     }
 
     if (tPtr->flags.secure)
-        free(text);
+        wfree(text);
     XCopyArea(screen->display, drawbuffer, view->window,
 	      screen->copyGC, 0,0, view->size.width,
 	      view->size.height,0,0);
@@ -924,240 +993,323 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
 {
     char buffer[64];
     KeySym ksym;
+    char *textEvent = NULL;
+    void *data = NULL;
     int count, refresh = 0;
     int control_pressed = 0;
+    int cancelSelection = 1;
+    Bool shifted, controled, modified;
+    Bool relay = True;
+    WMScreen *scr = tPtr->view->screen;
 
-    if (((XKeyEvent *) event)->state & WM_EMACSKEYMASK) {
+    event->xkey.state &= ~scr->ignoredModifierMask;
+    
+    /*printf("(%d,%d) -> ", tPtr->selection.position, tPtr->selection.count);*/
+    if (((XKeyEvent *) event)->state & WM_EMACSKEYMASK)
 	control_pressed = 1;
+    
+    shifted = event->xkey.state & ShiftMask;
+    controled = event->xkey.state & ControlMask;
+    if ((event->xkey.state & ~(ShiftMask|ControlMask)) != 0) {
+	modified = True;
+    } else {
+	modified = False;
     }
 
     count = XLookupString(&event->xkey, buffer, 63, &ksym, NULL);
     buffer[count] = '\0';
-
-    if (!(event->xkey.state & ShiftMask)) {
-        if (tPtr->selection.count)
-            refresh = 1;
-        tPtr->prevselection = tPtr->selection;
-        tPtr->selection.position = tPtr->cursorPosition;
-        tPtr->selection.count = 0;
-    }
-
-    /* Be careful in any case in this switch statement, never to call
-     * to more than a function that can generate text change notifications.
-     * Only one text change notification should be sent in any case.
-     * Else hazardous things can happen.
-     * Maybe we need a better solution than the function wrapper to inform
-     * functions that change text in text fields, if they need to send a
-     * change notification or not. -Dan
-     */
+    
     switch (ksym) {
      case XK_Tab:
+#ifdef XK_ISO_Left_Tab
      case XK_ISO_Left_Tab:
-	if (event->xkey.state & ShiftMask) {
-	    if (tPtr->view->prevFocusChain) {
-		W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
-				     tPtr->view->prevFocusChain);
-		tPtr->flags.notIllegalMovement = 1;
+#endif
+	if (!controled && !modified) {
+	    if (shifted) {
+		if (tPtr->view->prevFocusChain) {
+		    W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
+					 tPtr->view->prevFocusChain);
+		    tPtr->flags.notIllegalMovement = 1;
+		}
+		data = (void*)WMBacktabTextMovement;
+	    } else {
+		if (tPtr->view->nextFocusChain) {
+		    W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
+					 tPtr->view->nextFocusChain);
+		    tPtr->flags.notIllegalMovement = 1;
+		}
+		data = (void*)WMTabTextMovement;
 	    }
-	    NOTIFY(tPtr, didEndEditing, WMTextDidEndEditingNotification,
-		   (void*)WMBacktabTextMovement);
-	} else {
-	    if (tPtr->view->nextFocusChain) {
-		W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
-				     tPtr->view->nextFocusChain);
-		tPtr->flags.notIllegalMovement = 1;
-	    }
-	    NOTIFY(tPtr, didEndEditing, WMTextDidEndEditingNotification,
-		   (void*)WMTabTextMovement);
+	    textEvent = WMTextDidEndEditingNotification;
+
+	    relay = False;
+	}
+	break;
+
+     case XK_Escape:
+	if (!shifted && !controled && !modified) {
+	    data = (void*)WMEscapeTextMovement;
+	    textEvent = WMTextDidEndEditingNotification;
+	    
+	    relay = False;
 	}
 	break;
 
      case XK_Return:
-	NOTIFY(tPtr, didEndEditing, WMTextDidEndEditingNotification,
-	       (void*)WMReturnTextMovement);
+	if (!shifted && !controled && !modified) {
+	    data = (void*)WMReturnTextMovement;
+	    textEvent = WMTextDidEndEditingNotification;
+	    
+	    relay = False;
+	}
 	break;
 
      case WM_EMACSKEY_LEFT:
 	if (!control_pressed) {
 	    goto normal_key;
 	}
+#ifdef XK_KP_Left
      case XK_KP_Left:
+#endif
      case XK_Left:
-	if (tPtr->cursorPosition > 0) {
-	    paintCursor(tPtr);
-            if (event->xkey.state & ControlMask) {
-                int i;
-                for (i = tPtr->cursorPosition - 1; i >= 0; i--)
-                    if (tPtr->text[i] == ' ' || i == 0) {
-                        tPtr->cursorPosition = i;
-                        break;
-                    }
-            } else {
-		tPtr->cursorPosition--;
-            }
-	    if (tPtr->cursorPosition < tPtr->viewPosition) {
-		tPtr->viewPosition = tPtr->cursorPosition;
-		refresh = 1;
-	    } else {
+	if (!modified) {
+	    if (tPtr->cursorPosition > 0) {
 		paintCursor(tPtr);
+		if (event->xkey.state & ControlMask) {
+		    int i = tPtr->cursorPosition - 1;
+		    
+		    while (i > 0 && tPtr->text[i] != ' ') i--;
+		    while (i > 0 && tPtr->text[i] == ' ') i--;
+		    
+		    tPtr->cursorPosition = (i > 0) ? i + 1 : 0;
+		} else
+		    tPtr->cursorPosition--;
+		
+		if (tPtr->cursorPosition < tPtr->viewPosition) {
+		    tPtr->viewPosition = tPtr->cursorPosition;
+		    refresh = 1;
+		} else
+		    paintCursor(tPtr);
 	    }
+	    if (event->xkey.state & ShiftMask)
+		cancelSelection = 0;
+	    
+	    relay = False;
 	}
 	break;
 
     case WM_EMACSKEY_RIGHT:
-      if (!control_pressed) {
-	goto normal_key;
-      }
+        if (!control_pressed) {
+            goto normal_key;
+        }
+#ifdef XK_KP_Right
     case XK_KP_Right:
-     case XK_Right:
-	if (tPtr->cursorPosition < tPtr->textLen) {
-	    paintCursor(tPtr);
-            if (event->xkey.state & ControlMask) {
-                int i;
-                for (i = tPtr->cursorPosition + 1; i <= tPtr->textLen; i++)
-                    if (tPtr->text[i] == ' ' || i == tPtr->textLen) {
-                        tPtr->cursorPosition = i;
-                        break;
-                    }
-            } else {
-               tPtr->cursorPosition++;
-            }
-	    while (WMWidthOfString(tPtr->font,
-				&(tPtr->text[tPtr->viewPosition]),
-				tPtr->cursorPosition-tPtr->viewPosition)
-		   > tPtr->usableWidth) {
-		tPtr->viewPosition++;
-		refresh = 1;
-	    }
-	    if (!refresh)
+#endif
+    case XK_Right:
+	if (!modified) {
+	    if (tPtr->cursorPosition < tPtr->textLen) {
 		paintCursor(tPtr);
+		if (event->xkey.state & ControlMask) {
+		    int i = tPtr->cursorPosition;
+		    
+		    while (tPtr->text[i] && tPtr->text[i] != ' ') i++;
+		    while (tPtr->text[i] == ' ') i++;
+		    
+		    tPtr->cursorPosition = i;
+		} else {
+		    tPtr->cursorPosition++;
+		}
+		while (WMWidthOfString(tPtr->font,
+				       &(tPtr->text[tPtr->viewPosition]),
+				       tPtr->cursorPosition-tPtr->viewPosition)
+		       > tPtr->usableWidth) {
+		    tPtr->viewPosition++;
+		    refresh = 1;
+		}
+		if (!refresh)
+		    paintCursor(tPtr);
+	    }
+	    if (event->xkey.state & ShiftMask)
+		cancelSelection = 0;
+	    
+	    relay = False;
 	}
 	break;
 	
     case WM_EMACSKEY_HOME:
-      if (!control_pressed) {
-	goto normal_key;
-      }
+        if (!control_pressed) {
+            goto normal_key;
+        }
+#ifdef XK_KP_Home
     case XK_KP_Home:
-     case XK_Home:
-	if (tPtr->cursorPosition > 0) {
-	    paintCursor(tPtr);
-	    tPtr->cursorPosition = 0;
-	    if (tPtr->viewPosition > 0) {
-		tPtr->viewPosition = 0;
-		refresh = 1;
-	    } else {
+#endif
+    case XK_Home:
+	if (!modified && !controled) {
+	    if (tPtr->cursorPosition > 0) {
 		paintCursor(tPtr);
+		tPtr->cursorPosition = 0;
+		if (tPtr->viewPosition > 0) {
+		    tPtr->viewPosition = 0;
+		    refresh = 1;
+		} else
+		    paintCursor(tPtr);
 	    }
+	    if (event->xkey.state & ShiftMask)
+		cancelSelection = 0;
+	    
+	    relay = False;
 	}
 	break;
 	
     case WM_EMACSKEY_END:
-      if (!control_pressed) {
-	goto normal_key;
-      }
-     case XK_KP_End:
-     case XK_End:
-	if (tPtr->cursorPosition < tPtr->textLen) {
-	    paintCursor(tPtr);
-	    tPtr->cursorPosition = tPtr->textLen;
-	    tPtr->viewPosition = 0;
-	    while (WMWidthOfString(tPtr->font,
-				   &(tPtr->text[tPtr->viewPosition]),
-				   tPtr->textLen-tPtr->viewPosition)
-		   > tPtr->usableWidth) {
-		tPtr->viewPosition++;
-		refresh = 1;
-	    }
-	    if (!refresh)
+        if (!control_pressed) {
+            goto normal_key;
+        }
+#ifdef XK_KP_End
+    case XK_KP_End:
+#endif
+    case XK_End:
+	if (!modified && !controled) {
+	    if (tPtr->cursorPosition < tPtr->textLen) {
 		paintCursor(tPtr);
+		tPtr->cursorPosition = tPtr->textLen;
+		tPtr->viewPosition = 0;
+		while (WMWidthOfString(tPtr->font,
+				       &(tPtr->text[tPtr->viewPosition]),
+				       tPtr->textLen-tPtr->viewPosition)
+		       > tPtr->usableWidth) {
+		    tPtr->viewPosition++;
+		    refresh = 1;
+		}
+		if (!refresh)
+		    paintCursor(tPtr);
+	    }
+	    if (event->xkey.state & ShiftMask)
+		cancelSelection = 0;
+	    
+	    relay = False;
 	}
 	break;
 	
      case WM_EMACSKEY_BS:
-      if (!control_pressed) {
-	goto normal_key;
-      }
+         if (!control_pressed) {
+             goto normal_key;
+         }
      case XK_BackSpace:
-	if (tPtr->cursorPosition > 0) {
-            WMRange range;
-
-            if (tPtr->prevselection.count) {
-                range.position = tPtr->prevselection.count < 0 
-		    ? tPtr->prevselection.position + tPtr->prevselection.count 
-		    : tPtr->prevselection.position;
-
-		range.count = abs(tPtr->prevselection.count);
-	    } else {
-                range.position = tPtr->cursorPosition - 1;
-                range.count = 1;
-            }
-	    WMDeleteTextFieldRange(tPtr, range);
-            NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
-                   (void*)WMDeleteTextEvent);
+	if (!modified && !shifted && !controled) {
+	    if (tPtr->selection.count) {
+		WMDeleteTextFieldRange(tPtr, tPtr->selection);
+		data = (void*)WMDeleteTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    } else if (tPtr->cursorPosition > 0) {
+		WMRange range;
+		range.position = tPtr->cursorPosition - 1;
+		range.count = 1;
+		WMDeleteTextFieldRange(tPtr, range);
+		data = (void*)WMDeleteTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    }
+	    
+	    relay = False;
 	}
 	break;
 	
     case WM_EMACSKEY_DEL:
-      if (!control_pressed) {
-	goto normal_key;
-      }
+        if (!control_pressed) {
+            goto normal_key;
+        }
+#ifdef XK_KP_Delete
     case XK_KP_Delete:
-     case XK_Delete:
-	if (tPtr->cursorPosition < tPtr->textLen || tPtr->prevselection.count) {
-            WMRange range;
-
-            if (tPtr->prevselection.count) {
-                range.position = tPtr->prevselection.count < 0 
-		    ? tPtr->prevselection.position + tPtr->prevselection.count
-		    : tPtr->prevselection.position;
-
-                range.count = abs(tPtr->prevselection.count);
-            } else {
+#endif
+    case XK_Delete:
+	if (!modified && !controled && !shifted) {
+	    if (tPtr->selection.count) {
+		WMDeleteTextFieldRange(tPtr, tPtr->selection);
+		data = (void*)WMDeleteTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    } else if (tPtr->cursorPosition < tPtr->textLen) {
+		WMRange range;
 		range.position = tPtr->cursorPosition;
 		range.count = 1;
-            }
-	    WMDeleteTextFieldRange(tPtr, range);
-            NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
-                   (void*)WMDeleteTextEvent);
-	}
+		WMDeleteTextFieldRange(tPtr, range);
+		data = (void*)WMDeleteTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    }
+	    
+	    relay = False;
+        }
 	break;
 
     normal_key:
      default:
-	if (count > 0 && !iscntrl(buffer[0])) {
-	    WMRange range;
-
-	    if (tPtr->prevselection.count) {
-                range.position = tPtr->prevselection.count < 0 
-		    ? tPtr->prevselection.position + tPtr->prevselection.count
-		    : tPtr->prevselection.position;
-
-                range.count = abs(tPtr->prevselection.count);
-            } else {
-		range.position = tPtr->cursorPosition;
-		range.count = 1;
-            }
-            if (tPtr->prevselection.count)
-                WMDeleteTextFieldRange(tPtr, range);
-	    WMInsertTextFieldText(tPtr, buffer, tPtr->cursorPosition);
-            NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
-                   (void*)WMInsertTextEvent);
-	} else {
-	    return;
-	}
+	if (!controled && !modified) {
+	    if (count > 0 && isprint(buffer[0])) {
+		if (tPtr->selection.count)
+		    WMDeleteTextFieldRange(tPtr, tPtr->selection);
+		WMInsertTextFieldText(tPtr, buffer, tPtr->cursorPosition);
+		data = (void*)WMInsertTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    
+		relay = False;
+	    }
+        }
 	break;
     }
-    if (event->xkey.state & ShiftMask) {
-        if (tPtr->selection.count == 0)
-            tPtr->selection.position = tPtr->cursorPosition;
-        tPtr->selection.count = tPtr->cursorPosition - tPtr->selection.position;
-        refresh = 1;
+    
+    
+    if (relay) {
+	WMRelayToNextResponder(W_VIEW(tPtr), event);
+	return;
     }
-    tPtr->prevselection.count = 0;
-    if (refresh) {
+    
+
+    if (!cancelSelection) {
+    	if (tPtr->selection.count != tPtr->cursorPosition - tPtr->selection.position) {
+	    
+            tPtr->selection.count = tPtr->cursorPosition - tPtr->selection.position;
+
+	    WMPostNotificationName("_lostOwnership", NULL, tPtr);
+
+            refresh = 1;
+        }
+    } else {
+	
+	lostHandler(tPtr->view, XA_PRIMARY, NULL);
+	
+    	if (tPtr->selection.count) {
+    	    tPtr->selection.count = 0;
+            refresh = 1;
+	}
+    	tPtr->selection.position = tPtr->cursorPosition;
+    }
+
+    /*printf("(%d,%d)\n", tPtr->selection.position, tPtr->selection.count);*/
+
+    if (textEvent) {
+        WMNotification *notif = WMCreateNotification(textEvent, tPtr, data);
+
+        if (tPtr->delegate) {
+            if (textEvent==WMTextDidBeginEditingNotification &&
+                tPtr->delegate->didBeginEditing)
+                (*tPtr->delegate->didBeginEditing)(tPtr->delegate, notif);
+
+            else if (textEvent==WMTextDidEndEditingNotification &&
+                     tPtr->delegate->didEndEditing)
+                (*tPtr->delegate->didEndEditing)(tPtr->delegate, notif);
+
+            else if (textEvent==WMTextDidChangeNotification &&
+                     tPtr->delegate->didChange)
+                (*tPtr->delegate->didChange)(tPtr->delegate, notif);
+        }
+
+        WMPostNotification(notif);
+        WMReleaseNotification(notif);
+    }
+
+    if (refresh)
 	paintTextField(tPtr);
-    }
+
+    /*printf("(%d,%d)\n", tPtr->selection.position, tPtr->selection.count);*/
 }
 
 
@@ -1191,24 +1343,57 @@ pointToCursorPosition(TextField *tPtr, int x)
 }
 
 
+
+static void 
+pasteText(WMView *view, Atom selection, Atom target, Time timestamp, 
+	  void *cdata, WMData *data)
+{
+    TextField *tPtr = (TextField*)view->self;
+    char *str;
+    
+    tPtr->flags.waitingSelection = 0;
+
+    if (data != NULL) {
+	str = (char*)WMDataBytes(data);
+
+	WMInsertTextFieldText(tPtr, str, tPtr->cursorPosition);
+	NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
+	       (void*)WMInsertTextEvent);
+    } else {
+	int n;
+	
+	str = XFetchBuffer(tPtr->view->screen->display, &n, 0);
+		
+	if (str != NULL) {
+	    str[n] = 0;
+	    WMInsertTextFieldText(tPtr, str, tPtr->cursorPosition);
+	    XFree(str);
+	    NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
+		   (void*)WMInsertTextEvent);
+	}
+    }
+}
+
+
 static void
 handleTextFieldActionEvents(XEvent *event, void *data)
 {
     TextField *tPtr = (TextField*)data;
-    static int move;
+    static int move = 0;
+    static Time lastButtonReleasedEvent = 0;
+    Display *dpy = event->xany.display;
     
     CHECK_CLASS(data, WC_TextField);
 
     switch (event->type) {
      case KeyPress:
+	if (tPtr->flags.waitingSelection) {
+	    return;
+	}
         if (tPtr->flags.enabled && tPtr->flags.focused) {
             handleTextFieldKeyPress(tPtr, event);
-	    XGrabPointer(WMScreenDisplay(W_VIEW(tPtr)->screen),
-			 W_VIEW(tPtr)->window, False, 
-			 PointerMotionMask|ButtonPressMask|ButtonReleaseMask,
-			 GrabModeAsync, GrabModeAsync, None, 
-			 W_VIEW(tPtr)->screen->invisibleCursor,
-			 CurrentTime);
+	    XDefineCursor(dpy, W_VIEW(tPtr)->window, 
+			  W_VIEW(tPtr)->screen->invisibleCursor);
 	    tPtr->flags.pointerGrabbed = 1;
 	}
         break;
@@ -1217,7 +1402,11 @@ handleTextFieldActionEvents(XEvent *event, void *data)
 	
 	if (tPtr->flags.pointerGrabbed) {
 	    tPtr->flags.pointerGrabbed = 0;
-	    XUngrabPointer(WMScreenDisplay(W_VIEW(tPtr)->screen), CurrentTime);
+	    XDefineCursor(dpy, W_VIEW(tPtr)->window, 
+			  W_VIEW(tPtr)->screen->textCursor);
+	}
+	if (tPtr->flags.waitingSelection) {
+	    return;
 	}
 
         if (tPtr->flags.enabled && (event->xmotion.state & Button1Mask)) {
@@ -1235,40 +1424,36 @@ handleTextFieldActionEvents(XEvent *event, void *data)
 		tPtr->viewPosition--;
 	    }
 
-	    if (!tPtr->selection.count) {
-		tPtr->selection.position = tPtr->cursorPosition;
-	    }
-	    
 	    tPtr->cursorPosition = 
 		pointToCursorPosition(tPtr, event->xmotion.x);
 	    
 	    tPtr->selection.count = tPtr->cursorPosition - tPtr->selection.position;
 	    
-	    /*
-	     printf("notify %d %d\n",event->xmotion.x,tPtr->usableWidth);
-	     */
+	    if (tPtr->selection.count != 0) {
+		if (!tPtr->flags.ownsSelection) {
+		    WMCreateSelectionHandler(tPtr->view,
+					     XA_PRIMARY,
+					     event->xbutton.time,
+					     &selectionHandler, NULL);
+		    tPtr->flags.ownsSelection = 1;
+		}
+	    }
 	    
 	    paintCursor(tPtr);
 	    paintTextField(tPtr);
 
 	}
-        if (move) {
-            XSetSelectionOwner(tPtr->view->screen->display,
-                       XA_PRIMARY, tPtr->view->window, CurrentTime);
-            {
-                WMNotification *notif = WMCreateNotification("_lostOwnership",
-                        NULL,tPtr);
-                puts("notify it");
-                WMPostNotification(notif);
-                WMReleaseNotification(notif);
-            }
-        }
 	break;
 
      case ButtonPress:
 	if (tPtr->flags.pointerGrabbed) {
 	    tPtr->flags.pointerGrabbed = 0;
-	    XUngrabPointer(WMScreenDisplay(W_VIEW(tPtr)->screen), CurrentTime);
+	    XDefineCursor(dpy, W_VIEW(tPtr)->window, 
+			  W_VIEW(tPtr)->screen->textCursor);
+	    break;
+	}
+	
+	if (tPtr->flags.waitingSelection) {
 	    break;
 	}
 
@@ -1279,25 +1464,18 @@ handleTextFieldActionEvents(XEvent *event, void *data)
             textWidth = WMWidthOfString(tPtr->font, tPtr->text, tPtr->textLen);
             if (tPtr->flags.enabled && !tPtr->flags.focused) {
                 WMSetFocusToWidget(tPtr);
+		
             } else if (tPtr->flags.focused) {
+		tPtr->selection.position = tPtr->cursorPosition;
                 tPtr->selection.count = 0;
             }
-            if(textWidth < tPtr->usableWidth){
+            if(textWidth < tPtr->usableWidth) {
                 tPtr->cursorPosition = pointToCursorPosition(tPtr, 
 				     event->xbutton.x - tPtr->usableWidth
                                      + textWidth);
-            }
-            else tPtr->cursorPosition = pointToCursorPosition(tPtr, 
+            } else tPtr->cursorPosition = pointToCursorPosition(tPtr,
                                      event->xbutton.x);
-            /*
-            tPtr->cursorPosition = pointToCursorPosition(tPtr, 
-                                     event->xbutton.x);
-                tPtr->cursorPosition += tPtr->usableWidth - textWidth;
-            }
 
-            tPtr->cursorPosition = pointToCursorPosition(tPtr, 
-                                     event->xbutton.x);
-                                     */
             paintTextField(tPtr);
             break;
         
@@ -1307,43 +1485,83 @@ handleTextFieldActionEvents(XEvent *event, void *data)
                 tPtr->cursorPosition = pointToCursorPosition(tPtr, 
                                      event->xbutton.x);
                 paintTextField(tPtr);
-            } else if (tPtr->flags.focused) {
+            } else if (tPtr->flags.focused 
+		       && event->xbutton.button == Button1) {
                 tPtr->cursorPosition = pointToCursorPosition(tPtr, 
                                      event->xbutton.x);
+		tPtr->selection.position = tPtr->cursorPosition;
                 tPtr->selection.count = 0;
                 paintTextField(tPtr);
             }
             if (event->xbutton.button == Button2 && tPtr->flags.enabled) {
                 char *text;
+		int n;
 
-                text = W_GetTextSelection(tPtr->view->screen, XA_PRIMARY);
-                
-                if (!text) {
-                text = W_GetTextSelection(tPtr->view->screen,
-                              tPtr->view->screen->clipboardAtom);
-                }
-                if (!text) {
-		    text = W_GetTextSelection(tPtr->view->screen, 
-					      XA_CUT_BUFFER0);
-                }
-                if (text) {
-		    WMInsertTextFieldText(tPtr, text, tPtr->cursorPosition);
-		    XFree(text);
-                    NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
-                           (void*)WMInsertTextEvent);
-                }
+		if (!WMRequestSelection(tPtr->view, XA_PRIMARY, XA_STRING, 
+					event->xbutton.time,
+					pasteText, NULL)) {
+		    text = XFetchBuffer(tPtr->view->screen->display, &n, 0);
+		
+		    if (text) {
+			text[n] = 0;
+			WMInsertTextFieldText(tPtr, text, tPtr->cursorPosition);
+			XFree(text);
+			NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
+			       (void*)WMInsertTextEvent);
+		    }
+		} else {
+		    tPtr->flags.waitingSelection = 1;
+		}
             }
             break;
+	 default:
+	    break;
         }
 	break;
 
      case ButtonRelease:
 	if (tPtr->flags.pointerGrabbed) {
 	    tPtr->flags.pointerGrabbed = 0;
-	    XUngrabPointer(WMScreenDisplay(W_VIEW(tPtr)->screen), CurrentTime);
+	    XDefineCursor(dpy, W_VIEW(tPtr)->window, 
+			  W_VIEW(tPtr)->screen->textCursor);
+	}
+	if (tPtr->flags.waitingSelection) {
+	    break;
+	}
+	
+	if (tPtr->selection.count != 0) {
+	    int start, count;
+	    XRotateBuffers(dpy, 1);
+	    
+	    count = abs(tPtr->selection.count);
+	    if (tPtr->selection.count < 0)
+		start = tPtr->selection.position - count;
+	    else
+		start = tPtr->selection.position;
+	    
+	    XStoreBuffer(dpy, &tPtr->text[start], count, 0);
 	}
 
-        move = 0;
+	move = 0;
+		
+	if (event->xbutton.time - lastButtonReleasedEvent
+	    <= WINGsConfiguration.doubleClickDelay) {
+	    tPtr->selection.position = 0;
+	    tPtr->selection.count = tPtr->textLen;
+	    paintTextField(tPtr);
+	    
+	    if (!tPtr->flags.ownsSelection) {
+		WMCreateSelectionHandler(tPtr->view,
+					 XA_PRIMARY,
+					 event->xbutton.time,
+					 &selectionHandler, NULL);
+		tPtr->flags.ownsSelection = 1;
+	    }
+	    
+	    WMPostNotificationName("_lostOwnership", NULL, tPtr);
+	}
+	lastButtonReleasedEvent = event->xbutton.time;
+		
         break;
     }
 }
@@ -1358,11 +1576,11 @@ destroyTextField(TextField *tPtr)
 #endif
 
     WMReleaseFont(tPtr->font);
-    WMDeleteSelectionHandler(tPtr, XA_PRIMARY);
+    WMDeleteSelectionHandler(tPtr->view, XA_PRIMARY, CurrentTime);
     WMRemoveNotificationObserver(tPtr);
 
     if (tPtr->text)
-	free(tPtr->text);
+	wfree(tPtr->text);
 
-    free(tPtr);
+    wfree(tPtr);
 }
