@@ -25,16 +25,16 @@ static XSetWindowAttributes defAtts= {
     0,                          /* background_pixel */
     CopyFromParent,             /* border_pixmap */
     0,                          /* border_pixel */
-    NorthWestGravity,           /* bit_gravity */
-    NorthWestGravity,           /* win_gravity */
+    ForgetGravity,              /* bit_gravity */
+    ForgetGravity,		/* win_gravity */
     NotUseful,                  /* backing_store */
     (unsigned) ~0,              /* backing_planes */
     0,                          /* backing_pixel */
     False,                      /* save_under */
-    EVENT_MASK,   	         /* event_mask */
+    EVENT_MASK,   	        /* event_mask */
     0,                          /* do_not_propagate_mask */
     False,                      /* override_redirect */
-    CopyFromParent,             /* colormap */
+    None,			/* colormap */
     None                        /* cursor */
 };
 
@@ -102,28 +102,6 @@ adoptChildView(W_View *view, W_View *child)
 }
 
 
-
-static void
-handleEvents(XEvent *event, void *data)
-{
-    W_View *view = (W_View*)data;
-    
-    if (event->type == ConfigureNotify) {
-	
-	if (event->xconfigure.width != view->size.width
-	    || event->xconfigure.height != view->size.height) {
-	    
-	    if (view->flags.notifySizeChanged) {
-		view->size.width = event->xconfigure.width;
-		view->size.height = event->xconfigure.height;
-		WMPostNotificationName(WMViewSizeDidChangeNotification,
-				       view, NULL);
-	    }
-	}
-    }
-}
-
-
 static W_View*
 createView(W_Screen *screen, W_View *parent)
 {
@@ -134,22 +112,28 @@ createView(W_Screen *screen, W_View *parent)
     
     view = wmalloc(sizeof(W_View));
     memset(view, 0, sizeof(W_View));
-    
-    view->refCount = 1;
-    
+
     view->screen = screen;
 
     if (parent!=NULL) {
 	/* attributes are not valid for root window */
-	view->attribFlags = CWEventMask|CWColormap|CWBitGravity;
+	view->attribFlags = CWEventMask|CWBitGravity;
 	view->attribs = defAtts;
 
-	view->attribFlags |= CWBackPixel;
+	view->attribFlags |= CWBackPixel|CWColormap|CWBorderPixel;
 	view->attribs.background_pixel = W_PIXEL(screen->gray);
+	view->attribs.border_pixel = W_PIXEL(screen->black);
+	view->attribs.colormap = screen->colormap;
+
+	view->backColor = WMRetainColor(screen->gray);
 
 	adoptChildView(parent, view);
     }
 
+    view->refCount = 1;
+
+    view->eventHandlers = WMCreateBag(4);
+
     return view;
 }
 
@@ -175,6 +159,11 @@ W_CreateRootView(W_Screen *screen)
     view->flags.mapped = 1;
     view->flags.root = 1;
     
+    view->size.width = 
+	WidthOfScreen(ScreenOfDisplay(screen->display, screen->screen));
+    view->size.height =
+	HeightOfScreen(ScreenOfDisplay(screen->display, screen->screen));
+
     return view;
 }
 
@@ -191,13 +180,28 @@ W_CreateTopView(W_Screen *screen)
     view->flags.topLevel = 1;
     view->attribs.event_mask |= StructureNotifyMask;
 
-    /* catch changes in the toplevel window (resize from user etc.) */
-    WMCreateEventHandler(view, StructureNotifyMask, handleEvents, view);
-
     return view;
 }
 
 
+W_View*
+W_CreateUnmanagedTopView(W_Screen *screen)
+{
+    W_View *view;
+    
+    view = createView(screen, screen->rootView);
+    if (!view)
+	return NULL;
+
+    view->flags.topLevel = 1;
+    view->attribs.event_mask |= StructureNotifyMask;
+
+    view->attribFlags |= CWOverrideRedirect;
+    view->attribs.override_redirect = True;
+
+    return view;
+}
+
 
 void
 W_RealizeView(W_View *view)
@@ -217,7 +221,6 @@ W_RealizeView(W_View *view)
 
     if (!view->flags.realized) {
 	parentWID = view->parent->window;
-
 	view->window = XCreateWindow(dpy, parentWID, view->pos.x, view->pos.y,
 				     view->size.width, view->size.height, 0, 
 				     view->screen->depth, InputOutput,
@@ -246,47 +249,44 @@ W_RealizeView(W_View *view)
 }
 
 
-
-Bool
-W_CheckInternalMessage(W_Screen *scr, XClientMessageEvent *cev, int event)
-{
-    if (cev->message_type == scr->internalMessage
-	&& cev->format == 32 && cev->data.l[1] == event)
-	return True;
-    else
-	return False;
-}
-
-
 void
-W_ReparentView(W_View *view, W_View *newParent)
+W_ReparentView(W_View *view, W_View *newParent, int x, int y)
 {
-    int wasMapped;
     Display *dpy = view->screen->display;
     
     assert(!view->flags.topLevel);
-    
-    wasMapped = view->flags.mapped;
-    if (wasMapped)
-	W_UnmapView(view);
 
     unparentView(view);
     adoptChildView(newParent, view);
 
     if (view->flags.realized) {
 	if (newParent->flags.realized) {
-	    XReparentWindow(dpy, view->window, newParent->window, 0, 0);
+	    XReparentWindow(dpy, view->window, newParent->window, x, y);
 	} else {
 	    wwarning("trying to reparent realized view to unrealized parent");
 	    return;
 	}
     }
   
-    view->pos.x = 0;
-    view->pos.y = 0;
-    
-    if (wasMapped)
-	W_MapView(view);
+    view->pos.x = x;
+    view->pos.y = y;
+}
+
+
+void
+W_RaiseView(W_View *view)
+{
+    if (W_VIEW_REALIZED(view))
+	XRaiseWindow(W_VIEW_DISPLAY(view), W_VIEW_DRAWABLE(view));
+}
+
+
+
+void
+W_LowerView(W_View *view)
+{
+    if (W_VIEW_REALIZED(view))
+	XLowerWindow(W_VIEW_DISPLAY(view), W_VIEW_DRAWABLE(view));
 }
 
 
@@ -325,16 +325,33 @@ W_MapSubviews(W_View *view)
 }
 
 
+
+void
+W_UnmapSubviews(W_View *view)
+{
+    XUnmapSubwindows(view->screen->display, view->window);
+    XFlush(view->screen->display);
+    
+    view = view->childrenList;
+    while (view) {
+	view->flags.mapped = 0;
+	view->flags.mapWhenRealized = 0;
+	view = view->nextSister;
+    }
+}
+
+
+
 void
 W_UnmapView(W_View *view)
 {
     view->flags.mapWhenRealized = 0;
     if (!view->flags.mapped)
 	return;
-  
+
     XUnmapWindow(view->screen->display, view->window);
     XFlush(view->screen->display);
-    
+
     view->flags.mapped = 0;
 }
 
@@ -344,7 +361,9 @@ W_TopLevelOfView(W_View *view)
 {
     W_View *toplevel;
     
-    for (toplevel=view; !toplevel->flags.topLevel; toplevel=toplevel->parent);
+    for (toplevel=view;
+	 toplevel && !toplevel->flags.topLevel; 
+	 toplevel=toplevel->parent);
     
     return toplevel;
 }
@@ -359,6 +378,14 @@ destroyView(W_View *view)
 	return;
     view->flags.alreadyDead = 1;    
 
+    /* delete the balloon text for the view, if there's any */
+    WMSetBalloonTextForView(NULL, view);
+
+    if (view->nextFocusChain)
+        view->nextFocusChain->prevFocusChain = view->prevFocusChain;
+    if (view->prevFocusChain)
+        view->prevFocusChain->nextFocusChain = view->nextFocusChain;
+
     /* Do not leave focus in a inexisting control */
     if (W_FocusedViewOfToplevel(W_TopLevelOfView(view))==view)
 	W_SetFocusOfTopLevel(W_TopLevelOfView(view), NULL);
@@ -370,7 +397,7 @@ destroyView(W_View *view)
 	if (info) {
 	    if (info->toplevel==view) {
 		view->screen->focusInfo = info->next;
-		free(info);
+		wfree(info);
 	    } else {
 		while (info->next) {
 		    if (info->next->toplevel == view)
@@ -379,14 +406,14 @@ destroyView(W_View *view)
 		}
 		if (info->next) {
 		    W_FocusInfo *next = info->next->next;
-		    free(info->next);
+		    wfree(info->next);
 		    info->next = next;
 		} 
 		/* else the toplevel did not have any focused subview */
 	    }
 	}
     }
-    
+
     /* destroy children recursively */
     while (view->childrenList!=NULL) {
 	ptr = view->childrenList;
@@ -403,7 +430,8 @@ destroyView(W_View *view)
     W_CallDestroyHandlers(view);
 
     if (view->flags.realized) {
-	XDeleteContext(view->screen->display, view->window, ViewContext);
+	XDeleteContext(view->screen->display,
+		       view->window, ViewContext);
     
 	/* if parent is being destroyed, it will die naturaly */
 	if (!view->flags.parentDying || view->flags.topLevel)
@@ -415,7 +443,23 @@ destroyView(W_View *view)
 
     W_CleanUpEvents(view);
     
-    free(view);
+    WMFreeBag(view->eventHandlers);
+    view->eventHandlers = NULL;
+    
+    WMUnregisterViewDraggedTypes(view);
+    
+#if 0    
+    if (view->dragSourceProcs)
+	wfree(view->dragSourceProcs);
+
+    if (view->dragDestinationProcs)
+	wfree(view->dragDestinationProcs);
+    
+    if (scr->dragInfo.destView == view) {
+	scr->dragInfo.destView = NULL;
+    }
+#endif
+    wfree(view);
 }
 
 
@@ -423,7 +467,11 @@ destroyView(W_View *view)
 void
 W_DestroyView(W_View *view)
 {
-    W_ReleaseView(view);
+    view->refCount--;
+
+    if (view->refCount < 1) {
+        destroyView(view);
+    }
 }
 
 
@@ -433,14 +481,22 @@ W_MoveView(W_View *view, int x, int y)
 {
     assert(view->flags.root==0);
 
+    if (view->delegate && view->delegate->willMove) {
+	(*view->delegate->willMove)(view->delegate, view, &x, &y);
+    }
+
     if (view->pos.x == x && view->pos.y == y) 
 	return;
-    
+
     if (view->flags.realized) {
 	XMoveWindow(view->screen->display, view->window, x, y);
     }
     view->pos.x = x;
     view->pos.y = y;
+
+    if (view->delegate && view->delegate->didMove) {
+	(*view->delegate->didMove)(view->delegate, view);
+    }
 }
 
 
@@ -448,10 +504,14 @@ void
 W_ResizeView(W_View *view, unsigned int width, unsigned int height)
 {    
     int shrinked;
-    
+
+    if (view->delegate && view->delegate->willResize) {
+	(*view->delegate->willResize)(view->delegate, view, &width, &height);
+    }
+
     assert(width > 0);
     assert(height > 0);
-    
+
     if (view->size.width == width && view->size.height == height)
 	return;
     
@@ -462,37 +522,42 @@ W_ResizeView(W_View *view, unsigned int width, unsigned int height)
     } 
     view->size.width = width;
     view->size.height = height;
-    
+
+    if (view->delegate && view->delegate->didResize) {
+	(*view->delegate->didResize)(view->delegate, view);
+    }
+
     if (view->flags.notifySizeChanged)
 	WMPostNotificationName(WMViewSizeDidChangeNotification, view, NULL);
-
-    /* if the window was resized to be bigger, it will get an Expose 
-     * from the server */
-    if (view->flags.mapped && shrinked)
-	W_RedisplayView(view);
 }
 
 
 void
 W_RedisplayView(W_View *view)
 {
-    XExposeEvent ev;
+    XEvent ev;
     
     if (!view->flags.mapped)
 	return;
     
-    ev.type = Expose;
-    ev.display = view->screen->display;
-    ev.window = view->window;
-    ev.count = 0;
+    ev.xexpose.type = Expose;
+    ev.xexpose.display = view->screen->display;
+    ev.xexpose.window = view->window;
+    ev.xexpose.count = 0;
 
-    W_DispatchMessage(view, (XEvent*)&ev);
+    ev.xexpose.serial = 0;
+
+    WMHandleEvent(&ev);
 }
 
 
 void
 W_SetViewBackgroundColor(W_View *view, WMColor *color)
 {
+    if (view->backColor)
+	WMReleaseColor(view->backColor);
+    view->backColor = WMRetainColor(color);
+
     view->attribFlags |= CWBackPixel;
     view->attribs.background_pixel = color->color.pixel;
     if (view->flags.realized) {
@@ -503,6 +568,18 @@ W_SetViewBackgroundColor(W_View *view, WMColor *color)
 }
 
 
+void 
+W_SetViewCursor(W_View *view, Cursor cursor)
+{
+    view->cursor = cursor;
+    if (W_VIEW_REALIZED(view)) {
+	XDefineCursor(W_VIEW_DISPLAY(view), W_VIEW_DRAWABLE(view), cursor);
+    } else {
+	view->attribFlags |= CWCursor;
+	view->attribs.cursor = cursor;
+    }
+}
+
 
 W_View*
 W_FocusedViewOfToplevel(W_View *view)
@@ -591,15 +668,16 @@ WMView*
 W_RetainView(WMView *view)
 {
     view->refCount++;
+
     return view;
 }
 
 
-
 void
 W_ReleaseView(WMView *view)
 {
     view->refCount--;
+
     if (view->refCount < 1) {
 	destroyView(view);
     }
@@ -632,3 +710,25 @@ WMSetViewNotifySizeChanges(WMView *view, Bool flag)
     view->flags.notifySizeChanged = flag;
 }
 
+
+Window
+WMViewXID(WMView *view)
+{
+    return view->window;
+}
+
+
+WMPoint
+WMGetViewScreenPosition(WMView *view)
+{
+    WMScreen *scr = W_VIEW_SCREEN(view);
+    Window foo;
+    int x, y;
+    
+    XTranslateCoordinates(scr->display, W_VIEW_DRAWABLE(view),
+			  scr->rootWin, 0, 0, &x, &y, &foo);
+    
+    return wmkpoint(x, y);
+}
+
+			
