@@ -38,14 +38,14 @@
 #include "TexturePanel.h"
 
 typedef struct _Panel {
-    WMFrame *frame;
+    WMBox *box;
     char *sectionName;
 
     char *description;
 
     CallbackRec callbacks;
 
-    WMWindow *win;
+    WMWidget *parent;
 
     WMLabel *prevL;
 
@@ -82,6 +82,14 @@ typedef struct _Panel {
     WMFrame *taliF;
     WMButton *taliB[3];
 
+    /* root bg */
+    WMFrame *bgF;
+    
+    WMLabel *bgprevL;
+    WMButton *selbgB;
+
+    WMPopUpButton *modeB[3];
+    
     /* */
 
     int textureIndex[8];
@@ -531,6 +539,25 @@ renderTexture(WMScreen *scr, proplist_t texture, int width, int height,
 
 	image = RCreateImage(width, height, False);
 	RClearImage(image, &rcolor);
+    } else if (strcasecmp(type, "igradient")==0) {
+	int t1, t2;
+	RColor c1[2], c2[2];
+
+	str = PLGetString(PLGetArrayElement(texture, 1));
+	str2rcolor(rc, str, &c1[0]);
+	str = PLGetString(PLGetArrayElement(texture, 2));
+	str2rcolor(rc, str, &c1[1]);
+	str = PLGetString(PLGetArrayElement(texture, 3));
+	t1 = atoi(str);
+
+	str = PLGetString(PLGetArrayElement(texture, 4));
+	str2rcolor(rc, str, &c2[0]);
+	str = PLGetString(PLGetArrayElement(texture, 5));
+	str2rcolor(rc, str, &c2[1]);
+	str = PLGetString(PLGetArrayElement(texture, 6));
+	t2 = atoi(str);
+
+	image = RRenderInterwovenGradient(width, height, c1, t1, c2, t2);
     } else if (strcasecmp(&type[1], "gradient")==0) {
 	int style;
 	RColor rcolor2;
@@ -631,8 +658,8 @@ renderTexture(WMScreen *scr, proplist_t texture, int width, int height,
 	    image = RRenderMultiGradient(width, height, colors, style);
 	    
 	    for (i = 0; colors[i]!=NULL; i++)
-		free(colors[i]);
-	    free(colors);
+		wfree(colors[i]);
+	    wfree(colors);
 	}
     } else if (strcasecmp(&type[1], "pixmap")==0) {
 	RImage *timage = NULL;
@@ -671,7 +698,7 @@ renderTexture(WMScreen *scr, proplist_t texture, int width, int height,
 	    }
 
 	}
-	free(path);
+	wfree(path);
     }
 
     if (!image)
@@ -702,10 +729,10 @@ renderTexture(WMScreen *scr, proplist_t texture, int width, int height,
 static Pixmap
 renderMenu(_Panel *panel, proplist_t texture, int width, int iheight)
 {
-    WMScreen *scr = WMWidgetScreen(panel->win);
+    WMScreen *scr = WMWidgetScreen(panel->parent);
     Display *dpy = WMScreenDisplay(scr);
     Pixmap pix, tmp;
-    GC gc = XCreateGC(dpy, WMWidgetXID(panel->win), 0, NULL);
+    GC gc = XCreateGC(dpy, WMWidgetXID(panel->parent), 0, NULL);
     int i;
 
     switch (panel->menuStyle) {
@@ -737,7 +764,7 @@ renderPreview(_Panel *panel, GC gc, int part, int relief)
     WMListItem *item;
     TextureListItem *titem;
     Pixmap pix;
-    WMScreen *scr = WMWidgetScreen(panel->frame);
+    WMScreen *scr = WMWidgetScreen(panel->box);
 
     item = WMGetListItem(panel->texLs, panel->textureIndex[part]);
     titem = (TextureListItem*)item->clientData;
@@ -760,7 +787,7 @@ renderPreview(_Panel *panel, GC gc, int part, int relief)
 static void
 updatePreviewBox(_Panel *panel, int elements)
 {
-    WMScreen *scr = WMWidgetScreen(panel->win);
+    WMScreen *scr = WMWidgetScreen(panel->parent);
     Display *dpy = WMScreenDisplay(scr);
  /*   RContext *rc = WMScreenRContext(scr);*/
     int refresh = 0;
@@ -769,13 +796,13 @@ updatePreviewBox(_Panel *panel, int elements)
     int colorUpdate = 0;
     WMColor *black = WMBlackColor(scr);
 
-    gc = XCreateGC(dpy, WMWidgetXID(panel->win), 0, NULL);
+    gc = XCreateGC(dpy, WMWidgetXID(panel->parent), 0, NULL);
 
 
     if (panel->preview == None) {
 	WMColor *color;
 
-	panel->preview = XCreatePixmap(dpy, WMWidgetXID(panel->win),
+	panel->preview = XCreatePixmap(dpy, WMWidgetXID(panel->parent),
 				       240-4, 215-4, WMScreenDepth(scr));
 
 	color = WMCreateRGBColor(scr, 0x5100, 0x5100, 0x7100, True);
@@ -904,9 +931,9 @@ makeFileName(char *prefix)
     while (access(fname, F_OK)==0) {
 	char buf[30];
 
-	free(fname);
+	wfree(fname);
 	sprintf(buf, "%08lx.cache", time(NULL));
-	fname = wstrappend(prefix, buf);
+	fname = wstrconcat(prefix, buf);
     }
 
     return fname;
@@ -924,7 +951,7 @@ okNewTexture(void *data)
     char *str;
     proplist_t prop;
     TextureListItem *titem;
-    WMScreen *scr = WMWidgetScreen(panel->win);
+    WMScreen *scr = WMWidgetScreen(panel->parent);
 
     titem = wmalloc(sizeof(TextureListItem));
     memset(titem, 0, sizeof(TextureListItem));
@@ -973,7 +1000,7 @@ okEditTexture(void *data)
     if (titem->current) {
 	name = GetTexturePanelTextureName(panel->texturePanel);
 
-	free(titem->title);
+	wfree(titem->title);
 	titem->title = name;
     }
 
@@ -986,7 +1013,7 @@ okEditTexture(void *data)
 
     titem->ispixmap = isPixmap(prop);
 
-    free(titem->texture);
+    wfree(titem->texture);
     titem->texture = str;
 
     XFreePixmap(WMScreenDisplay(WMWidgetScreen(panel->texLs)), titem->preview);
@@ -1069,17 +1096,17 @@ deleteTexture(WMWidget *w, void *data)
 	titem2->selectedFor |= 1 << section;
     }
 
-    free(titem->title);
-    free(titem->texture);
+    wfree(titem->title);
+    wfree(titem->texture);
     PLRelease(titem->prop);
     if (titem->path) {
 	if (remove(titem->path) < 0 && errno != ENOENT) {
 	    wsyserror("could not remove file %s", titem->path);
 	}
-	free(titem->path);
+	wfree(titem->path);
     }
 
-    free(titem);
+    wfree(titem);
 
     WMRemoveListItem(panel->texLs, row);
     WMSetButtonEnabled(panel->delB, False);
@@ -1100,13 +1127,13 @@ extractTexture(WMWidget *w, void *data)
     WMSetFilePanelCanChooseDirectories(opanel, False);
     WMSetFilePanelCanChooseFiles(opanel, True);
 
-    if (WMRunModalFilePanelForDirectory(opanel, panel->win, wgethomedir(),
+    if (WMRunModalFilePanelForDirectory(opanel, panel->parent, wgethomedir(),
 					_("Select File"), NULL)) {
 	path = WMGetFilePanelFileName(opanel);
 
 	OpenExtractPanelFor(panel, path);
 
-	free(path);
+	wfree(path);
     }
 }
 
@@ -1118,7 +1145,7 @@ changePage(WMWidget *w, void *data)
     int section;
     WMListItem *item;
     TextureListItem *titem;
-    WMScreen *scr = WMWidgetScreen(panel->frame);
+    WMScreen *scr = WMWidgetScreen(panel->box);
     RContext *rc = WMScreenRContext(scr);
     static WMPoint positions[] = {
 	{5, 10},
@@ -1388,7 +1415,7 @@ fillColorList(_Panel *panel)
     WMColor *color;
     proplist_t list;
     WMUserDefaults *udb = WMGetStandardUserDefaults();
-    WMScreen *scr = WMWidgetScreen(panel->frame);
+    WMScreen *scr = WMWidgetScreen(panel->box);
     int i;
     
     list = WMGetUDObjectForKey(udb, "ColorList");
@@ -1425,7 +1452,7 @@ changeColorPage(WMWidget *w, void *data)
 {
     _Panel *panel = (_Panel*)data;
     int section;
-    WMScreen *scr = WMWidgetScreen(panel->frame);
+    WMScreen *scr = WMWidgetScreen(panel->box);
     RContext *rc = WMScreenRContext(scr);
     static WMPoint positions[] = {
 	{5, 10},
@@ -1494,7 +1521,7 @@ paintText(WMScreen *scr, Drawable d, WMColor *color, WMFont *font,
 static void
 updateColorPreviewBox(_Panel *panel, int elements)
 {
-    WMScreen *scr = WMWidgetScreen(panel->frame);
+    WMScreen *scr = WMWidgetScreen(panel->box);
     WMPixmap *pixmap;
     Pixmap d;
 
@@ -1599,7 +1626,7 @@ changedTabItem(struct WMTabViewDelegate *self, WMTabView *tabView,
 	}
 	changeColorPage(panel->colP, panel);
 	break;
-     case 2:
+     case 3:
 	switch (panel->oldTabItem) {
 	 case 0:
 	    changePage(NULL, panel);
@@ -1662,13 +1689,13 @@ createPanel(Panel *p)
 {
     _Panel *panel = (_Panel*)p;
     WMFont *font;
-    WMScreen *scr = WMWidgetScreen(panel->win);
+    WMScreen *scr = WMWidgetScreen(panel->parent);
     WMTabViewItem *item;
     int i;
     char *tmp;
     Bool ok = True;
     
-    panel->fprefix = wstrappend(wusergnusteppath(), "/.AppInfo");
+    panel->fprefix = wstrconcat(wusergnusteppath(), "/.AppInfo");
 
     if (access(panel->fprefix, F_OK)!=0) {
 	if (mkdir(panel->fprefix, 0755) < 0) {
@@ -1677,8 +1704,8 @@ createPanel(Panel *p)
 	}
     }
     if (ok) {
-	tmp = wstrappend(panel->fprefix, "/WPrefs/");
-	free(panel->fprefix);
+	tmp = wstrconcat(panel->fprefix, "/WPrefs/");
+	wfree(panel->fprefix);
 	panel->fprefix = tmp;
 	if (access(panel->fprefix, F_OK)!=0) {
 	    if (mkdir(panel->fprefix, 0755) < 0) {
@@ -1695,12 +1722,11 @@ createPanel(Panel *p)
     panel->offLed = WMCreatePixmapFromXPMData(scr, blueled2_xpm);
     panel->hand = WMCreatePixmapFromXPMData(scr, hand_xpm);
 
-    panel->frame = WMCreateFrame(panel->win);
-    WMResizeWidget(panel->frame, FRAME_WIDTH, FRAME_HEIGHT);
-    WMMoveWidget(panel->frame, FRAME_LEFT, FRAME_TOP);
+    panel->box = WMCreateBox(panel->parent);
+    WMSetBoxExpandsToParent(panel->box, 2, 2, 0, 0);
 
     /* preview box */
-    panel->prevL = WMCreateLabel(panel->frame);
+    panel->prevL = WMCreateLabel(panel->box);
     WMResizeWidget(panel->prevL, 240, FRAME_HEIGHT - 20);
     WMMoveWidget(panel->prevL, 15, 10);
     WMSetLabelRelief(panel->prevL, WRSunken);
@@ -1714,14 +1740,14 @@ createPanel(Panel *p)
 
     tabviewDelegate.data = panel;
 
-    panel->tabv = WMCreateTabView(panel->frame);
+    panel->tabv = WMCreateTabView(panel->box);
     WMResizeWidget(panel->tabv, 245, FRAME_HEIGHT - 20);
     WMMoveWidget(panel->tabv, 265, 10);
     WMSetTabViewDelegate(panel->tabv, &tabviewDelegate);
 
     /*** texture list ***/
 
-    panel->texF = WMCreateFrame(panel->frame);
+    panel->texF = WMCreateFrame(panel->box);
     WMSetFrameRelief(panel->texF, WRFlat);
 
     item = WMCreateTabViewItemWithIdentifier(0);
@@ -1771,7 +1797,7 @@ createPanel(Panel *p)
     WMSetButtonImagePosition(panel->newB, WIPAbove);
     WMSetButtonText(panel->newB, _("New"));
     WMSetButtonAction(panel->newB, newTexture, panel);
-    SetButtonAlphaImage(scr, panel->newB, TNEW_FILE);
+    SetButtonAlphaImage(scr, panel->newB, TNEW_FILE, NULL, NULL);
 
     WMSetBalloonTextForView(_("Create a new texture."),
 			    WMWidgetView(panel->newB));
@@ -1783,7 +1809,7 @@ createPanel(Panel *p)
     WMSetButtonImagePosition(panel->ripB, WIPAbove);
     WMSetButtonText(panel->ripB, _("Extract..."));
     WMSetButtonAction(panel->ripB, extractTexture, panel);
-    SetButtonAlphaImage(scr, panel->ripB, TEXTR_FILE);
+    SetButtonAlphaImage(scr, panel->ripB, TEXTR_FILE, NULL, NULL);
 
     WMSetBalloonTextForView(_("Extract texture(s) from a theme or a style file."),
 			    WMWidgetView(panel->ripB));
@@ -1796,7 +1822,7 @@ createPanel(Panel *p)
     WMSetButtonFont(panel->editB, font);
     WMSetButtonImagePosition(panel->editB, WIPAbove);
     WMSetButtonText(panel->editB, _("Edit"));
-    SetButtonAlphaImage(scr, panel->editB, TEDIT_FILE);
+    SetButtonAlphaImage(scr, panel->editB, TEDIT_FILE, NULL, NULL);
     WMSetButtonAction(panel->editB, editTexture, panel);
     WMSetBalloonTextForView(_("Edit the highlighted texture."),
 			    WMWidgetView(panel->editB));
@@ -1807,7 +1833,7 @@ createPanel(Panel *p)
     WMSetButtonFont(panel->delB, font);
     WMSetButtonImagePosition(panel->delB, WIPAbove);
     WMSetButtonText(panel->delB, _("Delete"));
-    SetButtonAlphaImage(scr, panel->delB, TDEL_FILE);
+    SetButtonAlphaImage(scr, panel->delB, TDEL_FILE, NULL, NULL);
     WMSetButtonEnabled(panel->delB, False);
     WMSetButtonAction(panel->delB, deleteTexture, panel);
     WMSetBalloonTextForView(_("Delete the highlighted texture."),
@@ -1818,7 +1844,7 @@ createPanel(Panel *p)
     WMMapSubwidgets(panel->texF);
 
     /*** colors ***/
-    panel->colF = WMCreateFrame(panel->frame);
+    panel->colF = WMCreateFrame(panel->box);
     WMSetFrameRelief(panel->colF, WRFlat);
 
     item = WMCreateTabViewItemWithIdentifier(1);
@@ -1867,12 +1893,39 @@ createPanel(Panel *p)
     }
 
     WMMapSubwidgets(panel->colF);
+    
+#ifdef unfinished
+    /*** root bg ***/
+    
+    panel->bgF = WMCreateFrame(panel->box);
+    WMSetFrameRelief(panel->bgF, WRFlat);
+    
+    item = WMCreateTabViewItemWithIdentifier(2);
+    WMSetTabViewItemView(item, WMWidgetView(panel->bgF));
+    WMSetTabViewItemLabel(item, _("Background"));
 
+    WMAddItemInTabView(panel->tabv, item);
+
+    panel->bgprevL = WMCreateLabel(panel->bgF);
+    WMResizeWidget(panel->bgprevL, 230, 155);
+    WMMoveWidget(panel->bgprevL, 5, 5);
+    WMSetLabelRelief(panel->bgprevL, WRSunken);
+
+    panel->selbgB = WMCreateCommandButton(panel->bgF);
+    WMMoveWidget(panel->selbgB, 5, 165);
+    WMResizeWidget(panel->selbgB, 100, 24);
+    WMSetButtonText(panel->selbgB, _("Browse..."));
+    
+    
+    
+    
+    WMMapSubwidgets(panel->bgF);
+#endif /* unfinished */
     /*** options ***/
-    panel->optF = WMCreateFrame(panel->frame);
+    panel->optF = WMCreateFrame(panel->box);
     WMSetFrameRelief(panel->optF, WRFlat);
 
-    item = WMCreateTabViewItemWithIdentifier(2);
+    item = WMCreateTabViewItemWithIdentifier(3);
     WMSetTabViewItemView(item, WMWidgetView(panel->optF));
     WMSetTabViewItemLabel(item, _("Options"));
 
@@ -1911,7 +1964,7 @@ createPanel(Panel *p)
 	    } else {
 		wwarning(_("could not load icon file %s"), path);
 	    }
-	    free(path);
+	    wfree(path);
 	}
     }
     WMGroupButtons(panel->mstyB[0], panel->mstyB[1]);
@@ -1951,8 +2004,8 @@ createPanel(Panel *p)
 
     /**/
 
-    WMRealizeWidget(panel->frame);
-    WMMapSubwidgets(panel->frame);
+    WMRealizeWidget(panel->box);
+    WMMapSubwidgets(panel->box);
 
     WMSetPopUpButtonSelectedItem(panel->secP, 0);
 
@@ -1964,7 +2017,7 @@ createPanel(Panel *p)
 
     fillColorList(panel);
 
-    panel->texturePanel = CreateTexturePanel(panel->win);
+    panel->texturePanel = CreateTexturePanel(panel->parent);
 }
 
 
@@ -2033,8 +2086,8 @@ showData(_Panel *panel)
 	if (!str)
 	    str = colorOptions[i*2+1];
 
-	if (!(color = WMCreateNamedColor(WMWidgetScreen(panel->frame), str, False))) {
-	    color = WMCreateNamedColor(WMWidgetScreen(panel->frame), "#000000", False);
+	if (!(color = WMCreateNamedColor(WMWidgetScreen(panel->box), str, False))) {
+	    color = WMCreateNamedColor(WMWidgetScreen(panel->box), "#000000", False);
 	}
 
 	panel->colors[i] = color;
@@ -2048,15 +2101,11 @@ showData(_Panel *panel)
     }
     updatePreviewBox(panel, EVERYTHING);
 
-
-    for (i = 0; i < 3; i++) {
-	WMSetButtonSelected(panel->mstyB[i], i==panel->menuStyle);
-	WMSetButtonSelected(panel->taliB[i], i==panel->titleAlignment);
-    }
+    WMSetButtonSelected(panel->mstyB[panel->menuStyle], True);
+    WMSetButtonSelected(panel->taliB[panel->titleAlignment], True);
 }
 
 
-
 static void
 storeData(_Panel *panel)
 {
@@ -2077,7 +2126,7 @@ storeData(_Panel *panel)
 
 	if (str) {
 	    SetStringForKey(str, colorOptions[i*2]);
-	    free(str);
+	    wfree(str);
 	}
     }
 
@@ -2146,7 +2195,7 @@ prepareForClose(_Panel *panel)
 
 	str = WMGetColorRGBDescription(color);
 	PLAppendArrayElement(textureList, PLMakeString(str));
-	free(str);
+	wfree(str);
     }
     WMSetUDObjectForKey(udb, textureList, "ColorList");
     PLRelease(textureList);
@@ -2169,7 +2218,7 @@ InitAppearance(WMScreen *scr, WMWindow *win)
     panel->description = _("Background texture configuration for windows,\n"
 			   "menus and icons.");
 
-    panel->win = win;
+    panel->parent = win;
 
     panel->callbacks.createWidgets = createPanel;
     panel->callbacks.updateDomain = storeData;
@@ -2204,10 +2253,10 @@ OpenExtractPanelFor(_Panel *panel, char *path)
     ExtractPanel *epanel;
     WMColor *color;
     WMFont *font;
-    WMScreen *scr = WMWidgetScreen(panel->win);
+    WMScreen *scr = WMWidgetScreen(panel->parent);
 
     epanel = wmalloc(sizeof(ExtractPanel));
-    epanel->win = WMCreatePanelWithStyleForWindow(panel->win, "extract",
+    epanel->win = WMCreatePanelWithStyleForWindow(panel->parent, "extract",
 						  WMTitledWindowMask
 						  |WMClosableWindowMask);
     WMResizeWidget(epanel->win, 245, 250);
