@@ -24,14 +24,14 @@
 #include "WPrefs.h"
 
 typedef struct _Panel {
-    WMFrame *frame;
+    WMBox *box;
     char *sectionName;
 
     char *description;
 
     CallbackRec callbacks;
     
-    WMWindow *win;
+    WMWidget *parent;
 
     WMButton *swi[8];
 
@@ -51,8 +51,9 @@ showData(_Panel *panel)
     WMSetButtonSelected(panel->swi[1], WMGetUDBoolForKey(udb, "NoXSetStuff"));
     WMSetButtonSelected(panel->swi[2], GetBoolForKey("SaveSessionOnExit"));
     WMSetButtonSelected(panel->swi[3], GetBoolForKey("UseSaveUnders"));
-    WMSetButtonSelected(panel->swi[4], GetBoolForKey("DisableBlinking"));
+    WMSetButtonSelected(panel->swi[4], GetBoolForKey("WindozeCycling"));
     WMSetButtonSelected(panel->swi[5], GetBoolForKey("DontConfirmKill"));
+    WMSetButtonSelected(panel->swi[6], GetBoolForKey("DisableBlinking"));
 }
 
 
@@ -61,26 +62,26 @@ createPanel(Panel *p)
 {
     _Panel *panel = (_Panel*)p;
     int i;
-    
-    panel->frame = WMCreateFrame(panel->win);
-    WMResizeWidget(panel->frame, FRAME_WIDTH, FRAME_HEIGHT);
-    WMMoveWidget(panel->frame, FRAME_LEFT, FRAME_TOP);
 
-    for (i=0; i<6; i++) {
-	panel->swi[i] = WMCreateSwitchButton(panel->frame);
+    panel->box = WMCreateBox(panel->parent);
+    WMSetBoxExpandsToParent(panel->box, 2, 2, 0, 0);
+
+    for (i=0; i<7; i++) {
+	panel->swi[i] = WMCreateSwitchButton(panel->box);
 	WMResizeWidget(panel->swi[i], FRAME_WIDTH-40, 25);
 	WMMoveWidget(panel->swi[i], 20, 20+i*25);
     }
     
     WMSetButtonText(panel->swi[0], _("Disable miniwindows (icons for miniaturized windows). For use with KDE/GNOME."));
     WMSetButtonText(panel->swi[1], _("Do not set non-WindowMaker specific parameters (do not use xset)"));
-    WMSetButtonText(panel->swi[2], _("Automatically save session when exiting WindowMaker"));
+    WMSetButtonText(panel->swi[2], _("Automatically save session when exiting Window Maker"));
     WMSetButtonText(panel->swi[3], _("Use SaveUnder in window frames, icons, menus and other objects"));
-    WMSetButtonText(panel->swi[4], _("Disable cycling color highlighting of icons."));
-    WMSetButtonText(panel->swi[5], _("Disable confirmation panel for the Kill command."));
+    WMSetButtonText(panel->swi[4], _("Use Windoze style cycling"));
+    WMSetButtonText(panel->swi[5], _("Disable confirmation panel for the Kill command"));
+    WMSetButtonText(panel->swi[6], _("Disable cycling color highlighting of icons"));
 
-    WMRealizeWidget(panel->frame);
-    WMMapSubwidgets(panel->frame);
+    WMRealizeWidget(panel->box);
+    WMMapSubwidgets(panel->box);
     
     showData(panel);
 }
@@ -97,13 +98,14 @@ storeDefaults(_Panel *panel)
 
     SetBoolForKey(WMGetButtonSelected(panel->swi[2]), "SaveSessionOnExit");
     SetBoolForKey(WMGetButtonSelected(panel->swi[3]), "UseSaveUnders");
-    SetBoolForKey(WMGetButtonSelected(panel->swi[4]), "DisableBlinking");
+    SetBoolForKey(WMGetButtonSelected(panel->swi[4]), "WindozeCycling");
     SetBoolForKey(WMGetButtonSelected(panel->swi[5]), "DontConfirmKill");
+    SetBoolForKey(WMGetButtonSelected(panel->swi[6]), "DisableBlinking");
 }
 
 
 Panel*
-InitExpert(WMScreen *scr, WMWindow *win)
+InitExpert(WMScreen *scr, WMWidget *parent)
 {
     _Panel *panel;
 
@@ -115,7 +117,7 @@ InitExpert(WMScreen *scr, WMWindow *win)
     panel->description = _("Options for people who know what they're doing...\n"
 			   "Also have some other misc. options.");
 
-    panel->win = win;
+    panel->parent = parent;
 
     panel->callbacks.createWidgets = createPanel;
     panel->callbacks.updateDomain = storeDefaults;
