@@ -38,7 +38,7 @@
 
 
 typedef struct _Panel {
-    WMFrame *frame;
+    WMBox *box;
 
     char *sectionName;
 
@@ -46,7 +46,7 @@ typedef struct _Panel {
 
     CallbackRec callbacks;
     
-    WMWindow *win;
+    WMWidget *parent;
     
     WMFrame *speedF;
     WMLabel *speedL;
@@ -66,14 +66,10 @@ typedef struct _Panel {
     WMLabel *listL;
     WMLabel *appL;
     WMLabel *selL;
-    WMLabel *mblL;
-    WMLabel *mbmL;
-    WMLabel *mbrL;
-    WMButton *lmb[3];
-    WMButton *amb[3];
-    WMButton *smb[3];
+    WMPopUpButton *listP;
+    WMPopUpButton *appP;
+    WMPopUpButton *selP;
 
-    
     WMButton *disaB;
 
     WMFrame *grabF;
@@ -92,25 +88,16 @@ typedef struct _Panel {
 #define DELAY_ICON "timer%i"
 #define DELAY_ICON_S "timer%is"
 
-#define MOUSEB_L "minimouseleft"
-#define MOUSEB_M "minimousemiddle"
-#define MOUSEB_R "minimouseright"
 
 /* need access to the double click variables */
-#include "WINGsP.h"
+#include <WINGs/WINGsP.h>
 
 
 
-static char *modifierNames[] = {
-    "Shift",
-	"Lock",
-	"Control",
-	"Mod1",
-	"Mod2",
-	"Mod3",
-	"Mod4",
-	"Mod5"
-};
+static char *modifierNames[8]; 
+
+
+static char *buttonNames[6];
 
 
 #define DELAY(i)		((i)*75+170)
@@ -149,11 +136,11 @@ speedChange(WMWidget *w, void *data)
 			    _("Error"),
 			    _("Invalid mouse acceleration value. Must be a positive real value."),
 			    _("OK"), NULL, NULL);
-	    free(tmp);
+	    wfree(tmp);
 	    return;
 	}
 	panel->acceleration = accel;
-	free(tmp);
+	wfree(tmp);
     } else {
 	i = (int)WMGetSliderValue(panel->speedS);
     
@@ -166,14 +153,14 @@ speedChange(WMWidget *w, void *data)
     tmp = WMGetTextFieldText(panel->threT);
     if (sscanf(tmp, "%i", &threshold)!=1 || threshold < 0 
 	|| threshold > panel->maxThreshold) {
-	WMRunAlertPanel(WMWidgetScreen(panel->win), GetWindow(panel), _("Error"),
+	WMRunAlertPanel(WMWidgetScreen(panel->parent), GetWindow(panel), _("Error"),
 			_("Invalid mouse acceleration threshold value. Must be the number of pixels to travel before accelerating."),
 			_("OK"), NULL, NULL);
     } else {
-	setMouseAccel(WMWidgetScreen(panel->win), panel->acceleration, 
+	setMouseAccel(WMWidgetScreen(panel->parent), panel->acceleration, 
 		      threshold);
     }
-    free(tmp);
+    wfree(tmp);
 }
 
 
@@ -212,22 +199,24 @@ getbutton(char *str)
     if (!str)
 	return -2;
 
-    if (strcasecmp(str, "left")==0)
+    if (strcasecmp(str, "none")==0)
 	return 0;
+    else if (strcasecmp(str, "left")==0)
+	return 1;
     else if (strcasecmp(str, "middle")==0)
-	return 1;
+	return 2;
     else if (strcasecmp(str, "right")==0)
-	return 2;
+	return 3;
     else if (strcasecmp(str, "button1")==0)
-	return 0;
-    else if (strcasecmp(str, "button2")==0)
 	return 1;
-    else if (strcasecmp(str, "button3")==0)
+    else if (strcasecmp(str, "button2")==0)
 	return 2;
-    else if (strcasecmp(str, "button4")==0
-	     || strcasecmp(str, "button5")==0) {
-	wwarning(_("mouse button %s not supported by WPrefs."), str);
-	return -2;
+    else if (strcasecmp(str, "button3")==0)
+	return 3;
+    else if (strcasecmp(str, "button4")==0)
+	return 4;
+    else if (strcasecmp(str, "button5")==0) {
+	return 5;
     } else {
 	return -1;
     }
@@ -254,48 +243,50 @@ showData(_Panel *panel)
     int a=-1, b=-1, c=-1;
     float accel;
     char buffer[32];
-    Display *dpy = WMScreenDisplay(WMWidgetScreen(panel->win));
+    Display *dpy = WMScreenDisplay(WMWidgetScreen(panel->parent));
 
     str = GetStringForKey("SelectWindowsMouseButton");
-    if (!str)
-	str = "left";
-    i = getbutton(str);
-    if (i==-1) {
+    if (str) {
+	i = getbutton(str);
+	if (i==-1) {
+	    a = 1;
+	    wwarning(_("bad value %s for option %s"),str, "SelectWindowsMouseButton");
+	} else if (i>=0) {
+	    a = i;
+	} 
+    } else {
 	a = 0;
-	wwarning(_("bad value %s for option %s"),str, "SelectWindowsMouseButton");
-	WMPerformButtonClick(panel->smb[0]);
-    } else if (i>=0) {
-	a = i;
-	WMPerformButtonClick(panel->smb[i]);
-    } 
+    }
+    WMSetPopUpButtonSelectedItem(panel->selP, a);
     
     str = GetStringForKey("WindowListMouseButton");
-    if (!str)
-	str = "middle";
-    i = getbutton(str);
-    if (i==-1) {
+    if (str) {
+	i = getbutton(str);
+	if (i==-1) {
+	    b = 2;
+	    wwarning(_("bad value %s for option %s"), str, "WindowListMouseButton");
+	} else if (i>=0) {
+	    b = i;
+	} 
+    } else {
 	b = 0;
-	wwarning(_("bad value %s for option %s"), str, "WindowListMouseButton");
-	WMPerformButtonClick(panel->lmb[1]);
-    } else if (i>=0) {
-	b = i;
-	WMPerformButtonClick(panel->lmb[i]);
-    } 
+    }
+    WMSetPopUpButtonSelectedItem(panel->listP, b);
     
     str = GetStringForKey("ApplicationMenuMouseButton");
-    if (!str)
-	str = "right";
-    i = getbutton(str);
-    if (i==-1) {
+    if (str) {
+	i = getbutton(str);
+	if (i==-1) {
+	    c = 3;
+	    wwarning(_("bad value %s for option %s"), str, "ApplicationMenuMouseButton");
+	} else if (i>=0) {
+	    c = i;
+	}
+    } else {
 	c = 0;
-	wwarning(_("bad value %s for option %s"), str, "ApplicationMenuMouseButton");
-	WMPerformButtonClick(panel->amb[2]);
-    } else if (i>=0) {
-	c = i;
-	WMPerformButtonClick(panel->amb[i]);
     }
-    
-    
+    WMSetPopUpButtonSelectedItem(panel->appP, c);
+
     WMSetButtonSelected(panel->disaB, GetBoolForKey("DisableWSMouseActions"));
     
     /**/
@@ -419,21 +410,24 @@ fillModifierPopUp(WMPopUpButton *pop)
 	    ptr = strstr(tmp, "_R");
 	    if (ptr)
 		*ptr = 0;
-	    sprintf(buffer, "%s (%s)", modifierNames[j], tmp);
+            sprintf(buffer, "%s (%s)", modifierNames[j], tmp);
+            /*sprintf(buffer, "%s", tmp);*/
 	    WMAddPopUpButtonItem(pop, buffer);
 	    for (i=k+1; i<a; i++) {
+		if (array[i] == NULL)
+		    continue;
 		if (strstr(array[i], tmp)) {
-		    free(array[i]);
+		    wfree(array[i]);
 		    array[i]=NULL;
 		    break;
 		}
 	    }
-	    free(tmp);
+	    wfree(tmp);
 	}
 	
 	while (--a>0) {
 	    if (array[a])
-		free(array[a]);
+		wfree(array[a]);
 	}
     }
     
@@ -443,65 +437,11 @@ fillModifierPopUp(WMPopUpButton *pop)
 
 
 
-static void
-mouseButtonClickA(WMWidget *w, void *data)
-{
-    _Panel *panel = (_Panel*)data;
-    int i;
-    
-    for (i=0; i<3; i++) {
-	if (panel->amb[i]==w)
-	    break;
-    }
-    if (i==3)
-	return;
-    if (WMGetButtonSelected(panel->lmb[i]))
-	WMSetButtonSelected(panel->lmb[i], False);
-    if (WMGetButtonSelected(panel->smb[i]))
-	WMSetButtonSelected(panel->smb[i], False);
-}
-
-static void
-mouseButtonClickL(WMWidget *w, void *data)
-{
-    _Panel *panel = (_Panel*)data;
-    int i;
-    
-    for (i=0; i<3; i++) {
-	if (panel->lmb[i]==w)
-	    break;
-    }
-    if (i==3)
-	return;
-    if (WMGetButtonSelected(panel->smb[i]))
-	WMSetButtonSelected(panel->smb[i], False);
-    if (WMGetButtonSelected(panel->amb[i]))
-	WMSetButtonSelected(panel->amb[i], False);    
-}
-
-static void
-mouseButtonClickS(WMWidget *w, void *data)
-{
-    _Panel *panel = (_Panel*)data;
-    int i;
-    
-    for (i=0; i<3; i++) {
-	if (panel->smb[i]==w)
-	    break;
-    }
-    if (i==3)
-	return;
-    if (WMGetButtonSelected(panel->lmb[i]))
-	WMSetButtonSelected(panel->lmb[i], False);
-    if (WMGetButtonSelected(panel->amb[i]))
-	WMSetButtonSelected(panel->amb[i], False);    
-}
-
 static void
 createPanel(Panel *p)
 {
     _Panel *panel = (_Panel*)p;
-    WMScreen *scr = WMWidgetScreen(panel->win);
+    WMScreen *scr = WMWidgetScreen(panel->parent);
     WMPixmap *icon;
     char *buf1, *buf2;
     int i;
@@ -512,12 +452,11 @@ createPanel(Panel *p)
     color.green = 0xae;
     color.blue = 0xaa;
     
-    panel->frame = WMCreateFrame(panel->win);
-    WMResizeWidget(panel->frame, FRAME_WIDTH, FRAME_HEIGHT);
-    WMMoveWidget(panel->frame, FRAME_LEFT, FRAME_TOP);
+    panel->box = WMCreateBox(panel->parent);
+    WMSetBoxExpandsToParent(panel->box, 2, 2, 0, 0);
     
     /**************** Mouse Speed ****************/
-    panel->speedF = WMCreateFrame(panel->frame);
+    panel->speedF = WMCreateFrame(panel->box);
     WMResizeWidget(panel->speedF, 245, 100);
     WMMoveWidget(panel->speedF, 15, 15);
     WMSetFrameTitle(panel->speedF, _("Mouse Speed"));
@@ -535,7 +474,7 @@ createPanel(Panel *p)
 	} else {
 	    wwarning(_("could not load icon %s"), path);
 	}
-	free(path);
+	wfree(path);
     }
 
     panel->speedS = WMCreateSlider(panel->speedF);
@@ -575,7 +514,7 @@ createPanel(Panel *p)
     
     /***************** Doubleclick Delay ****************/
 
-    panel->ddelaF = WMCreateFrame(panel->frame);
+    panel->ddelaF = WMCreateFrame(panel->box);
     WMResizeWidget(panel->ddelaF, 245, 95);
     WMMoveWidget(panel->ddelaF, 15, 125);
     WMSetFrameTitle(panel->ddelaF, _("Double-Click Delay"));
@@ -605,7 +544,7 @@ createPanel(Panel *p)
 	    } else {
 		wwarning(_("could not load icon file %s"), path);
 	    }
-	    free(path);
+	    wfree(path);
 	}
 	path = LocateImage(buf2);
 	if (path) {
@@ -616,11 +555,11 @@ createPanel(Panel *p)
 	    } else {
 		wwarning(_("could not load icon file %s"), path);
 	    }
-	    free(path);
+	    wfree(path);
 	}
     }
-    free(buf1);
-    free(buf2);
+    wfree(buf1);
+    wfree(buf2);
 
     panel->tester = CreateDoubleTest(panel->ddelaF, _("Test"));
     WMResizeWidget(panel->tester, 84, 29);
@@ -644,12 +583,12 @@ createPanel(Panel *p)
 	WMReleaseFont(font);
 	WMReleaseColor(color);
     }
-    WMSetLabelText(panel->ddelaL, "msec");
+    WMSetLabelText(panel->ddelaL, _("msec"));
 
     WMMapSubwidgets(panel->ddelaF);
     
     /* ************** Workspace Action Buttons **************** */
-    panel->menuF = WMCreateFrame(panel->frame);
+    panel->menuF = WMCreateFrame(panel->box);
     WMResizeWidget(panel->menuF, 240, 145);
     WMMoveWidget(panel->menuF, 270, 15);
     WMSetFrameTitle(panel->menuF, _("Workspace Mouse Actions"));
@@ -659,102 +598,48 @@ createPanel(Panel *p)
     WMMoveWidget(panel->disaB, 10, 20);
     WMSetButtonText(panel->disaB, _("Disable mouse actions"));
 
-    panel->mblL = WMCreateLabel(panel->menuF);
-    WMResizeWidget(panel->mblL, 16, 22);
-    WMMoveWidget(panel->mblL, 135, 40);
-    WMSetLabelImagePosition(panel->mblL, WIPImageOnly);
-    path = LocateImage(MOUSEB_L);
-    if (path) {
-	icon = WMCreatePixmapFromFile(scr, path);
-	if (icon) {
-	    WMSetLabelImage(panel->mblL, icon);
-	    WMReleasePixmap(icon);
-	} else {
-	    wwarning(_("could not load icon file %s"), path);
-	}
-	free(path);
-    }
-    panel->mbmL = WMCreateLabel(panel->menuF);
-    WMResizeWidget(panel->mbmL, 16, 22);
-    WMMoveWidget(panel->mbmL, 170, 40);
-    WMSetLabelImagePosition(panel->mbmL, WIPImageOnly);
-    path = LocateImage(MOUSEB_M);
-    if (path) {
-	icon = WMCreatePixmapFromFile(scr, path);
-	if (icon) {
-	    WMSetLabelImage(panel->mbmL, icon);
-	    WMReleasePixmap(icon);
-	} else {
-	    wwarning(_("could not load icon file %s"), path);
-	}
-	free(path);
-    }
-
-    panel->mbrL = WMCreateLabel(panel->menuF);
-    WMResizeWidget(panel->mbrL, 16, 22);
-    WMMoveWidget(panel->mbrL, 205, 40);
-    WMSetLabelImagePosition(panel->mbrL, WIPImageOnly);
-    path = LocateImage(MOUSEB_R);
-    if (path) {
-	icon = WMCreatePixmapFromFile(scr, path);
-	if (icon) {
-	    WMSetLabelImage(panel->mbrL, icon);
-	    WMReleasePixmap(icon);
-	} else {
-	    wwarning(_("could not load icon file %s"), path);
-	}
-	free(path);
-    }
-
+    
     panel->appL = WMCreateLabel(panel->menuF);
     WMResizeWidget(panel->appL, 125, 16);
-    WMMoveWidget(panel->appL, 5, 65);
+    WMMoveWidget(panel->appL, 5, 45);
     WMSetLabelTextAlignment(panel->appL, WARight);
     WMSetLabelText(panel->appL, _("Applications menu"));
+
+    panel->appP = WMCreatePopUpButton(panel->menuF);
+    WMResizeWidget(panel->appP, 95, 20);
+    WMMoveWidget(panel->appP, 135, 45);
     
     panel->listL = WMCreateLabel(panel->menuF);
     WMResizeWidget(panel->listL, 125, 16);
-    WMMoveWidget(panel->listL, 5, 90);
+    WMMoveWidget(panel->listL, 5, 80);
     WMSetLabelTextAlignment(panel->listL, WARight);
     WMSetLabelText(panel->listL, _("Window list menu"));
     
+    panel->listP = WMCreatePopUpButton(panel->menuF);
+    WMResizeWidget(panel->listP, 95, 20);
+    WMMoveWidget(panel->listP, 135, 80);
+
+    
     panel->selL = WMCreateLabel(panel->menuF);
     WMResizeWidget(panel->selL, 125, 16);
     WMMoveWidget(panel->selL, 5, 115);
     WMSetLabelTextAlignment(panel->selL, WARight);
     WMSetLabelText(panel->selL, _("Select windows"));
 
-    
-    for (i=0; i<3; i++) {
-	panel->amb[i] = WMCreateRadioButton(panel->menuF);
-	WMResizeWidget(panel->amb[i], 24, 24);
-	WMMoveWidget(panel->amb[i], 135+35*i, 65);
-	WMSetButtonText(panel->amb[i], NULL);
-	WMSetButtonAction(panel->amb[i], mouseButtonClickA, panel);
+    panel->selP = WMCreatePopUpButton(panel->menuF);
+    WMResizeWidget(panel->selP, 95, 20);
+    WMMoveWidget(panel->selP, 135, 115);
 
-	panel->lmb[i] = WMCreateRadioButton(panel->menuF);
-	WMResizeWidget(panel->lmb[i], 24, 24);
-	WMMoveWidget(panel->lmb[i], 135+35*i, 90);
-	WMSetButtonText(panel->lmb[i], NULL);	
-	WMSetButtonAction(panel->lmb[i], mouseButtonClickL, panel);
-
-	panel->smb[i] = WMCreateRadioButton(panel->menuF);
-	WMResizeWidget(panel->smb[i], 24, 24);
-	WMMoveWidget(panel->smb[i], 135+35*i, 115);
-	WMSetButtonText(panel->smb[i], NULL);	
-	WMSetButtonAction(panel->smb[i], mouseButtonClickS, panel);
-
-	if (i>0) {
-	    WMGroupButtons(panel->lmb[0], panel->lmb[i]);
-	    WMGroupButtons(panel->amb[0], panel->amb[i]);
-	    WMGroupButtons(panel->smb[0], panel->smb[i]);	    
-	}
+    for (i = 0; i < sizeof(buttonNames)/sizeof(char*); i++) {
+	WMAddPopUpButtonItem(panel->appP, buttonNames[i]);
+	WMAddPopUpButtonItem(panel->selP, buttonNames[i]);
+	WMAddPopUpButtonItem(panel->listP, buttonNames[i]);
     }
     
     WMMapSubwidgets(panel->menuF);
     
     /* ************** Grab Modifier **************** */
-    panel->grabF = WMCreateFrame(panel->frame);
+    panel->grabF = WMCreateFrame(panel->box);
     WMResizeWidget(panel->grabF, 240, 55);
     WMMoveWidget(panel->grabF, 270, 165);
     WMSetFrameTitle(panel->grabF, _("Mouse Grab Modifier"));
@@ -772,8 +657,8 @@ createPanel(Panel *p)
     
     WMMapSubwidgets(panel->grabF);
     
-    WMRealizeWidget(panel->frame);
-    WMMapSubwidgets(panel->frame);
+    WMRealizeWidget(panel->box);
+    WMMapSubwidgets(panel->box);
     
     
     showData(panel);
@@ -787,7 +672,7 @@ storeCommandInScript(char *cmd, char *line)
     FILE *f;
     char buffer[128];
 
-    path = wstrappend(wusergnusteppath(), "/Library/WindowMaker/autostart");
+    path = wstrconcat(wusergnusteppath(), "/Library/WindowMaker/autostart");
     
     f = fopen(path, "r");
     if (!f) {
@@ -805,11 +690,12 @@ storeCommandInScript(char *cmd, char *line)
 	char *tmppath;
 	FILE *fo;
 
-	tmppath = wstrappend(wusergnusteppath(), 
+	tmppath = wstrconcat(wusergnusteppath(), 
 			     "/Library/WindowMaker/autostart.tmp");
 	fo = fopen(tmppath, "w");
 	if (!fo) {
-	    wsyserror(_("could not create temporary file %s"), tmppath);
+            wsyserror(_("could not create temporary file %s"), tmppath);
+            wfree(tmppath);
 	    goto end;
 	}
 
@@ -842,13 +728,13 @@ storeCommandInScript(char *cmd, char *line)
 	if (rename(tmppath, path)!=0) {
 	    wsyserror(_("could not rename file %s to %s\n"), tmppath, path);
 	}
-	free(tmppath);
+	wfree(tmppath);
     }
     sprintf(buffer, "chmod u+x %s", path);
     system(buffer);
 
 end:
-    free(path);
+    wfree(path);
     if (f)
 	fclose(f);
 }
@@ -860,13 +746,13 @@ storeData(_Panel *panel)
     char buffer[64];
     int i;
     char *tmp, *p;
-    static char *button[3] = {"left", "middle", "right"};
+    static char *button[6] = {"None", "left", "middle", "right", "Button4", "Button5"};
     WMUserDefaults *udb = WMGetStandardUserDefaults();
 
     if (!WMGetUDBoolForKey(udb, "NoXSetStuff")) {
 	tmp = WMGetTextFieldText(panel->threT);
 	if (strlen(tmp)==0) {
-	    free(tmp);
+	    wfree(tmp);
 	    tmp = wstrdup("4");
 	}
 
@@ -874,7 +760,7 @@ storeData(_Panel *panel)
 		10, tmp);
 	storeCommandInScript(XSET" m", buffer);
 
-	free(tmp);
+	wfree(tmp);
     }
 
     tmp = WMGetTextFieldText(panel->ddelaT);
@@ -883,28 +769,16 @@ storeData(_Panel *panel)
 
     SetBoolForKey(WMGetButtonSelected(panel->disaB), "DisableWSMouseActions");
 
-    for (i=0; i<3; i++) {
-	if (WMGetButtonSelected(panel->amb[i]))
-	    break;
-    }
-    if (i<3)
-	SetStringForKey(button[i], "ApplicationMenuMouseButton");
+    i = WMGetPopUpButtonSelectedItem(panel->appP);
+    SetStringForKey(button[i], "ApplicationMenuMouseButton");
+
+    i = WMGetPopUpButtonSelectedItem(panel->listP);
+    SetStringForKey(button[i], "WindowListMouseButton");
     
-    for (i=0; i<3; i++) {
-	if (WMGetButtonSelected(panel->lmb[i]))
-	    break;
-    }
-    if (i<3)
-	SetStringForKey(button[i], "WindowListMouseButton");
+    i = WMGetPopUpButtonSelectedItem(panel->selP);
+    SetStringForKey(button[i], "SelectWindowsMouseButton");
     
-    for (i=0; i<3; i++) {
-	if (WMGetButtonSelected(panel->smb[i]))
-	    break;
-    }
-    if (i<3)
-	SetStringForKey(button[i], "SelectWindowsMouseButton");
-    
-    tmp = WMGetPopUpButtonItem(panel->grabP, 
+    tmp = WMGetPopUpButtonItem(panel->grabP,
 			       WMGetPopUpButtonSelectedItem(panel->grabP));
     tmp = wstrdup(tmp);
     p = strchr(tmp, ' ');
@@ -912,14 +786,30 @@ storeData(_Panel *panel)
 
     SetStringForKey(tmp, "ModifierKey");
 
-    free(tmp);
+    wfree(tmp);
 }
 
 
 Panel*
-InitMouseSettings(WMScreen *scr, WMWindow *win)
+InitMouseSettings(WMScreen *scr, WMWidget *parent)
 {
     _Panel *panel;
+
+    modifierNames[0] = wstrdup(_("Shift"));
+    modifierNames[1] = wstrdup(_("Lock"));
+    modifierNames[2] = wstrdup(_("Control"));
+    modifierNames[3] = wstrdup(_("Mod1"));
+    modifierNames[4] = wstrdup(_("Mod2"));
+    modifierNames[5] = wstrdup(_("Mod3"));
+    modifierNames[6] = wstrdup(_("Mod4"));
+    modifierNames[7] = wstrdup(_("Mod5"));
+			
+    buttonNames[0] = wstrdup(_("None"));
+    buttonNames[1] = wstrdup(_("Btn1 (left)"));
+    buttonNames[2] = wstrdup(_("Btn2 (middle)"));
+    buttonNames[3] = wstrdup(_("Btn3 (right)"));
+    buttonNames[4] = wstrdup(_("Btn4"));
+    buttonNames[5] = wstrdup(_("Btn5"));
     
     panel = wmalloc(sizeof(_Panel));
     memset(panel, 0, sizeof(_Panel));
@@ -929,7 +819,7 @@ InitMouseSettings(WMScreen *scr, WMWindow *win)
     panel->description = _("Mouse speed/acceleration, double click delay,\n"
 			   "mouse button bindings etc.");
 
-    panel->win = win;
+    panel->parent = parent;
 
     panel->callbacks.createWidgets = createPanel;
     panel->callbacks.updateDomain = storeData;
