@@ -2,7 +2,7 @@
  * 
  *  WPrefs - Window Maker Preferences Program
  * 
- *  Copyright (c) 1998 Alfredo K. Kojima
+ *  Copyright (c) 1998-2000 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,37 +25,37 @@
 #include <assert.h>
 
 
-extern Panel *InitWindowHandling(WMScreen *scr, WMWindow *win);
+extern Panel *InitWindowHandling(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitKeyboardSettings(WMScreen *scr, WMWindow *win);
+extern Panel *InitKeyboardSettings(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitMouseSettings(WMScreen *scr, WMWindow *win);
+extern Panel *InitMouseSettings(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitKeyboardShortcuts(WMScreen *scr, WMWindow *win);
+extern Panel *InitKeyboardShortcuts(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitWorkspace(WMScreen *scr, WMWindow *win);
+extern Panel *InitWorkspace(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitFocus(WMScreen *scr, WMWindow *win);
+extern Panel *InitFocus(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitPreferences(WMScreen *scr, WMWindow *win);
+extern Panel *InitPreferences(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitFont(WMScreen *scr, WMWindow *win);
+extern Panel *InitFont(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitConfigurations(WMScreen *scr, WMWindow *win);
+extern Panel *InitConfigurations(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitPaths(WMScreen *scr, WMWindow *win);
+extern Panel *InitPaths(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitMenu(WMScreen *scr, WMWindow *win);
+extern Panel *InitMenu(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitExpert(WMScreen *scr, WMWindow *win);
+extern Panel *InitExpert(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitMenuPreferences(WMScreen *scr, WMWindow *win);
+extern Panel *InitMenuPreferences(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitIcons(WMScreen *scr, WMWindow *win);
+extern Panel *InitIcons(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitThemes(WMScreen *scr, WMWindow *win);
+extern Panel *InitThemes(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitAppearance(WMScreen *scr, WMWindow *win);
+extern Panel *InitAppearance(WMScreen *scr, WMWidget *parent);
 
 
 
@@ -88,7 +88,7 @@ typedef struct _WPrefs {
     WMLabel *versionL;
     WMLabel *creditsL;
     WMLabel *statusL;
-    
+        
     Panel *currentPanel;
 } _WPrefs;
 
@@ -352,8 +352,8 @@ createMainWindow(WMScreen *scr)
     WMMoveWidget(WPrefs.creditsL, 10, FRAME_HEIGHT-60);
     WMSetLabelTextAlignment(WPrefs.creditsL, WACenter);
     WMSetLabelText(WPrefs.creditsL, _("Programming/Design: Alfredo K. Kojima\n"
-		   		"Artwork: Marco van Hylckama Vlieg and Largo\n"
-				      "More Programming: James Thompson"));
+		   		"Artwork: Marco van Hylckama Vlieg, Largo et al\n"
+				      "More Programming: James Thompson et al"));
    
     
     WMMapSubwidgets(WPrefs.win);
@@ -376,7 +376,10 @@ showPanel(Panel *panel)
 
     WMSetWindowTitle(WPrefs.win, rec->sectionName);
     
-    WMMapWidget(rec->frame);
+    if (rec->callbacks.showPanel)
+	(*rec->callbacks.showPanel)(panel);
+
+    WMMapWidget(rec->box);
 }
 
 
@@ -386,7 +389,10 @@ hidePanel(Panel *panel)
 {
     PanelRec *rec = (PanelRec*)panel;    
     
-    WMUnmapWidget(rec->frame);
+    WMUnmapWidget(rec->box);
+    
+    if (rec->callbacks.hidePanel)
+	(*rec->callbacks.hidePanel)(panel);
 }
 
 
@@ -404,17 +410,25 @@ savePanelData(Panel *panel)
 static void 
 changeSection(WMWidget *self, void *data)
 {
-    if (WPrefs.banner) {
-	WMDestroyWidget(WPrefs.banner);
-	WPrefs.banner = NULL;
+    if (WPrefs.currentPanel == data)
+	return;
+	
+    if (WPrefs.currentPanel == NULL) {
+	WMDestroyWidget(WPrefs.nameL);
+	WMDestroyWidget(WPrefs.creditsL);
+	WMDestroyWidget(WPrefs.versionL);
+	WMDestroyWidget(WPrefs.statusL);
+
+	WMSetFrameRelief(WPrefs.banner, WRGroove);
+
 /*	WMMapWidget(WPrefs.undosBtn);
 	WMMapWidget(WPrefs.undoBtn);
  */
 	WMMapWidget(WPrefs.saveBtn);
     }
-    
+
     showPanel(data);
-    
+
     if (WPrefs.currentPanel)
 	hidePanel(WPrefs.currentPanel);
     WPrefs.currentPanel = data;
@@ -422,8 +436,6 @@ changeSection(WMWidget *self, void *data)
 
 
 
-
-
 char*
 LocateImage(char *name)
 {
@@ -437,7 +449,7 @@ LocateImage(char *name)
 	sprintf(tmp, "%s.xpm", name);
 	path = WMPathForResourceOfType(tmp, "xpm");
     }
-    free(tmp);
+    wfree(tmp);
     if (!path) {
 	wwarning(_("could not locate image file %s\n"), name);
     }
@@ -578,7 +590,7 @@ SetButtonAlphaImage(WMScreen *scr, WMButton *bPtr, char *file,
 	WMReleasePixmap(icon);
 
     if (iconPath)
-	free(iconPath);
+	wfree(iconPath);
 }
 
 
@@ -612,7 +624,7 @@ AddSection(Panel *panel, char *iconFile)
 	}
 	SetButtonAlphaImage(WMWidgetScreen(bPtr), bPtr, iconFile,
 			    t1, t2);
-	free(t1);
+	wfree(t1);
     }
     WMMapWidget(bPtr);
 
@@ -664,7 +676,7 @@ Initialize(WMScreen *scr)
 	        WMReleasePixmap(icon);
 	    }
 	}
-	free(path);
+	wfree(path);
     }
     
     memset(&WPrefs, 0, sizeof(_WPrefs));
@@ -679,33 +691,34 @@ Initialize(WMScreen *scr)
 
     WMSetLabelText(WPrefs.statusL, _("Initializing configuration panels..."));
 
-    InitWindowHandling(scr, WPrefs.win);
-    InitFocus(scr, WPrefs.win);
-    InitMenuPreferences(scr, WPrefs.win);
-    InitIcons(scr, WPrefs.win);
-    InitPreferences(scr, WPrefs.win);
+    InitFocus(scr, WPrefs.banner);
+    InitWindowHandling(scr, WPrefs.banner);
 
-    InitPaths(scr, WPrefs.win);    
-    InitWorkspace(scr, WPrefs.win);
-    InitConfigurations(scr, WPrefs.win);
+    InitMenuPreferences(scr, WPrefs.banner);
+    InitIcons(scr, WPrefs.banner);
+    InitPreferences(scr, WPrefs.banner);
 
-    InitMenu(scr, WPrefs.win);
+    InitPaths(scr, WPrefs.banner);    
+    InitWorkspace(scr, WPrefs.banner);
+    InitConfigurations(scr, WPrefs.banner);
+
+    InitMenu(scr, WPrefs.banner);
 
 #ifdef not_yet_fully_implemented
-    InitKeyboardSettings(scr, WPrefs.win);
+    InitKeyboardSettings(scr, WPrefs.banner);
 #endif
-    InitKeyboardShortcuts(scr, WPrefs.win);
-    InitMouseSettings(scr, WPrefs.win);
+    InitKeyboardShortcuts(scr, WPrefs.banner);
+    InitMouseSettings(scr, WPrefs.banner);
 
-    InitAppearance(scr, WPrefs.win);
+    InitAppearance(scr, WPrefs.banner);
     
 #ifdef akk
-    InitFont(scr, WPrefs.win);
+    InitFont(scr, WPrefs.banner);
 #endif
 #ifdef not_yet_fully_implemented
-    InitThemes(scr, WPrefs.win);
+    InitThemes(scr, WPrefs.banner);
 #endif
-    InitExpert(scr, WPrefs.win);
+    InitExpert(scr, WPrefs.banner);
 
     WMRealizeWidget(WPrefs.scrollV);
 
@@ -747,14 +760,18 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
 		path);
 	WMRunAlertPanel(scr, mainw, _("Error"), mbuf, _("OK"), NULL, NULL);
     }
-    free(path);
+    wfree(path);
 
     path = getenv("WMAKER_BIN_NAME");
     if (!path)
 	path = "wmaker";
-    path = wstrappend(path, " --version");
+    {
+	char *command;
 
-    file = popen(path, "r");
+	command = wstrconcat(path, " --version");
+	file = popen(command, "r");
+	wfree(command);
+    }
     if (!file || !fgets(buffer, 1023, file)) {
 	wsyserror(_("could not extract version information from Window Maker"));
 	wfatal(_("Make sure wmaker is in your search path."));
@@ -789,9 +806,15 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
 	WMRunAlertPanel(scr, mainw, _("Warning"), mbuf, _("OK"), NULL, NULL);
     }
 
-    file = popen("wmaker --global_defaults_path", "r");
+    {
+	char *command;
+	
+	command = wstrconcat(path, " --global_defaults_path");
+	file = popen(command, "r");
+	wfree(command);
+    }
     if (!file || !fgets(buffer, 1023, file)) {
-	wsyserror(_("could not run \"wmaker --global_defaults_path\"."));
+	wsyserror(_("could not run \"%s --global_defaults_path\"."), path);
 	exit(1);
     } else {
 	char *ptr;
