@@ -34,17 +34,15 @@
 
 #include <wraster.h>
 
-#include <WINGs.h>
-#include <WUtil.h>
+#include <WINGs/WINGs.h>
 
 /* For EXTENDED_WINDOWSHORTCUT */
 #include "../src/wconfig.h"
 
 /****/
 
-#define WVERSION	"0.42"
-#define WMVERSION	"0.62.x"
-
+#define WVERSION	"0.44"
+#define WMVERSION	"0.64.x"
 
 extern char *NOptionValueChanged;
 
@@ -58,12 +56,14 @@ typedef struct {
     Bool (*requiresRestart)(Panel*);   /* return True if some static option was changed */
     void (*undoChanges)(Panel*);       /* reset values to those in the dictionary */
     void (*prepareForClose)(Panel*);   /* called when exiting WPrefs */
+    void (*showPanel)(Panel*);	       /* called when entering the panel */
+    void (*hidePanel)(Panel*);	       /* called when exiting the panel */
 } CallbackRec;
 
 
 /* all Panels must start with the following layout */
 typedef struct PanelRec {
-    WMFrame *frame;
+    WMBox *box;
 
     char *sectionName;		       /* section name to display in titlebar */
 
@@ -78,6 +78,9 @@ void AddSection(Panel *panel, char *iconFile);
 
 char *LocateImage(char *name);
 
+void SetButtonAlphaImage(WMScreen *scr, WMButton *bPtr, char *file,
+                         char *title1, char *title2);
+
 WMWindow *GetWindow(Panel *panel);
 
 /* manipulate the dictionary for the WindowMaker domain */
@@ -110,8 +113,8 @@ void AddDeadChildHandler(pid_t pid, void (*handler)(void*), void *data);
 
 
 #define FRAME_TOP	105
-#define FRAME_LEFT	-2
-#define FRAME_WIDTH	524
+#define FRAME_LEFT	-1
+#define FRAME_WIDTH	522
 #define FRAME_HEIGHT	235
 
 
@@ -124,8 +127,10 @@ void AddDeadChildHandler(pid_t pid, void (*handler)(void*), void *data);
 #if HAVE_LIBINTL_H && I18N
 # include <libintl.h>
 # define _(text) gettext(text)
+# define N_(text) (text)
 #else
 # define _(text) (text)
+# define N_(text) (text)
 #endif
 
 #endif
