@@ -25,11 +25,11 @@
 #include "wconfig.h"
 
 #include <assert.h>
+#include <limits.h>
 
-#include "WINGs.h"
-#include "WUtil.h"
+#include <WINGs/WINGs.h>
 
-#if HAVE_LIBINTL_H && I18N
+#if defined(HAVE_LIBINTL_H) && defined(I18N)
 # include <libintl.h>
 # define _(text) gettext(text)
 #else
@@ -54,16 +54,26 @@ typedef enum {
 } WClassType;
 
 
-/* generic window levels (a superset of the N*XTSTEP ones) */
+/*
+ * generic window levels (a superset of the N*XTSTEP ones)
+ * Applications should use levels between WMDesktopLevel and
+ * WMScreensaverLevel anything boyond that range is allowed,
+ * but discouraged.
+ */
 enum {
-    WMDesktopLevel = 0,
-	WMSunkenLevel = 1,
-	WMNormalLevel = 2,
+	WMBackLevel = INT_MIN+1,	/* Very lowest level */
+	WMDesktopLevel = -1000,		/* Lowest level of normal use */
+	WMSunkenLevel = -1,
+	WMNormalLevel = 0,
 	WMFloatingLevel = 3,
-	WMDockLevel = 4,
-	WMSubmenuLevel = 5,
-	WMMainMenuLevel = 6,
-	WMOuterSpaceLevel = 7
+	WMDockLevel = 5,
+	WMSubmenuLevel = 6,
+	WMMainMenuLevel = 20,
+	WMStatusLevel = 21,
+	WMModalLevel = 100,
+	WMPopUpLevel = 101,
+	WMScreensaverLevel = 1000,
+	WMOuterSpaceLevel = INT_MAX
 };
 
 /*
@@ -219,6 +229,13 @@ typedef enum {
 #define WD_BOTTOMRIGHT	7
 
 
+/* workspace border position */
+#define	WB_NONE		0
+#define	WB_LEFTRIGHT	1
+#define	WB_TOPBOTTOM	2
+#define	WB_ALLDIRS     	(WB_LEFTRIGHT|WB_TOPBOTTOM)
+
+
 /* program states */
 #define WSTATE_NORMAL		0
 #define WSTATE_NEED_EXIT	1
@@ -226,6 +243,7 @@ typedef enum {
 #define WSTATE_EXITING		3
 #define WSTATE_RESTARTING	4
 #define WSTATE_MODAL		5
+#define WSTATE_NEED_REREAD	6
 
 
 #define WCHECK_STATE(state)	(state == WProgramState)
@@ -308,6 +326,7 @@ typedef struct WPreferences {
     char constrain_window_size;	       /* don't let windows get bigger than 
 					* screen */
 
+    char windows_cycling;	       /* windoze cycling */
     char circ_raise;		       /* raise window after Alt-tabbing */
 
     char ignore_focus_click;
@@ -373,8 +392,8 @@ typedef struct WPreferences {
     unsigned int vedge_thickness;
     unsigned int vedge_hscrollspeed;
     unsigned int vedge_vscrollspeed;
-    unsigned int vedge_height; /* could be change to workspace specific one day */
-    unsigned int vedge_width;
+    unsigned int vedge_maxheight;
+    unsigned int vedge_maxwidth;
 #endif
 
     char ws_cycle;                     /* Cycle existing workspaces */
@@ -419,13 +438,18 @@ typedef struct WPreferences {
     int edge_resistance;
     char attract;
 
+    unsigned int workspace_border_size; /* Size in pixels of the workspace border */
+    char workspace_border_position;     /* Where to leave a workspace border */
+
     struct {
         unsigned int nodock:1;	       /* don't display the dock */
-        unsigned int noclip:1;        /* don't display the clip */
+        unsigned int noclip:1;         /* don't display the clip */
 	unsigned int nocpp:1;	       /* don't use cpp */
 	unsigned int noupdates:1;      /* don't require ~/GNUstep (-static) */
 	unsigned int noautolaunch:1;   /* don't autolaunch apps */
 	unsigned int norestore:1;      /* don't restore session */
+	unsigned int create_stdcmap:1; /* create std colormap */
+        unsigned int nopolling:1;      /* don't poll for defaults changes */
     } flags;			       /* internal flags */
 } WPreferences;
 
