@@ -55,8 +55,8 @@
 # include "kwm.h"
 #endif
 
-#ifdef WMSOUND
-#include "wmsound.h"
+#ifdef WSOUND
+#include "wsound.h"
 #endif
 
 
@@ -160,7 +160,7 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
 
     if (wwin == NULL) {
 	XSetInputFocus(dpy, scr->no_focus_win, RevertToParent, timestamp);
-	if (old_focused && !old_focused->flags.is_gnustep) {
+	if (old_focused) {
 	    wWindowUnfocus(old_focused);
 	}
 	if (oapp) {
@@ -174,7 +174,7 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
 	wKWMSendEventMessage(NULL, WKWMFocusWindow);
 #endif
 	return;
-    } else if (old_scr != scr && old_focused && !old_focused->flags.is_gnustep) {
+    } else if (old_scr != scr && old_focused) {
         wWindowUnfocus(old_focused);
     }
 
@@ -237,8 +237,7 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
 	}
     }
 
-    if (!wwin->flags.is_gnustep)
-	wWindowFocus(wwin, focused);
+    wWindowFocus(wwin, focused);
 
     if (napp && !wasfocused) {
 #ifdef USER_MENU
@@ -273,8 +272,8 @@ wShadeWindow(WWindow  *wwin)
 
     XLowerWindow(dpy, wwin->client_win);    
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_SHADE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_SHADE);
 #endif
 
 #ifdef ANIMATIONS
@@ -360,8 +359,8 @@ wUnshadeWindow(WWindow  *wwin)
     wwin->flags.mapped = 1;
     XMapWindow(dpy, wwin->client_win);
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_UNSHADE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_UNSHADE);
 #endif
 
 #ifdef ANIMATIONS
@@ -427,17 +426,38 @@ wMaximizeWindow(WWindow *wwin, int directions)
 {
     int new_width, new_height, new_x, new_y;
     WArea usableArea = wwin->screen_ptr->totalUsableArea;
+    WArea totalArea;
 
 
     if (WFLAGP(wwin, no_resizable))
 	return;
-
     
+    totalArea.x1 = 0;
+    totalArea.y1 = 0;
+    totalArea.x2 = wwin->screen_ptr->scr_width;
+    totalArea.y2 = wwin->screen_ptr->scr_height;
+
+#ifdef XINERAMA
+    if (wwin->screen_ptr->xine_count > 0
+	&& !(directions & MAX_IGNORE_XINERAMA)) {
+	WScreen *scr = wwin->screen_ptr;
+	WMRect rect;
+	
+	rect = wGetRectForHead(scr, wGetHeadForWindow(wwin));
+	totalArea.x1 = rect.pos.x;
+	totalArea.y1 = rect.pos.y;
+	totalArea.x2 = totalArea.x1 + rect.size.width;
+	totalArea.y2 = totalArea.y1 + rect.size.height;
+	
+	usableArea.x1 = WMAX(totalArea.x1, usableArea.x1);
+	usableArea.y1 = WMAX(totalArea.y1, usableArea.y1);
+	usableArea.x2 = WMIN(totalArea.x2, usableArea.x2);
+	usableArea.y2 = WMIN(totalArea.y2, usableArea.y2);
+    }
+#endif /* XINERAMA */
+
     if (WFLAGP(wwin, full_maximize)) {
-	usableArea.x1 = 0;
-	usableArea.y1 = 0;
-	usableArea.x2 = wwin->screen_ptr->scr_width;
-	usableArea.y2 = wwin->screen_ptr->scr_height;
+	usableArea = totalArea;
     }
 
     if (wwin->flags.shaded) {
@@ -494,8 +514,8 @@ wMaximizeWindow(WWindow *wwin, int directions)
     wKWMSendEventMessage(wwin, WKWMChangedClient);
 #endif
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_MAXIMIZE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_MAXIMIZE);
 #endif
 }
 
@@ -528,8 +548,8 @@ wUnmaximizeWindow(WWindow *wwin)
     wKWMSendEventMessage(wwin, WKWMChangedClient);
 #endif
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_UNMAXIMIZE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_UNMAXIMIZE);
 #endif
 }
 
@@ -949,8 +969,8 @@ wIconifyWindow(WWindow *wwin)
     unmapTransientsFor(wwin);
 
     if (present) {
-#ifdef WMSOUND
-	wSoundPlay(WMSOUND_ICONIFY);
+#ifdef WSOUND
+	wSoundPlay(WSOUND_ICONIFY);
 #endif
 
 	XUngrabPointer(dpy, CurrentTime);
@@ -1103,8 +1123,8 @@ wDeiconifyWindow(WWindow  *wwin)
 	XUnmapWindow(dpy, wwin->icon->core->window);
     }
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_DEICONIFY);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_DEICONIFY);
 #endif
 
     /* if the window is in another workspace, do it silently */
@@ -1221,8 +1241,8 @@ hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
 
     wClientSetState(wwin, IconicState, icon->icon_win);
     flushExpose();
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_HIDE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_HIDE);
 #endif
 #ifdef ANIMATIONS
     if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations &&
@@ -1436,8 +1456,8 @@ unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate,
     wwin->flags.hidden=0;
     wwin->flags.mapped=1;
     
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_UNHIDE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_UNHIDE);
 #endif
 #ifdef ANIMATIONS
     if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations 
