@@ -1,7 +1,8 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1998       Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +30,9 @@
 #include "WindowMaker.h"
 #include "menu.h"
 #include "window.h"
+#ifdef USER_MENU
+#include "usermenu.h"
+#endif /* USER_MENU */
 #include "icon.h"
 #include "appicon.h"
 #include "application.h"
@@ -42,8 +46,8 @@
 
 #include "dock.h"
 
-#ifdef WMSOUND
-#include "wmsound.h"
+#ifdef WSOUND
+#include "wsound.h"
 #endif
 
 
@@ -53,6 +57,8 @@ extern XContext wAppWinContext;
 extern XContext wWinContext;
 extern WPreferences wPreferences;
 
+extern WDDomain *WDWindowAttributes;
+
 /******** Local variables ********/
 
 
@@ -62,7 +68,6 @@ makeMainWindow(WScreen *scr, Window window)
     WWindow *wwin;
     XWindowAttributes attr;
 
-    
     if (!XGetWindowAttributes(dpy, window, &attr)) {
 	return NULL;
     }
@@ -78,8 +83,9 @@ makeMainWindow(WScreen *scr, Window window)
  */
     PropGetWMClass(window, &wwin->wm_class, &wwin->wm_instance);
 
+
     wDefaultFillAttributes(scr, wwin->wm_instance, wwin->wm_class,
-			   &wwin->window_flags, True);
+			   &wwin->user_flags, &wwin->defined_user_flags, True);
     
     XSelectInput(dpy, window, attr.your_event_mask | PropertyChangeMask
 		 | StructureNotifyMask );
@@ -105,103 +111,138 @@ findDockIconFor(WDock *dock, Window main_window)
 {
     WAppIcon *aicon = NULL;
 
-    aicon = wDockFindIconFor(dock, main_window);
+    aicon = wDockFindIconForWindow(dock, main_window);
     if (!aicon) {
         wDockTrackWindowLaunch(dock, main_window);
-        aicon = wDockFindIconFor(dock, main_window);
+        aicon = wDockFindIconForWindow(dock, main_window);
     }
     return aicon;
 }
 
 
 static void
-extractIconImage(WWindow *wwin)
+extractIcon(WWindow *wwin)
 {
     int argc;
     char **argv;
-    char *path = NULL;
     
-    if (!XGetCommand(dpy, wwin->client_win, &argv, &argc))
+    if (!XGetCommand(dpy, wwin->client_win, &argv, &argc) || argc < 1)
 	return;
 
+    wApplicationExtractDirPackIcon(wwin->screen_ptr,argv[0], 
+				   wwin->wm_instance,
+				   wwin->wm_class);
+    XFreeStringList(argv);
+}
+
+
+static void
+saveIconNameFor(char *iconPath, char *wm_instance, char *wm_class)
+{
+    proplist_t dict = WDWindowAttributes->dictionary;
+    proplist_t adict, key, iconk;
+    proplist_t val;
+    char *tmp;
+    int i;
+	
+    i = 0;
+    if (wm_instance)
+	i += strlen(wm_instance);
+    if (wm_class)
+	i += strlen(wm_class);
+
+    tmp = wmalloc(i+8);
+    *tmp = 0;
+    if (wm_class && wm_instance) {
+	sprintf(tmp, "%s.%s", wm_instance, wm_class);
+    } else {
+	if (wm_instance)
+	    strcat(tmp, wm_instance);
+	if (wm_class)
+	    strcat(tmp, wm_class);
+    }
+    
+    key = PLMakeString(tmp); 
+    wfree(tmp);
+    adict = PLGetDictionaryEntry(dict, key);
+    
+    iconk = PLMakeString("Icon");
+    
+    if (adict) {
+	val = PLGetDictionaryEntry(adict, iconk);
+    } else {
+	/* no dictionary for app, so create one */
+	adict = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
+	PLInsertDictionaryEntry(dict, key, adict);
+	PLRelease(adict);
+	val = NULL;
+    }
+    if (!val) {
+	val = PLMakeString(iconPath);
+	PLInsertDictionaryEntry(adict, iconk, val);
+	PLRelease(val);
+    }
+    PLRelease(key);
+
+    if (val && !wPreferences.flags.noupdates)
+	PLSave(dict, YES);
+}
+
+
+void
+wApplicationExtractDirPackIcon(WScreen *scr, char *path, 
+				char *wm_instance, char *wm_class)
+{
+    char *iconPath=NULL;
     /* Maybe the app is a .app and it has an icon in it, like
      * /usr/local/GNUstep/Apps/WPrefs.app/WPrefs.tiff
      */
-    if (argv[0] && strstr(argv[0], ".app")) {
+    if (strstr(path, ".app")) {
 	char *tmp;
 	
-	tmp = wmalloc(strlen(argv[0])+16);	
+	tmp = wmalloc(strlen(path)+16);
 	
-	if (wwin->screen_ptr->flags.supports_tiff) {
-	    strcpy(tmp, argv[0]);
+	if (scr->flags.supports_tiff) {
+	    strcpy(tmp, path);
 	    strcat(tmp, ".tiff");
 	    if (access(tmp, R_OK)==0)
-		path = tmp;
+		iconPath = tmp;
 	}
 	if (!path) {
-	    strcpy(tmp, argv[0]);
+	    strcpy(tmp, path);
 	    strcat(tmp, ".xpm");
 	    if (access(tmp, R_OK)==0)
-		path = tmp;
+		iconPath = tmp;
 	}
-	if (!path) 
-	    free(tmp);
+	if (!iconPath)
+	    wfree(tmp);
     }
-    XFreeStringList(argv);
     
-    if (path) {
-	proplist_t dict = wwin->screen_ptr->dwindow->dictionary;
-	proplist_t adict, key, iconk;
-	proplist_t val;
-	char *tmp;
-	int i;
-	
-	i = 0;
-	if (wwin->wm_instance)
-	    i += strlen(wwin->wm_instance);
-	if (wwin->wm_class)
-	    i += strlen(wwin->wm_class);
+    if (iconPath) {
+	saveIconNameFor(iconPath, wm_instance, wm_class);
 
-	tmp = wmalloc(i+8);
-	*tmp = 0;
-	if (wwin->wm_class && wwin->wm_instance) {
-	    sprintf(tmp, "%s.%s", wwin->wm_instance, wwin->wm_class);
-	} else {
-	    if (wwin->wm_instance)
-		strcat(tmp, wwin->wm_instance);
-	    if (wwin->wm_class)
-		strcat(tmp, wwin->wm_class);
-	}
-
-	key = PLMakeString(tmp); 
-	free(tmp);
-	adict = PLGetDictionaryEntry(dict, key);
-	
-	iconk = PLMakeString("Icon");
-
-	if (adict) {
-	    val = PLGetDictionaryEntry(adict, iconk);
-	} else {
-	    /* no dictionary for app, so create one */
-	    adict = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
-	    PLInsertDictionaryEntry(dict, key, adict);
-	    PLRelease(adict);
-	    val = NULL;
-	}
-	if (!val) {
-	    val = PLMakeString(path);
-	    PLInsertDictionaryEntry(adict, iconk, val);
-	    PLRelease(val);
-	}
-	PLRelease(key);
-
-	PLSave(dict, YES);
-	
-	free(path);
+	wfree(iconPath);
     }
 }
 
 
+static Bool
+extractClientIcon(WAppIcon *icon)
+{
+    char *path;
+
+    path = wIconStore(icon->icon);
+    if (!path)
+	return False;
+
+    saveIconNameFor(path, icon->wm_instance, icon->wm_class);
+
+    wfree(path);
+
+    return True;
+}
+
+
 WApplication*
 wApplicationCreate(WScreen *scr, Window main_window)
 {
@@ -243,34 +284,40 @@ wApplicationCreate(WScreen *scr, Window main_window)
     wapp->main_window = main_window;
     wapp->main_window_desc = makeMainWindow(scr, main_window);
     if (!wapp->main_window_desc) {
-	free(wapp);
+	wfree(wapp);
 	return NULL;
     }
     
-    extractIconImage(wapp->main_window_desc);
+    extractIcon(wapp->main_window_desc);
 
     leader = wWindowFor(main_window);
     if (leader) {
 	leader->main_window = main_window;
     }
     wapp->menu = wAppMenuGet(scr, main_window);
+#ifdef USER_MENU
+    if (!wapp->menu) wapp->menu = wUserMenuGet(scr, wapp->main_window_desc);
+#endif /* USER_MENU */
 
 
     /*
      * Set application wide attributes from the leader.
      */
-    wapp->flags.hidden = wapp->main_window_desc->window_flags.start_hidden;
+    wapp->flags.hidden = WFLAGP(wapp->main_window_desc, start_hidden);
+    
+    wapp->flags.emulated = WFLAGP(wapp->main_window_desc, emulate_appicon);
 
     /* application descriptor */
     XSaveContext(dpy, main_window, wAppWinContext, (XPointer)wapp);
 
-    if (!wapp->main_window_desc->window_flags.no_appicon) {
+    if (!WFLAGP(wapp->main_window_desc, no_appicon)) {
         wapp->app_icon = NULL;
         if (scr->last_dock)
             wapp->app_icon = findDockIconFor(scr->last_dock, main_window);
         /* check main dock if we did not find it in last dock */
-        if (!wapp->app_icon && scr->dock)
+        if (!wapp->app_icon && scr->dock) {
 	    wapp->app_icon = findDockIconFor(scr->dock, main_window);
+	}
         /* finally check clips */
         if (!wapp->app_icon) {
             int i;
@@ -313,15 +360,15 @@ wApplicationCreate(WScreen *scr, Window main_window)
     
     if (wapp->app_icon) {
         wapp->app_icon->main_window = main_window;
-#ifdef WMSOUND
-        wSoundServerGrab(wapp->app_icon->wm_class, main_window);
-#endif
     }
 
 #ifndef REDUCE_APPICONS
     if (wapp->app_icon && !wapp->app_icon->docked) {
 #else
     if (wapp->app_icon && !wapp->app_icon->docked && wapp->app_icon->num_apps == 1) {
+#ifdef THIS_SUCKS
+    }
+#endif
 #endif
         WIcon *icon = wapp->app_icon->icon;
         WDock *clip = scr->workspaces[scr->current_workspace]->clip;
@@ -329,22 +376,19 @@ wApplicationCreate(WScreen *scr, Window main_window)
 
         if (clip && clip->attract_icons && wDockFindFreeSlot(clip, &x, &y)) {
             wapp->app_icon->attracted = 1;
-            if (!clip->keep_attracted && !wapp->app_icon->icon->shadowed) {
+            if (!wapp->app_icon->icon->shadowed) {
                 wapp->app_icon->icon->shadowed = 1;
                 wapp->app_icon->icon->force_paint = 1;
                 /* We don't do an wAppIconPaint() here because it's in
-                 * wDockAttachIcon(). Dan.
+                 * wDockAttachIcon(). -Dan.
                  */
             }
             wDockAttachIcon(clip, wapp->app_icon, x, y);
         } else {
-            PlaceIcon(scr, &wapp->app_icon->x_pos, &wapp->app_icon->y_pos);
-#ifndef STRICTNS
-            wLowerFrame(icon->core);
-#endif
+            PlaceIcon(scr, &x, &y);
+	    wAppIconMove(wapp->app_icon, x, y);
+	    wLowerFrame(icon->core);
         }
-	XMoveWindow(dpy, icon->core->window, wapp->app_icon->x_pos, 
-		    wapp->app_icon->y_pos);
         if (!clip || !wapp->app_icon->attracted || !clip->collapsed)
 	    XMapWindow(dpy, icon->core->window);
     }
@@ -353,14 +397,25 @@ wApplicationCreate(WScreen *scr, Window main_window)
 	wArrangeIcons(scr, True);
     }
 
+    if (wapp->app_icon) {
+	char *tmp;
+
+	/* if the displayed icon was supplied by the client, save the icon */
+	tmp = wDefaultGetIconFile(scr, wapp->app_icon->wm_instance,
+				  wapp->app_icon->wm_class, True);
+	if (!tmp)
+	    extractClientIcon(wapp->app_icon);
+    }
+    
     wapp->prev = NULL; 
     wapp->next = scr->wapp_list;
     if (scr->wapp_list)
 	scr->wapp_list->prev = wapp;
     scr->wapp_list = wapp;
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_APPSTART);
+    
+#ifdef WSOUND
+    wSoundPlay(WSOUND_APPSTART);
 #endif
 
 #ifdef DEBUG
@@ -382,7 +437,7 @@ wApplicationDestroy(WApplication *wapp)
 
     if (!wapp)
       return;
-    
+  
     wapp->refcount--;
     if (wapp->refcount>0)
       return;
@@ -408,9 +463,7 @@ wApplicationDestroy(WApplication *wapp)
     XDeleteContext(dpy, wapp->main_window, wAppWinContext);
     wAppMenuDestroy(wapp->menu);
     if (wapp->app_icon) {
-        if (wapp->app_icon->docked
-            && (!wapp->app_icon->attracted || 
-		wapp->app_icon->dock->keep_attracted)) {
+        if (wapp->app_icon->docked && !wapp->app_icon->attracted) {
 #ifdef REDUCE_APPICONS
 	    if (napps == 0) {
 #endif
@@ -455,7 +508,7 @@ wApplicationDestroy(WApplication *wapp)
     	XSaveContext(dpy, wwin->client_win, wWinContext, 
 		     (XPointer)&wwin->client_descriptor);
     }
-    free(wapp);
+    wfree(wapp);
     
 #ifdef DEBUG
     printf("Destroyed application for %x\n", (unsigned)main_window);
@@ -464,8 +517,8 @@ wApplicationDestroy(WApplication *wapp)
         wArrangeIcons(scr, True);
     }
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_APPEXIT);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_APPEXIT);
 #endif
 }
 
