@@ -338,7 +338,7 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 #endif
             }
 	    if (tmp)
-		free(tmp);
+		wfree(tmp);
 	}
 	break;
 	
@@ -356,28 +356,29 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	    wIconChangeTitle(wwin->icon, new_title);
 	}
 	break;
-	
+
      case XA_WM_COMMAND:
 	if (wwin->main_window!=None) {
 	    WApplication *wapp = wApplicationOf(wwin->main_window);
 	    char **argv;
 	    int argc;
 
-	    if (wapp && wapp->app_icon) {
-		if (wapp->app_icon->command!=NULL)
-		    free(wapp->app_icon->command);
+            if (!wapp || !wapp->app_icon)
+		break;
 
-		if (XGetCommand(dpy, wwin->main_window, &argv, &argc)) {
-		    if (argc > 0 && argv != NULL)
-			wapp->app_icon->command = FlattenStringList(argv,argc);
-		    if (argv) {
-			XFreeStringList(argv);
-		    }
+	    if (XGetCommand(dpy, wwin->main_window, &argv, &argc)) {
+		if (argc > 0 && argv != NULL) {
+		    if (wapp->app_icon->command)
+		        wfree(wapp->app_icon->command);
+		    wapp->app_icon->command = wtokenjoin(argv, argc);
+		}
+		if (argv) {
+		    XFreeStringList(argv);
 		}
 	    }
 	}
 	break;
-	
+
      case XA_WM_HINTS:
 	/* WM_HINTS */
 
