@@ -67,6 +67,7 @@
 #include "actions.h"
 #include "dock.h"
 #include "workspace.h"
+#include "properties.h"
 
 
 /*
@@ -85,6 +86,9 @@ extern WDDomain *WDRootMenu;
 
 extern int wScreenCount;
 
+extern Atom _XA_WINDOWMAKER_ICON_SIZE;
+extern Atom _XA_WINDOWMAKER_ICON_TILE;
+
 /*
 extern proplist_t wDomainName;
 extern proplist_t wAttributeDomainName;
@@ -137,6 +141,7 @@ static int getRImage();
 
 /* value setting functions */
 static int setJustify();
+static int setClearance();
 static int setIfDockPresent();
 static int setStickyIcons();
 /*
@@ -150,7 +155,6 @@ static int setMenuTextFont();
 static int setIconTitleFont();
 static int setIconTitleColor();
 static int setIconTitleBack();
-static int setDisplayFont();
 static int setLargeDisplayFont();
 static int setWTitleColor();
 static int setFTitleBack();
@@ -174,10 +178,16 @@ static int setClipTitleFont();
 static int setClipTitleColor();
 
 static int setMenuStyle();
+#if 0
 static int setMultiByte();
-
+#endif
 static int updateUsableArea();
 
+#ifdef DEFINABLE_CURSOR
+extern Cursor wCursor[WCUR_LAST];
+static int getCursor();
+static int setCursor();
+#endif
 
 
 /*
@@ -227,7 +237,8 @@ static WOptionEnumeration seColormapModes[] = {
 };
 
 static WOptionEnumeration sePlacements[] = {
-    {"Auto", WPM_SMART, 0}, {"Smart", WPM_SMART, 1},
+    {"Auto", WPM_AUTO, 0},
+    {"Smart", WPM_SMART, 0},
     {"Cascade", WPM_CASCADE, 0},
     {"Random", WPM_RANDOM, 0},
     {"Manual", WPM_MANUAL, 0},
@@ -252,6 +263,7 @@ static WOptionEnumeration seSpeeds[] = {
 };
 
 static WOptionEnumeration seMouseButtons[] = {
+    {"None", -1, 0},
     {"Left", Button1, 0}, {"Button1", Button1, 1},
     {"Middle", Button2, 0}, {"Button2", Button2, 1},
     {"Right", Button3, 0}, {"Button3", Button3, 1},
@@ -304,7 +316,16 @@ static WOptionEnumeration seDisplayPositions[] = {
     {"topleft",		WD_TOPLEFT,	0},
     {"topright",	WD_TOPRIGHT, 	0},
     {"bottomleft",	WD_BOTTOMLEFT,	0},
-    {"bottomright",	WD_BOTTOMRIGHT,	0}
+    {"bottomright",	WD_BOTTOMRIGHT,	0},
+    {NULL,		0,		0}
+};
+
+static WOptionEnumeration seWorkspaceBorder[] = {
+    {"None",  		WB_NONE,	0},
+    {"LeftRight",      	WB_LEFTRIGHT,	0},
+    {"TopBottom",      	WB_TOPBOTTOM,  	0},
+    {"AllDirections",  	WB_ALLDIRS,	0},
+    {NULL,		0,		0}
 };
 
 
@@ -319,13 +340,13 @@ static WOptionEnumeration seDisplayPositions[] = {
  * created after these defaults are read
  */
 WDefaultEntry staticOptionList[] = {    
-    
-    {"DisableDithering",	"NO",			NULL,
-	&wPreferences.no_dithering,	getBool,	NULL
-    },
+
     {"ColormapSize",	"4",			NULL,
 	&wPreferences.cmap_size,	getInt,		NULL
     },
+    {"DisableDithering",	"NO",			NULL,
+	&wPreferences.no_dithering,	getBool,	NULL
+    },
     /* static by laziness */
     {"IconSize",	"64",			NULL,
 	&wPreferences.icon_size,	getInt,		NULL
@@ -350,10 +371,12 @@ WDefaultEntry staticOptionList[] = {
     },
     {"DisableMiniwindows", "NO",		NULL,
 	&wPreferences.disable_miniwindows, getBool,	NULL
-    },
-    {"MultiByteText", "NO", 			NULL,
-	&wPreferences.multi_byte_text, getBool,		setMultiByte
     }
+#if 0
+    ,{"MultiByteText", "NO", 			NULL,
+	&wPreferences.multi_byte_text, getBool,	setMultiByte
+    }
+#endif
 };
 
 
@@ -390,6 +413,9 @@ WDefaultEntry optionList[] = {
     {"RaiseDelay",	"0",			NULL,
 	  &wPreferences.raise_delay,	getInt,		NULL
     },
+    {"WindozeCycling",  "YES",			NULL,
+	  &wPreferences.windows_cycling,getBool,	NULL
+    },
     {"CirculateRaise",	"NO",			NULL,
 	  &wPreferences.circ_raise, 	getBool, 	NULL
     },
@@ -405,6 +431,29 @@ WDefaultEntry optionList[] = {
     {"WorkspaceNameDisplayPosition", "center",	seDisplayPositions,
 	  &wPreferences.workspace_name_display_position, getEnum, NULL
     },
+    {"WorkspaceBorder", "None",			seWorkspaceBorder,
+	  &wPreferences.workspace_border_position, getEnum, updateUsableArea
+    },
+    {"WorkspaceBorderSize", "0",       		NULL,
+	  &wPreferences.workspace_border_size, getInt, updateUsableArea
+    },
+#ifdef VIRTUAL_DESKTOP
+    {"VirtualEdgeThickness", "1",   NULL,
+        &wPreferences.vedge_thickness, getInt,        NULL
+    },
+    {"VirtualEdgeHorizonScrollSpeed", "1",  NULL,
+        &wPreferences.vedge_hscrollspeed, getInt,     NULL
+    },
+    {"VirtualEdgeVerticalScrollSpeed", "1", NULL,
+        &wPreferences.vedge_vscrollspeed, getInt,     NULL
+    },
+    {"VirtualEdgeMaximumWidth", "3000",    NULL,
+        &wPreferences.vedge_maxwidth, getInt,        NULL
+    },
+    {"VirtualEdgeMaximumHeight", "3000",   NULL,
+        &wPreferences.vedge_maxheight, getInt,       NULL
+    },
+#endif
     {"StickyIcons", "NO",			NULL,
 	  &wPreferences.sticky_icons, getBool, setStickyIcons
     },
@@ -492,17 +541,12 @@ WDefaultEntry optionList[] = {
     {"EdgeResistance", 	"30",			NULL,
 	  &wPreferences.edge_resistance,getInt,		NULL
     },
+    {"Attraction", 	"NO",			NULL,
+	  &wPreferences.attract,  getBool,    NULL
+    },
     {"DisableBlinking",	"NO",		NULL,
 	   &wPreferences.dont_blink,	getBool,	NULL
     },
-#ifdef WEENDOZE_CYCLE
-    {"WindozeCycling","NO", 			NULL,
-	    &wPreferences.windoze_cycling,	getBool, NULL
-    },
-    {"PopupSwitchMenu","YES",			NULL, 
-	    &wPreferences.popup_switchmenu,	getBool, NULL
-    },
-#endif /* WEENDOZE_CYCLE */
       /* style options */
     {"MenuStyle", 	"normal",  		seMenuStyles,
 	&wPreferences.menu_style, getEnum, 	setMenuStyle
@@ -531,6 +575,15 @@ WDefaultEntry optionList[] = {
     {"WindowTitleFont",	DEF_TITLE_FONT,	       	NULL,
 	  NULL,				getFont,	setWinTitleFont,
     },
+    {"WindowTitleExtendSpace",	DEF_WINDOW_TITLE_EXTEND_SPACE,	       	NULL,
+      &wPreferences.window_title_clearance,	getInt,		setClearance
+    },
+    {"MenuTitleExtendSpace",	DEF_MENU_TITLE_EXTEND_SPACE,	       	NULL,
+      &wPreferences.menu_title_clearance,	getInt,		setClearance
+    },
+    {"MenuTextExtendSpace",	DEF_MENU_TEXT_EXTEND_SPACE,	       	NULL,
+      &wPreferences.menu_text_clearance,	getInt,		setClearance
+    },
     {"MenuTitleFont",	DEF_MENU_TITLE_FONT,	NULL,
 	  NULL,				getFont,	setMenuTitleFont
     },
@@ -543,9 +596,6 @@ WDefaultEntry optionList[] = {
     {"ClipTitleFont",	DEF_CLIP_TITLE_FONT,	NULL,
 	  NULL,				getFont,	setClipTitleFont
     },
-    {"DisplayFont",	DEF_INFO_TEXT_FONT,	NULL,
-	  NULL,				getFont,	setDisplayFont
-    },
     {"LargeDisplayFont",DEF_WORKSPACE_NAME_FONT, NULL,
 	  NULL,				getFont,	setLargeDisplayFont
     },
@@ -642,10 +692,10 @@ WDefaultEntry optionList[] = {
     {"VMaximizeKey", "None",			(void*)WKBD_VMAXIMIZE,
 	  NULL,				getKeybind,	setKeyGrab
     },
-    {"RaiseKey", "Meta+Up",			(void*)WKBD_RAISE,
+    {"RaiseKey", "\"Meta+Up\"",			(void*)WKBD_RAISE,
 	  NULL,				getKeybind,	setKeyGrab
     },
-    {"LowerKey", "Meta+Down",			(void*)WKBD_LOWER,
+    {"LowerKey", "\"Meta+Down\"",			(void*)WKBD_LOWER,
 	  NULL,				getKeybind,	setKeyGrab
     },
     {"RaiseLowerKey", "None",			(void*)WKBD_RAISELOWER,
@@ -717,7 +767,6 @@ WDefaultEntry optionList[] = {
     {"WindowShortcut4Key","None",		(void*)WKBD_WINDOW4,
 	    NULL, 			getKeybind,	setKeyGrab
     }
-#ifdef EXTEND_WINDOWSHORTCUT
     ,{"WindowShortcut5Key","None",		(void*)WKBD_WINDOW5,
 	    NULL, 			getKeybind,	setKeyGrab
     },
@@ -735,34 +784,53 @@ WDefaultEntry optionList[] = {
     },
     {"WindowShortcut10Key","None",		(void*)WKBD_WINDOW10,
 	    NULL, 			getKeybind,	setKeyGrab
-    }
-#endif /* EXTEND_WINDOWSHORTCUT */
+    },
+    {"ScreenSwitchKey",  "None",		(void*)WKBD_SWITCH_SCREEN,
+	    NULL, 			getKeybind,	setKeyGrab	
+    },
 
 #ifdef KEEP_XKB_LOCK_STATUS
-    ,{"ToggleKbdModeKey", "None",                      (void*)WKBD_TOGGLE,
+    {"ToggleKbdModeKey", "None",                      (void*)WKBD_TOGGLE,
 	    NULL,                       getKeybind,     setKeyGrab
     },
     {"KbdModeLock", "NO",  	                  NULL,
 	    &wPreferences.modelock, 	getBool,	NULL
-    }
+    },
 #endif /* KEEP_XKB_LOCK_STATUS */
-#ifdef TITLE_TEXT_SHADOW
-    ,{"FShadowColor",     "black",                (void*)WS_SFOCUSED,
-          NULL,                         getColor,       setWTitleColor
+#ifdef DEFINABLE_CURSOR
+    {"NormalCursor", "(builtin, left_ptr)",	(void*)WCUR_ROOT,
+          NULL,				getCursor,	setCursor
     },
-    {"PShadowColor",     "black",                (void*)WS_SPFOCUSED,
-          NULL,                         getColor,       setWTitleColor
+    {"MoveCursor", "(builtin, fleur)",		(void*)WCUR_MOVE,
+          NULL,				getCursor,	setCursor
     },
-    {"UShadowColor",     "grey50",               (void*)WS_SUNFOCUSED,
-         NULL,                         getColor,       setWTitleColor
+#if 0
+    {"TopLeftResizeCursor", "(builtin, sizing)",	(void*)WCUR_TOPLEFTRESIZE,
+          NULL,				getCursor,	setCursor
     },
-    {"MShadowColor",     "black",               (void*)WS_SMENU,
-          NULL,                         getColor,       setMenuTitleColor
+    {"TopRightResizeCursor", "(builtin, sizing)",	(void*)WCUR_TOPRIGHTRESIZE,
+          NULL,				getCursor,	setCursor
     },
-    {"Shadow", "Yes",                    NULL,
-         &wPreferences.title_shadow, getBool,  setJustify
+    {"BottomLeftResizeCursor", "(builtin, sizing)",	(void*)WCUR_BOTTOMLEFTRESIZE,
+          NULL,				getCursor,	setCursor
+    },
+    {"BottomRightResizeCursor", "(builtin, sizing)",	(void*)WCUR_BOTTOMRIGHTRESIZE,
+          NULL,				getCursor,	setCursor
+    },
+    {"VerticalResizeCursor", "(builtin, sizing)",	(void*)WCUR_VERTICALRESIZE,
+          NULL,				getCursor,	setCursor
+    },
+    {"HorizonResizeCursor", "(builtin, sizing)",	(void*)WCUR_HORIZONRESIZE,
+          NULL,				getCursor,	setCursor
+    },
+    {"WaitCursor", "(builtin, watch)",		(void*)WCUR_WAIT,
+          NULL,				getCursor,	setCursor
+    },
+    {"ArrowCursor", "(builtin, top_left_arrow)",	(void*)WCUR_ARROW,
+          NULL,				getCursor,	setCursor
     }
-#endif /* TITLE_TEXT_SHADOW */
+#endif
+#endif /* DEFINABLE_CURSOR */
 };
 
 
@@ -847,8 +915,8 @@ wDefaultsDestroyDomain(WDDomain *domain)
 {
     if (domain->dictionary)
 	PLRelease(domain->dictionary);
-    free(domain->path);
-    free(domain);
+    wfree(domain->path);
+    wfree(domain);
 }
 
 
@@ -890,7 +958,7 @@ wDefaultsInitDomain(char *domain, Bool requireDictionary)
     }
 
     /* global system dictionary */
-    sprintf(path, "%s/%s", SYSCONFDIR, domain);
+    sprintf(path, "%s/WindowMaker/%s", SYSCONFDIR, domain);
     if (stat(path, &stbuf)>=0) {
         shared_dict = ReadProplistFromFile(path);
         if (shared_dict) {
@@ -914,8 +982,8 @@ wDefaultsInitDomain(char *domain, Bool requireDictionary)
 		}
             }
         } else {
-	    wwarning(_("could not load domain %s from global defaults database"),
-		     domain);
+	    wwarning(_("could not load domain %s from global defaults database (%s)"),
+		     domain, path);
 	}
     }
 
@@ -985,7 +1053,7 @@ wDefaultsCheckDomains(void *foo)
 	WDWindowMaker->timestamp = stbuf.st_mtime;	
 
 	/* global dictionary */
-	sprintf(path, "%s/WindowMaker", SYSCONFDIR);
+	sprintf(path, "%s/WindowMaker/WindowMaker", SYSCONFDIR);
 	if (stat(path, &stbuf)>=0) {
 	    shared_dict = ReadProplistFromFile(path);
 	    if (shared_dict && !PLIsDictionary(shared_dict)) {
@@ -1064,28 +1132,28 @@ wDefaultsCheckDomains(void *foo)
 
 #ifndef LITE
     if (stat(WDRootMenu->path, &stbuf)>=0
-	&& WDRootMenu->timestamp < stbuf.st_mtime) {
-	dict = ReadProplistFromFile(WDRootMenu->path);
+        && WDRootMenu->timestamp < stbuf.st_mtime) {
+        dict = ReadProplistFromFile(WDRootMenu->path);
 #ifdef HEARTBEAT
-	puts("Checking WMRootMenu domain");
+        puts("Checking WMRootMenu domain");
 #endif
-	if (dict) {
-	    if (!PLIsArray(dict) && !PLIsString(dict)) {
-		PLRelease(dict);
-		dict = NULL;
-		wwarning(_("Domain %s (%s) of defaults database is corrupted!"),
-			 "WMRootMenu", WDRootMenu->path);
-	    } else {
-		if (WDRootMenu->dictionary) {
-		    PLRelease(WDRootMenu->dictionary);
-		}
-		WDRootMenu->dictionary = dict;
-	    }
-	} else {
-	    wwarning(_("could not load domain %s from user defaults database"),
-		     "WMRootMenu");
-	}
-	WDRootMenu->timestamp = stbuf.st_mtime;
+        if (dict) {
+            if (!PLIsArray(dict) && !PLIsString(dict)) {
+                PLRelease(dict);
+                dict = NULL;
+                wwarning(_("Domain %s (%s) of defaults database is corrupted!"),
+                         "WMRootMenu", WDRootMenu->path);
+            } else {
+                if (WDRootMenu->dictionary) {
+                    PLRelease(WDRootMenu->dictionary);
+                }
+                WDRootMenu->dictionary = dict;
+            }
+        } else {
+            wwarning(_("could not load domain %s from user defaults database"),
+                     "WMRootMenu");
+        }
+        WDRootMenu->timestamp = stbuf.st_mtime;
     }
 #endif /* !LITE */
 
@@ -1100,6 +1168,7 @@ wReadDefaults(WScreen *scr, proplist_t new_dict)
     proplist_t plvalue, old_value;
     WDefaultEntry *entry;
     int i, must_update;
+    int update_workspace_back = 0;	       /* kluge :/ */
     int needs_refresh;
     void *tdata;
     proplist_t old_dict = (WDWindowMaker->dictionary!=new_dict 
@@ -1138,16 +1207,33 @@ wReadDefaults(WScreen *scr, proplist_t new_dict)
 	} else if (!PLIsEqual(plvalue, old_value)) {	    
 	    /* value has changed */
 	} else {
-	    /* value was not changed since last time */
-	    continue;
+	    
+	    if (strcmp(entry->key, "WorkspaceBack") == 0
+		&& update_workspace_back
+		&& scr->flags.backimage_helper_launched) {
+	    } else {
+		/* value was not changed since last time */
+		continue;
+	    }
 	}
 	
 	if (plvalue) {
 #ifdef DEBUG
-	    printf("Updating %s to %s\n", entry->key, PLGetDescription(plvalue));
+	    printf("Updating %s to %s\n", entry->key, 
+		   PLGetDescription(plvalue));
 #endif
 	    /* convert data */
 	    if ((*entry->convert)(scr, entry, plvalue, entry->addr, &tdata)) {
+		/*
+		 * If the WorkspaceSpecificBack data has been changed
+		 * so that the helper will be launched now, we must be
+		 * sure to send the default background texture config
+		 * to the helper.
+		 */
+		if (strcmp(entry->key, "WorkspaceSpecificBack") == 0
+		    && !scr->flags.backimage_helper_launched) {
+		    update_workspace_back = 1;
+		}
 		if (entry->update) {
 		    needs_refresh |= 
 			(*entry->update)(scr, entry, tdata, entry->extra_data);
@@ -1231,7 +1317,7 @@ wDefaultUpdateIcons(WScreen *scr)
             RImage *new_image;
 
             if (aicon->icon->file)
-                free(aicon->icon->file);
+                wfree(aicon->icon->file);
             aicon->icon->file = wstrdup(file);
 
             new_image = wDefaultGetImage(scr, aicon->wm_instance,
@@ -1256,7 +1342,7 @@ wDefaultUpdateIcons(WScreen *scr)
                 RImage *new_image;
 
                 if (wwin->icon->file)
-                    free(wwin->icon->file);
+                    wfree(wwin->icon->file);
                 wwin->icon->file = wstrdup(file);
 
                 new_image = wDefaultGetImage(scr, wwin->wm_instance,
@@ -1272,10 +1358,14 @@ wDefaultUpdateIcons(WScreen *scr)
 
 /* --------------------------- Local ----------------------- */
 
-#define STRINGP(x) if (!PLIsString(value)) { \
+#define GET_STRING_OR_DEFAULT(x, var) if (!PLIsString(value)) { \
 	wwarning(_("Wrong option format for key \"%s\". Should be %s."), \
 		entry->key, x); \
-	return False; }
+	wwarning(_("using default \"%s\" instead"), entry->default_value); \
+	var = entry->default_value;\
+	} else var = PLGetString(value)\
+
+	
 
 
 
@@ -1329,9 +1419,7 @@ getBool(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
     char *val;
     int second_pass=0;
 
-    STRINGP("Boolean");
-
-    val = PLGetString(value);
+    GET_STRING_OR_DEFAULT("Boolean", val);
 
 again:
     if ((val[1]=='\0' && (val[0]=='y' || val[0]=='Y'))
@@ -1380,9 +1468,7 @@ getInt(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
     char *val;
     
     
-    STRINGP("Integer");
-    
-    val = PLGetString(value);
+    GET_STRING_OR_DEFAULT("Integer", val);
     
     if (sscanf(val, "%i", &data)!=1) {
         wwarning(_("can't convert \"%s\" to integer for key \"%s\""),
@@ -1496,9 +1582,7 @@ getString(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
 {
     static char *data;
     
-    STRINGP("String");
-    
-    data = PLGetString(value);
+    GET_STRING_OR_DEFAULT("String", data);
     
     if (!data) {
         data = PLGetString(entry->plvalue);
@@ -1577,7 +1661,7 @@ again:
     ptr--; *(ptr--) = 0;
 
     if (*(char**)addr!=NULL) {
-	free(*(char**)addr);
+	wfree(*(char**)addr);
     }
     *(char**)addr = data;
 
@@ -1614,6 +1698,8 @@ getEnum(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
  * (dgradient <color> <color>)
  * (mhgradient <color> <color> ...)
  * (mvgradient <color> <color> ...)
+ * (mdgradient <color> <color> ...)
+ * (igradient <color1> <color1> <thickness1> <color2> <color2> <thickness2>)
  * (tpixmap <file> <color>)
  * (spixmap <file> <color>)
  * (cpixmap <file> <color>)
@@ -1713,7 +1799,66 @@ parse_texture(WScreen *scr, proplist_t pl)
 	color2.blue = xcolor.blue >> 8;
 
 	texture = (WTexture*)wTextureMakeGradient(scr, type, &color1, &color2);
-	
+
+    } else if (strcasecmp(val, "igradient")==0) {
+	RColor colors1[2], colors2[2];
+	int th1, th2;
+	XColor xcolor;
+        int i;
+
+	if (nelem != 7) {
+	    wwarning(_("bad number of arguments in gradient specification"));
+	    return NULL;
+	}
+
+	/* get from color */
+	for (i = 0; i < 2; i++) {
+	    elem = PLGetArrayElement(pl, 1+i);
+	    if (!elem || !PLIsString(elem))
+		return NULL;
+	    val = PLGetString(elem);
+
+	    if (!XParseColor(dpy, scr->w_colormap, val, &xcolor)) {
+		wwarning(_("\"%s\" is not a valid color name"), val);
+		return NULL;
+	    }
+	    colors1[i].alpha = 255;
+	    colors1[i].red = xcolor.red >> 8;
+	    colors1[i].green = xcolor.green >> 8;
+	    colors1[i].blue = xcolor.blue >> 8;
+	}
+	elem = PLGetArrayElement(pl, 3);
+	if (!elem || !PLIsString(elem))
+	    return NULL;
+	val = PLGetString(elem);
+	th1 = atoi(val);
+
+
+	/* get from color */
+	for (i = 0; i < 2; i++) {
+	    elem = PLGetArrayElement(pl, 4+i);
+	    if (!elem || !PLIsString(elem))
+		return NULL;
+	    val = PLGetString(elem);
+
+	    if (!XParseColor(dpy, scr->w_colormap, val, &xcolor)) {
+		wwarning(_("\"%s\" is not a valid color name"), val);
+		return NULL;
+	    }
+	    colors2[i].alpha = 255;
+	    colors2[i].red = xcolor.red >> 8;
+	    colors2[i].green = xcolor.green >> 8;
+	    colors2[i].blue = xcolor.blue >> 8;
+	}
+	elem = PLGetArrayElement(pl, 6);
+	if (!elem || !PLIsString(elem))
+	    return NULL;
+	val = PLGetString(elem);
+	th2 = atoi(val);
+
+	texture = (WTexture*)wTextureMakeIGradient(scr, th1, colors1,
+						   th2, colors2);
+
     } else if (strcasecmp(val, "mhgradient")==0
 	       || strcasecmp(val, "mvgradient")==0
 	       || strcasecmp(val, "mdgradient")==0) {
@@ -1742,9 +1887,9 @@ parse_texture(WScreen *scr, proplist_t pl)
 	    elem = PLGetArrayElement(pl, i+1);
 	    if (!elem || !PLIsString(elem)) {
 		for (--i; i>=0; --i) {
-		    free(colors[i]);
+		    wfree(colors[i]);
 		}
-		free(colors);
+		wfree(colors);
 		return NULL;
 	    }
 	    val = PLGetString(elem);
@@ -1752,9 +1897,9 @@ parse_texture(WScreen *scr, proplist_t pl)
 	    if (!XParseColor(dpy, scr->w_colormap, val, &color)) {
 		wwarning(_("\"%s\" is not a valid color name"), val);
 		for (--i; i>=0; --i) {
-		    free(colors[i]);
+		    wfree(colors[i]);
 		}
-		free(colors);
+		wfree(colors);
 		return NULL;
 	    } else {
 		colors[i] = wmalloc(sizeof(RColor));
@@ -1905,7 +2050,7 @@ parse_texture(WScreen *scr, proplist_t pl)
 	for (i = 0; i < argc - 1; i++) {
 	    elem = PLGetArrayElement(pl, 3 + i);
 	    if (!elem || !PLIsString(elem)) {
-		free(argv);
+		wfree(argv);
 
 		return NULL;
 	    }
@@ -1998,7 +2143,6 @@ again:
 }
 
 
-
 static int
 getWSBackground(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 		void *addr, void **ret)
@@ -2110,9 +2254,7 @@ getFont(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
     static WMFont *font;
     char *val;
 
-    STRINGP("Font");
-
-    val = PLGetString(value);
+    GET_STRING_OR_DEFAULT("Font", val);
 
     font = WMCreateFont(scr->wmscreen, val);
     if (!font)
@@ -2141,9 +2283,8 @@ getColor(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
     char *val;
     int second_pass=0;
 
-    STRINGP("Color");
+    GET_STRING_OR_DEFAULT("Color", val);
     
-    val = PLGetString(value);
 
 again:
     if (!wGetColor(scr, val, &color)) {
@@ -2183,9 +2324,7 @@ getKeybind(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
     char buf[128], *b;
 
 
-    STRINGP("Key spec");
-
-    val = PLGetString(value);
+    GET_STRING_OR_DEFAULT("Key spec", val);
 
     if (!val || strcasecmp(val, "NONE")==0) {
 	shortcut.keycode = 0;
@@ -2244,9 +2383,8 @@ getModMask(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
     static unsigned int mask;
     char *str;
 
-    STRINGP("Modifier Key");
+    GET_STRING_OR_DEFAULT("Modifier Key", str);
 
-    str = PLGetString(value);
     if (!str)
 	return False;
 
@@ -2278,9 +2416,7 @@ getRImages(WScreen *scr, WDefaultEntry *entry, proplist_t value,
     int i, n;
     int w, h;
 
-    STRINGP("Image File Path");
-
-    str = PLGetString(value);
+    GET_STRING_OR_DEFAULT("Image File Path", str);
     if (!str)
 	return False;
 
@@ -2307,17 +2443,288 @@ getRImages(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 }
 #endif
 
+#ifdef DEFINABLE_CURSOR
 
-/* ---------------- value setting functions --------------- */
-static int
-setJustify(WScreen *scr, WDefaultEntry *entry, WTexture **texture, void *foo)
+# include <X11/cursorfont.h>
+typedef struct 
 {
-    return REFRESH_WINDOW_TITLE_COLOR;
+   char *name;
+   int id;
+} WCursorLookup;
+
+#define CURSOR_ID_NONE	(XC_num_glyphs)
+
+static WCursorLookup cursor_table[] =
+{
+    { "X_cursor",		XC_X_cursor },
+    { "arrow",			XC_arrow },
+    { "based_arrow_down",	XC_based_arrow_down },
+    { "based_arrow_up",		XC_based_arrow_up },
+    { "boat",			XC_boat },
+    { "bogosity",		XC_bogosity },
+    { "bottom_left_corner",	XC_bottom_left_corner },
+    { "bottom_right_corner",	XC_bottom_right_corner },
+    { "bottom_side",		XC_bottom_side },
+    { "bottom_tee",		XC_bottom_tee },
+    { "box_spiral",		XC_box_spiral },
+    { "center_ptr",		XC_center_ptr },
+    { "circle",			XC_circle },
+    { "clock",			XC_clock },
+    { "coffee_mug",		XC_coffee_mug },
+    { "cross",			XC_cross },
+    { "cross_reverse",		XC_cross_reverse },
+    { "crosshair",		XC_crosshair },
+    { "diamond_cross",		XC_diamond_cross },
+    { "dot",			XC_dot },
+    { "dotbox",			XC_dotbox },
+    { "double_arrow",		XC_double_arrow },
+    { "draft_large",		XC_draft_large },
+    { "draft_small",		XC_draft_small },
+    { "draped_box",		XC_draped_box },
+    { "exchange",		XC_exchange },
+    { "fleur",			XC_fleur },
+    { "gobbler",		XC_gobbler },
+    { "gumby",			XC_gumby },
+    { "hand1",			XC_hand1 },
+    { "hand2",			XC_hand2 },
+    { "heart",			XC_heart },
+    { "icon",			XC_icon },
+    { "iron_cross",		XC_iron_cross },
+    { "left_ptr",		XC_left_ptr },
+    { "left_side",		XC_left_side },
+    { "left_tee",		XC_left_tee },
+    { "leftbutton",		XC_leftbutton },
+    { "ll_angle",		XC_ll_angle },
+    { "lr_angle",		XC_lr_angle },
+    { "man",			XC_man },
+    { "middlebutton",		XC_middlebutton },
+    { "mouse",			XC_mouse },
+    { "pencil",			XC_pencil },
+    { "pirate",			XC_pirate },
+    { "plus",			XC_plus },
+    { "question_arrow",		XC_question_arrow },
+    { "right_ptr",		XC_right_ptr },
+    { "right_side",		XC_right_side },
+    { "right_tee",		XC_right_tee },
+    { "rightbutton",		XC_rightbutton },
+    { "rtl_logo",		XC_rtl_logo },
+    { "sailboat",		XC_sailboat },
+    { "sb_down_arrow",		XC_sb_down_arrow },
+    { "sb_h_double_arrow",	XC_sb_h_double_arrow },
+    { "sb_left_arrow",		XC_sb_left_arrow },
+    { "sb_right_arrow",		XC_sb_right_arrow },
+    { "sb_up_arrow",		XC_sb_up_arrow },
+    { "sb_v_double_arrow",	XC_sb_v_double_arrow },
+    { "shuttle",		XC_shuttle },
+    { "sizing",			XC_sizing },
+    { "spider",			XC_spider },
+    { "spraycan",		XC_spraycan },
+    { "star",			XC_star },
+    { "target",			XC_target },
+    { "tcross",			XC_tcross },
+    { "top_left_arrow",		XC_top_left_arrow },
+    { "top_left_corner",	XC_top_left_corner },
+    { "top_right_corner",	XC_top_right_corner },
+    { "top_side",		XC_top_side },
+    { "top_tee",		XC_top_tee },
+    { "trek",			XC_trek },
+    { "ul_angle",		XC_ul_angle },
+    { "umbrella",		XC_umbrella },
+    { "ur_angle",		XC_ur_angle },
+    { "watch",			XC_watch },
+    { "xterm",			XC_xterm },
+    { NULL,			CURSOR_ID_NONE }
+};
+
+static void
+check_bitmap_status(int status, char *filename, Pixmap bitmap)
+{
+    switch(status) {
+     case BitmapOpenFailed:
+	wwarning(_("failed to open bitmap file \"%s\""), filename);
+	break;
+     case BitmapFileInvalid:
+	wwarning(_("\"%s\" is not a valid bitmap file"), filename);
+	break;
+     case BitmapNoMemory:
+	wwarning(_("out of memory reading bitmap file \"%s\""), filename);
+	break;
+     case BitmapSuccess:
+	XFreePixmap(dpy, bitmap);
+	break;
+    }
+}
+
+/*
+ * (none)
+ * (builtin, <cursor_name>)
+ * (bitmap, <cursor_bitmap>, <cursor_mask>)
+ */
+static int
+parse_cursor(WScreen *scr, proplist_t pl, Cursor *cursor)
+{
+    proplist_t elem;
+    char *val;
+    int nelem;
+    int status = 0;
+    
+    nelem = PLGetNumberOfElements(pl);
+    if (nelem < 1) {
+	return(status);
+    }
+    elem = PLGetArrayElement(pl, 0);
+    if (!elem || !PLIsString(elem)) {
+	return(status);
+    }
+    val = PLGetString(elem);
+    
+    if (0 == strcasecmp(val, "none")) {
+	status = 1;
+	*cursor = None;
+    } else if (0 == strcasecmp(val, "builtin")) {
+	int i;
+	int cursor_id = CURSOR_ID_NONE;
+	
+	if (2 != nelem) {
+	    wwarning(_("bad number of arguments in cursor specification"));
+	    return(status);
+	}
+	elem = PLGetArrayElement(pl, 1);
+	if (!elem || !PLIsString(elem)) {
+	    return(status);
+	}
+	val = PLGetString(elem);
+
+	for (i = 0; NULL != cursor_table[i].name; i++) {
+	    if (0 == strcasecmp(val, cursor_table[i].name)) {
+		cursor_id = cursor_table[i].id;
+		break;
+	    }
+	}
+	if (CURSOR_ID_NONE == cursor_id) {
+	    wwarning(_("unknown builtin cursor name \"%s\""), val);
+	} else {
+	    *cursor = XCreateFontCursor(dpy, cursor_id);
+	    status = 1;
+	}
+    } else if (0 == strcasecmp(val, "bitmap")) {
+	char *bitmap_name;
+	char *mask_name;
+	int bitmap_status;
+	int mask_status;
+	Pixmap bitmap;
+	Pixmap mask;
+	unsigned int w, h;
+	int x, y;
+	XColor fg, bg;
+	
+	if (3 != nelem) {
+	    wwarning(_("bad number of arguments in cursor specification"));
+	    return(status);
+	}
+	elem = PLGetArrayElement(pl, 1);
+	if (!elem || !PLIsString(elem)) {
+	    return(status);
+	}
+	val = PLGetString(elem);
+	bitmap_name = FindImage(wPreferences.pixmap_path, val);
+	if (!bitmap_name) {
+	    wwarning(_("could not find cursor bitmap file \"%s\""), val);
+	    return(status);
+	}
+	elem = PLGetArrayElement(pl, 2);
+	if (!elem || !PLIsString(elem)) {
+	    wfree(bitmap_name);
+	    return(status);
+	}
+	val = PLGetString(elem);
+	mask_name = FindImage(wPreferences.pixmap_path, val);
+	if (!mask_name) {
+	    wfree(bitmap_name);
+	    wwarning(_("could not find cursor bitmap file \"%s\""), val);
+	    return(status);
+	}
+	mask_status = XReadBitmapFile(dpy, scr->w_win, mask_name, &w, &h,
+				      &mask, &x, &y);
+	bitmap_status = XReadBitmapFile(dpy, scr->w_win, bitmap_name, &w, &h,
+					&bitmap, &x, &y);
+	if ((BitmapSuccess == bitmap_status) &&
+	    (BitmapSuccess == mask_status)) {
+	    fg.pixel = scr->black_pixel;
+	    bg.pixel = scr->white_pixel;
+	    XQueryColor(dpy, scr->w_colormap, &fg);
+	    XQueryColor(dpy, scr->w_colormap, &bg);
+	    *cursor = XCreatePixmapCursor(dpy, bitmap, mask, &fg, &bg, x, y);
+	    status = 1;
+	}
+	check_bitmap_status(bitmap_status, bitmap_name, bitmap);
+	check_bitmap_status(mask_status, mask_name, mask);
+	wfree(bitmap_name);
+	wfree(mask_name);
+    }
+    return(status);
 }
 
 
 static int
-setIfDockPresent(WScreen *scr, WDefaultEntry *entry, int *flag, long which)
+getCursor(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
+	  void **ret)
+{
+    static Cursor cursor;
+    int status;
+    int changed = 0;
+    
+again:
+    if (!PLIsArray(value)) {
+	wwarning(_("Wrong option format for key \"%s\". Should be %s."),
+		 entry->key, "cursor specification");
+	if (!changed) {
+	    value = entry->plvalue;
+	    changed = 1;
+	    wwarning(_("using default \"%s\" instead"), entry->default_value);
+	    goto again;
+	}
+	return(False);
+    }
+    status = parse_cursor(scr, value, &cursor);
+    if (!status) {
+	wwarning(_("Error in cursor specification for key \"%s\""), entry->key);
+	if (!changed) {
+	    value = entry->plvalue;
+	    changed = 1;
+	    wwarning(_("using default \"%s\" instead"), entry->default_value);
+	    goto again;
+	}
+	return(False);
+    }
+    if (ret) {
+	*ret = &cursor;
+    }
+    if (addr) {
+	*(Cursor *)addr = cursor;
+    }
+    return(True);
+}
+#undef CURSOR_ID_NONE
+
+#endif /* DEFINABLE_CURSOR */
+
+
+/* ---------------- value setting functions --------------- */
+static int
+    setJustify(WScreen *scr, WDefaultEntry *entry, WTexture **texture, void *foo)
+{
+    return REFRESH_WINDOW_TITLE_COLOR;
+}
+
+static int
+setClearance(WScreen *scr, WDefaultEntry *entry, void *bar, void *foo)
+{
+    return REFRESH_WINDOW_FONT|REFRESH_BUTTON_IMAGES|REFRESH_MENU_TITLE_FONT|REFRESH_MENU_FONT;
+}
+
+static int
+setIfDockPresent(WScreen *scr, WDefaultEntry *entry, char *flag, long which)
 {
     switch (which) {
     case WM_DOCK:
@@ -2382,6 +2789,11 @@ setIconTile(WScreen *scr, WDefaultEntry *entry, WTexture **texture, void *foo)
     }
 
     scr->icon_tile = img;
+    
+    
+    /* put the icon in the noticeboard hint */
+    PropSetIconTileHint(scr, img);
+    
 
     if (!wPreferences.flags.noclip) {
 	if (scr->clip_tile) {
@@ -2485,28 +2897,6 @@ setClipTitleFont(WScreen *scr, WDefaultEntry *entry, WMFont *font, void *foo)
 }
 
 
-static int
-setDisplayFont(WScreen *scr, WDefaultEntry *entry, WMFont *font, void *foo)
-{
-    if (scr->info_text_font) {
-	WMReleaseFont(scr->info_text_font);
-    }
-
-    scr->info_text_font = font;
-
-    /* This test works because the scr structure is initially zeroed out
-       and None = 0. Any other time, the window should be valid. */
-    if (scr->geometry_display != None) {
-        wGetGeometryWindowSize(scr, &scr->geometry_display_width,
-			       &scr->geometry_display_height);
-	XResizeWindow(dpy, scr->geometry_display,
-	    scr->geometry_display_width, scr->geometry_display_height);
-    }
-
-    return 0;
-}
-
-
 static int
 setLargeDisplayFont(WScreen *scr, WDefaultEntry *entry, WMFont *font, void *foo)
 {
@@ -2604,29 +2994,12 @@ setWTitleColor(WScreen *scr, WDefaultEntry *entry, XColor *color, long index)
 static int 
 setMenuTitleColor(WScreen *scr, WDefaultEntry *entry, XColor *color, long index)
 {
-#ifdef TITLE_TEXT_SHADOW
-    if (index == WS_SMENU){
-        if (scr->menu_title_pixel[WS_SMENU]!=scr->white_pixel &&
-             scr->menu_title_pixel[WS_SMENU]!=scr->black_pixel) {
-             wFreeColor(scr, scr->menu_title_pixel[WS_SMENU]);
-        }
-        scr->menu_title_pixel[WS_SMENU] = color->pixel;
-    }
-    else {
-        if (scr->menu_title_pixel[0]!=scr->white_pixel &&
-             scr->menu_title_pixel[0]!=scr->black_pixel) {
-             wFreeColor(scr, scr->menu_title_pixel[0]);
-        }
-        scr->menu_title_pixel[0] = color->pixel;
-    }
-#else /* !TITLE_TEXT_SHADOW */
     if (scr->menu_title_pixel[0]!=scr->white_pixel &&
-	scr->menu_title_pixel[0]!=scr->black_pixel) {
-	wFreeColor(scr, scr->menu_title_pixel[0]);
+            scr->menu_title_pixel[0]!=scr->black_pixel) {
+            wFreeColor(scr, scr->menu_title_pixel[0]);
     }
     
     scr->menu_title_pixel[0] = color->pixel;
-#endif /* !TITLE_TEXT_SHADOW */
     XSetForeground(dpy, scr->menu_title_gc, color->pixel);
     
     return REFRESH_MENU_TITLE_COLOR;
@@ -2792,6 +3165,7 @@ setWorkspaceSpecificBack(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 
 	    SendHelperMessage(scr, 'P', -1, wPreferences.pixmap_path);
 	}
+	
     }
 
     for (i = 0; i < PLGetNumberOfElements(value); i++) {
@@ -2801,7 +3175,7 @@ setWorkspaceSpecificBack(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 
 	    SendHelperMessage(scr, 'S', i+1, str);
 
-	    free(str);
+	    wfree(str);
 	} else {
 	    SendHelperMessage(scr, 'U', i+1, NULL);
 	}
@@ -2827,13 +3201,13 @@ setWorkspaceBack(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 	    str = PLGetDescription(value);
 	    if (str) {
 		SendHelperMessage(scr, 'S', 0, str);
-		free(str);
+		wfree(str);
 		SendHelperMessage(scr, 'C', scr->current_workspace+1, NULL);
 	    } else {
 		SendHelperMessage(scr, 'U', 0, NULL);
 	    }
 	}
-    } else {
+    } else if (PLGetNumberOfElements(value) > 0) {
 	char *command;
 	char *text;
 
@@ -2844,9 +3218,9 @@ setWorkspaceBack(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 	    sprintf(command, "wmsetbg -d -S -p '%s' &", text);
 	else
 	    sprintf(command, "wmsetbg -d -p '%s' &", text);
-	free(text);
+	wfree(text);
 	system(command);
-	free(command);
+	wfree(command);
     }
     PLRelease(value);
 
@@ -2862,10 +3236,6 @@ setWidgetColor(WScreen *scr, WDefaultEntry *entry, WTexture **texture, void *foo
     }
     scr->widget_texture = *(WTexSolid**)texture;
 
-    if (scr->geometry_display != None)
-    	XSetWindowBackground(dpy, scr->geometry_display, 
-			     scr->widget_texture->normal.pixel);
-
     return 0;
 }
 
@@ -3006,7 +3376,7 @@ setButtonImages(WScreen *scr, WDefaultEntry *entry, int *value, void *foo)
  * wmaker panels will have the same dbl-click values. 
  * TODO: figure a better way of dealing with it.
  */
-#include "WINGsP.h"
+#include <WINGs/WINGsP.h>
 
 static int
 setDoubleClick(WScreen *scr, WDefaultEntry *entry, int *value, void *foo)
@@ -3022,9 +3392,9 @@ setDoubleClick(WScreen *scr, WDefaultEntry *entry, int *value, void *foo)
 }
 
 
-
+#if 0
 static int
-setMultiByte(WScreen *scr, WDefaultEntry *entry, int *value, void *foo)
+setMultiByte(WScreen *scr, WDefaultEntry *entry, char *value, void *foo)
 {
     extern _WINGsConfiguration WINGsConfiguration;
 
@@ -3032,4 +3402,25 @@ setMultiByte(WScreen *scr, WDefaultEntry *entry, int *value, void *foo)
     
     return 0;
 }
+#endif
+
+#ifdef DEFINABLE_CURSOR
+static int
+setCursor(WScreen *scr, WDefaultEntry *entry, Cursor *cursor, long index)
+{
+   if (None != wCursor[index])
+    {
+       XFreeCursor(dpy, wCursor[index]);
+    }
+   
+   wCursor[index] = *cursor;
+
+   if ((WCUR_ROOT == index) && (None != *cursor))
+    {
+       XDefineCursor(dpy, scr->root_win, *cursor);
+    }
+
+   return 0;
+}
+#endif /* DEFINABLE_CURSOR*/
 
