@@ -36,6 +36,10 @@
 #include <dirent.h>
 #include <limits.h>
 
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+
 #include <signal.h>
 #ifdef __FreeBSD__
 #include <sys/signal.h>
@@ -75,7 +79,7 @@ wMessageDialog(WScreen *scr, char *title, char *message,
 			       defBtn, altBtn, othBtn);    
 
     parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 400, 180, 0, 0, 0);
-    
+
     XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
     wwin = wManageInternalWindow(scr, parent, None, NULL, 
@@ -84,15 +88,10 @@ wMessageDialog(WScreen *scr, char *title, char *message,
     wwin->client_leader = WMWidgetXID(panel->win);
 
     WMMapWidget(panel->win);
-    
+
     wWindowMap(wwin);
 
-    while (!panel->done) {
-	XEvent event;
-	
-	WMNextEvent(dpy, &event);
-	WMHandleEvent(&event);
-    }
+    WMRunModalLoop(WMWidgetScreen(panel->win), WMWidgetView(panel->win));
 
     result = panel->result;
 
@@ -117,7 +116,7 @@ wInputDialog(WScreen *scr, char *title, char *message, char **text)
     WMInputPanel *panel;
     char *result;
 
-    
+
     panel = WMCreateInputPanel(scr->wmscreen, NULL, title, message, *text,
 			       _("OK"), _("Cancel"));
 
@@ -134,16 +133,11 @@ wInputDialog(WScreen *scr, char *title, char *message, char **text)
     wwin->client_leader = WMWidgetXID(panel->win);
 
     WMMapWidget(panel->win);
-    
+
     wWindowMap(wwin);
-    
-    while (!panel->done) {
-	XEvent event;
-	
-	WMNextEvent(dpy, &event);
-	WMHandleEvent(&event);
-    }
-    
+
+    WMRunModalLoop(WMWidgetScreen(panel->win), WMWidgetView(panel->win));
+
     if (panel->result == WAPRDefault)
 	result = WMGetTextFieldText(panel->text);
     else
@@ -159,7 +153,7 @@ wInputDialog(WScreen *scr, char *title, char *message, char **text)
 	return False;
     else {
         if (*text)
-            free(*text);
+            wfree(*text);
         *text = result;
 
         return True;
@@ -184,7 +178,10 @@ typedef struct IconPanel {
     
     WMList *dirList;
     WMList *iconList;
+    WMFont *normalfont;
     
+    WMButton *previewButton;
+
     WMLabel *iconView;
     
     WMLabel *fileLabel;
@@ -197,6 +194,7 @@ typedef struct IconPanel {
 #endif
     short done;
     short result;
+    short preview;
 } IconPanel;
 
 
@@ -208,6 +206,9 @@ listPixmaps(WScreen *scr, WMList *lPtr, char *path)
     DIR *dir;
     char pbuf[PATH_MAX+16];
     char *apath;
+    IconPanel *panel = WMGetHangedData(lPtr);
+
+    panel->preview = False;
     
     apath = wexpandpath(path);
     dir = opendir(apath);
@@ -221,8 +222,8 @@ listPixmaps(WScreen *scr, WMList *lPtr, char *path)
 	strcat(msg, path);
 	
 	wMessageDialog(scr, _("Error"), msg, _("OK"), NULL, NULL);
-	free(msg);
-	free(apath);
+	wfree(msg);
+	wfree(apath);
 	return;
     }
 
@@ -243,12 +244,14 @@ listPixmaps(WScreen *scr, WMList *lPtr, char *path)
 	
 	if (statb.st_mode & (S_IRUSR|S_IRGRP|S_IROTH)
 	    && statb.st_mode & (S_IFREG|S_IFLNK)) {
-	    WMAddSortedListItem(lPtr, dentry->d_name);
+	    WMAddListItem(lPtr, dentry->d_name);
 	}
     }
+    WMSortListItems(lPtr);
 
     closedir(dir);
-    free(apath);
+    wfree(apath);
+    panel->preview = True;
 }
 
 
@@ -311,10 +314,10 @@ listCallback(void *self, void *data)
 	strcpy(path, tmp);
 	strcat(path, "/");
 	strcat(path, iconFile);
-	free(tmp);
+	wfree(tmp);
 	WMSetTextFieldText(panel->fileField, path);
 	setViewedImage(panel, path);
-	free(path);
+	wfree(path);
     }
 }
 
@@ -337,13 +340,101 @@ listIconPaths(WMList *lPtr)
 	 * directories searched */
 	if (access(tmp, X_OK)==0)
 	    WMAddListItem(lPtr, path);
-	free(tmp);
+	wfree(tmp);
     } while ((path=strtok(NULL, ":"))!=NULL);
 
-    free(paths);
+    wfree(paths);
 }
 
 
+void
+drawIconProc(WMList *lPtr, int index, Drawable d, char *text,
+	     int state, WMRect *rect) 
+{
+    IconPanel *panel = WMGetHangedData(lPtr);
+    GC gc = panel->scr->draw_gc;
+    GC copygc = panel->scr->copy_gc;
+    char *buffer, *dirfile;
+    WMPixmap *pixmap;
+    WMColor *blackcolor;
+    WMColor *whitecolor;
+    WMSize size;
+    WMScreen *wmscr=WMWidgetScreen(panel->win);
+    int width;
+
+    if(!panel->preview) return;
+
+    width = rect->size.width;
+
+    blackcolor = WMBlackColor(wmscr);
+    whitecolor = WMWhiteColor(wmscr);
+
+    dirfile = wexpandpath(WMGetListSelectedItem(panel->dirList)->text);
+    buffer = wmalloc(strlen(dirfile)+strlen(text)+4);
+    sprintf(buffer, "%s/%s", dirfile, text);
+    wfree(dirfile);
+
+    pixmap = WMCreatePixmapFromFile(WMWidgetScreen(panel->win), buffer);
+    wfree(buffer);
+    if (!pixmap) {
+        WMRemoveListItem(lPtr, index);
+        return;
+    }
+
+    XClearArea(dpy, d, rect->pos.x, rect->pos.y, width, rect->size.height, 
+	       False);
+    XSetClipMask(dpy, gc, None);
+    /*
+    XDrawRectangle(dpy, d, WMColorGC(whitecolor), rect->pos.x + 5, 
+		   rect->pos.y +5, width - 10, 54);
+     */
+    XDrawLine(dpy, d, WMColorGC(whitecolor), rect->pos.x,
+	      rect->pos.y+rect->size.height-1, rect->pos.x+width, 
+	      rect->pos.y+rect->size.height-1);
+   
+    
+    if (state&WLDSSelected) {
+        XFillRectangle(dpy, d, WMColorGC(whitecolor), rect->pos.x,
+		       rect->pos.y, width, rect->size.height);
+    }
+
+    size = WMGetPixmapSize(pixmap);
+
+    XSetClipMask(dpy, copygc, WMGetPixmapMaskXID(pixmap));
+    XSetClipOrigin(dpy, copygc, rect->pos.x + (width-size.width)/2,
+		   rect->pos.y+2);
+    XCopyArea(dpy, WMGetPixmapXID(pixmap), d, copygc, 0, 0,
+	      size.width>100?100:size.width, size.height>64?64:size.height, 
+	      rect->pos.x + (width-size.width)/2, rect->pos.y+2);
+
+    {
+        int i,j;
+	int fheight = WMFontHeight(panel->normalfont);
+	int tlen = strlen(text);
+	int twidth = WMWidthOfString(panel->normalfont, text, tlen);
+	int ofx, ofy;
+	
+	ofx = rect->pos.x + (width - twidth)/2;
+	ofy = rect->pos.y + 64 - fheight;
+	
+        for(i=-1;i<2;i++)
+	    for(j=-1;j<2;j++)
+		WMDrawString(wmscr, d, WMColorGC(whitecolor), 
+			     panel->normalfont, ofx+i, ofy+j,
+			     text, tlen);
+
+	WMDrawString(wmscr, d, WMColorGC(blackcolor), panel->normalfont, 
+		     ofx, ofy, text, tlen);
+    }
+
+    WMReleasePixmap(pixmap);
+    /* I hope it is better to do not use cache / on my box it is fast nuff */
+    XFlush(dpy);
+
+    WMReleaseColor(blackcolor);
+    WMReleaseColor(whitecolor);
+}
+
 
 static void
 buttonCallback(void *self, void *clientData)
@@ -358,6 +449,14 @@ buttonCallback(void *self, void *clientData)
     } else if (bPtr==panel->cancelButton) {
 	panel->done = True;
 	panel->result = False;
+    } else if (bPtr==panel->previewButton) {
+        /**** Previewer ****/
+        WMSetButtonEnabled(bPtr, False);
+        WMSetListUserDrawItemHeight(panel->iconList, 68);
+        WMSetListUserDrawProc(panel->iconList, drawIconProc);
+        WMRedisplayWidget(panel->iconList);
+        /* for draw proc to access screen/gc */
+        /*** end preview ***/
     }
 #if 0
     else if (bPtr==panel->chooseButton) {
@@ -370,7 +469,7 @@ buttonCallback(void *self, void *clientData)
 	    path = WMGetFilePanelFile(op);
 	    WMSetTextFieldText(panel->fileField, path);
 	    setViewedImage(panel, path);
-	    free(path);
+	    wfree(path);
 	}
 	WMDestroyFilePanel(op);
     }
@@ -396,6 +495,7 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
     WMResizeWidget(panel->win, 450, 280);
     
     boldFont = WMBoldSystemFontOfSize(scr->wmscreen, 12);
+    panel->normalfont = WMSystemFontOfSize(WMWidgetScreen(panel->win), 12);
     
     panel->dirLabel = WMCreateLabel(panel->win);
     WMResizeWidget(panel->dirLabel, 200, 20);
@@ -436,10 +536,18 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
     WMResizeWidget(panel->iconList, 140, 170);
     WMMoveWidget(panel->iconList, 215, 30); 
     WMSetListAction(panel->iconList, listCallback, panel);
-   
+
+    WMHangData(panel->iconList,panel);
+
+    panel->previewButton = WMCreateCommandButton(panel->win);
+    WMResizeWidget(panel->previewButton, 75, 26);
+    WMMoveWidget(panel->previewButton, 365, 130);
+    WMSetButtonText(panel->previewButton, _("Preview"));
+    WMSetButtonAction(panel->previewButton, buttonCallback, panel);
+
     panel->iconView = WMCreateLabel(panel->win);
     WMResizeWidget(panel->iconView, 75, 75);
-    WMMoveWidget(panel->iconView, 365, 60);
+    WMMoveWidget(panel->iconView, 365, 40);
     WMSetLabelImagePosition(panel->iconView, WIPOverlaps);
     WMSetLabelRelief(panel->iconView, WRSunken);
     WMSetLabelTextAlignment(panel->iconView, WACenter);
@@ -483,7 +591,7 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
     {
 	char *tmp;
 
-	tmp = malloc((instance ? strlen(instance) : 0)
+	tmp = wmalloc((instance ? strlen(instance) : 0)
 		     + (class ? strlen(class) : 0) + 32);
 
 	if (tmp && (instance || class))
@@ -494,7 +602,7 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
 	wwin = wManageInternalWindow(scr, parent, None, tmp,
 				     (scr->scr_width - 450)/2,
 				     (scr->scr_height - 280)/2, 450, 280);
-	free(tmp);
+	wfree(tmp);
     }
     
     /* put icon paths in the list */
@@ -518,7 +626,7 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
 	 * would be loaded by default with the current search path */
 	*file = WMGetListSelectedItem(panel->iconList)->text;
 	if ((*file)[0]==0) {
-	    free(*file);
+	    wfree(*file);
 	    *file = NULL;
 	} else {
 	    defaultPath = FindImage(wPreferences.icon_path, *file);
@@ -528,21 +636,23 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
 		*file = wantedPath;
 	    } else {
 		*file = wstrdup(*file);
-		free(wantedPath);
+		wfree(wantedPath);
 	    }
-	    free(defaultPath);
+	    wfree(defaultPath);
 	}
     } else {
 	*file = NULL;
     }
 
+    WMReleaseFont(panel->normalfont);
+
     WMUnmapWidget(panel->win);
 
     WMDestroyWidget(panel->win);    
 
     wUnmanageWindow(wwin, False, False);
 
-    free(panel);
+    wfree(panel);
 
     XDestroyWindow(dpy, parent);
     
@@ -588,8 +698,8 @@ typedef struct {
 
 
 #define COPYRIGHT_TEXT  \
-     "Copyright \xa9 1997~1999 Alfredo K. Kojima <kojima@windowmaker.org>\n"\
-     "Copyright \xa9 1998,1999 Dan Pascu <dan@windowmaker.org>"
+     "Copyright \xa9 1997~2001 Alfredo K. Kojima <kojima@windowmaker.org>\n"\
+     "Copyright \xa9 1998~2001 Dan Pascu <dan@windowmaker.org>"
 
  
 
@@ -618,7 +728,7 @@ destroyInfoPanel(WCoreWindow *foo, void *data, XEvent *event)
 
     WMDestroyWidget(thePanel->win);
 
-    free(thePanel);
+    wfree(thePanel);
     
     thePanel = NULL;
 }
@@ -848,8 +958,6 @@ handleLogoPush(XEvent *event, void *data)
 "             .....XmnbN:q&Bo....             ",
 "                 ............                "};
     static char *msgs[] = {
-	"Sloppy focus is a *?#@",
-	    "Repent! Sloppy focus users will burn in hell!!!",
 	    "Have a nice day!"
     };
 
@@ -875,7 +983,7 @@ handleLogoPush(XEvent *event, void *data)
 	    }
 
 	    panel->icon = RLoadImage(panel->scr->rcontext, path, 0);
-	    free(path);
+	    wfree(path);
 	    if (!panel->icon) {
 		broken = 1;
 		return;
@@ -923,7 +1031,7 @@ handleLogoPush(XEvent *event, void *data)
 	WMDeleteTimerHandler(panel->timer);
 	panel->timer = NULL;
 
-	sprintf(version, "Version %s", VERSION);
+	sprintf(version, _("Version %s"), VERSION);
 	WMSetLabelText(panel->versionL, version);
     }
 
@@ -1025,10 +1133,10 @@ wShowInfoPanel(WScreen *scr)
 	font = NULL;
     }
     WMSetLabelTextAlignment(panel->name2L, WACenter);
-    WMSetLabelText(panel->name2L, "Window Manager for X");
+    WMSetLabelText(panel->name2L, _("Window Manager for X"));
 
     
-    sprintf(version, "Version %s", VERSION);
+    sprintf(version, _("Version %s"), VERSION);
     panel->versionL = WMCreateLabel(panel->win);
     WMResizeWidget(panel->versionL, 310, 16);
     WMMoveWidget(panel->versionL, 30, 95);
@@ -1049,32 +1157,42 @@ wShowInfoPanel(WScreen *scr)
 
     switch (scr->w_depth) {
      case 15:
-	strcpy(version, "32 thousand");
+	strcpy(version, _("32 thousand"));
 	break;
      case 16:
-	strcpy(version, "64 thousand");
+	strcpy(version, _("64 thousand"));
 	break;
      case 24:
      case 32:
-	strcpy(version, "16 million");
+	strcpy(version, _("16 million"));
 	break;
      default:
 	sprintf(version, "%d", 1<<scr->w_depth);
 	break;
     }
 
-    sprintf(buffer, "Using visual 0x%x: %s %ibpp (%s colors)\n",
+    sprintf(buffer, _("Using visual 0x%x: %s %ibpp (%s colors)\n"),
 	    (unsigned)scr->w_visual->visualid,
 	    visuals[scr->w_visual->class], scr->w_depth, version);
 
-    strcat(buffer, "Supported image formats: ");
+#if defined(HAVE_MALLOC_H) && defined(HAVE_MALLINFO)
+    {
+	struct mallinfo ma = mallinfo();
+	sprintf(buffer+strlen(buffer), 
+		_("Total allocated memory: %i kB. Total memory in use: %i kB.\n"),
+		(ma.arena+ma.hblkhd)/1024, (ma.uordblks+ma.hblkhd)/1024);
+	
+    }
+#endif
+    
+    strcat(buffer, _("Supported image formats: "));
     strl = RSupportedFileFormats();
     for (i=0; strl[i]!=NULL; i++) {
 	strcat(buffer, strl[i]);
 	strcat(buffer, " ");
     }
 
-    strcat(buffer, "\nAdditional Support For: ");
+    strcat(buffer, _("\nAdditional Support For: "));
     {
 	char *list[8];
 	char buf[80];
@@ -1092,15 +1210,15 @@ wShowInfoPanel(WScreen *scr)
 #ifdef OLWM_HINTS
 	list[j++] = "OLWM";
 #endif
-#ifdef WMSOUND
-	list[j++] = "Sound";
+#ifdef WSOUND
+	list[j++] = _("Sound");
 #endif
 
 	buf[0] = 0;
 	for (i = 0; i < j; i++) {
 	    if (i > 0) {
 		if (i == j - 1)
-		    strcat(buf, " and ");
+		    strcat(buf, _(" and "));
 		else
 		    strcat(buf, ", ");
 	    }
@@ -1129,7 +1247,7 @@ wShowInfoPanel(WScreen *scr)
 
     WMMapWidget(panel->win);
 
-    wwin = wManageInternalWindow(scr, parent, None, "Info",
+    wwin = wManageInternalWindow(scr, parent, None, _("Info"),
 				 (scr->scr_width - 382)/2,
 				 (scr->scr_height - 230)/2, 382, 230);
 
@@ -1152,7 +1270,7 @@ wShowInfoPanel(WScreen *scr)
 	panel->timer = WMAddTimerHandler(100, logoPushCallback, panel);
 	panel->cycle = 0;
 	panel->x = 1;
-	panel->str = "Merry Christmas!";
+	panel->str = _("Merry X'mas!");
 	panel->oldPix = WMRetainPixmap(WMGetLabelImage(panel->logoL));
     } 
 #endif
@@ -1176,22 +1294,6 @@ typedef struct {
 } LegalPanel;
 
 
-
-#define LICENSE_TEXT \
-	"    Window Maker is free software; you can redistribute it and/or modify "\
-	"it under the terms of the GNU General Public License as published "\
-	"by the Free Software Foundation; either version 2 of the License, "\
-	"or (at your option) any later version.\n\n\n"\
-	"    Window Maker is distributed in the hope that it will be useful, but "\
-	"WITHOUT ANY WARRANTY; without even the implied warranty of "\
-	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU "\
-	"General Public License for more details.\n\n\n"\
-	"    You should have received a copy of the GNU General Public License "\
-	"along with this program; if not, write to the Free Software "\
-	"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA "\
-	"02111-1307, USA."
- 
-
 static LegalPanel *legalPanel = NULL;
 
 static void
@@ -1203,7 +1305,7 @@ destroyLegalPanel(WCoreWindow *foo, void *data, XEvent *event)
 
     wUnmanageWindow(legalPanel->wwin, False, False);
 
-    free(legalPanel);
+    wfree(legalPanel);
     
     legalPanel = NULL;
 }
@@ -1233,10 +1335,23 @@ wShowLegalPanel(WScreen *scr)
 
     
     panel->licenseL = WMCreateLabel(panel->win);
+    WMSetLabelWraps(panel->licenseL, True);
     WMResizeWidget(panel->licenseL, 400, 230);
     WMMoveWidget(panel->licenseL, 10, 10);
     WMSetLabelTextAlignment(panel->licenseL, WALeft);
-    WMSetLabelText(panel->licenseL, LICENSE_TEXT);
+    WMSetLabelText(panel->licenseL,
+      _("    Window Maker is free software; you can redistribute it and/or\n"
+	"modify it under the terms of the GNU General Public License as\n"
+	"published by the Free Software Foundation; either version 2 of the\n"
+	"License, or (at your option) any later version.\n\n\n"
+	"    Window Maker is distributed in the hope that it will be useful,\n"
+	"but WITHOUT ANY WARRANTY; without even the implied warranty\n"
+	"of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
+	"See the GNU General Public License for more details.\n\n\n"
+	"    You should have received a copy of the GNU General Public\n"
+	"License along with this program; if not, write to the Free Software\n"
+	"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA\n"
+	"02111-1307, USA."));
     WMSetLabelRelief(panel->licenseL, WRGroove);
 
     WMRealizeWidget(panel->win);
@@ -1246,7 +1361,7 @@ wShowLegalPanel(WScreen *scr)
 
     XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
-    wwin = wManageInternalWindow(scr, parent, None, "Legal",
+    wwin = wManageInternalWindow(scr, parent, None, _("Legal"),
 				 (scr->scr_width - 420)/2,
 				 (scr->scr_height - 250)/2, 420, 250);
 
@@ -1366,7 +1481,7 @@ getWindowMakerIconImage(WMScreen *scr)
                 pix = WMCreatePixmapFromRImage(scr, image, 0);
                 RDestroyImage(image);
             }
-            free(path);
+            wfree(path);
         }
     }
 
@@ -1503,7 +1618,7 @@ wShowCrashingDialogPanel(int whatSig)
 
     WMUnmapWidget(panel->win);
     WMDestroyWidget(panel->win);
-    free(panel);
+    wfree(panel);
 
     return action;
 }
@@ -1556,19 +1671,6 @@ drawGNUstepLogo(Display *dpy, Drawable d, int width, int height,
 }
 
 
-
-#define GNUSTEP_TEXT \
-	"Window Maker is part of the GNUstep project.\n"\
-	"The GNUstep project aims to create a free\n"\
-	"implementation of the OpenStep(tm) specification\n"\
-	"which is a object-oriented framework for\n"\
-	"creating advanced graphical, multi-platform\n"\
-	"applications. Aditionally, a development and\n"\
-	"user desktop enviroment will be created on top\n"\
-	"of the framework. For more information about\n"\
-	"GNUstep, please visit: www.gnustep.org"
-
-
 typedef struct {
     WScreen *scr;
     
@@ -1592,7 +1694,7 @@ destroyGNUstepPanel(WCoreWindow *foo, void *data, XEvent *event)
 
     wUnmanageWindow(gnustepPanel->wwin, False, False);
 
-    free(gnustepPanel);
+    wfree(gnustepPanel);
 
     gnustepPanel = NULL;
 }
@@ -1654,7 +1756,16 @@ wShowGNUstepPanel(WScreen *scr)
     WMMoveWidget(panel->textL, 30, 50);
     WMSetLabelTextAlignment(panel->textL, WARight);
     WMSetLabelImagePosition(panel->textL, WIPOverlaps);
-    WMSetLabelText(panel->textL, GNUSTEP_TEXT);
+    WMSetLabelText(panel->textL,
+	_("Window Maker is part of the GNUstep project.\n"\
+	"The GNUstep project aims to create a free\n"\
+	"implementation of the OpenStep(tm) specification\n"\
+	"which is a object-oriented framework for\n"\
+	"creating advanced graphical, multi-platform\n"\
+	"applications. Additionally, a development and\n"\
+	"user desktop enviroment will be created on top\n"\
+	"of the framework. For more information about\n"\
+	"GNUstep, please visit: www.gnustep.org"));
     WMSetLabelImage(panel->textL, pixmap);
 
     WMReleasePixmap(pixmap);
@@ -1666,7 +1777,7 @@ wShowGNUstepPanel(WScreen *scr)
 
     XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
-    wwin = wManageInternalWindow(scr, parent, None, "About GNUstep",
+    wwin = wManageInternalWindow(scr, parent, None, _("About GNUstep"),
 				 (scr->scr_width - 325)/2,
 				 (scr->scr_height - 200)/2, 325, 200);
 
