@@ -29,7 +29,6 @@
 #include <unistd.h>
 #include <string.h>
 
-#include <nana.h>
 
 
 #include <X11/Xlib.h>
@@ -194,7 +193,7 @@ wDeleteDeathHandler(WMagicNumber id)
 
     WMRemoveFromBag(deathHandlers, handler);
 
-    free(handler);
+    wfree(handler);
 }
 
 
@@ -220,6 +219,9 @@ DispatchEvent(XEvent *event)
 	Shutdown(WSRestartPreparationMode);
 	/* received SIGHUP */
 	Restart(NULL, True);
+    } else if (WCHECK_STATE(WSTATE_NEED_REREAD)) {
+        WCHANGE_STATE(WSTATE_NORMAL);
+        wDefaultsCheckDomains("bla");
     }
 
     /* for the case that all that is wanted to be dispatched is
@@ -600,6 +602,7 @@ handleButtonPress(XEvent *event)
 {
     WObjDescriptor *desc;
     WScreen *scr;
+    
 #ifdef DEBUG
     L("got button press");
 #endif
@@ -637,13 +640,9 @@ handleButtonPress(XEvent *event)
 	}
 #ifdef MOUSE_WS_SWITCH
 	else if (event->xbutton.button==Button5) {
-
 	    wWorkspaceRelativeChange(scr, -1);
-
 	} else if (event->xbutton.button==Button4) {
-
 	    wWorkspaceRelativeChange(scr, 1);
-
 	}
 #endif /* MOUSE_WS_SWITCH */
 #ifdef GNOME_STUFF
@@ -669,12 +668,12 @@ handleButtonPress(XEvent *event)
 	    XAllowEvents(dpy, AsyncPointer, CurrentTime);
 	}
      
-	if (wPreferences.focus_mode == WKF_CLICK) {
+/*	if (wPreferences.focus_mode == WKF_CLICK) {*/
 	    if (wPreferences.ignore_focus_click) {
 		XAllowEvents(dpy, AsyncPointer, CurrentTime);
 	    }
 	    XAllowEvents(dpy, ReplayPointer, CurrentTime);
-	}
+/*	}*/
 	XSync(dpy, 0);
     } else if (desc->parent_type == WCLASS_APPICON
 	       || desc->parent_type == WCLASS_MINIWINDOW
@@ -999,24 +998,25 @@ handleEnterNotify(XEvent *event)
 #endif
 
 #ifdef VIRTUAL_DESKTOP
+    /* TODO: acceleration code */
     if (wPreferences.vedge_thickness) {
         int x,y;
         if (event->xcrossing.window == scr->virtual_edge_r) {
             XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, scr->scr_width - wPreferences.vedge_thickness - 1, event->xcrossing.y_root);
             wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
-            wWorkspaceSetViewPort(scr, scr->current_workspace, x+30, y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x + VIRTUALEDGE_SCROLL_HSTEP, y);
         } else if (event->xcrossing.window == scr->virtual_edge_l) {
             XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, wPreferences.vedge_thickness + 1, event->xcrossing.y_root);
             wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
-            wWorkspaceSetViewPort(scr, scr->current_workspace, x-30, y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x - VIRTUALEDGE_SCROLL_HSTEP, y);
         } else if (event->xcrossing.window == scr->virtual_edge_u) {
             XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, event->xcrossing.x_root, wPreferences.vedge_thickness + 1);
             wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
-            wWorkspaceSetViewPort(scr, scr->current_workspace, x, y-30);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x, y - VIRTUALEDGE_SCROLL_VSTEP);
         } else if (event->xcrossing.window == scr->virtual_edge_d) {
             XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, event->xcrossing.x_root, scr->scr_height - wPreferences.vedge_thickness - 1);
             wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
-            wWorkspaceSetViewPort(scr, scr->current_workspace, x, y+30);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x, y + VIRTUALEDGE_SCROLL_VSTEP);
         }
     }
 #endif
@@ -1303,138 +1303,6 @@ windowUnderPointer(WScreen *scr)
 
 
 
-static void
-doWindozeCycle(WWindow *wwin, XEvent *event, Bool next)
-{
-    WScreen *scr = wScreenForRootWindow(event->xkey.root);
-    Bool done = False;
-    Bool openedSwitchMenu = False;
-    WWindow *newFocused;
-    WWindow *oldFocused;
-    int modifiers;
-    XModifierKeymap *keymap;
-
-    if (!wwin)
-	return;
-
-    keymap = XGetModifierMapping(dpy);
-
-    
-    XGrabKeyboard(dpy, scr->root_win, False, GrabModeAsync, GrabModeAsync, 
-		  CurrentTime);
-
-    if (next) {
-	newFocused = NextToFocusAfter(wwin);
-    } else {
-	newFocused = NextToFocusBefore(wwin); 
-    }
-
-    scr->flags.doing_alt_tab = 1;
-
-
-    if (wPreferences.circ_raise)
-	XRaiseWindow(dpy, newFocused->frame->core->window);
-    wWindowFocus(newFocused, scr->focused_window);
-    oldFocused = newFocused;
-
-#if 0
-    if (wPreferences.popup_switchmenu && 
-	(!scr->switch_menu || !scr->switch_menu->flags.mapped)) {
-
-	OpenSwitchMenu(scr, scr->scr_width/2, scr->scr_height/2, False);
-	openedSwitchMenu = True;
-    }
-#endif	
-    while (!done) {
-	XEvent ev;
-
-	WMMaskEvent(dpy,KeyPressMask|KeyReleaseMask|ExposureMask, &ev);
-
-	if (ev.type != KeyRelease && ev.type != KeyPress) {
-	    WMHandleEvent(&ev);
-	    continue;
-	}
-	/* ignore CapsLock */
-	modifiers = ev.xkey.state & ValidModMask;
-
-	if (ev.type == KeyPress) {
-	    if (wKeyBindings[WKBD_FOCUSNEXT].keycode == ev.xkey.keycode
-		&& wKeyBindings[WKBD_FOCUSNEXT].modifier == modifiers) {
-
-		UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-		newFocused = NextToFocusAfter(newFocused);
-		wWindowFocus(newFocused, oldFocused);
-		oldFocused = newFocused;
-	    
-		if (wPreferences.circ_raise) {
-		    /* restore order */
-		    CommitStacking(scr);
-		    XRaiseWindow(dpy, newFocused->frame->core->window);
-		}
-
-		UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-
-	    } else if (wKeyBindings[WKBD_FOCUSPREV].keycode == ev.xkey.keycode
-		       && wKeyBindings[WKBD_FOCUSPREV].modifier == modifiers) {
-
-		UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-		newFocused = NextToFocusBefore(newFocused);
-		wWindowFocus(newFocused, oldFocused);
-		oldFocused = newFocused;
-
-		if (wPreferences.circ_raise) {
-		    /* restore order */
-		    CommitStacking(scr);
-		    XRaiseWindow(dpy, newFocused->frame->core->window);
-		}
-		UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-
-	    } else {
-		handleKeyPress(&ev);
-	    }
-	    
-	    /*
-	    else if (wKeyBindings[WKBD_LOWER].keycode == ev.xkey.keycode
-		       && wKeyBindings[WKBD_LOWER].modifier == modifiers) {
-
-		wLowerFrame(newFocused->frame->core);
-
-	    } else if (wKeyBindings[WKBD_RAISE].keycode == ev.xkey.keycode
-		       && wKeyBindings[WKBD_RAISE].modifier == modifiers) {
-
-		wRaiseFrame(newFocused->frame->core);
-	    }*/
-	} else if (ev.type == KeyRelease) {
-	    int i;
-
-	    for (i = 0; i < 8 * keymap->max_keypermod; i++) {
-		if (keymap->modifiermap[i] == ev.xkey.keycode &&
-		    wKeyBindings[WKBD_FOCUSNEXT].modifier 
-		    & 1<<(i/keymap->max_keypermod)) {
-		    done = True;
-		    break;
-		}
-	    }
-	}
-    }
-    XFreeModifiermap(keymap);
-
-    XUngrabKeyboard(dpy, CurrentTime);
-    wSetFocusTo(scr, newFocused);
-
-    if (wPreferences.circ_raise) {
-	wRaiseFrame(newFocused->frame->core);
-    	CommitStacking(scr);
-    }
-
-    scr->flags.doing_alt_tab = 0;
-    if (openedSwitchMenu) 
-	OpenSwitchMenu(scr, scr->scr_width/2, scr->scr_height/2, False);
-}
-
-
-
-
 static void
 handleKeyPress(XEvent *event)
 {
@@ -1605,17 +1473,11 @@ handleKeyPress(XEvent *event)
 	}
         break;
      case WKBD_FOCUSNEXT:
-	if (wKeyBindings[WKBD_FOCUSNEXT].modifier != 0)
-	    doWindozeCycle(wwin, event, True);
-	else
-	    puts("NEXT");
+	StartWindozeCycle(wwin, event, True);
 	break;
 
      case WKBD_FOCUSPREV:
-	if (wKeyBindings[WKBD_FOCUSPREV].modifier != 0)
-	    doWindozeCycle(wwin, event, False);
-	else
-	    puts("PREV");
+	StartWindozeCycle(wwin, event, False);
 	break;
 
 #if (defined(__STDC__) && !defined(UNIXCPP)) || defined(ANSICPP)
@@ -1652,14 +1514,12 @@ handleKeyPress(XEvent *event)
      case WKBD_WINDOW2:
      case WKBD_WINDOW3:
      case WKBD_WINDOW4:
-#ifdef EXTEND_WINDOWSHORTCUT
      case WKBD_WINDOW5:
      case WKBD_WINDOW6:
      case WKBD_WINDOW7:
      case WKBD_WINDOW8:
      case WKBD_WINDOW9:
      case WKBD_WINDOW10:
-#endif
 
 #define INITBAG(bag)  if (bag) WMEmptyBag(bag); else bag = WMCreateBag(4)
 
@@ -1735,6 +1595,30 @@ handleKeyPress(XEvent *event)
 #undef INITBAG
 
         break;
+	
+     case WKBD_SWITCH_SCREEN:
+	if (wScreenCount > 1) {
+	    WScreen *scr2;
+	    int i;
+	    
+	    /* find index of this screen */
+	    for (i = 0; i < wScreenCount; i++) {
+		if (wScreenWithNumber(i) == scr)
+		    break;
+	    }
+	    i++;
+	    if (i >= wScreenCount) {
+		i = 0;
+	    }
+	    scr2 = wScreenWithNumber(i);
+	    
+	    if (scr2) {
+		XWarpPointer(dpy, scr->root_win, scr2->root_win, 0, 0, 0, 0,
+			     scr2->scr_width/2, scr2->scr_height/2);
+	    }
+	}
+	break;
+	
      case WKBD_NEXTWSLAYER:
      case WKBD_PREVWSLAYER:
 	{
