@@ -63,9 +63,10 @@ static void updateTitlebar(WFrameWindow *fwin);
 
 WFrameWindow*
 wFrameWindowCreate(WScreen *scr, int wlevel, int x, int y, 
-		   int width, int height, int flags,
+		   int width, int height, int *clearance, int flags,
 		   WTexture **title_texture, WTexture **resize_texture,
-		   unsigned long *color, GC *gc, WMFont **font)
+		   unsigned long *color,
+           GC *gc, WMFont **font)
 {
     WFrameWindow *fwin;
     
@@ -79,15 +80,17 @@ wFrameWindowCreate(WScreen *scr, int wlevel, int x, int y,
     fwin->title_texture = title_texture;
     fwin->resizebar_texture = resize_texture;
     fwin->title_pixel = color;
+    fwin->title_clearance = clearance;
     fwin->title_gc = gc;
     fwin->font = font;
 #ifdef KEEP_XKB_LOCK_STATUS
     fwin->languagemode = XkbGroup1Index;
     fwin->last_languagemode = XkbGroup2Index;
 #endif
-    
+
     fwin->core = wCoreCreateTopLevel(scr, x, y, width, height, 
-				     FRAME_BORDER_WIDTH);
+				     (flags & WFF_BORDER) 
+				     ? FRAME_BORDER_WIDTH : 0);
     if (wPreferences.use_saveunders) {
 	unsigned long vmask;
 	XSetWindowAttributes attribs;
@@ -129,7 +132,7 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	height = fwin->core->height - fwin->top_width - fwin->bottom_width;
 
     if (flags & WFF_TITLEBAR)
-	theight = WMFontHeight(*fwin->font) + TITLEBAR_EXTRA_HEIGHT;
+	theight = WMFontHeight(*fwin->font) + (*fwin->title_clearance + TITLEBAR_EXTEND_SPACE) * 2;
     else
 	theight = 0;
     
@@ -153,12 +156,12 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 		}
 #ifdef XKB_BUTTON_HINT
 		if (fwin->language_button)
-        if (fwin->flags.hide_left_button || !fwin->left_button
-            || fwin->flags.lbutton_dont_fit) {
-		    wCoreConfigure(fwin->language_button, 0, 0, bsize, bsize);
-        } else {
-		    wCoreConfigure(fwin->language_button, bsize, 0, bsize, bsize);
-        }
+		    if (fwin->flags.hide_left_button || !fwin->left_button
+			|| fwin->flags.lbutton_dont_fit) {
+			wCoreConfigure(fwin->language_button, 0, 0, bsize, bsize);
+		    } else {
+			wCoreConfigure(fwin->language_button, bsize, 0, bsize, bsize);
+		    }
 #endif
 
 		
@@ -174,10 +177,9 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
         
 #ifdef XKB_BUTTON_HINT
 		if (fwin->language_button) {
-            wCoreConfigure(fwin->language_button, 6 + bsize, (theight-bsize)/2,
-                    bsize, bsize);
-
-        }
+		    wCoreConfigure(fwin->language_button, 6 + bsize, (theight-bsize)/2,
+				   bsize, bsize);
+		}
 #endif
 		
 		if (fwin->right_button) {
@@ -189,23 +191,23 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	} else {
 	    /* we had a titlebar, but now we don't need it anymore */
 	    for (i=0; i < (fwin->flags.single_texture ? 1 : 3); i++) {
-            FREE_PIXMAP(fwin->title_back[i]);
-            if (wPreferences.new_style) {
-                FREE_PIXMAP(fwin->lbutton_back[i]);
-                FREE_PIXMAP(fwin->rbutton_back[i]);
+		FREE_PIXMAP(fwin->title_back[i]);
+		if (wPreferences.new_style) {
+		    FREE_PIXMAP(fwin->lbutton_back[i]);
+		    FREE_PIXMAP(fwin->rbutton_back[i]);
 #ifdef XKB_BUTTON_HINT
-                FREE_PIXMAP(fwin->languagebutton_back[i]);
+		    FREE_PIXMAP(fwin->languagebutton_back[i]);
 #endif
-            }
+		}
 	    }
 	    if (fwin->left_button)
 		wCoreDestroy(fwin->left_button);
 	    fwin->left_button = NULL;
 	    
 #ifdef XKB_BUTTON_HINT
-        if (fwin->language_button)
-            wCoreDestroy(fwin->language_button);
-        fwin->language_button = NULL;
+	    if (fwin->language_button)
+		wCoreDestroy(fwin->language_button);
+	    fwin->language_button = NULL;
 #endif
 
 	    if (fwin->right_button)
@@ -225,59 +227,59 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	    
 	    fwin->flags.titlebar = 1;
 	    fwin->titlebar = wCoreCreate(fwin->core, 0, 0, width+1, theight);
-	    
+
 	    if (flags & WFF_LEFT_BUTTON) {
-            fwin->flags.left_button = 1;
-            if (wPreferences.new_style) {
-                fwin->left_button = wCoreCreate(fwin->core, 0, 0,
-                        bsize, bsize);
-                if (width < theight*4) {
-                    fwin->flags.lbutton_dont_fit = 1;
-                } else {
-                    XMapRaised(dpy, fwin->left_button->window);
-                }
-            } else {
-                fwin->left_button = 
-                    wCoreCreate(fwin->titlebar, 3, (theight-bsize)/2,
-                            bsize, bsize);
-
-                XSetWindowBackground(dpy, fwin->left_button->window,
-                        scr->widget_texture->normal.pixel);
-
-                if (width < theight*3) {
-                    fwin->flags.lbutton_dont_fit = 1;
-                } else {
-                    XMapRaised(dpy, fwin->left_button->window);
-                }
-            }
+		fwin->flags.left_button = 1;
+		if (wPreferences.new_style) {
+		    fwin->left_button = wCoreCreate(fwin->core, 0, 0,
+						    bsize, bsize);
+		    if (width < theight*4) {
+			fwin->flags.lbutton_dont_fit = 1;
+		    } else {
+			XMapRaised(dpy, fwin->left_button->window);
+		    }
+		} else {
+		    fwin->left_button = 
+			wCoreCreate(fwin->titlebar, 3, (theight-bsize)/2,
+				    bsize, bsize);
+		    
+		    XSetWindowBackground(dpy, fwin->left_button->window,
+					 scr->widget_texture->normal.pixel);
+		    
+		    if (width < theight*3) {
+			fwin->flags.lbutton_dont_fit = 1;
+		    } else {
+			XMapRaised(dpy, fwin->left_button->window);
+		    }
+		}
 	    }
         
 #ifdef XKB_BUTTON_HINT
 	    if (flags & WFF_LANGUAGE_BUTTON) {
-            fwin->flags.language_button = 1;
-            if (wPreferences.new_style) {
-                fwin->language_button = wCoreCreate(fwin->core,
-                        bsize, 0, bsize, bsize);
-
-                if (width < theight*4) {
-                    fwin->flags.languagebutton_dont_fit = 1;
-                } else {
-                    XMapRaised(dpy, fwin->language_button->window);
-                }
-            } else {
-                fwin->language_button = 
-                    wCoreCreate(fwin->titlebar, bsize + 6, (theight-bsize)/2,
+		fwin->flags.language_button = 1;
+		if (wPreferences.new_style) {
+		    fwin->language_button = wCoreCreate(fwin->core,
+							bsize, 0, bsize, bsize);
+		    
+		    if (width < theight*4) {
+			fwin->flags.languagebutton_dont_fit = 1;
+		    } else {
+			XMapRaised(dpy, fwin->language_button->window);
+		    }
+		} else {
+		    fwin->language_button = 
+			wCoreCreate(fwin->titlebar, bsize + 6, (theight-bsize)/2,
 				    bsize, bsize); 
-                
-                XSetWindowBackground(dpy, fwin->language_button->window,
+		    
+		    XSetWindowBackground(dpy, fwin->language_button->window,
 					 scr->widget_texture->normal.pixel); 
-
-                if (width < theight*3) {
-                    fwin->flags.languagebutton_dont_fit = 1;
-                } else {
-                    XMapRaised(dpy, fwin->language_button->window);
-                }
-            }
+		    
+		    if (width < theight*3) {
+			fwin->flags.languagebutton_dont_fit = 1;
+		    } else {
+			XMapRaised(dpy, fwin->language_button->window);
+		    }
+		}
 	    }
 #endif
 	    
@@ -311,14 +313,10 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	}
     }
     checkTitleSize(fwin);
-
-    if (flags & WFF_RESIZEBAR) {
-	fwin->bottom_width = RESIZEBAR_HEIGHT;
-    } else {
-	fwin->bottom_width = 0;
-    } 
     
     if (flags & WFF_RESIZEBAR) {
+	fwin->bottom_width = RESIZEBAR_HEIGHT;
+	
 	if (!fwin->resizebar) {
 	    fwin->flags.resizebar = 1;
 	    fwin->resizebar = wCoreCreate(fwin->core, 0,
@@ -342,6 +340,8 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 	    }
 	}
     } else {
+	fwin->bottom_width = 0;
+	
 	if (fwin->resizebar) {
 	    fwin->bottom_width = 0;
 	    wCoreDestroy(fwin->resizebar);
@@ -355,6 +355,13 @@ wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags)
 			   height + fwin->top_width + fwin->bottom_width);
     }
     
+    
+    if (flags & WFF_BORDER) {
+	XSetWindowBorderWidth(dpy, fwin->core->window, FRAME_BORDER_WIDTH);
+    } else {
+	XSetWindowBorderWidth(dpy, fwin->core->window, 0);
+    }
+    
     /* setup object descriptors */
     
     if (fwin->titlebar) {
@@ -427,7 +434,7 @@ wFrameWindowDestroy(WFrameWindow *fwin)
     wCoreDestroy(fwin->core);
     
     if (fwin->title)
-	free(fwin->title);
+	wfree(fwin->title);
     
     for (i=0; i < (fwin->flags.single_texture ? 1 : 3); i++) {
 	FREE_PIXMAP(fwin->title_back[i]);
@@ -440,7 +447,7 @@ wFrameWindowDestroy(WFrameWindow *fwin)
         }
     }
     
-    free(fwin);
+    wfree(fwin);
 }
 
 
@@ -463,7 +470,7 @@ updateTitlebar(WFrameWindow *fwin)
     int x, w;
     int theight;
     
-    theight = WMFontHeight(*fwin->font) + TITLEBAR_EXTRA_HEIGHT;
+	theight = WMFontHeight(*fwin->font) + (*fwin->title_clearance + TITLEBAR_EXTEND_SPACE) * 2;
 
     x = 0;
     w = fwin->core->width + 1;
@@ -654,7 +661,7 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
     
 #ifdef XKB_BUTTON_HINT
     if (language) {
-	    timg = RGetSubImage(img, bwidth, 0, bwidth, bheight);
+	    timg = RGetSubImage(img, bwidth * left, 0, bwidth, bheight);
 	} else
 	    timg = NULL;
 #endif
@@ -792,9 +799,10 @@ updateTexture(WFrameWindow *fwin)
 					       fwin->lbutton_back[i]);
 
 #ifdef XKB_BUTTON_HINT
-		if (fwin->language_button && fwin->languagebutton_back[i])
+		if (fwin->language_button && fwin->languagebutton_back[i]) {
 		    XSetWindowBackgroundPixmap(dpy, fwin->language_button->window,
 					       fwin->languagebutton_back[i]);
+        }
 #endif
 		
 		if (fwin->right_button && fwin->rbutton_back[i])
@@ -931,6 +939,8 @@ remakeTexture(WFrameWindow *fwin, int state)
 void
 wFrameWindowPaint(WFrameWindow *fwin)
 {
+    WScreen *scr = fwin->screen_ptr;
+
     if (fwin->flags.is_client_window_frame)
 	fwin->flags.justification = wPreferences.title_justification;
 
@@ -961,15 +971,15 @@ wFrameWindowPaint(WFrameWindow *fwin)
 	
 	updateTexture(fwin);
     }
-    
+
     if (fwin->titlebar && !fwin->flags.repaint_only_resizebar
 	&& fwin->title_texture[fwin->flags.state]->any.type==WTEX_SOLID) {
-	wDrawBevel(fwin->titlebar->window, fwin->titlebar->width,
-		   fwin->titlebar->height,
-		   (WTexSolid*)fwin->title_texture[fwin->flags.state], 
-		   WREL_RAISED);
+        wDrawBevel(fwin->titlebar->window, fwin->titlebar->width,
+                fwin->titlebar->height,
+                (WTexSolid*)fwin->title_texture[fwin->flags.state], 
+                WREL_RAISED);
     }
-    
+
     if (fwin->resizebar	&& !fwin->flags.repaint_only_titlebar
 	&& fwin->resizebar_texture[0]->any.type == WTEX_SOLID) {
 	Window win;
@@ -977,14 +987,14 @@ wFrameWindowPaint(WFrameWindow *fwin)
 	int cw;
 	GC light_gc, dim_gc;
 	WTexSolid *texture = (WTexSolid*)fwin->resizebar_texture[0];
-	
+
 	w = fwin->resizebar->width;
 	h = fwin->resizebar->height;
 	cw = fwin->resizebar_corner_width;
 	light_gc = texture->light_gc;
 	dim_gc = texture->dim_gc;
 	win = fwin->resizebar->window;
-	
+
 	XDrawLine(dpy, win, dim_gc, 0, 0, w, 0);
 	XDrawLine(dpy, win, light_gc, 0, 1, w, 1);
 	
@@ -1005,11 +1015,11 @@ wFrameWindowPaint(WFrameWindow *fwin)
     
     
     if (fwin->title && fwin->titlebar && !fwin->flags.repaint_only_resizebar) {
-	int x, w;
-	int lofs = 6, rofs = 6;
-	int titlelen;
-	char *title;
-	int allButtons = 1;
+        int x, w;
+        int lofs = 6, rofs = 6;
+        int titlelen;
+        char *title;
+        int allButtons = 1;
 
 
         if (!wPreferences.new_style) {
@@ -1036,7 +1046,7 @@ wFrameWindowPaint(WFrameWindow *fwin)
 
 #ifdef XKB_BUTTON_HINT
         fwin->languagebutton_image =
-            fwin->screen_ptr->b_pixmaps[WBUT_XKBGROUP1 + fwin->languagemode];
+            scr->b_pixmaps[WBUT_XKBGROUP1 + fwin->languagemode];
 #endif
 
 	title = ShrinkString(*fwin->font, fwin->title,
@@ -1061,30 +1071,14 @@ wFrameWindowPaint(WFrameWindow *fwin)
 	    break;
 	}
 
-#ifdef TITLE_TEXT_SHADOW
-       if(wPreferences.title_shadow){
-          int shadowx,shadowy;
-          XSetForeground(dpy, *fwin->title_gc, 
-              fwin->title_pixel[fwin->flags.state+3]);
-           for(shadowx=0;shadowx<TITLE_TEXT_SHADOW_WIDTH;shadowx++)
-           for(shadowy=0;shadowy<TITLE_TEXT_SHADOW_HEIGHT;shadowy++)
-		   WMDrawString(fwin->screen_ptr->wmscreen, fwin->titlebar->window,
-				*fwin->title_gc, *fwin->font,
-				x + shadowx + TITLE_TEXT_SHADOW_X_OFFSET, 
-				TITLEBAR_EXTRA_HEIGHT/2
-				+ shadowy + TITLE_TEXT_SHADOW_Y_OFFSET, title,
-				titlelen);
-       }
-#endif /* TITLE_TEXT_SHADOW */
-
 	XSetForeground(dpy, *fwin->title_gc, 
 		       fwin->title_pixel[fwin->flags.state]);
 	
-	WMDrawString(fwin->screen_ptr->wmscreen, fwin->titlebar->window, 
-		     *fwin->title_gc, *fwin->font, x, TITLEBAR_EXTRA_HEIGHT/2, 
-		     title, titlelen);
+    WMDrawString(scr->wmscreen, fwin->titlebar->window, 
+            *fwin->title_gc, *fwin->font, x, *fwin->title_clearance + TITLEBAR_EXTEND_SPACE, 
+            title, titlelen);
 
-	free(title);
+	wfree(title);
 	
 	if (fwin->left_button)
 	    handleButtonExpose(&fwin->left_button->descriptor, NULL);
@@ -1231,7 +1225,7 @@ wFrameWindowChangeTitle(WFrameWindow *fwin, char *new_title)
     }
 
     if (fwin->title)
-	free(fwin->title);
+	wfree(fwin->title);
     
     fwin->title = wstrdup(new_title);
     
