@@ -78,9 +78,8 @@ WShortKey wKeyBindings[WKBD_LAST];
 
 /* defaults domains */
 WDDomain *WDWindowMaker = NULL;
-WDDomain *WDRootMenu = NULL;
 WDDomain *WDWindowAttributes = NULL;
-
+WDDomain *WDRootMenu = NULL;
 
 /* XContexts */
 XContext wWinContext;
@@ -249,18 +248,18 @@ shellCommandHandler(pid_t pid, unsigned char status, _tuple *data)
     if (status == 127) {
 	char *buffer;
 
-	buffer = wstrappend(_("Could not execute command: "), data->command);
+	buffer = wstrconcat(_("Could not execute command: "), data->command);
 
 	wMessageDialog(data->scr, _("Error"), buffer, _("OK"), NULL, NULL);
-	free(buffer);
+	wfree(buffer);
     } else if (status != 127) {
 	/*
 	printf("%s: %i\n", data->command, status);
 	 */
     }
 
-    free(data->command);
-    free(data);
+    wfree(data->command);
+    wfree(data);
 }
 
 
@@ -427,12 +426,14 @@ print_help()
     puts(_(" --no-dock		do not open the application Dock"));
     puts(_(" --no-clip		do not open the workspace Clip"));
     puts(_(" --no-autolaunch	do not autolaunch applications"));
-    puts(_(" --dont-restore	do not restore saved session"));
+    puts(_(" --dont-restore		do not restore saved session"));
 
-    puts(_(" --locale locale		locale to use"));
+    puts(_(" --locale locale	locale to use"));
 
+    puts(_(" --create-stdcmap	create the standard colormap hint in PseudoColor visuals"));
     puts(_(" --visual-id visualid	visual id of visual to use"));
     puts(_(" --static		do not update or save configurations"));
+    puts(_(" --no-polling		do not periodically check for configuration updates"));
 #ifdef DEBUG
     puts(_(" --synchronous		turn on synchronous display mode"));
 #endif
@@ -455,7 +456,7 @@ check_defaults()
 	exit(1);
     }
     
-    free(path);
+    wfree(path);
 }
 
 
@@ -463,10 +464,10 @@ static void
 execInitScript()
 {
     char *file;
-    char *paths = wstrappend(wusergnusteppath(), ":"DEF_CONFIG_PATHS);
+    char *paths = wstrconcat(wusergnusteppath(), ":"DEF_CONFIG_PATHS);
 
     file = wfindfile(paths, DEF_INIT_SCRIPT);
-    free(paths);
+    wfree(paths);
 
     if (file) {
 	if (system(file) != 0) {
@@ -479,7 +480,7 @@ execInitScript()
 	    exit(1);
 	}
 #endif
-	free(file);
+	wfree(file);
     }
 }
 
@@ -488,10 +489,10 @@ void
 ExecExitScript()
 {
     char *file;
-    char *paths = wstrappend(wusergnusteppath(), ":"DEF_CONFIG_PATHS);
+    char *paths = wstrconcat(wusergnusteppath(), ":"DEF_CONFIG_PATHS);
 
     file = wfindfile(paths, DEF_EXIT_SCRIPT);
-    free(paths);
+    wfree(paths);
 
     if (file) {
 	if (system(file) != 0) {
@@ -504,7 +505,7 @@ ExecExitScript()
 	    exit(1);
 	}
 #endif
-	free(file);
+	wfree(file);
     }
 }
 
@@ -539,7 +540,7 @@ getFullPath(char *path)
 	
 
     } else {
-	return wstrappend(path);
+	return wstrconcat(path);
     }
 
     return tmp;
@@ -561,7 +562,7 @@ main(int argc, char **argv)
 
     /* for telling WPrefs what's the name of the wmaker binary being ran */
 
-    str = wstrappend("WMAKER_BIN_NAME=", argv[0]);
+    str = wstrconcat("WMAKER_BIN_NAME=", argv[0]);
     putenv(str);
 
     ArgCount = argc;
@@ -620,7 +621,8 @@ main(int argc, char **argv)
 		    exit(0);
 		}
 		Locale = argv[i];
-	    } else if (strcmp(argv[i], "-display")==0) {
+	    } else if (strcmp(argv[i], "-display")==0
+	    	       || strcmp(argv[i], "--display")==0) {
 		i++;
 		if (i>=argc) {
 		    wwarning(_("too few arguments for %s"), argv[i-1]);
@@ -642,6 +644,10 @@ main(int argc, char **argv)
 		       || strcmp(argv[i], "--static")==0) {
 
 		wPreferences.flags.noupdates = 1;
+            } else if (strcmp(argv[i], "-nopolling")==0
+                       || strcmp(argv[i], "--no-polling")==0) {
+
+		wPreferences.flags.nopolling = 1;
 #ifdef XSMP_ENABLED
 	    } else if (strcmp(argv[i], "-clientid")==0
 		       || strcmp(argv[i], "-restore")==0) {
