@@ -33,7 +33,7 @@
 #include <math.h>
 #include <time.h>
 
-#include <WUtil.h>
+#include <WINGs/WUtil.h>
 #include <wraster.h>
 
 
@@ -164,7 +164,7 @@ MakeCPPArgs(char *path)
 	    *buf = 0; /* trunc filename */
 	    putdef(line, " -I", tmp);
 	}
-	free(tmp);
+	wfree(tmp);
     }
 
 
@@ -202,64 +202,6 @@ MakeCPPArgs(char *path)
 
 
 
-
-WWindow*
-NextToFocusAfter(WWindow *wwin)
-{
-    WWindow *tmp = wwin->prev;
-
-    while (tmp) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-	    return tmp;
-	}
-	tmp = tmp->prev;
-    }
-
-    tmp = wwin;
-    /* start over from the beginning of the list */
-    while (tmp->next)
-	tmp = tmp->next;
-
-    while (tmp && tmp != wwin) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-	    return tmp;
-	}
-	tmp = tmp->prev;
-    }
-
-    return wwin;
-}
-
-
-WWindow*
-NextToFocusBefore(WWindow *wwin)
-{
-    WWindow *tmp = wwin->next;
-
-    while (tmp) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-	    return tmp;
-	}
-	tmp = tmp->next;
-    }
-
-    /* start over from the beginning of the list */
-    tmp = wwin;
-    while (tmp->prev)
-	tmp = tmp->prev;
-
-    while (tmp && tmp != wwin) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-
-	    return tmp;
-	}
-	tmp = tmp->next;
-    }
-
-    return wwin;
-}
-
-
 #if 0
 /*
  * Is win2 below win1?
@@ -295,7 +237,8 @@ isBelow(WWindow *win1, WWindow *win2)
  *  XFetchName Wrapper
  *
  */
-Bool wFetchName(dpy, win, winname)
+Bool
+wFetchName(dpy, win, winname)
 Display *dpy;
 Window win;
 char **winname;
@@ -339,7 +282,8 @@ char **winname;
  *
  */
 
-Bool wGetIconName(dpy, win, iconname)
+Bool
+wGetIconName(dpy, win, iconname)
 Display *dpy;
 Window win;
 char **iconname;
@@ -501,7 +445,7 @@ ShrinkString(WMFont *font, char *string, int width)
     if (pos) {
 	*pos = 0;
 	p = strlen(text);
-	w1 = WMWidthOfString(font, text, p);
+    w1 = WMWidthOfString(font, text, p);
 	if (w1 > width) {
 	    w1 = 0;
 	    p = 0;
@@ -519,7 +463,6 @@ ShrinkString(WMFont *font, char *string, int width)
     }
     strcat(text, "...");
     width -= WMWidthOfString(font, "...", 3);
-    
     pos = string;
     p1=0;
     p2=p;
@@ -560,199 +503,113 @@ FindImage(char *paths, char *file)
 }
 
 
-char*
-FlattenStringList(char **list, int count)
-{
-    int i, j;
-    char *flat_string, *wspace;
-
-    j = 0;
-    for (i=0; i<count; i++) {
-        if (list[i]!=NULL && list[i][0]!=0) {
-            j += strlen(list[i]);
-            if (strpbrk(list[i], " \t"))
-                j += 2;
-        }
-    }
-    
-    flat_string = malloc(j+count+1);
-    if (!flat_string) {
-	return NULL;
-    }
-
-    *flat_string = 0;
-    for (i=0; i<count; i++) {
-	if (list[i]!=NULL && list[i][0]!=0) {
-	    if (i>0)
-		strcat(flat_string, " ");
-            wspace = strpbrk(list[i], " \t");
-            if (wspace)
-                strcat(flat_string, "\"");
-	    strcat(flat_string, list[i]);
-            if (wspace)
-                strcat(flat_string, "\"");
-	}
-    }
-
-    return flat_string;
-}
-
-
-
-/*
- *----------------------------------------------------------------------
- * ParseCommand --
- * 	Divides a command line into a argv/argc pair.
- *---------------------------------------------------------------------- 
- */
-#define PRC_ALPHA	0
-#define PRC_BLANK	1
-#define PRC_ESCAPE	2
-#define PRC_DQUOTE	3
-#define PRC_EOS		4
-#define PRC_SQUOTE	5
-
-typedef struct {
-    short nstate;
-    short output;
-} DFA;
-
-
-static DFA mtable[9][6] = {
-    {{3,1},{0,0},{4,0},{1,0},{8,0},{6,0}},
-    {{1,1},{1,1},{2,0},{3,0},{5,0},{1,1}},
-    {{1,1},{1,1},{1,1},{1,1},{5,0},{1,1}},
-    {{3,1},{5,0},{4,0},{1,0},{5,0},{6,0}},
-    {{3,1},{3,1},{3,1},{3,1},{5,0},{3,1}},
-    {{-1,-1},{0,0},{0,0},{0,0},{0,0},{0,0}}, /* final state */
-    {{6,1},{6,1},{7,0},{6,1},{5,0},{3,0}},
-    {{6,1},{6,1},{6,1},{6,1},{5,0},{6,1}},
-    {{-1,-1},{0,0},{0,0},{0,0},{0,0},{0,0}}, /* final state */
-};
-
-char*
-next_token(char *word, char **next)
-{
-    char *ptr;
-    char *ret, *t;
-    int state, ctype;
-
-    t = ret = wmalloc(strlen(word)+1);
-    ptr = word;
-    
-    state = 0;
-    *t = 0;
-    while (1) {
-	if (*ptr==0) 
-	    ctype = PRC_EOS;
-	else if (*ptr=='\\')
-	    ctype = PRC_ESCAPE;
-	else if (*ptr=='"')
-	    ctype = PRC_DQUOTE;
-	else if (*ptr=='\'')
-	    ctype = PRC_SQUOTE;
-	else if (*ptr==' ' || *ptr=='\t')
-	    ctype = PRC_BLANK;
-	else
-	    ctype = PRC_ALPHA;
-
-	if (mtable[state][ctype].output) {
-	    *t = *ptr; t++;
-	    *t = 0;
-	}
-	state = mtable[state][ctype].nstate;
-	ptr++;
-	if (mtable[state][0].output<0) {
-	    break;
-	}
-    }
-
-    if (*ret==0)
-	t = NULL;
-    else
-	t = wstrdup(ret);
-
-    free(ret);
-    
-    if (ctype==PRC_EOS)
-	*next = NULL;
-    else
-	*next = ptr;
-    
-    return t;
-}
-
-
-void
-ParseCommand(char *command, char ***argv, int *argc)
-{
-    WMBag *bag = WMCreateBag(4);
-    char *token, *line;
-    int count, j;
-
-    line = command;
-    do {
-	token = next_token(line, &line);
-	if (token) {
-	    WMPutInBag(bag, token);
-	}
-    } while (token!=NULL && line!=NULL);
-
-    count = WMGetBagItemCount(bag);
-    *argv = wmalloc(sizeof(char*)*count);
-    for (j = 0; j < count; j++) {
-	(*argv)[j] = WMGetFromBag(bag, j);
-    }
-    *argc = count;
-
-    WMFreeBag(bag);
-}
-
-#if 0
 static void
-timeup(void *foo)
+timeoutHandler(void *data)
 {
-    *(int*)foo=1;
+    *(int*)data = 1;
 }
-#endif
+
+
+static char*
+getTextSelection(WScreen *screen, Atom selection)
+{
+    int buffer = -1;
+    
+    switch (selection) {
+     case XA_CUT_BUFFER0:
+	buffer = 0;
+	break;
+     case XA_CUT_BUFFER1:
+	buffer = 1;
+	break;
+     case XA_CUT_BUFFER2:
+	buffer = 2;
+	break;
+     case XA_CUT_BUFFER3:
+	buffer = 3;
+	break;
+     case XA_CUT_BUFFER4:
+	buffer = 4;
+	break;
+     case XA_CUT_BUFFER5:
+	buffer = 5;
+	break;
+     case XA_CUT_BUFFER6:
+	buffer = 6;
+	break;
+     case XA_CUT_BUFFER7:
+	buffer = 7;
+	break;
+    }
+    if (buffer >= 0) {
+	char *data;
+	int size;
+
+	data = XFetchBuffer(dpy, &size, buffer);
+
+	return data;
+    } else {
+	char *data;
+	int bits;
+	Atom rtype;
+	unsigned long len, bytes;
+	WMHandlerID timer;
+	int timeout = 0;
+	XEvent ev;
+	static Atom clipboard = 0;
+	
+	if (!clipboard) 
+	    clipboard = XInternAtom(dpy, "CLIPBOARD", False);
+	
+	XDeleteProperty(dpy, screen->info_window, clipboard);
+	
+	XConvertSelection(dpy, selection, XA_STRING,
+			  clipboard, screen->info_window,
+			  CurrentTime);
+	
+	timer = WMAddTimerHandler(1000, timeoutHandler, &timeout);
+	
+	while (!XCheckTypedWindowEvent(dpy, screen->info_window,
+				       SelectionNotify, &ev) && !timeout);
+	
+	if (!timeout) {
+	    WMDeleteTimerHandler(timer);
+	} else {
+	    wwarning("selection retrieval timed out");
+	    return NULL;
+	}
+
+	/* nobody owns the selection or the current owner has
+	 * nothing to do with what we need */
+	if (ev.xselection.property == None) {
+	    return NULL;
+	}
+
+	if (XGetWindowProperty(dpy, screen->info_window,
+			       clipboard, 0, 1024,
+			       False, XA_STRING, &rtype, &bits, &len,
+			       &bytes, (unsigned char**)&data)!=Success) {
+	    return NULL;
+	}
+	if (rtype!=XA_STRING || bits!=8) {
+	    wwarning("invalid data in text selection");
+	    if (data)
+		XFree(data);
+	    return NULL;
+	}
+	return data;
+    }
+}
+
 static char*
 getselection(WScreen *scr)
 {
     char *tmp;
-    extern char *W_GetTextSelection(); /* in WINGs */
     
-    tmp = W_GetTextSelection(scr->wmscreen, XA_PRIMARY);
+    tmp = getTextSelection(scr, XA_PRIMARY);
     if (!tmp)
-	tmp = W_GetTextSelection(scr->wmscreen, XA_CUT_BUFFER0);
+	tmp = getTextSelection(scr, XA_CUT_BUFFER0);
     return tmp;
-
-#if 0
-    XEvent event;
-    int timeover=0;
-    WMHandlerID *id;
-    
-#ifdef DEBUG
-    puts("getting selection");
-#endif
-    RequestSelection(dpy, scr->no_focus_win, LastTimestamp);
-    /* timeout on 1 sec. */
-    id = WMAddTimerHandler(1000, timeup, &timeover);
-    while (!timeover) {
-	WMNextEvent(dpy, &event);
-	if (event.type == SelectionNotify 
-	    && event.xany.window==scr->no_focus_win) {
-	    WMDeleteTimerHandler(id);
-#ifdef DEBUG
-	    puts("selection ok");
-#endif
-	    return GetSelection(dpy, scr->no_focus_win);
-	} else {
-	    WMHandleEvent(&event);
-	}
-    }
-    wwarning(_("selection timed-out"));
-    return NULL;
-#endif
 }
 
 
@@ -872,7 +729,7 @@ get_dnd_selection(WScreen *scr)
 	return NULL;
     }
 
-    flat_string = FlattenStringList(list, count);
+    flat_string = wtokenjoin(list, count);
     if (!flat_string) {
 	wwarning(_("out of memory while getting data from DND drop"));
     }
@@ -1023,9 +880,9 @@ ExpandOptions(WScreen *scr, char *cmdline)
 #if defined(OFFIX_DND) || defined(XDND)
 	     case 'd':
 #ifdef XDND
-        if(scr->xdestring) {
-            dropped_thing = wstrdup(scr->xdestring);
-        }
+		if(scr->xdestring) {
+		    dropped_thing = wstrdup(scr->xdestring);
+		}
 #endif
 		if (!dropped_thing) {
 		    dropped_thing = get_dnd_selection(scr);
@@ -1066,6 +923,7 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		strcat(out,selection);
 		optr+=slen;
 		break;
+
 	     default:
 		out[optr++]='%';
 		out[optr++]=cmdline[ptr];
@@ -1080,7 +938,7 @@ ExpandOptions(WScreen *scr, char *cmdline)
     return out;
     
     error:
-    free(out);
+    wfree(out);
     if (selection)
       XFree(selection);
     return NULL;
@@ -1164,18 +1022,6 @@ keysymToString(KeySym keysym, unsigned int state)
 }
 #endif
 
-static char *
-appendrealloc(char *a, char *b)
-{
-    if (a == NULL)
-	return wstrdup(b);
-    else {
-	char *c = wstrappend(a, b);
-	free(a);
-	return c;
-    }
-}
-
 
 char*
 GetShortcutString(char *text)
@@ -1202,33 +1048,33 @@ GetShortcutString(char *text)
 	modmask |= mod;
 	
 	if (strcasecmp(text, "Meta")==0) {
-	    buffer = appendrealloc(buffer, "M+");
+	    buffer = wstrappend(buffer, "M+");
 	} else if (strcasecmp(text, "Alt")==0) {
-	    buffer = appendrealloc(buffer, "A+");
+	    buffer = wstrappend(buffer, "A+");
 	} else if (strcasecmp(text, "Shift")==0) {
-	    buffer = appendrealloc(buffer, "Sh+");
+	    buffer = wstrappend(buffer, "Sh+");
 	} else if (strcasecmp(text, "Mod1")==0) {
-	    buffer = appendrealloc(buffer, "M1+"); 
+	    buffer = wstrappend(buffer, "M1+"); 
 	} else if (strcasecmp(text, "Mod2")==0) {
-	    buffer = appendrealloc(buffer, "M2+"); 
+	    buffer = wstrappend(buffer, "M2+"); 
 	} else if (strcasecmp(text, "Mod3")==0) {
-	    buffer = appendrealloc(buffer, "M3+");
+	    buffer = wstrappend(buffer, "M3+");
 	} else if (strcasecmp(text, "Mod4")==0) {
-	    buffer = appendrealloc(buffer, "M4+"); 
+	    buffer = wstrappend(buffer, "M4+"); 
 	} else if (strcasecmp(text, "Mod5")==0) {
-	    buffer = appendrealloc(buffer, "M5+"); 
+	    buffer = wstrappend(buffer, "M5+"); 
 	} else if (strcasecmp(text, "Control")==0) {
 	    control = 1;
 	} else {
-	    buffer = appendrealloc(buffer, text);
+	    buffer = wstrappend(buffer, text);
 	}
 	text = k+1;
     }
 
     if (control) {
-	buffer = appendrealloc(buffer, "^");
+	buffer = wstrappend(buffer, "^");
     }
-    buffer = appendrealloc(buffer, text);
+    buffer = wstrappend(buffer, text);
     
     /* get key */
 /*    ksym = XStringToKeysym(text);
@@ -1236,7 +1082,7 @@ GetShortcutString(char *text)
     puts(tmp);
     buffer = wstrappend(buffer, tmp);
 */
-    free(tmp);
+    wfree(tmp);
 
     return buffer;
 }
@@ -1284,14 +1130,14 @@ EscapeWM_CLASS(char *name, char *class)
     if (ename && eclass) {
 	ret = wmalloc(strlen(ename)+strlen(eclass)+4);
 	sprintf(ret, "%s.%s", ename, eclass);
-	free(ename);
-	free(eclass);
+	wfree(ename);
+	wfree(eclass);
     } else if (ename) {
 	ret = wstrdup(ename);
-	free(ename);
+	wfree(ename);
     } else {
 	ret = wstrdup(eclass);
-	free(eclass);
+	wfree(eclass);
     }
 
     return ret;
@@ -1359,11 +1205,11 @@ UnescapeWM_CLASS(char *str, char **name, char **class)
     (*class)[k] = 0;
     
     if (!*name) {
-	free(*name);
+	wfree(*name);
 	*name = NULL;
     }
     if (!*class) {
-	free(*class);
+	wfree(*class);
 	*class = NULL;
     }
 }
@@ -1400,5 +1246,5 @@ SendHelperMessage(WScreen *scr, char type, int workspace, char *msg)
     if (write(scr->helper_fd, buffer, len+4) < 0) {
 	wsyserror(_("could not send message to background image helper"));
     }
-    free(buffer);
+    wfree(buffer);
 }
