@@ -25,7 +25,6 @@
 
 #ifndef LITE
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -55,7 +54,6 @@
 #include "xmodifier.h"
 #include <proplist.h>
 
-#include "list.h"
 
 
 extern char *Locale;
@@ -167,7 +165,7 @@ execCommand(WMenu *menu, WMenuEntry *entry)
 
     if (cmdline) {
 	ExecuteShellCommand(menu->frame->screen_ptr, cmdline);
-	free(cmdline);
+	wfree(cmdline);
     }
     XUngrabPointer(dpy, CurrentTime);
     XSync(dpy, 0);
@@ -457,7 +455,7 @@ removeShortcutsForMenu(WMenu *menu)
     while (ptr!=NULL) {
 	tmp = ptr->next;
 	if (ptr->menu == menu) {
-	    free(ptr);
+	    wfree(ptr);
 	} else {
 	    ptr->next = newList;
 	    newList = ptr;
@@ -492,7 +490,7 @@ addShortcut(char *file, char *shortcutDefinition, WMenu *menu,
 	mod = wXModifierFromKey(b);
 	if (mod<0) {
 	    wwarning(_("%s:invalid key modifier \"%s\""), file, b);
-	    free(ptr);
+	    wfree(ptr);
 	    return False;
 	}
 	ptr->modifier |= mod;
@@ -506,7 +504,7 @@ addShortcut(char *file, char *shortcutDefinition, WMenu *menu,
     if (ksym==NoSymbol) {
 	wwarning(_("%s:invalid kbd shortcut specification \"%s\" for entry %s"),
 		 file, shortcutDefinition, entry->text);
-	free(ptr);
+	wfree(ptr);
 	return False;
     }
     
@@ -514,7 +512,7 @@ addShortcut(char *file, char *shortcutDefinition, WMenu *menu,
     if (ptr->keycode==0) {
 	wwarning(_("%s:invalid key in shortcut \"%s\" for entry %s"), file,
 		 shortcutDefinition, entry->text);
-	free(ptr);
+	wfree(ptr);
 	return False;
     }
 
@@ -603,7 +601,7 @@ static void
 separateCommand(char *line, char ***file, char **command)
 {
     char *token, *tmp = line;
-    LinkedList *list = NULL;
+    WMBag *bag = WMCreateBag(4);
     int count, i;
 
     *file = NULL;
@@ -618,20 +616,19 @@ separateCommand(char *line, char ***file, char **command)
                     wwarning(_("%s: missing command"), line);
                 break;
             }
-            list = list_cons(token, list);
+	    WMPutInBag(bag, token);
         }
     } while (token!=NULL && tmp!=NULL);
 
-    count = list_length(list);
+    count = WMGetBagItemCount(bag);
     if (count>0) {
         *file = wmalloc(sizeof(char*)*(count+1));
-        i = count;
         (*file)[count] = NULL;
-        while (list!=NULL) {
-            (*file)[--i] = list->head;
-            list_remove_head(&list);
+        for (i = 0; i < count; i++) {
+            (*file)[i] = WMGetFromBag(bag, i);
         }
     }
+    WMFreeBag(bag);
 }
 
 
@@ -647,7 +644,7 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
     time_t last=0;
     
     separateCommand((char*)entry->clientdata, &path, &cmd);
-    if (!path || *path==NULL || **path==0) {
+    if (path == NULL || *path==NULL || **path==0) {
 	wwarning(_("invalid OPEN_MENU specification: %s"),
 		 (char*)entry->clientdata);
 	return;
@@ -680,10 +677,10 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
 	    }
 	    
 	    tmp = wexpandpath(path[i]);
-	    free(path[i]);
+	    wfree(path[i]);
 	    lpath = getLocalizedMenuFile(tmp);
 	    if (lpath) {
-		free(tmp);
+		wfree(tmp);
 		path[i] = lpath;
 		lpath = NULL;
 	    } else {
@@ -744,10 +741,10 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
 finish:
     i = 0;
     while (path[i]!=NULL)
-        free(path[i++]);
-    free(path);
+        wfree(path[i++]);
+    wfree(path);
     if (cmd)
-	free(cmd);
+	wfree(cmd);
 }
 
 
@@ -809,7 +806,7 @@ addMenuEntry(WMenu *menu, char *title, char *shortcut, char *command,
 		     file_name, command);
 	else {
 	    entry = wMenuAddCallback(menu, title, execCommand, 
-				     wstrappend("exec ", params));
+				     wstrconcat("exec ", params));
 	    entry->free_cdata = free;
 	    shortcutOk = True;
 	}
@@ -1108,7 +1105,7 @@ readMenuFile(WScreen *scr, char *file_name)
 	    wwarning(_("could not make arguments for menu file preprocessor"));
 	} else {
 	    sprintf(command, "%s %s %s", CPP_PATH, args, file_name);
-	    free(args);
+	    wfree(args);
 	    file = popen(command, "r");
 	    if (!file) {
 		wsyserror(_("%s:could not open/preprocess menu file"), 
@@ -1208,7 +1205,7 @@ readMenuPipe(WScreen *scr, char **file_name)
        } else {
            sprintf(command, "%s | %s %s", filename, CPP_PATH, args);
 
-           free(args);
+           wfree(args);
            file = popen(command, "r");
            if (!file) {
                wsyserror(_("%s:could not open/preprocess menu file"), filename);
@@ -1270,9 +1267,12 @@ typedef struct {
 
 
 static int
-myCompare(dir_data *d1, dir_data *d2)
+myCompare(const void *d1, const void *d2)
 {
-    return strcmp(d1->name, d2->name);
+    dir_data *p1 = *(dir_data**)d1;
+    dir_data *p2 = *(dir_data**)d2;
+
+    return strcmp(p1->name, p2->name);
 }
 
 
@@ -1304,11 +1304,15 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
     struct stat stat_buf;
     WMenu *menu=NULL;
     char *buffer;
-    LinkedList *dirs = NULL, *files = NULL;
+    WMBag *dirs = NULL, *files = NULL;
     int length, i, have_space=0;
     dir_data *data;
     int stripExtension = 0;
 
+    
+    dirs = WMCreateBag(16);
+    files = WMCreateBag(16);
+    
     i=0;
     while (path[i]!=NULL) {
 	if (strcmp(path[i], "-noext")==0) {
@@ -1361,7 +1365,7 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 			data->name = wstrdup(dentry->d_name);
 			data->index = i;
 
-                        list_insert_sorted(data, &dirs, (int(*)())myCompare);
+			WMPutInBag(dirs, data);
                     }
                 } else if (S_ISREG(stat_buf.st_mode) || isFilePack) {
                     /* Hack because access always returns X_OK success for user root */
@@ -1374,29 +1378,37 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 			data->name = wstrdup(dentry->d_name);
 			data->index = i;
 
-                        list_insert_sorted(data, &files, (int(*)())myCompare);
+			WMPutInBag(files, data);
                     }
                 }
             }
-            free(buffer);
+            wfree(buffer);
         }
 
         closedir(dir);
         i++;
     }
-
-    if (!dirs && !files)
+    
+    if (!WMGetBagItemCount(dirs) && !WMGetBagItemCount(files)) {
+	WMFreeBag(dirs);
+	WMFreeBag(files);
         return NULL;
+    }
+
+    WMSortBag(dirs, myCompare);
+    WMSortBag(files, myCompare);
 
     menu = wMenuCreate(scr, title, False);
     menu->on_destroy = removeShortcutsForMenu;
-    
-    while (dirs != NULL) {
+
+    for (i = 0; i < WMGetBagItemCount(dirs); i++) {
         /* New directory. Use same OPEN_MENU command that was used
          * for the current directory. */
-        dir_data *d = (dir_data*)dirs->head;
+        dir_data *d = (dir_data*)WMGetFromBag(dirs, i);
 
         length = strlen(path[d->index])+strlen(d->name)+6;
+	if (stripExtension)
+	    length += 7;
         if (command)
             length += strlen(command) + 6;
         buffer = malloc(length);
@@ -1405,16 +1417,18 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
                      path[d->index]);
             break;
         }
+	
+	buffer[0] = '\0';
+        if (stripExtension)
+            strcat(buffer, "-noext ");
 
         have_space = strchr(path[d->index], ' ')!=NULL ||
                      strchr(d->name, ' ')!=NULL;
-        if (have_space) {
-            buffer[0] = '"';
-            buffer[1] = 0;
-            strcat(buffer, path[d->index]);
-        } else {
-            strcpy(buffer, path[d->index]);
-        }
+
+	if (have_space)
+            strcat(buffer, "\"");
+	strcat(buffer, path[d->index]);
+
         strcat(buffer, "/");
         strcat(buffer, d->name);
         if (have_space)
@@ -1426,24 +1440,21 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 
         addMenuEntry(menu, d->name, NULL, "OPEN_MENU", buffer, path[d->index]);
 
-        free(buffer);
-        if (dirs->head) {
-            if (d->name)
-                free(d->name);
-            free(dirs->head);
-        }
-        list_remove_head(&dirs);
+        wfree(buffer);
+	if (d->name)
+	    wfree(d->name);
+	wfree(d);
     }
 
-    while (files != NULL) {
+    for (i = 0; i < WMGetBagItemCount(files); i++) {
         /* executable: add as entry */
-        dir_data *f = (dir_data*) files->head;;
+        dir_data *f = (dir_data*)WMGetFromBag(files, i);
 
         length = strlen(path[f->index])+strlen(f->name)+6;
         if (command)
             length += strlen(command);
 
-        buffer = wmalloc(length);
+        buffer = malloc(length);
         if (!buffer) {
             wsyserror(_("out of memory while constructing directory menu %s"),
                      path[f->index]);
@@ -1479,14 +1490,14 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 	}
         addMenuEntry(menu, f->name, NULL, "SHEXEC", buffer, path[f->index]);
 
-        free(buffer);
-        if (files->head) {
-            if (f->name)
-                free(f->name);
-            free(files->head);
-        }
-        list_remove_head(&files);
+        wfree(buffer);
+	if (f->name)
+	    wfree(f->name);
+	wfree(f);
     }
+    
+    WMFreeBag(files);
+    WMFreeBag(dirs);
 
     return menu;
 }
@@ -1550,14 +1561,14 @@ configureMenu(WScreen *scr, proplist_t definition)
 	if (!path) {
 	    wsyserror(_("could not find menu file \"%s\" referenced in WMRootMenu"),
 		     tmp);
-	    free(tmp);
+	    wfree(tmp);
 	    return NULL;
 	}
 
 	if (stat(path, &stat_buf)<0) {
 	    wsyserror(_("could not access menu \"%s\" referenced in WMRootMenu"), path);
-	    free(path);
-	    free(tmp);
+	    wfree(path);
+	    wfree(tmp);
 	    return NULL;
 	}
 
@@ -1576,8 +1587,8 @@ configureMenu(WScreen *scr, proplist_t definition)
 	} else {
 	    menu = NULL;
 	}
-	free(path);
-	free(tmp);
+	wfree(path);
+	wfree(tmp);
 
 	return menu;
     }
@@ -1591,7 +1602,7 @@ configureMenu(WScreen *scr, proplist_t definition)
 	tmp = PLGetDescription(elem);
 	wwarning(_("%s:format error in root menu configuration \"%s\""), 
 		 "WMRootMenu", tmp);
-	free(tmp);
+	wfree(tmp);
 	return NULL;
     }
     mtitle = PLGetString(elem);
@@ -1667,7 +1678,7 @@ configureMenu(WScreen *scr, proplist_t definition)
 	tmp = PLGetDescription(elem);
 	wwarning(_("%s:format error in root menu configuration \"%s\""), 
 		 "WMRootMenu", tmp);
-	free(tmp);
+	wfree(tmp);
     }
     
     return menu;
