@@ -91,7 +91,6 @@
 
 #include "dock.h"
 
-#include "list.h"
 
 #include <proplist.h>
 
@@ -126,6 +125,7 @@ static proplist_t sShaded;
 static proplist_t sMiniaturized;
 static proplist_t sHidden;
 static proplist_t sGeometry;
+static proplist_t sShortcutMask;
 
 static proplist_t sDock;
 
@@ -148,6 +148,7 @@ make_keys()
     sHidden = PLMakeString("Hidden");
     sGeometry = PLMakeString("Geometry");
     sDock = PLMakeString("Dock");
+    sShortcutMask = PLMakeString("ShortcutMask");
 
     sYes = PLMakeString("Yes");
     sNo = PLMakeString("No");
@@ -186,6 +187,24 @@ getBool(proplist_t value)
 }
 
 
+static unsigned
+getInt(proplist_t value)
+{
+    char *val;
+    unsigned n;
+    
+    if (!PLIsString(value))
+	return 0;
+    val = PLGetString(value);
+    if (!val)
+	return 0;
+    if (sscanf(val, "%u", &n) != 1)
+	return 0;
+
+    return n;
+}
+
+
 
 static proplist_t
 makeWindowState(WWindow *wwin, WApplication *wapp)
@@ -194,10 +213,12 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
     Window win;
     int argc;
     char **argv;
+    int i;
+    unsigned mask;
     char *class, *instance, *command=NULL, buffer[256];
     proplist_t win_state, cmd, name, workspace;
     proplist_t shaded, miniaturized, hidden, geometry;
-    proplist_t dock;
+    proplist_t dock, shortcut;
 
     if (wwin->main_window!=None && wwin->main_window!=wwin->client_win)
         win = wwin->main_window;
@@ -205,7 +226,7 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
         win = wwin->client_win;
 
     if (XGetCommand(dpy, win, &argv, &argc) && argc>0) {
-        command = FlattenStringList(argv, argc);
+        command = wtokenjoin(argv, argc);
         XFreeStringList(argv);
     }
     if (!command)
@@ -232,6 +253,16 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
         sprintf(buffer, "%ix%i+%i+%i", wwin->client.width, wwin->client.height,
 		wwin->frame_x, wwin->frame_y);
         geometry = PLMakeString(buffer);
+	
+	for (mask = 0, i = 0; i < MAX_WINDOW_SHORTCUTS; i++) {
+	    if (scr->shortcutWindows[i] != NULL &&
+		WMGetFirstInBag(scr->shortcutWindows[i], wwin) != WBNotFound) {
+		mask |= 1<<i;
+	    }
+	}
+
+	sprintf(buffer, "%u", mask);
+	shortcut = PLMakeString(buffer);
 
         win_state = PLMakeDictionaryFromEntries(sName, name,
                                                 sCommand, cmd,
@@ -239,6 +270,7 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
                                                 sShaded, shaded,
                                                 sMiniaturized, miniaturized,
                                                 sHidden, hidden,
+						sShortcutMask, shortcut,
                                                 sGeometry, geometry,
                                                 NULL);
 
@@ -246,6 +278,7 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
         PLRelease(cmd);
         PLRelease(workspace);
         PLRelease(geometry);
+	PLRelease(shortcut);
         if (wapp && wapp->app_icon && wapp->app_icon->dock) {
             int i;
             char *name;
@@ -269,7 +302,7 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
 
     if (instance) XFree(instance);
     if (class) XFree(class);
-    if (command) free(command);
+    if (command) wfree(command);
 
     return win_state;
 }
@@ -281,7 +314,7 @@ wSessionSaveState(WScreen *scr)
     WWindow *wwin = scr->focused_window;
     proplist_t win_info, wks;
     proplist_t list=NULL;
-    LinkedList *wapp_list=NULL;
+    WMBag *wapp_list=NULL;
 
 
     make_keys();
@@ -294,10 +327,13 @@ wSessionSaveState(WScreen *scr)
 
     list = PLMakeArrayFromElements(NULL);
 
+    wapp_list = WMCreateBag(16);
+
     while (wwin) {
         WApplication *wapp=wApplicationOf(wwin->main_window);
 
-        if (wwin->transient_for==None && list_find(wapp_list, wapp)==NULL
+        if (wwin->transient_for==None 
+	    && WMGetFirstInBag(wapp_list, wapp)==WBNotFound
 	    && !WFLAGP(wwin, dont_save_session)) {
             /* A entry for this application was not yet saved. Save one. */
             if ((win_info = makeWindowState(wwin, wapp))!=NULL) {
@@ -308,7 +344,7 @@ wSessionSaveState(WScreen *scr)
                  * application list, so no multiple entries for the same
                  * application are saved.
                  */
-                wapp_list = list_cons(wapp, wapp_list);
+		WMPutInBag(wapp_list, wapp);
             }
         }
         wwin = wwin->prev;
@@ -321,7 +357,7 @@ wSessionSaveState(WScreen *scr)
     PLInsertDictionaryEntry(scr->session_state, sWorkspace, wks);
     PLRelease(wks);
 
-    list_free(wapp_list);
+    WMFreeBag(wapp_list);
 }
 
 
@@ -345,7 +381,7 @@ execCommand(WScreen *scr, char *command, char *host)
     char **argv;
     int argc;
 
-    ParseCommand(command, &argv, &argc);
+    wtokensplit(command, &argv, &argc);
 
     if (argv==NULL) {
         return 0;
@@ -368,8 +404,8 @@ execCommand(WScreen *scr, char *command, char *host)
         exit(111);
     }
     while (argc > 0)
-        free(argv[--argc]);
-    free(argv);
+        wfree(argv[--argc]);
+    wfree(argv);
     return pid;
 }
 
@@ -380,6 +416,7 @@ getWindowState(WScreen *scr, proplist_t win_state)
     WSavedState *state = wmalloc(sizeof(WSavedState));
     proplist_t value;
     char *tmp;
+    unsigned mask;
     int i;
 
     memset(state, 0, sizeof(WSavedState));
@@ -405,20 +442,19 @@ getWindowState(WScreen *scr, proplist_t win_state)
         state->miniaturized = getBool(value);
     if ((value = PLGetDictionaryEntry(win_state, sHidden))!=NULL)
         state->hidden = getBool(value);
-
+    if ((value = PLGetDictionaryEntry(win_state, sShortcutMask))!=NULL) {
+        mask = getInt(value);
+	state->window_shortcuts = mask;
+    }
+    
     value = PLGetDictionaryEntry(win_state, sGeometry);
     if (value && PLIsString(value)) {
-        if (sscanf(PLGetString(value), "%ix%i+%i+%i",
+        if (!(sscanf(PLGetString(value), "%ix%i+%i+%i",
                    &state->w, &state->h, &state->x, &state->y)==4 &&
-            (state->w>0 && state->h>0)) {
-            state->use_geometry = 1;
-        } else if (sscanf(PLGetString(value), "%i,%i,%i,%i",
-                   &state->x, &state->y, &state->w, &state->h)==4 &&
-            (state->w>0 && state->h>0)) { 
-	    /* TODO: remove redundant sscanf() in version 0.20.x */
-            state->use_geometry = 1;
-        }
-
+	      (state->w>0 && state->h>0))) {
+	    state->w = 0;
+	    state->h = 0;
+	}
     }
 
     return state;
@@ -508,7 +544,8 @@ wSessionRestoreState(WScreen *scr)
                 btn = dock->icon_array[j];
                 if (btn && SAME(instance, btn->wm_instance) &&
                     SAME(class, btn->wm_class) &&
-                    SAME(command, btn->command)) {
+                    SAME(command, btn->command) &&
+		    !btn->launching) {
                     found = 1;
                     break;
                 }
@@ -520,11 +557,11 @@ wSessionRestoreState(WScreen *scr)
         } else if ((pid = execCommand(scr, command, host)) > 0) {
             wWindowAddSavedState(instance, class, command, pid, state);
         } else {
-            free(state);
+            wfree(state);
         }
 
-        if (instance) free(instance);
-        if (class) free(class);
+        if (instance) wfree(instance);
+        if (class) wfree(class);
     }
     /* clean up */
     PLSetStringCmpHook(StringCompareHook);
@@ -1113,17 +1150,17 @@ fail:
     SmcSaveYourselfDone(smc_conn, ok);
     
     if (prop[0].vals)
-	free(prop[0].vals);
+	wfree(prop[0].vals);
     if (prop[1].vals)
-	free(prop[1].vals);
+	wfree(prop[1].vals);
     if (discardCmd)
-	free(discardCmd);
+	wfree(discardCmd);
 
     if (!ok) {
 	remove(statefile);
     }
     if (statefile)
-	free(statefile);
+	wfree(statefile);
 }
 
 
