@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <errno.h>
 #include <signal.h>
 #include <sys/wait.h>
 #ifdef __FreeBSD__
@@ -78,6 +79,10 @@ extern const char * const sys_siglist[];
 # define SA_RESTART 0
 #endif
 
+/* Just in case, for weirdo systems */
+#ifndef SA_NODEFER
+# define SA_NODEFER 0
+#endif
 
 /****** Global Variables ******/
 
@@ -273,6 +278,22 @@ handleSig(int sig)
 	    WMAddIdleHandler(delayedAction, NULL);
 	}
         return;
+    } else if (sig == SIGUSR2) {
+#ifdef SYS_SIGLIST_DECLARED
+        wwarning(_("got signal %i (%s) - rereading defaults\n"), sig, sys_siglist[sig]);
+#else
+        wwarning(_("got signal %i - rereading defaults\n"), sig);
+#endif
+
+        WCHANGE_STATE(WSTATE_NEED_REREAD);
+        /* setup idle handler, so that this will be handled when
+         * the select() is returned becaused of the signal, even if
+         * there are no X events in the queue */
+        if (!WDelayedActionSet) {
+            WDelayedActionSet = 1;
+            WMAddIdleHandler(delayedAction, NULL);
+	}
+        return;
     } else if (sig == SIGTERM || sig == SIGHUP) {
 #ifdef SYS_SIGLIST_DECLARED
         wwarning(_("got signal %i (%s) - exiting...\n"), sig, sys_siglist[sig]);
@@ -337,12 +358,12 @@ handleSig(int sig)
 
         if (crashAction == WMRestart) {
             /* we try to restart Window Maker */
-            wwarning(_("trying to restart Window Maker..."));
+            wmessage(_("trying to restart Window Maker..."));
             Restart(NULL, False);
             /* fallback to alternate window manager then */
         }
 
-    	wwarning(_("trying to start alternate window manager..."));
+    	wmessage(_("trying to start alternate window manager..."));
 
         Restart(FALLBACK_WINDOWMANAGER, False);
         Restart("fvwm", False);
@@ -377,8 +398,13 @@ buryChild(int foo)
     int status;
     
     /* R.I.P. */
-    pid = waitpid(-1, &status, WNOHANG);
-    if (pid>0) {
+    /* If 2 or more kids exit in a small time window, before this handler gets
+     * the chance to get invoked, the SIGCHLD signals will be merged and only
+     * one SIGCHLD signal will be sent to us. We use a while loop to get all
+     * exited child status because we can't count on the number of SIGCHLD
+     * signals to know exactly how many kids have exited. -Dan
+     */
+    while ((pid=waitpid(-1, &status, WNOHANG))>0 || (pid<0 && errno==EINTR)) {
 	NotifyDeadProcess(pid, WEXITSTATUS(status));
 	/* 
 	 * Make sure that the kid will be buried even if there are
@@ -387,7 +413,7 @@ buryChild(int foo)
 	if (!WDelayedActionSet) {
 	    WDelayedActionSet = 1;
 	    WMAddIdleHandler(delayedAction, NULL);
-	}
+        }
     }
 }
 
@@ -786,6 +812,7 @@ StartUp(Bool defaultScreenOnly)
      * -Dan */
     sig_action.sa_flags = SA_RESTART;
     sigaction(SIGUSR1, &sig_action, NULL);
+    sigaction(SIGUSR2, &sig_action, NULL);
 
     /* ignore dead pipe */
     sig_action.sa_handler = ignoreSig;
@@ -901,7 +928,7 @@ StartUp(Bool defaultScreenOnly)
 	    wScreen[j]->last_dock = wScreen[j]->dock;
 	
 	manageAllWindows(wScreen[j]);
-	
+		
 	/* restore saved menus */
 	wMenuRestoreState(wScreen[j]);
 	
@@ -930,7 +957,7 @@ StartUp(Bool defaultScreenOnly)
 	/* go to workspace where we were before restart */
 	if (ws_state) { 
 	    wWorkspaceForceChange(wScreen[j], ws_state->workspace);
-	    free(ws_state);
+	    wfree(ws_state);
 	} else {
 	    wSessionRestoreLastWorkspace(wScreen[j]);
 	}
@@ -945,7 +972,7 @@ StartUp(Bool defaultScreenOnly)
 	Exit(1);
     }
 
-    if (!wPreferences.flags.noupdates) {
+    if (!wPreferences.flags.nopolling && !wPreferences.flags.noupdates) {
 	/* setup defaults file polling */
 	WMAddTimerHandler(3000, wDefaultsCheckDomains, NULL);
     }
