@@ -1,5 +1,5 @@
 /* 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997 Shige Abe and 
  *		       Alfredo K. Kojima
@@ -23,6 +23,8 @@
 
 #include "wconfig.h"
 
+#ifndef LITE
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -64,20 +66,8 @@ focusWindow(WMenu *menu, WMenuEntry *entry)
 
     wwin = (WWindow*)entry->clientdata;
     scr = wwin->screen_ptr;
-    
-    wWorkspaceChange(menu->frame->screen_ptr, wwin->frame->workspace);
 
-    if (wwin->flags.shaded) {
-	wUnshadeWindow(wwin);
-    }
-    if (wwin->flags.hidden) {
-	wUnhideApplication(wApplicationOf(wwin->main_window), False, False);
-    } else if (wwin->flags.miniaturized) {
-	wDeiconifyWindow(wwin);
-    } else {
-	wSetFocusTo(menu->frame->screen_ptr, wwin);
-	wRaiseFrame(wwin->frame->core);
-    }
+    wMakeWindowVisible(wwin);
 
     x = wwin->frame_x;
     y = wwin->frame_y;
@@ -112,12 +102,12 @@ OpenSwitchMenu(WScreen *scr, int x, int y, int keyboard)
 		if (keyboard)
 		    wMenuMapAt(switchmenu, 0, 0, True);
 		else
-		    wMenuMapCopyAt(switchmenu, x-switchmenu->frame->core->width/2,
-			       y-switchmenu->frame->top_width/2);
+		    wMenuMapCopyAt(switchmenu, 
+				   x-switchmenu->frame->core->width/2, y);
 	    }
 	} else {
-	    wMenuMapAt(switchmenu, x-switchmenu->frame->core->width/2,
-		       y-switchmenu->frame->top_width/2, keyboard);
+	    wMenuMapAt(switchmenu, x-switchmenu->frame->core->width/2, y,
+		       keyboard);
 	}
 	return;
     }
@@ -135,16 +125,44 @@ OpenSwitchMenu(WScreen *scr, int x, int y, int keyboard)
     if (switchmenu) {
 	if (!switchmenu->flags.realized)
 	    wMenuRealize(switchmenu);
-	wMenuMapAt(switchmenu, x-switchmenu->frame->core->width/2,
-		   y-switchmenu->frame->top_width/2, keyboard);
+	wMenuMapAt(switchmenu, x-switchmenu->frame->core->width/2, y, 
+		   keyboard);
     }
 }
 
+
+static int
+menuIndexForWindow(WMenu *menu, WWindow *wwin, int old_pos)
+{
+    int idx;
+
+    if (menu->entry_no == 0)
+	return -1;
+
+#define WS(i)  ((WWindow*)menu->entries[i]->clientdata)->frame->workspace
+    if (old_pos >= 0) {
+	if (WS(old_pos) >= wwin->frame->workspace
+	    && (old_pos == 0 || WS(old_pos-1) <= wwin->frame->workspace)) {
+	    return old_pos;
+	}
+    }
+#undef WS
+
+    for (idx = 0; idx < menu->entry_no; idx++) {
+	WWindow *tw = (WWindow*)menu->entries[idx]->clientdata;
+
+	if (!IS_OMNIPRESENT(tw) 
+	    && tw->frame->workspace > wwin->frame->workspace) {
+	    break;
+	}
+    }
+
+    return idx;
+}
+
+
 /*
- *
  * Update switch menu
- *
- *
  */
 void 
 UpdateSwitchMenu(WScreen *scr, WWindow *wwin, int action)
@@ -164,31 +182,51 @@ UpdateSwitchMenu(WScreen *scr, WWindow *wwin, int action)
      *    2.  When a window is destroyed.
      *
      *    3.  When a window changes it's title.
+     * 	  4.  When a window changes its workspace.
      */
     if (action == ACTION_ADD) {
 	char *t;
-	if (wwin->flags.internal_window 
-	    || wwin->window_flags.skip_window_list)
-	  return;
+	int idx;
+
+	if (wwin->flags.internal_window || WFLAGP(wwin, skip_window_list))
+	    return;
 	    
 	if (wwin->frame->title)
 	  sprintf(title, "%s", wwin->frame->title);
 	else
 	  sprintf(title, "%s", DEF_WINDOW_TITLE);
 	t = ShrinkString(scr->menu_entry_font, title, MAX_WINDOWLIST_WIDTH);
-	entry = wMenuAddCallback(switchmenu, t, focusWindow, wwin);
-	free(t);
+
+	if (IS_OMNIPRESENT(wwin))
+	    idx = -1;
+	else {
+	    idx = menuIndexForWindow(switchmenu, wwin, -1);
+	}
+	
+	entry = wMenuInsertCallback(switchmenu, idx, t, focusWindow, wwin);
+	wfree(t);
 
 	entry->flags.indicator = 1;
 	entry->rtext = wmalloc(MAX_WORKSPACENAME_WIDTH+8);
-	if (wwin->window_flags.omnipresent)
+	if (IS_OMNIPRESENT(wwin))
 	    sprintf(entry->rtext, "[*]");
 	else
 	    sprintf(entry->rtext, "[%s]", 
 		    scr->workspaces[wwin->frame->workspace]->name);
-	
-	if (wwin->flags.focused)
+
+	if (wwin->flags.hidden) {
+	    entry->flags.indicator_type = MI_HIDDEN;
 	    entry->flags.indicator_on = 1;
+	} else if (wwin->flags.miniaturized) {
+	    entry->flags.indicator_type = MI_MINIWINDOW;
+	    entry->flags.indicator_on = 1;
+	} else if (wwin->flags.focused) {
+	    entry->flags.indicator_type = MI_DIAMOND;
+	    entry->flags.indicator_on = 1;
+	} else if (wwin->flags.shaded) {
+	    entry->flags.indicator_type = MI_SHADED;
+	    entry->flags.indicator_on = 1;
+	}
 
 	wMenuRealize(switchmenu);
 	checkVisibility = 1;
@@ -207,24 +245,54 @@ UpdateSwitchMenu(WScreen *scr, WWindow *wwin, int action)
 		    
 		 case ACTION_CHANGE:
 		    if (entry->text)
-		      free(entry->text);
+		      wfree(entry->text);
 		    
 		    if (wwin->frame->title)
 		      sprintf(title, "%s", wwin->frame->title);
 		    else
 		      sprintf(title, "%s", DEF_WINDOW_TITLE);
 
-		    t = ShrinkString(scr->menu_entry_font, title, 
-				     MAX_WINDOWLIST_WIDTH);
+		    t = ShrinkString(scr->menu_entry_font, title, MAX_WINDOWLIST_WIDTH);
 		    entry->text = t;
-		    /* fall through to update workspace number */
+
+		    wMenuRealize(switchmenu);
+		    checkVisibility = 1;
+		    break;
+
 		 case ACTION_CHANGE_WORKSPACE:
 		    if (entry->rtext) {
-			if (wwin->window_flags.omnipresent)
+			int idx = -1;
+			char *t, *rt;
+			int it, ion;
+			
+			if (IS_OMNIPRESENT(wwin)) {
 			    sprintf(entry->rtext, "[*]");
-			else
+			} else {
 			    sprintf(entry->rtext, "[%s]", 
 				    scr->workspaces[wwin->frame->workspace]->name);
+			}
+
+			rt = entry->rtext;
+			entry->rtext = NULL;
+			t = entry->text;
+			entry->text = NULL;
+			
+			it = entry->flags.indicator_type;
+			ion = entry->flags.indicator_on;
+
+			wMenuRemoveItem(switchmenu, i);
+
+			if (!IS_OMNIPRESENT(wwin) && idx < 0) {
+			    idx = menuIndexForWindow(switchmenu, wwin, i);
+			}
+
+			entry = wMenuInsertCallback(switchmenu, idx, t,
+						    focusWindow, wwin);
+			wfree(t);
+			entry->rtext = rt;
+			entry->flags.indicator = 1;
+			entry->flags.indicator_type = it;
+			entry->flags.indicator_on = ion;
 		    }
 		    wMenuRealize(switchmenu);
 		    checkVisibility = 1;
@@ -238,11 +306,13 @@ UpdateSwitchMenu(WScreen *scr, WWindow *wwin, int action)
 		    } else if (wwin->flags.miniaturized) {
 			entry->flags.indicator_type = MI_MINIWINDOW;
 			entry->flags.indicator_on = 1;
+		    } else if (wwin->flags.shaded && !wwin->flags.focused) {
+			entry->flags.indicator_type = MI_SHADED;
+			entry->flags.indicator_on = 1;
 		    } else {
 			entry->flags.indicator_on = wwin->flags.focused;
 			entry->flags.indicator_type = MI_DIAMOND;
 		    }
-		    wMenuPaint(switchmenu);
 		    break;
 		}
 		break;
@@ -278,8 +348,8 @@ UpdateSwitchMenuWorkspace(WScreen *scr, int workspace)
 	wwin = (WWindow*)menu->entries[i]->clientdata;
 
 	if (wwin->frame->workspace==workspace 
-	    && !wwin->window_flags.omnipresent) {
-	    if (wwin->window_flags.omnipresent)
+	    && !IS_OMNIPRESENT(wwin)) {
+	    if (IS_OMNIPRESENT(wwin))
 		sprintf(menu->entries[i]->rtext, "[*]");
 	    else
 		sprintf(menu->entries[i]->rtext, "[%s]", 
@@ -290,3 +360,6 @@ UpdateSwitchMenuWorkspace(WScreen *scr, int workspace)
     if (!menu->flags.realized)
 	wMenuRealize(menu);
 }
+
+
+#endif /* !LITE */
