@@ -1,5 +1,5 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -24,6 +24,12 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
+#ifdef TEXTURE_PLUGIN
+# ifdef HAVE_DLFCN_H
+#  include <dlfcn.h>
+# endif
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -38,21 +44,16 @@
 extern WPreferences wPreferences;
 
 
-static Pixmap renderTexture(WScreen *scr, int width, int height,
-			     WTexture *texture, int rel);
-
-
 static void bevelImage(RImage *image, int relief);
 
 
 
-WTexSolid *
+WTexSolid*
 wTextureMakeSolid(WScreen *scr, XColor *color)
 {
     WTexSolid *texture;
     int gcm;
     XGCValues gcv;
-    unsigned int r, g, b;
         
     texture = wmalloc(sizeof(WTexture));
 
@@ -68,36 +69,44 @@ wTextureMakeSolid(WScreen *scr, XColor *color)
 	texture->dim.red = 0x6185;
 	texture->dim.green = 0x6185;
 	texture->dim.blue = 0x6185;
-	texture->dark.red = 0;
-	texture->dark.green = 0;
-	texture->dark.blue = 0;
     } else {
-	r = (color->red * 17)/10;
-	g = (color->green * 17)/10;
-	b = (color->blue * 17)/10;
-	texture->light.red = (r > 0xffff ? 0xffff : r);
-	texture->light.green = (g > 0xffff ? 0xffff : g);
-	texture->light.blue = (b > 0xffff ? 0xffff : b);
-	r = (color->red * 5)/10;
-	g = (color->green * 5)/10;
-	b = (color->blue * 5)/10;
-	texture->dim.red = r;
-	texture->dim.green = g;
-	texture->dim.blue = b;
-	texture->dark.red = 0;
-	texture->dark.green = 0;
-	texture->dark.blue = 0;
+	RColor rgb;
+	RHSVColor hsv, hsv2;
+	int v;
+
+	rgb.red = color->red >> 8;
+	rgb.green = color->green >> 8;
+	rgb.blue = color->blue >> 8;
+	RRGBtoHSV(&rgb, &hsv);
+	RHSVtoRGB(&hsv, &rgb);
+	hsv2 = hsv;
+
+	v = hsv.value*16/10;
+	hsv.value = (v > 255 ? 255 : v);
+	RHSVtoRGB(&hsv, &rgb);
+	texture->light.red = rgb.red << 8;
+	texture->light.green = rgb.green << 8;
+	texture->light.blue = rgb.blue << 8;
+
+	hsv2.value = hsv2.value/2;
+	RHSVtoRGB(&hsv2, &rgb);
+	texture->dim.red = rgb.red << 8;
+	texture->dim.green = rgb.green << 8;
+	texture->dim.blue = rgb.blue << 8;
     }
+    texture->dark.red = 0;
+    texture->dark.green = 0;
+    texture->dark.blue = 0;
     XAllocColor(dpy, scr->w_colormap, &texture->light);
     XAllocColor(dpy, scr->w_colormap, &texture->dim);
     XAllocColor(dpy, scr->w_colormap, &texture->dark);
-    
+
     gcm = GCForeground|GCBackground|GCGraphicsExposures;
     gcv.graphics_exposures = False;
-    
+
     gcv.background = gcv.foreground = texture->light.pixel;
     texture->light_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
-    
+
     gcv.background = gcv.foreground = texture->dim.pixel;	
     texture->dim_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
     
@@ -134,11 +143,11 @@ wTextureDestroy(WScreen *scr, WTexture *texture)
 	return;
     }
 #endif
-    
+
     /* 
      * some stupid servers don't like white or black being freed...
      */
-#define CANFREE(c) (c!=scr->black_pixel && c!=scr->white_pixel)
+#define CANFREE(c) (c!=scr->black_pixel && c!=scr->white_pixel && c!=0)
     switch (texture->any.type) {
      case WTEX_SOLID:
 	XFreeGC(dpy, texture->solid.light_gc);
@@ -160,10 +169,30 @@ wTextureDestroy(WScreen *scr, WTexture *texture)
      case WTEX_MVGRADIENT:
      case WTEX_MDGRADIENT:
 	for (i=0; texture->mgradient.colors[i]!=NULL; i++) {
-	    free(texture->mgradient.colors[i]);
+	    wfree(texture->mgradient.colors[i]);
 	}
-	free(texture->mgradient.colors);
+	wfree(texture->mgradient.colors);
 	break;
+
+     case WTEX_THGRADIENT:
+     case WTEX_TVGRADIENT:
+     case WTEX_TDGRADIENT:	
+	RDestroyImage(texture->tgradient.pixmap);
+	break;
+
+#ifdef TEXTURE_PLUGIN
+     case WTEX_FUNCTION:
+#ifdef HAVE_DLFCN_H
+        if (texture->function.handle) {
+	    dlclose(texture->function.handle);
+	}
+#endif
+	for (i = 0; i < texture->function.argc; i++) {
+	    wfree(texture->function.argv[i]);
+	}
+	wfree(texture->function.argv);
+	break;
+#endif /* TEXTURE_PLUGIN */
     }
     if (CANFREE(texture->any.color.pixel))
       colors[count++] = texture->any.color.pixel;
@@ -172,20 +201,21 @@ wTextureDestroy(WScreen *scr, WTexture *texture)
 	
 	/* ignore error from buggy servers that don't know how
 	 * to do reference counting for colors. */
+	XSync(dpy,0);
 	oldhandler = XSetErrorHandler(dummyErrorHandler);
-	XFreeColors(dpy, scr->colormap, colors, count, 0);
+	XFreeColors(dpy, scr->w_colormap, colors, count, 0);
 	XSync(dpy,0);
 	XSetErrorHandler(oldhandler);
     }
     XFreeGC(dpy, texture->any.gc);
-    free(texture);
+    wfree(texture);
 #undef CANFREE    
 }
 
 
 
 WTexGradient*
-wTextureMakeGradient(WScreen *scr, int style, XColor *from, XColor *to)
+wTextureMakeGradient(WScreen *scr, int style, RColor *from, RColor *to)
 {
     WTexGradient *texture;
     XGCValues gcv;
@@ -199,9 +229,9 @@ wTextureMakeGradient(WScreen *scr, int style, XColor *from, XColor *to)
     texture->color1 = *from;
     texture->color2 = *to;
 
-    texture->normal.red = (from->red + to->red)/2;
-    texture->normal.green = (from->green + to->green)/2;
-    texture->normal.blue = (from->blue + to->blue)/2;
+    texture->normal.red = (from->red + to->red)<<7;
+    texture->normal.green = (from->green + to->green)<<7;
+    texture->normal.blue = (from->blue + to->blue)<<7;
 
     XAllocColor(dpy, scr->w_colormap, &texture->normal);
     gcv.background = gcv.foreground = texture->normal.pixel;
@@ -214,6 +244,45 @@ wTextureMakeGradient(WScreen *scr, int style, XColor *from, XColor *to)
 
 
 
+
+WTexIGradient*
+wTextureMakeIGradient(WScreen *scr, int thickness1, RColor colors1[2],
+		      int thickness2, RColor colors2[2])
+{
+    WTexIGradient *texture;
+    XGCValues gcv;
+    int i;
+
+    
+    texture = wmalloc(sizeof(WTexture));
+    memset(texture, 0, sizeof(WTexture));
+    texture->type = WTEX_IGRADIENT;
+    for (i = 0; i < 2; i++) {
+	texture->colors1[i] = colors1[i];
+	texture->colors2[i] = colors2[i];
+    }
+    texture->thickness1 = thickness1;
+    texture->thickness2 = thickness2;
+    if (thickness1 >= thickness2) {
+	texture->normal.red = (colors1[0].red + colors1[1].red)<<7;
+	texture->normal.green = (colors1[0].green + colors1[1].green)<<7;
+	texture->normal.blue = (colors1[0].blue + colors1[1].blue)<<7;
+    } else {
+	texture->normal.red = (colors2[0].red + colors2[1].red)<<7;
+	texture->normal.green = (colors2[0].green + colors2[1].green)<<7;
+	texture->normal.blue = (colors2[0].blue + colors2[1].blue)<<7;
+    }
+    XAllocColor(dpy, scr->w_colormap, &texture->normal);
+    gcv.background = gcv.foreground = texture->normal.pixel;
+    gcv.graphics_exposures = False;
+    texture->normal_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground
+				   |GCGraphicsExposures, &gcv);
+
+    return texture;
+}
+
+
+
 WTexMGradient*
 wTextureMakeMGradient(WScreen *scr, int style, RColor **colors)
 {
@@ -264,11 +333,11 @@ wTextureMakePixmap(WScreen *scr, int style, char *pixmap_file, XColor *color)
     image = RLoadImage(scr->rcontext, file, 0);
     if (!image) {
 	wwarning(_("could not load texture pixmap \"%s\":%s"), file,
-			RErrorString);
-	free(file);
+			RMessageForError(RErrorCode));
+	wfree(file);
 	return NULL;
     }
-    free(file);
+    wfree(file);
 
     texture = wmalloc(sizeof(WTexture));
     memset(texture, 0, sizeof(WTexture));
@@ -288,12 +357,114 @@ wTextureMakePixmap(WScreen *scr, int style, char *pixmap_file, XColor *color)
     return texture;
 }
 
+WTexTGradient*
+wTextureMakeTGradient(WScreen *scr, int style, RColor *from, RColor *to,
+		      char *pixmap_file, int opacity)
+{
+    WTexTGradient *texture;
+    XGCValues gcv;
+    RImage *image;
+    char *file;
+
+    file = FindImage(wPreferences.pixmap_path, pixmap_file);
+    if (!file) {
+        wwarning(_("image file \"%s\" used as texture could not be found."),
+                 pixmap_file);
+	return NULL;
+    }
+    image = RLoadImage(scr->rcontext, file, 0);
+    if (!image) {
+	wwarning(_("could not load texture pixmap \"%s\":%s"), file,
+		 RMessageForError(RErrorCode));
+	wfree(file);
+	return NULL;
+    }
+    wfree(file);
+
+    texture = wmalloc(sizeof(WTexture));
+    memset(texture, 0, sizeof(WTexture));
+    texture->type = style;
+    
+    texture->opacity = opacity;
+
+    texture->color1 = *from;
+    texture->color2 = *to;
+
+    texture->normal.red = (from->red + to->red)<<7;
+    texture->normal.green = (from->green + to->green)<<7;
+    texture->normal.blue = (from->blue + to->blue)<<7;
+
+    XAllocColor(dpy, scr->w_colormap, &texture->normal);
+    gcv.background = gcv.foreground = texture->normal.pixel;
+    gcv.graphics_exposures = False;
+    texture->normal_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground
+				   |GCGraphicsExposures, &gcv);
+
+    texture->pixmap = image;
+
+    return texture;
+}
+
+
+#ifdef TEXTURE_PLUGIN
+WTexFunction*
+wTextureMakeFunction(WScreen *scr, char *lib, char *func, int argc, char **argv)
+{
+    XColor fallbackColor;
+    XGCValues gcv;
+    WTexFunction *texture;
+    
+    texture = wmalloc(sizeof(WTexture));
+    texture->type = WTEX_FUNCTION;
+    texture->handle = NULL;
+    texture->render = 0;
+    texture->argc = argc;
+    texture->argv = argv;
+    
+    fallbackColor.red = 0x8000;
+    fallbackColor.green = 0x8000;
+    fallbackColor.blue = 0x8000;
+    
+    gcv.background = gcv.foreground = fallbackColor.pixel;
+    gcv.graphics_exposures = False;
+    texture->normal_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground
+				   |GCGraphicsExposures, &gcv);
+    
+# ifdef HAVE_DLFCN_H
+    /* open the library */
+    texture->handle = dlopen(lib, RTLD_LAZY);
+    if (!texture->handle) {
+        wwarning(_("library \"%s\" cound not be opened."), lib);
+	wfree(argv);
+	wfree(texture);
+	return NULL;
+    }
+    
+    /* find the function */
+    texture->render = dlsym(texture->handle, func);
+    if (!texture->render) {
+        wwarning(_("function \"%s\" not found in library \"%s\""), func, lib);
+	wfree(argv);
+	dlclose(texture->handle);
+	wfree(texture);
+	return NULL;
+    }
+# else
+    wwarning(_("function textures not supported on this system, sorry."));
+# endif
+    
+    /* success! */
+    return texture;
+}
+#endif /* TEXTURE_PLUGIN */
+
 
 RImage*
-wTextureRenderImage(WTexture *texture, int width, int height, int relief)
+wTextureRenderImage(WTexture *texture, int width, int height,
+		    int relief)
 {
-    RImage *image;
-    RColor color1, color2;
+    RImage *image = NULL;
+    RColor color1;
     int d;
     int subtype;
     
@@ -310,12 +481,28 @@ wTextureRenderImage(WTexture *texture, int width, int height, int relief)
 	break;
 	
      case WTEX_PIXMAP:
-	if (texture->pixmap.subtype == WTP_TILE)
+	if (texture->pixmap.subtype == WTP_TILE) {
             image = RMakeTiledImage(texture->pixmap.pixmap, width, height);
-	else
+        } else if (texture->pixmap.subtype == WTP_CENTER) {
+            color1.red = texture->pixmap.normal.red>>8;
+            color1.green = texture->pixmap.normal.green>>8;
+            color1.blue = texture->pixmap.normal.blue>>8;
+            color1.alpha = 255;
+            image = RMakeCenteredImage(texture->pixmap.pixmap, width, height,
+                                       &color1);
+        } else {
             image = RScaleImage(texture->pixmap.pixmap, width, height);
+        }
 	break;
 	
+     case WTEX_IGRADIENT:
+	image = RRenderInterwovenGradient(width, height,
+					  texture->igradient.colors1,
+					  texture->igradient.thickness1,
+					  texture->igradient.colors2,
+					  texture->igradient.thickness2);
+	break;
+
      case WTEX_HGRADIENT:
 	subtype = RGRD_HORIZONTAL;
 	goto render_gradient;
@@ -327,14 +514,9 @@ wTextureRenderImage(WTexture *texture, int width, int height, int relief)
      case WTEX_DGRADIENT:
 	subtype = RGRD_DIAGONAL;
      render_gradient:
-	color1.red = texture->gradient.color1.red >> 8;
-	color1.green = texture->gradient.color1.green >> 8;
-	color1.blue = texture->gradient.color1.blue >> 8;
-	color2.red = texture->gradient.color2.red >> 8;
-	color2.green = texture->gradient.color2.green >> 8;
-	color2.blue = texture->gradient.color2.blue >> 8;
 
-	image = RRenderGradient(width, height, &color1, &color2, subtype);
+	image = RRenderGradient(width, height, &texture->gradient.color1, 
+				&texture->gradient.color2, subtype);
 	break;
 	
      case WTEX_MHGRADIENT:
@@ -352,15 +534,76 @@ wTextureRenderImage(WTexture *texture, int width, int height, int relief)
 				     &(texture->mgradient.colors[1]),
 				     subtype);
 	break;
-	
+
+     case WTEX_THGRADIENT:
+	subtype = RGRD_HORIZONTAL;
+	goto render_tgradient;
+
+     case WTEX_TVGRADIENT:
+	subtype = RGRD_VERTICAL;
+	goto render_tgradient;
+
+     case WTEX_TDGRADIENT:
+	subtype = RGRD_DIAGONAL;
+     render_tgradient:
+	{
+	    RImage *grad;
+
+	    image = RMakeTiledImage(texture->tgradient.pixmap, width, height);
+	    if (!image)
+		break;
+
+	    grad = RRenderGradient(width, height, &texture->tgradient.color1,
+				   &texture->tgradient.color2, subtype);
+	    if (!grad) {
+		RDestroyImage(image);
+		image = NULL;
+		break;
+	    }
+
+	    RCombineImagesWithOpaqueness(image, grad,
+					 texture->tgradient.opacity);
+	    RDestroyImage(grad);
+	}
+	break;
+
+#ifdef TEXTURE_PLUGIN
+     case WTEX_FUNCTION:
+#ifdef HAVE_DLFCN_H
+	if (texture->function.render) {
+            image = texture->function.render (
+	        texture->function.argc, texture->function.argv,
+		width, height, relief);
+	}
+#endif
+	if (!image) {
+	    RErrorCode = RERR_INTERNAL;
+	}
+        break;
+#endif /* TEXTURE_PLUGIN */
+
      default:
 	puts("ERROR in wTextureRenderImage()");
 	image = NULL;
+	break;
     }
 
     if (!image) {
-	wwarning(_("could not render texture: %s"), RErrorString);
-	return None;
+	RColor gray;
+
+	wwarning(_("could not render texture: %s"), RMessageForError(RErrorCode));
+
+	image = RCreateImage(width, height, False);
+	if (image == NULL) {
+	    wwarning(_("could not allocate image buffer"));
+	    return NULL;
+	}
+       
+	gray.red = 190;
+	gray.green = 190;
+	gray.blue = 190;
+        gray.alpha = 255;
+	RClearImage(image, &gray);
     }
 
 
@@ -401,107 +644,47 @@ wTextureRenderImage(WTexture *texture, int width, int height, int relief)
 }
 
 
-/* used only for menu entries */
-void
-wTextureRender(WScreen *scr, WTexture *texture, Pixmap *data, 
-	       int width, int height, int relief)
-{
-    if (!texture)
-      return;
-/*    
-    switch (texture->any.type) {
-     case WTEX_DGRADIENT:
-     case WTEX_VGRADIENT:
-     case WTEX_HGRADIENT:
-     case WTEX_MHGRADIENT:
-     case WTEX_MVGRADIENT:
-     case WTEX_MDGRADIENT:
-     case WTEX_PIXMAP:
-	*/
-	if (!*data) {
-	    *data = renderTexture(scr, width, height, texture, relief);
-	}/*
-	break;
-    }*/
-}
-
-
 
 static void
 bevelImage(RImage *image, int relief)
 {
     int width = image->width;
     int height = image->height;
-    int x, y, ofs;
-    unsigned char *r, *g, *b;
-
-    r = image->data[0];
-    g = image->data[1];
-    b = image->data[2];
+    RColor color;
+    
     switch (relief) {
      case WREL_MENUENTRY:
-	r++; 	g++;	b++;
-	for (x=0; x<(width-2); x++) {
-	    *r = (*r<175 ? *r+80 : 255);
-	    *g = (*g<175 ? *g+80 : 255);
-	    *b = (*b<175 ? *b+80 : 255);	    
-	    r++; g++; b++;
-	}
-	r = image->data[0];
-	g = image->data[1];
-	b = image->data[2];
-	for (y=0; y<height; y++) {
-	    ofs = y*width;
-	    r[ofs] = (r[ofs] <175 ? r[ofs]+80 : 255);
-	    g[ofs] = (g[ofs] <175 ? g[ofs]+80 : 255);
-	    b[ofs] = (b[ofs] <175 ? b[ofs]+80 : 255);
-	    ofs = y*width+(width-1);
-	    r[ofs] = (r[ofs] >40 ? r[ofs]-40 : 0);
-	    g[ofs] = (g[ofs] >40 ? g[ofs]-40 : 0);
-	    b[ofs] = (b[ofs] >40 ? b[ofs]-40 : 0);
-	}
-	r = image->data[0] + (height-2)*width + 1;
-	g = image->data[1] + (height-2)*width + 1;
-	b = image->data[2] + (height-2)*width + 1;
-	for (x=1; x<width; x++) {
-	    *r = (*r>40 ? *r-40 : 0);
-	    *g = (*g>40 ? *g-40 : 0);
-	    *b = (*b>40 ? *b-40 : 0);
-	    r++; g++; b++;
-	}
-	memset(r, 0, width);
-	memset(g, 0, width);
-	memset(b, 0, width);	
+	color.red = color.green = color.blue = 80;
+	color.alpha = 0;
+	/**/
+	ROperateLine(image, RAddOperation, 1, 0, width-2, 0, &color);
+	/**/
+
+	ROperateLine(image, RAddOperation, 0, 0, 0, height-1, &color);
+
+	color.red = color.green = color.blue = 40;
+	color.alpha = 0;
+	ROperateLine(image, RSubtractOperation, width-1, 0, width-1, 
+		     height-1, &color);
+
+	/**/
+	ROperateLine(image, RSubtractOperation, 1, height-2, width-2,
+		     height-2, &color);
+
+	color.red = color.green = color.blue = 0;
+	color.alpha = 255;
+	RDrawLine(image, 0, height-1, width-1, height-1, &color);
+	/**/
 	break;
 
     }
 }
  
-
-
-static Pixmap
-renderTexture(WScreen *scr, int width, int height, WTexture *texture,
-	       int rel)
-{
-    RImage *img;
-    Pixmap pix;
-
-    img = wTextureRenderImage(texture, width, height, rel);
-    
-    if (!img) {
-	wwarning(_("could not render texture: %s"), RErrorString);
-	return None;
-    }
-    if (!RConvertImage(scr->rcontext, img, &pix)) {
-	wwarning(_("error rendering image:%s"), RErrorString);
-    }
-    RDestroyImage(img);
-    return pix;
-}
  
 
 void
-wDrawBevel(WCoreWindow *core, WTexSolid *texture, int relief)
+wDrawBevel(Drawable d, unsigned width, unsigned height,
+	   WTexSolid *texture, int relief)
 {
     GC light, dim, dark;
     XSegment segs[4];
@@ -518,53 +701,54 @@ wDrawBevel(WCoreWindow *core, WTexSolid *texture, int relief)
      case WREL_RAISED:
      case WREL_ICON:
 	segs[0].x1 = 1;
-	segs[0].x2 = core->width - 2;
-	segs[0].y2 = segs[0].y1 = core->height - 2;
-	segs[1].x1 = core->width - 2;
+	segs[0].x2 = width - 2;
+	segs[0].y2 = segs[0].y1 = height - 2;
+	segs[1].x1 = width - 2;
 	segs[1].y1 = 1;
-	segs[1].x2 = core->width - 2;
-	segs[1].y2 = core->height - 2;
-	XDrawSegments(dpy, core->window, dim, segs, 2);
+	segs[1].x2 = width - 2;
+	segs[1].y2 = height - 2;
+	XDrawSegments(dpy, d, dim, segs, 2);
 	segs[0].x1 = 0;
-	segs[0].x2 = core->width - 1;
-	segs[0].y2 = segs[0].y1 = core->height - 1;
-	segs[1].x1 = segs[1].x2 = core->width - 1;
+	segs[0].x2 = width - 1;
+	segs[0].y2 = segs[0].y1 = height - 1;
+	segs[1].x1 = segs[1].x2 = width - 1;
 	segs[1].y1 = 0;
-	segs[1].y2 = core->height - 1;
-	XDrawSegments(dpy, core->window, dark, segs, 2);
+	segs[1].y2 = height - 1;
+	XDrawSegments(dpy, d, dark, segs, 2);
 	segs[0].x1 = segs[0].y1 = segs[0].y2 = 0;
-	segs[0].x2 = core->width - 2;
+	segs[0].x2 = width - 2;
 	segs[1].x1 = segs[1].y1 = 0;
 	segs[1].x2 = 0;
-	segs[1].y2 = core->height - 2;
-	XDrawSegments(dpy, core->window, light, segs, 2);
+	segs[1].y2 = height - 2;
+	XDrawSegments(dpy, d, light, segs, 2);
 	if (relief==WREL_ICON) {
 	    segs[0].x1 = segs[0].y1 = segs[0].y2 = 1;
-	    segs[0].x2 = core->width - 2;
+	    segs[0].x2 = width - 2;
 	    segs[1].x1 = segs[1].y1 = 1;
 	    segs[1].x2 = 1;
-	    segs[1].y2 = core->height - 2;
-	    XDrawSegments(dpy, core->window, light, segs, 2);
+	    segs[1].y2 = height - 2;
+	    XDrawSegments(dpy, d, light, segs, 2);
 	}
 	break;
-		
+#ifdef unused		
      case WREL_SUNKEN:
 	segs[0].x1 = 0;
-	segs[0].x2 = core->width - 1;
+	segs[0].x2 = width - 1;
 	segs[0].y2 = segs[0].y1 = 0;
 	segs[1].x1 = segs[1].x2 = 0;
 	segs[1].y1 = 0;
-	segs[1].y2 = core->height - 1;
-	XDrawSegments(dpy, core->window, dark, segs, 2);
+	segs[1].y2 = height - 1;
+	XDrawSegments(dpy, d, dark, segs, 2);
 	
 	segs[0].x1 = 0;
-	segs[0].y1 = segs[0].y2 = core->height - 1;
-	segs[0].x2 = core->width - 1;
-	segs[1].x2 = segs[1].x1 = core->width - 1;
+	segs[0].y1 = segs[0].y2 = height - 1;
+	segs[0].x2 = width - 1;
+	segs[1].x2 = segs[1].x1 = width - 1;
 	segs[1].y1 = 1;
-	segs[1].y2 = core->height - 1;
-	XDrawSegments(dpy, core->window, light, segs, 2);
-	break;	
+	segs[1].y2 = height - 1;
+	XDrawSegments(dpy, d, light, segs, 2);
+	break;
+#endif
     }
 }
 
