@@ -51,7 +51,7 @@
 #include "stacking.h"
 #include "defaults.h"
 #include "workspace.h"
-#include "list.h"
+
 
 #ifdef MWM_HINTS
 # include "motif.h"
@@ -145,7 +145,15 @@ appearanceObserver(void *self, WMNotification *notif)
 	return;
 
     if (flags & WFontSettings) {
-	wWindowConfigureBorders(wwin);
+        wWindowConfigureBorders(wwin);
+        if(wwin->flags.shaded) {
+            wFrameWindowResize(wwin->frame, wwin->frame->core->width,
+                    wwin->frame->top_width - 1);
+
+            wwin->client.y = wwin->frame_y - wwin->client.height 
+                + wwin->frame->top_width;
+            wWindowSynthConfigureNotify(wwin);
+        }
     }
     if (flags & WTextureSettings) {
 	wwin->frame->flags.need_texture_remake = 1;
@@ -207,24 +215,28 @@ wWindowDestroy(WWindow *wwin)
 {
     int i;
 
+    if (wwin->screen_ptr->cmap_window == wwin) {
+	wwin->screen_ptr->cmap_window = NULL;
+    }
+    
     WMRemoveNotificationObserver(wwin);
     
     wwin->flags.destroyed = 1;
 
     for (i = 0; i < MAX_WINDOW_SHORTCUTS; i++) {
-        wwin->screen_ptr->shortcutSelectedWindows[i] = list_remove_elem(wwin->screen_ptr->shortcutSelectedWindows[i], wwin);
-        if (wwin->screen_ptr->shortcutWindow[i] == wwin) {
-            if (wwin->screen_ptr->shortcutSelectedWindows[i]) {
-                LinkedList *list = wwin->screen_ptr->shortcutSelectedWindows[i];
-                wwin->screen_ptr->shortcutWindow[i] = 
-                    list->head;
-            }
-            else wwin->screen_ptr->shortcutWindow[i] = NULL;
-        }
+	if (!wwin->screen_ptr->shortcutWindows[i])
+	    continue;
+
+	WMRemoveFromBag(wwin->screen_ptr->shortcutWindows[i], wwin);
+
+	if (!WMGetBagItemCount(wwin->screen_ptr->shortcutWindows[i])) {
+	    WMFreeBag(wwin->screen_ptr->shortcutWindows[i]);
+	    wwin->screen_ptr->shortcutWindows[i] = NULL;
+	}
     }
 
     if (wwin->normal_hints)
-      free(wwin->normal_hints);
+      XFree(wwin->normal_hints);
     
     if (wwin->wm_hints)
       XFree(wwin->wm_hints);
@@ -236,7 +248,7 @@ wWindowDestroy(WWindow *wwin)
       XFree(wwin->wm_class);
 
     if (wwin->wm_gnustep_attr)
-      free(wwin->wm_gnustep_attr);
+      wfree(wwin->wm_gnustep_attr);
 
     if (wwin->cmap_windows)
       XFree(wwin->cmap_windows);
@@ -262,28 +274,48 @@ static void
 setupGNUstepHints(WWindow *wwin, GNUstepWMAttributes *gs_hints)
 {
     if (gs_hints->flags & GSWindowStyleAttr) {
-	wwin->client_flags.no_titlebar = 
-		  ((gs_hints->window_style & WMTitledWindowMask)?0:1);
+	if (gs_hints->window_style == WMBorderlessWindowMask) {
+	    wwin->client_flags.no_border = 1;
+	    wwin->client_flags.no_titlebar = 1;
+	    wwin->client_flags.no_closable = 1;
+	    wwin->client_flags.no_miniaturizable = 1;
+	    wwin->client_flags.no_resizable = 1;
+	    wwin->client_flags.no_close_button = 1;
+	    wwin->client_flags.no_miniaturize_button = 1;
+	    wwin->client_flags.no_resizebar = 1;
+	} else {
+	    wwin->client_flags.no_close_button =
+		((gs_hints->window_style & WMClosableWindowMask)?0:1);
 
-	wwin->client_flags.no_close_button =
-		  ((gs_hints->window_style & WMClosableWindowMask)?0:1);
+	    wwin->client_flags.no_closable =
+		((gs_hints->window_style & WMClosableWindowMask)?0:1);
 
-	wwin->client_flags.no_closable =
-		  ((gs_hints->window_style & WMClosableWindowMask)?0:1);
+	    wwin->client_flags.no_miniaturize_button =
+		((gs_hints->window_style & WMMiniaturizableWindowMask)?0:1);
 
-	wwin->client_flags.no_miniaturize_button =
-		  ((gs_hints->window_style & WMMiniaturizableWindowMask)?0:1);
+	    wwin->client_flags.no_miniaturizable = 
+		wwin->client_flags.no_miniaturize_button;
 
-	wwin->client_flags.no_miniaturizable =
-		  ((gs_hints->window_style & WMMiniaturizableWindowMask)?0:1);
+	    wwin->client_flags.no_resizebar =
+		((gs_hints->window_style & WMResizableWindowMask)?0:1);
 
-	wwin->client_flags.no_resizebar =
-		  ((gs_hints->window_style & WMResizableWindowMask)?0:1);
+	    wwin->client_flags.no_resizable = wwin->client_flags.no_resizebar;
 
-	wwin->client_flags.no_resizable =
-		  ((gs_hints->window_style & WMResizableWindowMask)?0:1);
+	    /* these attributes supposedly imply in the existence
+	     * of a titlebar */
+	    if (gs_hints->window_style & (WMResizableWindowMask|
+					  WMClosableWindowMask|
+					  WMMiniaturizableWindowMask)) {
+		wwin->client_flags.no_titlebar = 0;
+	    } else {
+		wwin->client_flags.no_titlebar = 
+		    ((gs_hints->window_style & WMTitledWindowMask)?0:1);
+	    }
+
+	}
     } else {
 	/* setup the defaults */
+	wwin->client_flags.no_border = 0;
 	wwin->client_flags.no_titlebar = 0;
 	wwin->client_flags.no_closable = 0;
 	wwin->client_flags.no_miniaturizable = 0;
@@ -364,26 +396,12 @@ wWindowSetupInitialAttributes(WWindow *wwin, int *level, int *workspace)
 
 	if (wwin->wm_gnustep_attr->flags & GSWindowLevelAttr) {
 
-	    switch (wwin->wm_gnustep_attr->window_level) {
-	     case WMNormalWindowLevel:
-		*level = WMNormalLevel;
-		break;
-	     case WMFloatingWindowLevel:
-		*level = WMFloatingLevel;
-		break;
-	     case WMDockWindowLevel:
-		*level = WMDockLevel;
-		break;
-	     case WMSubmenuWindowLevel:
-		*level = WMSubmenuLevel;
-		break;
-	     case WMMainMenuWindowLevel:
-		*level = WMMainMenuLevel;
-		break;
-	     default:
-		*level = WMNormalLevel;
-		break;
-	    }
+	    *level = wwin->wm_gnustep_attr->window_level;
+	    /*
+	     * INT_MIN is the only illegal window level.
+	     */
+	    if (*level == INT_MIN)
+		*level = INT_MIN + 1;
 	} else {
 	    /* setup defaults */
 	    *level = WMNormalLevel;
@@ -391,18 +409,22 @@ wWindowSetupInitialAttributes(WWindow *wwin, int *level, int *workspace)
     } else {
 	int tmp_workspace = -1;
 	int tmp_level = -1;
+	Bool check;
+	
+	check = False;
 	
 #ifdef MWM_HINTS
 	wMWMCheckClientHints(wwin);
 #endif /* MWM_HINTS */
 
-#ifdef KWM_HINTS
-	wKWMCheckClientHints(wwin, &tmp_level, &tmp_workspace);
-#endif /* KWM_HINTS */
-	
 #ifdef GNOME_STUFF
-	wGNOMECheckClientHints(wwin, &tmp_level, &tmp_workspace);
+	check = wGNOMECheckClientHints(wwin, &tmp_level, &tmp_workspace);
 #endif /* GNOME_STUFF */
+	
+#ifdef KWM_HINTS
+	if (!check)
+	    wKWMCheckClientHints(wwin, &tmp_level, &tmp_workspace);
+#endif /* KWM_HINTS */
 
 #ifdef OLWM_HINTS
 	wOLWMCheckClientHints(wwin);
@@ -619,6 +641,9 @@ wManageWindow(WScreen *scr, Window window)
     XSetWindowBorderWidth(dpy, window, 0);
 
     /* get hints from GNUstep app */
+    if (wwin->wm_class != 0 && strcmp(wwin->wm_class, "GNUstep") == 0) {
+	wwin->flags.is_gnustep = 1;
+    }
     if (!PropGetGNUstepWMAttr(window, &wwin->wm_gnustep_attr)) {
 	wwin->wm_gnustep_attr = NULL;
     }
@@ -734,12 +759,18 @@ wManageWindow(WScreen *scr, Window window)
 	wwin->flags.maximized = MAX_VERTICAL|MAX_HORIZONTAL;
     }
 
+    {
+	Bool bla;
+	
+	bla = False;
 #ifdef GNOME_STUFF
-    wGNOMECheckInitialClientState(wwin);
+	bla = wGNOMECheckInitialClientState(wwin);
 #endif
 #ifdef KWM_HINTS
-    wKWMCheckClientInitialState(wwin);
+	if (!bla)
+	    wKWMCheckClientInitialState(wwin);
 #endif
+    }
 
     /* apply previous state if it exists and we're in startup */
     if (scr->flags.startup && wm_state >= 0) {
@@ -773,6 +804,7 @@ wManageWindow(WScreen *scr, Window window)
 	    !WFLAGP(wwin, no_miniaturizable)) {
             wwin->flags.miniaturized = win_state->state->miniaturized;
         }
+
         if (!IS_OMNIPRESENT(wwin)) {
             int w = wDefaultGetStartWorkspace(scr, wwin->wm_instance,
                                               wwin->wm_class);
@@ -797,17 +829,44 @@ wManageWindow(WScreen *scr, Window window)
 	    wwin->flags.shaded = wstate->shaded;
 	    wwin->flags.hidden = wstate->hidden;
 	    wwin->flags.miniaturized = wstate->miniaturized;
-	    workspace = wstate->workspace;
+	    wwin->flags.maximized = wstate->maximized;
+	    if (wwin->flags.maximized) {
+		wwin->old_geometry.x = wstate->x;
+		wwin->old_geometry.y = wstate->y;
+		wwin->old_geometry.width = wstate->w;
+		wwin->old_geometry.height = wstate->h;
+	    }
 
-	    if (scr->flags.startup && wstate->window_shortcuts >= 0) {
+	    workspace = wstate->workspace;
+	} else {
+	    wstate = NULL;
+	}
+
+	/* restore window shortcut */
+	if (wstate != NULL || win_state != NULL) {
+	    unsigned mask = 0;
+
+	    if (win_state != NULL)
+		mask = win_state->state->window_shortcuts;
+	    
+	    if (wstate != NULL && mask == 0)
+		mask = wstate->window_shortcuts;
+
+	    if (mask > 0) {
 		int i;
 
 		for (i = 0; i < MAX_WINDOW_SHORTCUTS; i++) {
-		    if (wstate->window_shortcuts & (1<<i))
-			scr->shortcutWindow[i] = wwin;
+		    if (mask & (1<<i)) {
+			if (!scr->shortcutWindows[i])
+			    scr->shortcutWindows[i] = WMCreateBag(4);
+
+			WMPutInBag(scr->shortcutWindows[i], wwin);
+		    }
 		}
 	    }
-	    free(wstate);
+	}
+	if (wstate != NULL) {
+	    wfree(wstate);
 	}
     }
 
@@ -846,7 +905,7 @@ wManageWindow(WScreen *scr, Window window)
     }
 
     /* setup window geometry */
-    if (win_state && win_state->state->use_geometry) {
+    if (win_state && win_state->state->w > 0) {
         width = win_state->state->w;
         height = win_state->state->h;
     }
@@ -859,7 +918,7 @@ wManageWindow(WScreen *scr, Window window)
     {
 	Bool dontBring = False;
 
-	if (win_state && win_state->state->use_geometry) {
+	if (win_state && win_state->state->w > 0) {
 	    x = win_state->state->x;
 	    y = win_state->state->y;
 	} else if ((wwin->transient_for==None
@@ -869,7 +928,27 @@ wManageWindow(WScreen *scr, Window window)
 		   && !wwin->flags.miniaturized
 		   && !wwin->flags.maximized
 		   && !(wwin->normal_hints->flags & (USPosition|PPosition))) {
-	    PlaceWindow(wwin, &x, &y, width, height);
+	    
+	    if (transientOwner && transientOwner->flags.mapped) {
+		int offs = WMAX(20, 2*transientOwner->frame->top_width);
+		
+		x = transientOwner->frame_x + 
+		    abs((transientOwner->frame->core->width - width)/2) + offs;
+		y = transientOwner->frame_y +
+		    abs((transientOwner->frame->core->height - height)/3) + offs;
+
+		if (x < 0)
+		    x = 0;
+		else if (x + width > scr->scr_width)
+		    x = scr->scr_width - width;
+
+		if (y < 0)
+		    y = 0;
+		else if (y + height > scr->scr_height)
+		    y = scr->scr_height - height;
+	    } else {
+		PlaceWindow(wwin, &x, &y, width, height);
+	    }
 	    if (wPreferences.window_placement == WPM_MANUAL)
 		dontBring = True;
 	} 
@@ -899,9 +978,12 @@ wManageWindow(WScreen *scr, Window window)
 	foo |= WFF_TITLEBAR;
     if (!WFLAGP(wwin, no_resizebar))
 	foo |= WFF_RESIZEBAR;
+    if (!WFLAGP(wwin, no_border))
+	foo |= WFF_BORDER;
 
     wwin->frame = wFrameWindowCreate(scr, window_level, 
-				     x, y, width, height, foo,
+				     x, y, width, height,
+				     &wPreferences.window_title_clearance, foo,
 				     scr->window_title_texture,
 				     scr->resizebar_texture,
 				     scr->window_title_pixel, 
@@ -1255,7 +1337,7 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
     wwin->frame_y = wwin->client.y;
 
     
-    foo = WFF_RIGHT_BUTTON;
+    foo = WFF_RIGHT_BUTTON|WFF_BORDER;
     foo |= WFF_TITLEBAR;
 #ifdef XKB_BUTTON_HINT
     foo |= WFF_LANGUAGE_BUTTON;
@@ -1263,7 +1345,8 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
 
     wwin->frame = wFrameWindowCreate(scr, WMFloatingLevel,
 				     wwin->frame_x, wwin->frame_y,
-				     width, height, foo,
+				     width, height,
+                     &wPreferences.window_title_clearance, foo,
 				     scr->window_title_texture,
 				     scr->resizebar_texture,
 				     scr->window_title_pixel, 
@@ -1348,7 +1431,8 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
 	wwin->prev = NULL;
     }
 
-    wFrameWindowChangeState(wwin->frame, WS_UNFOCUSED);
+    if (wwin->flags.is_gnustep == 0)
+	wFrameWindowChangeState(wwin->frame, WS_UNFOCUSED);
 
 /*    if (wPreferences.auto_focus)*/
 	wSetFocusTo(scr, wwin);
@@ -1394,8 +1478,7 @@ wUnmanageWindow(WWindow *wwin, Bool restore, Bool destroyed)
 
     /* First close attribute editor window if open */
     if (wwin->flags.inspector_open) {
-        WWindow *pwin = wwin->inspector->frame; /* the inspector window */
-        (*pwin->frame->on_click_right)(NULL, pwin, NULL);
+	wCloseInspectorForWindow(wwin);
     }
 
     /* Close window menu if it's open for this window */
@@ -1463,28 +1546,40 @@ wUnmanageWindow(WWindow *wwin, Bool restore, Bool destroyed)
 	    scr->focused_window->next = NULL;
 	}
 
-	/* if in click to focus mode and the window
-	 * was a transient, focus the owner window  
-	 */
-	tmp = NULL;
 	if (wPreferences.focus_mode==WKF_CLICK) {
-	    tmp = wWindowFor(wwin->transient_for);
-	    if (tmp && (!tmp->flags.mapped || WFLAGP(tmp, no_focusable))) {
-		tmp = NULL;
-	    }
-	}
-	/* otherwise, focus the next one in the focus list */
-	if (!tmp) {
-	    tmp = scr->focused_window;
-	    while (tmp) {
-		if (!WFLAGP(tmp, no_focusable)
-		    && (tmp->flags.mapped || tmp->flags.shaded))
-		    break;
-		tmp = tmp->prev;
-	    }
-	}
-	if (wPreferences.focus_mode==WKF_CLICK) {
-	    newFocusedWindow = tmp;
+
+           /* if in click to focus mode and the window
+            * was a transient, focus the owner window  
+            */
+           tmp = NULL;
+           if (wPreferences.focus_mode==WKF_CLICK) {
+               tmp = wWindowFor(wwin->transient_for);
+               if (tmp && (!tmp->flags.mapped || WFLAGP(tmp, no_focusable))) {
+                   tmp = NULL;
+               }
+           }
+           /* otherwise, focus the next one in the focus list */
+           if (!tmp) {
+              tmp = scr->focused_window;
+              while (tmp) {               /* look for one in the window list first */
+                 if (!WFLAGP(tmp, no_focusable) && !WFLAGP(tmp, skip_window_list)
+                     && (tmp->flags.mapped || tmp->flags.shaded))
+                     break;
+                 tmp = tmp->prev;
+              }
+              if (!tmp) {                 /* if unsuccessful, choose any focusable window */
+                 tmp = scr->focused_window;
+                 while (tmp) {
+                    if (!WFLAGP(tmp, no_focusable)
+                         && (tmp->flags.mapped || tmp->flags.shaded))
+                         break;
+                    tmp = tmp->prev;
+                 }
+              }
+           }
+
+	   newFocusedWindow = tmp;
+
 	} else if (wPreferences.focus_mode==WKF_SLOPPY
 		   || wPreferences.focus_mode==WKF_POINTER) {
             unsigned int mask;
@@ -1523,7 +1618,8 @@ wUnmanageWindow(WWindow *wwin, Bool restore, Bool destroyed)
 
     if (wasFocused) {
 	if (newFocusedWindow != owner && owner) {
-	    wFrameWindowChangeState(owner->frame, WS_UNFOCUSED);
+	    if (wwin->flags.is_gnustep == 0)
+	      wFrameWindowChangeState(owner->frame, WS_UNFOCUSED);
 	}
 	wSetFocusTo(scr, newFocusedWindow);
     }
@@ -1578,7 +1674,8 @@ wWindowFocus(WWindow *wwin, WWindow *owin)
 
     wwin->flags.semi_focused = 0;
 
-    wFrameWindowChangeState(wwin->frame, WS_FOCUSED);
+    if (wwin->flags.is_gnustep == 0)
+      wFrameWindowChangeState(wwin->frame, WS_FOCUSED);
 
     wwin->flags.focused = 1;
 
@@ -1633,7 +1730,8 @@ wWindowUnfocus(WWindow *wwin)
 {
     CloseWindowMenu(wwin->screen_ptr);
 
-    wFrameWindowChangeState(wwin->frame, wwin->flags.semi_focused 
+    if (wwin->flags.is_gnustep == 0)
+      wFrameWindowChangeState(wwin->frame, wwin->flags.semi_focused 
 			    ? WS_PFOCUSED : WS_UNFOCUSED);
 
     if (wwin->transient_for!=None 
@@ -1760,8 +1858,11 @@ wWindowConstrainSize(WWindow *wwin, int *nwidth, int *nheight)
 	height = (((height - minH) / hinc) * hinc) + minH;
     }
 
-    *nwidth = width;
-    *nheight = height;
+    /* broken stupid apps may cause preposterous values for these.. */
+    if (width > 0)
+	*nwidth = width;
+    if (height > 0)
+	*nheight = height;
 }
 
 
@@ -1788,8 +1889,10 @@ wWindowChangeWorkspace(WWindow *wwin, int workspace)
 		wapp->last_workspace = workspace;
 	    }
 	    if (wwin->flags.miniaturized) {
-		XUnmapWindow(dpy, wwin->icon->core->window);
-		wwin->icon->mapped = 0;
+		if (wwin->icon) {
+		    XUnmapWindow(dpy, wwin->icon->core->window);
+		    wwin->icon->mapped = 0;
+		}
 	    } else {
 		unmap = 1;
 		wSetFocusTo(scr, NULL);
@@ -1798,8 +1901,10 @@ wWindowChangeWorkspace(WWindow *wwin, int workspace)
     } else {
 	/* brought to current workspace. Map window */
 	if (wwin->flags.miniaturized && !wPreferences.sticky_icons) {
-	    XMapWindow(dpy, wwin->icon->core->window);
-	    wwin->icon->mapped = 1;
+	    if (wwin->icon) {
+		XMapWindow(dpy, wwin->icon->core->window);
+		wwin->icon->mapped = 1;
+	    }
 	} else if (!wwin->flags.mapped && 
 	    !(wwin->flags.miniaturized || wwin->flags.hidden)) {
 	    wWindowMap(wwin);
@@ -1936,6 +2041,10 @@ int req_width, req_height;	       /* new size of the client */
     }
     wwin->frame_x = req_x;
     wwin->frame_y = req_y;
+    if (!WFLAGP(wwin, no_border)) {
+	wwin->client.x += FRAME_BORDER_WIDTH;
+	wwin->client.y += FRAME_BORDER_WIDTH;
+    }
 
 #ifdef SHAPE
     if (wShapeSupported && wwin->flags.shaped && resize) {
@@ -1972,6 +2081,10 @@ int req_x, req_y;		       /* new position of the frame */
 
     wwin->client.x = req_x;
     wwin->client.y = req_y + wwin->frame->top_width;
+    if (!WFLAGP(wwin, no_border)) {
+	wwin->client.x += FRAME_BORDER_WIDTH;
+	wwin->client.y += FRAME_BORDER_WIDTH;
+    }
 
     XMoveWindow(dpy, wwin->frame->core->window, req_x, req_y);
 
@@ -1985,36 +2098,36 @@ int req_x, req_y;		       /* new position of the frame */
 }
 
 
-void 
+void
 wWindowUpdateButtonImages(WWindow *wwin)
 {
     WScreen *scr = wwin->screen_ptr;
     Pixmap pixmap, mask;
     WFrameWindow *fwin = wwin->frame;
-    
+
     if (WFLAGP(wwin, no_titlebar))
 	return;
 
     /* miniaturize button */
-    
+
     if (!WFLAGP(wwin, no_miniaturize_button)) {
 	if (wwin->wm_gnustep_attr
 	    && wwin->wm_gnustep_attr->flags & GSMiniaturizePixmapAttr) {
 	    pixmap = wwin->wm_gnustep_attr->miniaturize_pixmap;
-	      
+	
 	    if (wwin->wm_gnustep_attr->flags&GSMiniaturizeMaskAttr) {
 		mask = wwin->wm_gnustep_attr->miniaturize_mask;
 	    } else {
 		mask = None;
 	    }
 
-	    if (fwin->lbutton_image 
+	    if (fwin->lbutton_image
 		&& (fwin->lbutton_image->image != pixmap
 		    || fwin->lbutton_image->mask != mask)) {
 		wPixmapDestroy(fwin->lbutton_image);
 		fwin->lbutton_image = NULL;
 	    }
-	    
+	
 	    if (!fwin->lbutton_image) {
 		fwin->lbutton_image = wPixmapCreate(scr, pixmap, mask);
 		fwin->lbutton_image->client_owned = 1;
@@ -2027,55 +2140,64 @@ wWindowUpdateButtonImages(WWindow *wwin)
 	    fwin->lbutton_image = scr->b_pixmaps[WBUT_ICONIFY];
 	}
     }
-    
+
 #ifdef XKB_BUTTON_HINT
     if (!WFLAGP(wwin, no_language_button)) {
-	    if (fwin->languagebutton_image && !fwin->languagebutton_image->shared) {
+	    if (fwin->languagebutton_image &&  
+		!fwin->languagebutton_image->shared) {
 		wPixmapDestroy(fwin->languagebutton_image);
 	    }
-	    fwin->languagebutton_image = scr->b_pixmaps[WBUT_XKBGROUP1 + fwin->languagemode];
+	fwin->languagebutton_image = 
+	    scr->b_pixmaps[WBUT_XKBGROUP1 + fwin->languagemode];
     }
 #endif
-    
+
     /* close button */
 
+/* redefine WFLAGP to MGFLAGP to allow broken close operation */
+#define MGFLAGP(wwin, FLAG)	(wwin)->client_flags.FLAG
+
     if (!WFLAGP(wwin, no_close_button)) {
-	if (wwin->wm_gnustep_attr
-	    && wwin->wm_gnustep_attr->flags&GSClosePixmapAttr) {
-	    pixmap = wwin->wm_gnustep_attr->close_pixmap;
-	    
-	    if (wwin->wm_gnustep_attr->flags&GSCloseMaskAttr) {
-		mask = wwin->wm_gnustep_attr->close_mask;
-	    } else {
-		mask = None;
-	    }
+        if (wwin->wm_gnustep_attr 
+	    && wwin->wm_gnustep_attr->flags & GSClosePixmapAttr) {
+            pixmap = wwin->wm_gnustep_attr->close_pixmap;
+	
+            if (wwin->wm_gnustep_attr->flags&GSCloseMaskAttr)
+                 mask = wwin->wm_gnustep_attr->close_mask;
+            else
+                 mask = None;
+
+           if (fwin->rbutton_image && (fwin->rbutton_image->image != pixmap
+				       || fwin->rbutton_image->mask != mask)) {
+	       wPixmapDestroy(fwin->rbutton_image);
+	       fwin->rbutton_image = NULL;
+	   }
+			
+           if (!fwin->rbutton_image) {
+               fwin->rbutton_image = wPixmapCreate(scr, pixmap, mask);
+               fwin->rbutton_image->client_owned = 1;
+               fwin->rbutton_image->client_owned_mask = 1;
+	   }
 
-	    if (fwin->rbutton_image
-		&& (fwin->rbutton_image->image != pixmap
-		    || fwin->rbutton_image->mask != mask)) {
-		wPixmapDestroy(fwin->rbutton_image);
-		fwin->rbutton_image = NULL;
-	    }
-	    
-	    if (!fwin->rbutton_image) {
-		fwin->rbutton_image = wPixmapCreate(scr, pixmap, mask);
-		fwin->rbutton_image->client_owned = 1;
-		fwin->rbutton_image->client_owned_mask = 1;
-	    }
 	} else if (WFLAGP(wwin, kill_close)) {
-	    if (fwin->rbutton_image && !fwin->rbutton_image->shared) {
+
+	    if (fwin->rbutton_image && !fwin->rbutton_image->shared)
 		wPixmapDestroy(fwin->rbutton_image);
-	    }
+
 	    fwin->rbutton_image = scr->b_pixmaps[WBUT_KILL];
-	} else if (WFLAGP(wwin, broken_close)) {
-	    if (fwin->rbutton_image && !fwin->rbutton_image->shared) {
+	    
+	} else if (MGFLAGP(wwin, broken_close)) {
+
+	    if (fwin->rbutton_image && !fwin->rbutton_image->shared) 
 		wPixmapDestroy(fwin->rbutton_image);
-	    }
+
 	    fwin->rbutton_image = scr->b_pixmaps[WBUT_BROKENCLOSE];
+
 	} else {
-	    if (fwin->rbutton_image && !fwin->rbutton_image->shared) {
+
+	    if (fwin->rbutton_image && !fwin->rbutton_image->shared) 
 		wPixmapDestroy(fwin->rbutton_image);
-	    }
+
 	    fwin->rbutton_image = scr->b_pixmaps[WBUT_CLOSE];
 	}
     }
@@ -2108,6 +2230,8 @@ wWindowConfigureBorders(WWindow *wwin)
 	    flags |= WFF_TITLEBAR;
 	if (!WFLAGP(wwin, no_resizebar))
 	    flags |= WFF_RESIZEBAR;
+	if (!WFLAGP(wwin, no_border))
+	    flags |= WFF_BORDER;
 	if (wwin->flags.shaded)
 	    flags |= WFF_IS_SHADED;
 
@@ -2179,9 +2303,22 @@ wWindowSaveState(WWindow *wwin)
     data[1] = wwin->flags.miniaturized;
     data[2] = wwin->flags.shaded;
     data[3] = wwin->flags.hidden;
+    data[4] = wwin->flags.maximized;
+    if (wwin->flags.maximized == 0) {
+	data[5] = wwin->frame_x;
+	data[6] = wwin->frame_y;
+	data[7] = wwin->frame->core->width;
+	data[8] = wwin->frame->core->height;
+    } else {
+	data[5] = wwin->old_geometry.x;
+	data[6] = wwin->old_geometry.y;
+	data[7] = wwin->old_geometry.width;
+	data[8] = wwin->old_geometry.height;
+    }
 
     for (i = 0; i < MAX_WINDOW_SHORTCUTS; i++) {
-	if (wwin->screen_ptr->shortcutWindow[i] == wwin)
+	if (wwin->screen_ptr->shortcutWindows[i] &&
+	    WMCountInBag(wwin->screen_ptr->shortcutWindows[i], wwin))
 	    data[9] |= 1<<i;
     }
     XChangeProperty(dpy, wwin->client_win, _XA_WINDOWMAKER_STATE,
@@ -2212,7 +2349,7 @@ getSavedState(Window window, WSavedState **state)
 	(*state)->miniaturized = data[1];
 	(*state)->shaded = data[2];
 	(*state)->hidden = data[3];
-	(*state)->use_geometry = data[4];
+	(*state)->maximized = data[4];
 	(*state)->x = data[5];
 	(*state)->y = data[6];
 	(*state)->w = data[7];
@@ -2290,7 +2427,7 @@ wWindowSetShape(WWindow *wwin)
 			    0, wwin->frame->top_width, urec, count,
 			    ShapeSet, Unsorted);
     XFlush(dpy);
-    free(urec);
+    wfree(urec);
     return;
 
 alt_code:
@@ -2402,9 +2539,10 @@ wWindowResetMouseGrabs(WWindow *wwin)
 			  GrabModeAsync, None, None);
     }
     
-    if (!wwin->flags.focused && !WFLAGP(wwin, no_focusable)) {
+    if (!wwin->flags.focused && !WFLAGP(wwin, no_focusable)
+	&& !wwin->flags.is_gnustep) {
 	/* the passive grabs to focus the window */
-	if (wPreferences.focus_mode == WKF_CLICK)
+/*	if (wPreferences.focus_mode == WKF_CLICK) */
 	    XGrabButton(dpy, AnyButton, AnyModifier, wwin->client_win,
 			True, ButtonPressMask, GrabModeSync, GrabModeAsync,
 			None, None);
@@ -2416,17 +2554,13 @@ wWindowResetMouseGrabs(WWindow *wwin)
 void
 wWindowUpdateGNUstepAttr(WWindow *wwin, GNUstepWMAttributes *attr)
 {
-    
-    if (attr->flags & GSExtraFlagsAttr) {
-	if (WFLAGP(wwin, broken_close) != 
-	    (attr->extra_flags & GSDocumentEditedFlag)) {
-
-	    wwin->client_flags.broken_close = !WFLAGP(wwin, broken_close);
-
-	    wWindowUpdateButtonImages(wwin);
-	}
-    }
-    
+   if (attr->flags & GSExtraFlagsAttr) {
+       if (MGFLAGP(wwin, broken_close) != 
+	   (attr->extra_flags & GSDocumentEditedFlag)) {
+	   wwin->client_flags.broken_close = !MGFLAGP(wwin, broken_close);
+	   wWindowUpdateButtonImages(wwin);
+       }
+   }
 }
 
 
@@ -2478,7 +2612,7 @@ wWindowGetSavedState(Window win)
 
     if (XGetCommand(dpy, win, &argv, &argc)) {
 	if (argc > 0)
-	    command = FlattenStringList(argv, argc);
+	    command = wtokenjoin(argv, argc);
         XFreeStringList(argv);
     }
     if (!command)
@@ -2502,7 +2636,7 @@ wWindowGetSavedState(Window win)
            class, command);
 #endif
 
-    if (command) free(command);
+    if (command) wfree(command);
     if (instance) XFree(instance);
     if (class) XFree(class);
 
@@ -2525,11 +2659,11 @@ wWindowDeleteSavedState(WMagicNumber id)
         printf("Deleted WindowState with ID %p, for %s.%s : \"%s\"\n",
                wstate, wstate->instance, wstate->class, wstate->command);
 #endif
-        if (wstate->instance) free(wstate->instance);
-        if (wstate->class)    free(wstate->class);
-        if (wstate->command)  free(wstate->command);
-        free(wstate->state);
-        free(wstate);
+        if (wstate->instance) wfree(wstate->instance);
+        if (wstate->class)    wfree(wstate->class);
+        if (wstate->command)  wfree(wstate->command);
+        wfree(wstate->state);
+        wfree(wstate);
     } else {
 	while (tmp->next) {
 	    if (tmp->next==wstate) {
@@ -2538,11 +2672,11 @@ wWindowDeleteSavedState(WMagicNumber id)
                 printf("Deleted WindowState with ID %p, for %s.%s : \"%s\"\n",
                        wstate, wstate->instance, wstate->class, wstate->command);
 #endif
-                if (wstate->instance) free(wstate->instance);
-                if (wstate->class)    free(wstate->class);
-                if (wstate->command)  free(wstate->command);
-                free(wstate->state);
-                free(wstate);
+                if (wstate->instance) wfree(wstate->instance);
+                if (wstate->class)    wfree(wstate->class);
+                if (wstate->command)  wfree(wstate->command);
+                wfree(wstate->state);
+                wfree(wstate);
 		break;
 	    }
 	    tmp = tmp->next;
@@ -2567,11 +2701,11 @@ wWindowDeleteSavedStatesForPID(pid_t pid)
         printf("Deleted WindowState with ID %p, for %s.%s : \"%s\"\n",
                wstate, wstate->instance, wstate->class, wstate->command);
 #endif
-        if (wstate->instance) free(wstate->instance);
-        if (wstate->class)    free(wstate->class);
-        if (wstate->command)  free(wstate->command);
-        free(wstate->state);
-        free(wstate);
+        if (wstate->instance) wfree(wstate->instance);
+        if (wstate->class)    wfree(wstate->class);
+        if (wstate->command)  wfree(wstate->command);
+        wfree(wstate->state);
+        wfree(wstate);
     } else {
 	while (tmp->next) {
 	    if (tmp->next->pid==pid) {
@@ -2581,11 +2715,11 @@ wWindowDeleteSavedStatesForPID(pid_t pid)
                 printf("Deleted WindowState with ID %p, for %s.%s : \"%s\"\n",
                        wstate, wstate->instance, wstate->class, wstate->command);
 #endif
-                if (wstate->instance) free(wstate->instance);
-                if (wstate->class)    free(wstate->class);
-                if (wstate->command)  free(wstate->command);
-                free(wstate->state);
-                free(wstate);
+                if (wstate->instance) wfree(wstate->instance);
+                if (wstate->class)    wfree(wstate->class);
+                if (wstate->command)  wfree(wstate->command);
+                wfree(wstate->state);
+                wfree(wstate);
 		break;
 	    }
 	    tmp = tmp->next;
@@ -2676,10 +2810,11 @@ titlebarDblClick(WCoreWindow *sender, void *data, XEvent *event)
 
 	    /* maximize window */
 	    if (dir !=0 && !WFLAGP(wwin, no_resizable)) {
-		if (wwin->flags.maximized) 
+		int ndir = dir ^ wwin->flags.maximized;
+		if (wwin->flags.maximized != 0)
 		    wUnmaximizeWindow(wwin);
-		 else
-		    wMaximizeWindow(wwin, dir);
+		if (ndir != 0)
+		    wMaximizeWindow(wwin, ndir);
 	    }
 	}
     } else if (event->xbutton.button==Button3) {
@@ -2701,8 +2836,8 @@ frameMouseDown(WObjDescriptor *desc, XEvent *event)
     
     CloseWindowMenu(wwin->screen_ptr);
 
-    if (wPreferences.focus_mode==WKF_CLICK
-	&& !(event->xbutton.state&ControlMask)
+    if (/*wPreferences.focus_mode==WKF_CLICK
+	&&*/ !(event->xbutton.state&ControlMask)
 	&& !WFLAGP(wwin, no_focusable)) {
 	wSetFocusTo(wwin->screen_ptr, wwin);
     }
