@@ -45,9 +45,87 @@
 #include "actions.h"
 #include "winspector.h"
 #include "dock.h"
+#include "client.h"
 
 #include <proplist.h>
 
+
+
+
+
+
+
+typedef struct InspectorPanel {
+    struct InspectorPanel *nextPtr;
+
+    WWindow *frame;
+
+    WWindow *inspected;       /* the window that's being inspected */
+
+    WMWindow *win;
+    
+    Window parent;
+
+    /* common stuff */
+    WMButton *revertBtn;
+    WMButton *applyBtn;
+    WMButton *saveBtn;
+
+    WMPopUpButton *pagePopUp;
+
+    /* first page. general stuff */
+    
+    WMFrame *specFrm;
+    WMButton *instRb;
+    WMButton *clsRb;
+    WMButton *bothRb;
+    WMButton *defaultRb;
+
+    WMButton *selWinB;
+    
+    WMLabel *specLbl;
+    
+    /* second page. attributes */
+
+    WMFrame *attrFrm;
+    WMButton *attrChk[11];
+
+    /* 3rd page. more attributes */
+    WMFrame *moreFrm;
+#ifdef XKB_BUTTON_HINT
+    WMButton *moreChk[9];
+#else
+    WMButton *moreChk[8];
+#endif
+
+    /* 4th page. icon and workspace */
+    WMFrame *iconFrm;
+    WMLabel *iconLbl;
+    WMLabel *fileLbl;
+    WMTextField *fileText;
+    WMButton *alwChk;
+    /*
+    WMButton *updateIconBtn;
+     */
+    WMButton *browseIconBtn;
+    
+    WMFrame *wsFrm;
+    WMPopUpButton *wsP;
+    
+    /* 5th page. application wide attributes */
+    WMFrame *appFrm;
+    WMButton *appChk[2];
+
+    unsigned int done:1;
+    unsigned int destroyed:1;
+    unsigned int choosingIcon:1;
+} InspectorPanel;
+
+
+
+
+extern Cursor wCursor[WCUR_LAST];
+
 extern WDDomain *WDWindowAttributes;
 
 static InspectorPanel *panelList=NULL;
@@ -58,6 +136,7 @@ static proplist_t ANoTitlebar = NULL;
 static proplist_t ANoResizebar;
 static proplist_t ANoMiniaturizeButton;
 static proplist_t ANoCloseButton;
+static proplist_t ANoBorder;
 static proplist_t ANoHideOthers;
 static proplist_t ANoMouseBindings;
 static proplist_t ANoKeyBindings;
@@ -95,8 +174,23 @@ static proplist_t Yes, No;
 #define PHEIGHT	350
 
 
+
+static char *spec_text;
+
+
 static void applySettings(WMButton *button, InspectorPanel *panel);
 
+
+
+
+#define UNDEFINED_POS 0xffffff
+
+static InspectorPanel *createInspectorForWindow(WWindow *wwin, 
+						int xpos, int ypos,
+						Bool showSelectPanel);
+
+
+
 static void
 make_keys()
 {
@@ -108,6 +202,7 @@ make_keys()
     ANoResizebar = PLMakeString("NoResizebar");
     ANoMiniaturizeButton = PLMakeString("NoMiniaturizeButton");
     ANoCloseButton = PLMakeString("NoCloseButton");
+    ANoBorder = PLMakeString("NoBorder");    
     ANoHideOthers = PLMakeString("NoHideOthers");
     ANoMouseBindings = PLMakeString("NoMouseBindings");
     ANoKeyBindings = PLMakeString("NoKeyBindings");
@@ -150,7 +245,7 @@ freeInspector(InspectorPanel *panel)
 
     XDestroyWindow(dpy, panel->parent);
 
-    free(panel);
+    wfree(panel);
 }
 
 
@@ -200,7 +295,7 @@ wDestroyInspectorPanels()
         panel->inspected->flags.inspector_open = 0;
         panel->inspected->inspector = NULL;
 
-        free(panel);
+        wfree(panel);
     }
 }
 
@@ -259,7 +354,7 @@ showIconFor(WMScreen *scrPtr, InspectorPanel *panel,
     if ((flags & USE_TEXT_FIELD) != 0) {
         file = WMGetTextFieldText(panel->fileText);
         if (file && file[0] == 0) {
-            free(file);
+            wfree(file);
             file = NULL;
         }
     } else {
@@ -289,13 +384,13 @@ showIconFor(WMScreen *scrPtr, InspectorPanel *panel,
 		    file);
             wMessageDialog(panel->frame->screen_ptr, _("Error"), buf, 
 			   _("OK"), NULL, NULL);
-	    free(buf);
-	    free(file);
+	    wfree(buf);
+	    wfree(file);
             return -1;
         }
 
         pixmap = WMCreatePixmapFromFile(scrPtr, path);
-        free(path);
+        wfree(path);
 
         if (!pixmap) {
 	    char *buf;
@@ -305,11 +400,11 @@ showIconFor(WMScreen *scrPtr, InspectorPanel *panel,
 		    file, RMessageForError(RErrorCode));
             wMessageDialog(panel->frame->screen_ptr, _("Error"), buf,
 			   _("OK"), NULL, NULL);
-	    free(buf);
-	    free(file);
+	    wfree(buf);
+	    wfree(file);
             return -1;
         }
-        free(file);
+        wfree(file);
     }
 
     WMSetLabelImage(panel->iconLbl, pixmap);
@@ -371,12 +466,13 @@ getBool(proplist_t value)
  */
 
 
-static void
+static int
 insertAttribute(proplist_t dict, proplist_t window, proplist_t attr,
-                proplist_t value, int *modified, int flags)
+                proplist_t value, int flags)
 {
     proplist_t def_win, def_value=NULL;
     int update = 0;
+    int modified = 0;
 
     if (!(flags & UPDATE_DEFAULTS) && dict) {
         if ((def_win = PLGetDictionaryEntry(dict, AnyWindow)) != NULL) {
@@ -398,8 +494,10 @@ insertAttribute(proplist_t dict, proplist_t window, proplist_t attr,
 
     if (update) {
         PLInsertDictionaryEntry(window, attr, value);
-        *modified = 1;
+        modified = 1;
     }
+
+    return modified;
 }
 
 
@@ -410,7 +508,7 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     WDDomain *db = WDWindowAttributes;
     proplist_t dict = db->dictionary;
     proplist_t winDic, value, key;
-    char buffer[256], *icon_file;
+    char *icon_file;
     int flags = 0;
     int different = 0;
 
@@ -422,14 +520,16 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     else if (WMGetButtonSelected(panel->clsRb) != 0)
         key = PLMakeString(wwin->wm_class);
     else if (WMGetButtonSelected(panel->bothRb) != 0) {
+		char *buffer;
+
+		buffer = wmalloc(strlen(wwin->wm_instance)+strlen(wwin->wm_class)+4);
         strcat(strcat(strcpy(buffer, wwin->wm_instance), "."), wwin->wm_class);
         key = PLMakeString(buffer);
-    }
-    else if (WMGetButtonSelected(panel->defaultRb) != 0) {
+        wfree(buffer);
+    } else if (WMGetButtonSelected(panel->defaultRb) != 0) {
         key = PLRetain(AnyWindow);
         flags = UPDATE_DEFAULTS;
-    }
-    else
+    } else
         key = NULL;
 
     if (!key)
@@ -462,10 +562,10 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     if (icon_file) {
         if (icon_file[0] != 0) {
             value = PLMakeString(icon_file);
-            insertAttribute(dict, winDic, AIcon, value, &different, flags);
+            different |= insertAttribute(dict, winDic, AIcon, value, flags);
             PLRelease(value);
         }
-        free(icon_file);
+        wfree(icon_file);
     }
 
     {
@@ -475,7 +575,8 @@ saveSettings(WMButton *button, InspectorPanel *panel)
 
 	if (i>=0 && i < panel->frame->screen_ptr->workspace_count) {
 	    value = PLMakeString(panel->frame->screen_ptr->workspaces[i]->name);
-	    insertAttribute(dict, winDic, AStartWorkspace, value, &different, flags);
+	    different |= insertAttribute(dict, winDic, AStartWorkspace, value, 
+					 flags);
 	    PLRelease(value);
         }
     }
@@ -483,66 +584,69 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     flags |= IS_BOOLEAN;
 
     value = (WMGetButtonSelected(panel->alwChk)!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AAlwaysUserIcon,    value, &different, flags);
+    different |= insertAttribute(dict, winDic, AAlwaysUserIcon, value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[0])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ANoTitlebar,        value, &different, flags);
+    different |= insertAttribute(dict, winDic, ANoTitlebar,     value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[1])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ANoResizebar,       value, &different, flags);
+    different |= insertAttribute(dict, winDic, ANoResizebar,     value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[2])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ANoCloseButton,       value, &different, flags);
+    different |= insertAttribute(dict, winDic, ANoCloseButton,   value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[3])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ANoMiniaturizeButton, value, &different, flags);
+    different |= insertAttribute(dict, winDic, ANoMiniaturizeButton, value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[4])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AKeepOnTop,         value, &different, flags);
+    different |= insertAttribute(dict, winDic, ANoBorder, value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[5])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AKeepOnBottom,         value, &different, flags);
+    different |= insertAttribute(dict, winDic, AKeepOnTop,       value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[6])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AOmnipresent,       value, &different, flags);
+    different |= insertAttribute(dict, winDic, AKeepOnBottom,    value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[7])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AStartMiniaturized, value, &different, flags);
+    different |= insertAttribute(dict, winDic, AOmnipresent,     value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[8])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AStartMaximized,    value, &different, flags);
+    different |= insertAttribute(dict, winDic, AStartMiniaturized, value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[9])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ASkipWindowList,    value, &different, flags);
+    different |= insertAttribute(dict, winDic, AStartMaximized,  value, flags);
+
+    value = (WMGetButtonSelected(panel->attrChk[10])!=0) ? Yes : No;
+    different |= insertAttribute(dict, winDic, ASkipWindowList,  value, flags);
 
 
     value = (WMGetButtonSelected(panel->moreChk[0])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ANoHideOthers,      value, &different, flags);
+    different |= insertAttribute(dict, winDic, ANoHideOthers,    value, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[1])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ANoKeyBindings,     value, &different, flags);
+    different |= insertAttribute(dict, winDic, ANoKeyBindings,   value, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[2])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ANoMouseBindings,   value, &different, flags);
+    different |= insertAttribute(dict, winDic, ANoMouseBindings, value, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[3])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AKeepInsideScreen,  value, &different, flags);
+    different |= insertAttribute(dict, winDic, AKeepInsideScreen,value, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[4])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AUnfocusable, value, &different, flags);
+    different |= insertAttribute(dict, winDic, AUnfocusable, value, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[5])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ADontSaveSession,   value, &different, flags);
+    different |= insertAttribute(dict, winDic, ADontSaveSession, value, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[6])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AEmulateAppIcon,   value, &different, flags);
+    different |= insertAttribute(dict, winDic, AEmulateAppIcon,  value, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[7])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, AFullMaximize,   value, &different, flags);
+    different |= insertAttribute(dict, winDic, AFullMaximize, value, flags);
 
 #ifdef XKB_BUTTON_HINT
     value = (WMGetButtonSelected(panel->moreChk[8])!=0) ? Yes : No;
-    insertAttribute(dict, winDic, ANoLanguageButton,  value, &different, flags);
+    different |= insertAttribute(dict, winDic, ANoLanguageButton, value, flags);
 #endif
 
     /* application wide settings for when */
@@ -550,10 +654,10 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     if (panel->inspected->main_window == panel->inspected->client_win) {
 	
 	value = (WMGetButtonSelected(panel->appChk[0])!=0) ? Yes : No;
-	insertAttribute(dict, winDic, AStartHidden, value, &different, flags);
+	different |= insertAttribute(dict, winDic, AStartHidden, value, flags);
 
 	value = (WMGetButtonSelected(panel->appChk[1])!=0) ? Yes : No;
-	insertAttribute(dict, winDic, ANoAppIcon, value, &different, flags);
+	different |= insertAttribute(dict, winDic, ANoAppIcon, value, flags);
     } 
 
     PLRemoveDictionaryEntry(dict, key);
@@ -575,15 +679,19 @@ saveSettings(WMButton *button, InspectorPanel *panel)
 	wapp = wApplicationOf(panel->inspected->main_window);
 	if (wapp) {
 	    char *iconFile;
+		char *buffer;
 
 	    appDic = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
 
 	    assert(wapp->main_window_desc->wm_instance!=NULL);
 	    assert(wapp->main_window_desc->wm_class!=NULL);
 
+		buffer = wmalloc(strlen(wapp->main_window_desc->wm_instance)
+						 +strlen(wapp->main_window_desc->wm_class)+4);
 	    strcat(strcpy(buffer, wapp->main_window_desc->wm_instance), ".");
-	    strcat(buffer, wwin->wm_class);
+	    strcat(buffer, wapp->main_window_desc->wm_class);
 	    key = PLMakeString(buffer);
+            wfree(buffer);
 
 	    iconFile = wDefaultGetIconFile(wwin->screen_ptr, 
 					   wapp->main_window_desc->wm_instance,
@@ -592,16 +700,18 @@ saveSettings(WMButton *button, InspectorPanel *panel)
 
 	    if (iconFile && iconFile[0]!=0) {
 		value = PLMakeString(iconFile);
-		insertAttribute(dict, appDic, AIcon, value, &different, 
-				flags&~IS_BOOLEAN);
+		different |= insertAttribute(dict, appDic, AIcon, value,
+					     flags&~IS_BOOLEAN);
 		PLRelease(value);
 	    }
 		
 	    value = (WMGetButtonSelected(panel->appChk[0])!=0) ? Yes : No;
-	    insertAttribute(dict, appDic, AStartHidden,  value, &different, flags);
+	    different |= insertAttribute(dict, appDic, AStartHidden,  value, 
+					 flags);
 
 	    value = (WMGetButtonSelected(panel->appChk[1])!=0) ? Yes : No;
-	    insertAttribute(dict, appDic, ANoAppIcon,  value, &different, flags);
+	    different |= insertAttribute(dict, appDic, ANoAppIcon,  value, 
+					 flags);
 
 	    PLRemoveDictionaryEntry(dict, key);
 	    if (different) {
@@ -641,7 +751,7 @@ makeAppIconFor(WApplication *wapp)
 
         if (clip && clip->attract_icons && wDockFindFreeSlot(clip, &x, &y)) {
             wapp->app_icon->attracted = 1;
-	    if (!clip->keep_attracted && !wapp->app_icon->icon->shadowed) {
+	    if (!wapp->app_icon->icon->shadowed) {
 		wapp->app_icon->icon->shadowed = 1;
 		wapp->app_icon->icon->force_paint = 1;
 	    }
@@ -665,8 +775,7 @@ removeAppIconFor(WApplication *wapp)
     if (!wapp->app_icon)
         return;
 
-    if (wapp->app_icon->docked && 
-	(!wapp->app_icon->attracted || wapp->app_icon->dock->keep_attracted)) {
+    if (wapp->app_icon->docked && !wapp->app_icon->attracted) {
         wapp->app_icon->running = 0;
         /* since we keep it, we don't care if it was attracted or not */
         wapp->app_icon->attracted = 0;
@@ -709,12 +818,13 @@ applySettings(WMButton *button, InspectorPanel *panel)
     WSETUFLAG(wwin, no_resizebar, WMGetButtonSelected(panel->attrChk[1]));
     WSETUFLAG(wwin, no_close_button, WMGetButtonSelected(panel->attrChk[2]));
     WSETUFLAG(wwin, no_miniaturize_button, WMGetButtonSelected(panel->attrChk[3]));
-    floating                   =  WMGetButtonSelected(panel->attrChk[4]);
-    sunken                     =  WMGetButtonSelected(panel->attrChk[5]);
-    WSETUFLAG(wwin, omnipresent, WMGetButtonSelected(panel->attrChk[6]));
-    WSETUFLAG(wwin, start_miniaturized, WMGetButtonSelected(panel->attrChk[7]));
-    WSETUFLAG(wwin, start_maximized, WMGetButtonSelected(panel->attrChk[8]));
-    skip_window_list           =  WMGetButtonSelected(panel->attrChk[9]);
+    WSETUFLAG(wwin, no_border, WMGetButtonSelected(panel->attrChk[4]));
+    floating                   =  WMGetButtonSelected(panel->attrChk[5]);
+    sunken                     =  WMGetButtonSelected(panel->attrChk[6]);
+    WSETUFLAG(wwin, omnipresent, WMGetButtonSelected(panel->attrChk[7]));
+    WSETUFLAG(wwin, start_miniaturized, WMGetButtonSelected(panel->attrChk[8]));
+    WSETUFLAG(wwin, start_maximized, WMGetButtonSelected(panel->attrChk[9]));
+    skip_window_list           =  WMGetButtonSelected(panel->attrChk[10]);
 
     WSETUFLAG(wwin, no_hide_others, WMGetButtonSelected(panel->moreChk[0]));
     WSETUFLAG(wwin, no_bind_keys, WMGetButtonSelected(panel->moreChk[1]));
@@ -796,12 +906,12 @@ applySettings(WMButton *button, InspectorPanel *panel)
             char *file = WMGetTextFieldText(panel->fileText);
 
 	    if (file[0] == 0) {
-		free(file);
+		wfree(file);
 		file = NULL;
 	    }
 	    wIconChangeImageFile(wapp->app_icon->icon, file);
 	    if (file)
-		free(file);
+		wfree(file);
 	    wAppIconPaint(wapp->app_icon);
         }
     }
@@ -834,7 +944,7 @@ revertSettings(WMButton *button, InspectorPanel *panel)
 
     wWindowSetupInitialAttributes(wwin, &level, &workspace);
 
-    for (i=0; i < 10; i++) {
+    for (i=0; i < 11; i++) {
 	int flag = 0;
 	
 	switch (i) {
@@ -851,21 +961,24 @@ revertSettings(WMButton *button, InspectorPanel *panel)
 	    flag = WFLAGP(wwin, no_miniaturize_button);
 	    break;
 	 case 4:
-            flag = WFLAGP(wwin, floating);
+	    flag = WFLAGP(wwin, no_border);
 	    break;
 	 case 5:
-            flag = WFLAGP(wwin, sunken);
+            flag = WFLAGP(wwin, floating);
 	    break;
 	 case 6:
-	    flag = WFLAGP(wwin, omnipresent);
+            flag = WFLAGP(wwin, sunken);
 	    break;
 	 case 7:
-	    flag = WFLAGP(wwin, start_miniaturized);
+	    flag = WFLAGP(wwin, omnipresent);
 	    break;
 	 case 8:
-	    flag = WFLAGP(wwin, start_maximized!=0);
+	    flag = WFLAGP(wwin, start_miniaturized);
 	    break;
 	 case 9:
+	    flag = WFLAGP(wwin, start_maximized!=0);
+	    break;
+	 case 10:
 	    flag = WFLAGP(wwin, skip_window_list);
 	    break;
 	}
@@ -934,6 +1047,11 @@ revertSettings(WMButton *button, InspectorPanel *panel)
     } else {
 	WMSetPopUpButtonSelectedItem(panel->wsP, 0);
     }
+    
+    /* must auto apply, so that there wno't be internal
+     * inconsistencies between the state in the flags and
+     * the actual state of the window */
+    applySettings(panel->applyBtn, panel);
 }
 
 
@@ -959,7 +1077,7 @@ chooseIconCallback(WMWidget *self, void *clientData)
 	    WMSetTextFieldText(panel->fileText, file);
 	    showIconFor(WMWidgetScreen(self), panel, NULL, NULL,
 			USE_TEXT_FIELD);
-	    free(file);
+	    wfree(file);
 	}
 	WMSetButtonEnabled(panel->browseIconBtn, True);
     } else {
@@ -991,7 +1109,8 @@ selectSpecification(WMWidget *bPtr, void *data)
     char *str;
     WWindow *wwin = panel->inspected;
 
-    if (bPtr == panel->defaultRb) {
+
+    if (bPtr == panel->defaultRb && (wwin->wm_instance || wwin->wm_class)) {
 	WMSetButtonEnabled(panel->applyBtn, False);
     } else {
 	WMSetButtonEnabled(panel->applyBtn, True);
@@ -1000,18 +1119,59 @@ selectSpecification(WMWidget *bPtr, void *data)
     str = wmalloc(16 + strlen(wwin->wm_instance ? wwin->wm_instance : "?")
 		  + strlen(wwin->wm_class ? wwin->wm_class : "?"));
 
-    sprintf(str, "Inspecting  %s.%s",
+    sprintf(str, _("Inspecting  %s.%s"),
 	    wwin->wm_instance ? wwin->wm_instance : "?",
 	    wwin->wm_class ? wwin->wm_class : "?");
 
     wFrameWindowChangeTitle(panel->frame->frame, str);
 
-    free(str);
+    wfree(str);
 }
 
 
+static void
+selectWindow(WMWidget *bPtr, void *data)
+{
+    InspectorPanel *panel = (InspectorPanel*)data;
+    WWindow *wwin = panel->inspected;
+    WScreen *scr = wwin->screen_ptr;
+    XEvent event;
+    WWindow *iwin;
+    
+    if (XGrabPointer(dpy, scr->root_win, True,
+		     ButtonPressMask, GrabModeAsync, GrabModeAsync, None,
+		     wCursor[WCUR_SELECT], CurrentTime)!=GrabSuccess) {
+	wwarning("could not grab mouse pointer");
+	return;
+    }
+    
+    WMSetLabelText(panel->specLbl, _("Click in the window you wish to inspect."));
+
+    WMMaskEvent(dpy, ButtonPressMask, &event);
+
+    XUngrabPointer(dpy, CurrentTime);
+
+    iwin = wWindowFor(event.xbutton.subwindow);
+    
+    if (iwin && !iwin->flags.internal_window && iwin != wwin
+	&& !iwin->flags.inspector_open) {
+
+	iwin->flags.inspector_open = 1;
+	iwin->inspector = createInspectorForWindow(iwin,
+						   panel->frame->frame_x,
+						   panel->frame->frame_y,
+						   True);
+	wCloseInspectorForWindow(wwin);
+    } else {
+	WMSetLabelText(panel->specLbl, spec_text);
+    }
+}
+
+
+
 static InspectorPanel*
-createInspectorForWindow(WWindow *wwin)
+createInspectorForWindow(WWindow *wwin, int xpos, int ypos, 
+			 Bool showSelectPanel)
 {
     WScreen *scr = wwin->screen_ptr;
     InspectorPanel *panel;
@@ -1024,6 +1184,10 @@ createInspectorForWindow(WWindow *wwin)
     WMPixmap *pixmap;
 #endif
 
+    spec_text = _("The configuration will apply to all\n"
+			"windows that have their WM_CLASS\n"
+			"property set to the above selected\n"
+			"name, when saved.");
 
     panel = wmalloc(sizeof(InspectorPanel));
     memset(panel, 0, sizeof(InspectorPanel));
@@ -1084,7 +1248,7 @@ createInspectorForWindow(WWindow *wwin)
     panel->specFrm = WMCreateFrame(panel->win);
     WMSetFrameTitle(panel->specFrm, _("Window Specification"));
     WMMoveWidget(panel->specFrm, 15, 65);
-    WMResizeWidget(panel->specFrm, frame_width, 105);
+    WMResizeWidget(panel->specFrm, frame_width, 145);
 
 
     panel->defaultRb = WMCreateRadioButton(panel->specFrm);
@@ -1097,14 +1261,15 @@ createInspectorForWindow(WWindow *wwin)
     if (wwin->wm_class && wwin->wm_instance) {
 	char *str, *tmp;
 
-        tmp = wstrappend(wwin->wm_instance, ".");
-        str = wstrappend(tmp, wwin->wm_class);
+        tmp = wstrconcat(wwin->wm_instance, ".");
+        str = wstrconcat(tmp, wwin->wm_class);
+
 	panel->bothRb = WMCreateRadioButton(panel->specFrm);
 	WMMoveWidget(panel->bothRb, 10, 18);
 	WMResizeWidget(panel->bothRb, frame_width - (2 * 10), 20);
 	WMSetButtonText(panel->bothRb, str);
-        free(tmp);
-        free(str);
+        wfree(tmp);
+        wfree(str);
 	WMGroupButtons(panel->defaultRb, panel->bothRb);
 
 	if (!selectedBtn)
@@ -1139,17 +1304,21 @@ createInspectorForWindow(WWindow *wwin)
 	WMSetButtonAction(panel->clsRb, selectSpecification, panel);
     }
 
-    if (selectedBtn)
-        WMSetButtonSelected(selectedBtn, True);
-
+    
+    panel->selWinB = WMCreateCommandButton(panel->specFrm);
+    WMMoveWidget(panel->selWinB, 20, 145-24 - 10);
+    WMResizeWidget(panel->selWinB, frame_width - 2*10 - 20, 24);
+    WMSetButtonText(panel->selWinB, _("Select Window"));
+    WMSetButtonAction(panel->selWinB, selectWindow, panel);
+    
+    
     panel->specLbl = WMCreateLabel(panel->win);
-    WMMoveWidget(panel->specLbl, 15, 170);
+    WMMoveWidget(panel->specLbl, 15, 210);
     WMResizeWidget(panel->specLbl, frame_width, 100);
-    WMSetLabelText(panel->specLbl,
-		   _("The configuration will apply to all\n"
-		     "windows that have their WM_CLASS property"
-		     " set to the above selected\nname, when saved."));
-    WMSetLabelTextAlignment(panel->specLbl, WACenter);
+    WMSetLabelText(panel->specLbl, spec_text);
+    WMSetLabelWraps(panel->specLbl, True);
+    
+    WMSetLabelTextAlignment(panel->specLbl, WALeft);
     
     /**** attributes ****/
     panel->attrFrm = WMCreateFrame(panel->win);
@@ -1157,7 +1326,7 @@ createInspectorForWindow(WWindow *wwin)
     WMMoveWidget(panel->attrFrm, 15, 45);
     WMResizeWidget(panel->attrFrm, frame_width, 250);
 
-    for (i=0; i < 10; i++) {
+    for (i=0; i < 11; i++) {
 	char *caption = NULL;
 	int flag = 0;
 	char *descr = NULL;
@@ -1188,34 +1357,39 @@ createInspectorForWindow(WWindow *wwin)
 	    descr = _("Remove the `miniaturize window' button of the window.");
 	    break;
 	 case 4:
+	    caption = _("Disable Border");
+	    flag = WFLAGP(wwin, no_border);
+	    descr = _("Remove the 1 pixel black border around the window.");
+	    break;
+	 case 5:
 	    caption = _("Keep on Top / Floating");
 	    flag = WFLAGP(wwin, floating);
 	    descr = _("Keep the window over other windows, not allowing\n"
 		      "them to covert it.");
 	    break;
-	 case 5:
+	 case 6:
 	    caption = _("Keep at Bottom / Sunken");
 	    flag = WFLAGP(wwin, sunken);
 	    descr = _("Keep the window under all other windows.");
 	    break;
-	 case 6:
+	 case 7:
 	    caption = _("Omnipresent");
 	    flag = WFLAGP(wwin, omnipresent);
 	    descr = _("Make window occupy all workspaces.");
 	    break;
-	 case 7:
+	 case 8:
 	    caption = _("Start Miniaturized");
 	    flag = WFLAGP(wwin, start_miniaturized);
 	    descr = _("Make the window be automatically miniaturized when it's\n"
 		    "first shown.");
 	    break;
-	 case 8:
+	 case 9:
 	    caption = _("Start Maximized");
 	    flag = WFLAGP(wwin, start_maximized!=0);
 	    descr = _("Make the window be automatically maximized when it's\n"
 		    "first shown.");
 	    break;
-	 case 9:
+	 case 10:
 	    caption = _("Skip Window List");
 	    flag = WFLAGP(wwin, skip_window_list);
 	    descr = _("Do not list the window in the window list menu.");
@@ -1237,11 +1411,13 @@ createInspectorForWindow(WWindow *wwin)
     WMMoveWidget(panel->moreFrm, 15, 45);
     WMResizeWidget(panel->moreFrm, frame_width, 250);
 
+    for (i=0;
 #ifdef XKB_BUTTON_HINT
-    for (i=0; i < 9; i++) {
+	 i < 9;
 #else
-    for (i=0; i < 8; i++) {
+	 i < 8;
 #endif
+	 i++) {
 	char *caption = NULL;
 	int flag = 0;
 	char *descr = NULL;
@@ -1326,24 +1502,17 @@ createInspectorForWindow(WWindow *wwin)
     WMSetFrameTitle(panel->iconFrm, _("Miniwindow Image"));
 
     panel->iconLbl = WMCreateLabel(panel->iconFrm);
-    WMMoveWidget(panel->iconLbl, PWIDTH - (2 * 15) - 22 - 64, 30);
+    WMMoveWidget(panel->iconLbl, PWIDTH - (2 * 15) - 22 - 64, 20);
     WMResizeWidget(panel->iconLbl, 64, 64);
     WMSetLabelRelief(panel->iconLbl, WRGroove);
     WMSetLabelImagePosition(panel->iconLbl, WIPImageOnly);
     
     panel->browseIconBtn = WMCreateCommandButton(panel->iconFrm);
     WMSetButtonAction(panel->browseIconBtn, chooseIconCallback, panel);
-    WMMoveWidget(panel->browseIconBtn, 22, 30);
-    WMResizeWidget(panel->browseIconBtn, 100, 26);
+    WMMoveWidget(panel->browseIconBtn, 22, 32);
+    WMResizeWidget(panel->browseIconBtn, 120, 26);
     WMSetButtonText(panel->browseIconBtn, _("Browse..."));
 
-#if 0
-    panel->updateIconBtn = WMCreateCommandButton(panel->iconFrm);
-    WMSetButtonAction(panel->updateIconBtn, (WMAction*)updateIcon, panel);
-    WMMoveWidget(panel->updateIconBtn, 22, 65);
-    WMResizeWidget(panel->updateIconBtn, 100, 26);
-    WMSetButtonText(panel->updateIconBtn, _("Update"));
-#endif
 #ifdef wrong_behaviour
     WMSetButtonImagePosition(panel->updateIconBtn, WIPRight);
     pixmap = WMGetSystemPixmap(scr->wmscreen, WSIReturnArrow);
@@ -1355,20 +1524,21 @@ createInspectorForWindow(WWindow *wwin)
 #endif
 
     panel->fileLbl = WMCreateLabel(panel->iconFrm);
-    WMMoveWidget(panel->fileLbl, 20, 95);
+    WMMoveWidget(panel->fileLbl, 20, 85);
     WMResizeWidget(panel->fileLbl, PWIDTH - (2 * 15) - (2 * 20), 14);
     WMSetLabelText(panel->fileLbl, _("Icon File Name:"));
 
     panel->fileText = WMCreateTextField(panel->iconFrm);
-    WMMoveWidget(panel->fileText, 20, 115);
-    WMResizeWidget(panel->fileText, PWIDTH - (2 * 15) - (2 * 15), 20);
+    WMMoveWidget(panel->fileText, 20, 105);
+    WMResizeWidget(panel->fileText, PWIDTH - (2 * 20) - (2 * 15), 20);
     WMSetTextFieldText(panel->fileText, NULL);
     WMAddNotificationObserver(textEditedObserver, panel, 
 			      WMTextDidEndEditingNotification, 
 			      panel->fileText);
+
     panel->alwChk = WMCreateSwitchButton(panel->iconFrm);
-    WMMoveWidget(panel->alwChk, 20, 140);
-    WMResizeWidget(panel->alwChk, PWIDTH - (2 * 15) - (2 * 15), 20);
+    WMMoveWidget(panel->alwChk, 20, 130);
+    WMResizeWidget(panel->alwChk, PWIDTH - (2 * 15) - (2 * 15), 30);
     WMSetButtonText(panel->alwChk, _("Ignore client supplied icon"));
     WMSetButtonSelected(panel->alwChk, WFLAGP(wwin, always_user_icon));
 
@@ -1480,8 +1650,13 @@ createInspectorForWindow(WWindow *wwin)
     if (panel->appFrm)
 	WMMapSubwidgets(panel->appFrm);
 
-    WMSetPopUpButtonSelectedItem(panel->pagePopUp, 1);
-    changePage(panel->pagePopUp, panel);
+    if (showSelectPanel) {
+	WMSetPopUpButtonSelectedItem(panel->pagePopUp, 0);
+	changePage(panel->pagePopUp, panel);
+    } else {
+	WMSetPopUpButtonSelectedItem(panel->pagePopUp, 1);
+	changePage(panel->pagePopUp, panel);
+    }
 
     
     parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, PWIDTH, PHEIGHT, 
@@ -1494,12 +1669,17 @@ createInspectorForWindow(WWindow *wwin)
 
     XSetTransientForHint(dpy, parent, wwin->client_win);
 
-    x = wwin->frame_x+wwin->frame->core->width/2;
-    y = wwin->frame_y+wwin->frame->top_width*2;
-    if (y + PHEIGHT > scr->scr_height)
-	y = scr->scr_height - PHEIGHT - 30;
-    if (x + PWIDTH > scr->scr_width)
-	x = scr->scr_width - PWIDTH;
+    if (xpos == UNDEFINED_POS) {
+	x = wwin->frame_x+wwin->frame->core->width/2;
+	y = wwin->frame_y+wwin->frame->top_width*2;
+	if (y + PHEIGHT > scr->scr_height)
+	    y = scr->scr_height - PHEIGHT - 30;
+	if (x + PWIDTH > scr->scr_width)
+	    x = scr->scr_width - PWIDTH;
+    } else {
+	x = xpos;
+	y = ypos;
+    }
 
     panel->frame = wManageInternalWindow(scr, parent, wwin->client_win, 
 					 "Inspector", x, y, PWIDTH, PHEIGHT);
@@ -1507,6 +1687,8 @@ createInspectorForWindow(WWindow *wwin)
     if (!selectedBtn)
 	selectedBtn = panel->defaultRb;
 
+    WMSetButtonSelected(selectedBtn, True);
+
     selectSpecification(selectedBtn, panel);
 
     /* kluge to know who should get the key events */
@@ -1537,7 +1719,58 @@ wShowInspectorForWindow(WWindow *wwin)
 
     make_keys();
     wwin->flags.inspector_open = 1;
-    wwin->inspector = createInspectorForWindow(wwin);;
+    wwin->inspector = createInspectorForWindow(wwin, UNDEFINED_POS,
+					       UNDEFINED_POS, False);
+}
+
+
+
+    
+void
+wHideInspectorForWindow(WWindow *wwin)
+{
+    WWindow *pwin = wwin->inspector->frame;
+
+    wWindowUnmap(pwin);
+    pwin->flags.hidden = 1;
+
+    wClientSetState(pwin, IconicState, None);
+}
+
+
+
+void
+wUnhideInspectorForWindow(WWindow *wwin)
+{
+    WWindow *pwin = wwin->inspector->frame;
+
+    pwin->flags.hidden = 0;
+    pwin->flags.mapped = 1;
+    XMapWindow(dpy, pwin->client_win);
+    XMapWindow(dpy, pwin->frame->core->window);
+    wClientSetState(pwin, NormalState, None);
+}
+
+
+
+WWindow*
+wGetWindowOfInspectorForWindow(WWindow *wwin)
+{    
+    if (wwin->inspector) {
+	assert(wwin->flags.inspector_open != 0);
+	
+	return wwin->inspector->frame;
+    } else
+	return NULL;
+}
+
+
+void
+wCloseInspectorForWindow(WWindow *wwin)
+{
+    WWindow *pwin = wwin->inspector->frame; /* the inspector window */
+
+    (*pwin->frame->on_click_right)(NULL, pwin, NULL);
 }
 
 
