@@ -46,13 +46,12 @@
 
 #include "../src/wconfig.h"
 
-#include "../WINGs/WINGs.h"
-#include "../WINGs/WUtil.h"
-#include "../wrlib/wraster.h"
+#include <WINGs/WINGs.h>
+#include <wraster.h>
 
 #include <proplist.h>
 
-#define PROG_VERSION	"wmsetbg (Window Maker) 2.3"
+#define PROG_VERSION	"wmsetbg (Window Maker) 2.7"
 
 
 #define WORKSPACE_COUNT (MAX_WORKSPACES+1)
@@ -112,7 +111,7 @@ loadImage(RContext *rc, char *file)
 	wwarning("%s:could not load image file used in texture:%s", path,
 		 RMessageForError(RErrorCode));
     }
-    free(path);
+    wfree(path);
 
     return image;
 }
@@ -210,12 +209,12 @@ parseTexture(RContext *rc, char *text)
 	 case 'H':
 	    gtype = RHorizontalGradient;
 	    iwidth = scrWidth;
-	    iheight = 8;
+	    iheight = 32;
 	    break;
 	 case 'V':
 	 case 'v':
 	    gtype = RVerticalGradient;
-	    iwidth = 8;
+	    iwidth = 32;
 	    iheight = scrHeight;
 	    break;
 	 default:
@@ -269,8 +268,8 @@ parseTexture(RContext *rc, char *text)
 		wwarning("could not parse texture %s", text);
 
 		for (j = 0; colors[j]!=NULL; j++)
-		    free(colors[j]);
-		free(colors);
+		    wfree(colors[j]);
+		wfree(colors);
 		goto error;
 	    }
 	    tmp = PLGetString(val);
@@ -280,16 +279,16 @@ parseTexture(RContext *rc, char *text)
 			 tmp, text);
 
 		for (j = 0; colors[j]!=NULL; j++)
-		    free(colors[j]);
-		free(colors);
+		    wfree(colors[j]);
+		wfree(colors);
 		goto error;
 	    }
 	    if (!(colors[i-2] = malloc(sizeof(RColor)))) {
 		wwarning("out of memory while parsing texture");
 
 		for (j = 0; colors[j]!=NULL; j++)
-		    free(colors[j]);
-		free(colors);
+		    wfree(colors[j]);
+		wfree(colors);
 		goto error;
 	    }
 
@@ -303,12 +302,12 @@ parseTexture(RContext *rc, char *text)
 	 case 'H':
 	    gtype = RHorizontalGradient;
 	    iwidth = scrWidth;
-	    iheight = 8;
+	    iheight = 32;
 	    break;
 	 case 'V':
 	 case 'v':
 	    gtype = RVerticalGradient;
-	    iwidth = 8;
+	    iwidth = 32;
 	    iheight = scrHeight;
 	    break;
 	 default:
@@ -321,8 +320,8 @@ parseTexture(RContext *rc, char *text)
 	image = RRenderMultiGradient(iwidth, iheight, colors, gtype);
 	
 	for (j = 0; colors[j]!=NULL; j++)
-	    free(colors[j]);
-	free(colors);
+	    wfree(colors[j]);
+	wfree(colors);
 
 	if (!image) {
 	    wwarning("could not render gradient texture:%s",
@@ -351,6 +350,8 @@ parseTexture(RContext *rc, char *text)
 	RImage *image = NULL;
 	int w, h;
 	int iwidth, iheight;
+	RColor rcolor;
+
 
 	GETSTRORGOTO(val, tmp, 1, error);
 /*
@@ -375,12 +376,18 @@ parseTexture(RContext *rc, char *text)
 	    goto error;
 	}
 	if (!XAllocColor(dpy, DefaultColormap(dpy, scr), &color)) {
-	    RColor rcolor;
-
 	    rcolor.red = color.red >> 8;
 	    rcolor.green = color.green >> 8;
 	    rcolor.blue = color.blue >> 8;
 	    RGetClosestXColor(rc, &rcolor, &color);
+	} else {
+	    rcolor.red = 0;
+	    rcolor.green = 0;
+	    rcolor.blue = 0;
+	}
+	/* for images with a transparent color */
+	if (image->data[3]) {
+	    RCombineImageWithColor(image, &rcolor);
 	}
 
 	switch (toupper(type[0])) {
@@ -410,7 +417,7 @@ parseTexture(RContext *rc, char *text)
 		    w = (scrHeight*iwidth)/iheight;
 		}
 	    }
-	    {
+	    if (w != image->width || h != image->height) {
 		RImage *simage;
 
 		if (smooth)
@@ -425,9 +432,10 @@ parseTexture(RContext *rc, char *text)
 		}
 		RDestroyImage(image);
 		image = simage;
-		iwidth = image->width;
-		iheight = image->height;
 	    }
+	    iwidth = image->width;
+	    iheight = image->height;
+
 	    /* fall through */
 	 case 'C':
 	    {
@@ -529,7 +537,6 @@ parseTexture(RContext *rc, char *text)
 
 	image = loadImage(rc, file);
 	if (!image) {
-	    RDestroyImage(gradient);
 	    goto error;
 	}
 
@@ -557,7 +564,8 @@ parseTexture(RContext *rc, char *text)
 	if (!gradient) {
 	    wwarning("could not render texture:%s",
 		     RMessageForError(RErrorCode));
-	    RDestroyImage(gradient);
+            RDestroyImage(gradient);
+            RDestroyImage(image);
 	    goto error;
 	}
 
@@ -577,7 +585,7 @@ parseTexture(RContext *rc, char *text)
 	if (!RConvertImage(rc, tiled, &pixmap)) {
 	    wwarning("could not convert texture:%s", 
 		     RMessageForError(RErrorCode));
-	    RDestroyImage(image);
+	    RDestroyImage(tiled);
 	    goto error;
 	}
 	texture->width = tiled->width;
@@ -650,7 +658,7 @@ function_cleanup:
         if (argv) {
 	    int i;
 	    for (i=0; i<argc; i++) {
-	        free(argv[i]);
+	        wfree(argv[i]);
 	    }
 	}
 	if (handle) {
@@ -677,7 +685,7 @@ function_cleanup:
 
 error:
     if (texture)
-	free(texture);
+	wfree(texture);
     if (texarray)
 	PLRelease(texarray);
 
@@ -700,8 +708,8 @@ freeTexture(BackgroundTexture *texture)
     if (texture->pixmap) {
 	XFreePixmap(dpy, texture->pixmap);
     }
-    free(texture->spec);
-    free(texture);
+    wfree(texture->spec);
+    wfree(texture);
 }
 
 
@@ -843,8 +851,9 @@ setPixmapProperty(Pixmap pixmap)
 void
 changeTexture(BackgroundTexture *texture)
 {
-    if (!texture)
+    if (!texture) {
 	return;
+    }
 
     if (texture->solid) {
 	XSetWindowBackground(dpy, root, texture->color.pixel);
@@ -959,10 +968,11 @@ helperLoop(RContext *rc)
 #ifdef DEBUG
 	    printf("change texture %i\n", workspace);
 #endif
-	    if (!textures[workspace])
+	    if (!textures[workspace]) {
 		changeTexture(textures[0]);
-	    else
+	    } else {
 		changeTexture(textures[workspace]);
+	    }
 	    break;
 
 	 case 'P':
@@ -970,7 +980,7 @@ helperLoop(RContext *rc)
 	    printf("change pixmappath %s\n", &buffer[1]);
 #endif
 	    if (PixmapPath)
-		free(PixmapPath);
+		wfree(PixmapPath);
 	    PixmapPath = wstrdup(&buffer[1]);
 	    break;
 
@@ -1000,8 +1010,9 @@ updateDomain(char *domain, char *key, char *texture)
 {
     char *program = "wdwrite";
 
-    system(wstrappend("wdwrite ", 
-		      wstrappend(domain, smooth ? " SmoothWorkspaceBack YES"
+    /* here is a mem leak */
+    system(wstrconcat("wdwrite ", 
+		      wstrconcat(domain, smooth ? " SmoothWorkspaceBack YES"
 				 : " SmoothWorkspaceBack NO")));
 
     execlp(program, program, domain, key, texture, NULL);
@@ -1015,7 +1026,7 @@ globalDefaultsPathForDomain(char *domain)
 {
     char path[1024];
 
-    sprintf(path, "%s/%s", SYSCONFDIR, domain);
+    sprintf(path, "%s/WindowMaker/%s", SYSCONFDIR, domain);
 
     return wstrdup(path);
 }
@@ -1037,7 +1048,7 @@ getValueForKey(char *domain, char *keyName)
     if (!d) {
 	wwarning("could not open domain file %s", path);
     }
-    free(path);
+    wfree(path);
 
     if (d && !PLIsDictionary(d)) {
 	PLRelease(d);
@@ -1056,7 +1067,7 @@ getValueForKey(char *domain, char *keyName)
 	    d = NULL;
 	} else {
 	    d = PLGetProplistWithPath(path);
-	    free(path);
+	    wfree(path);
 	}
 
 	if (d && !PLIsDictionary(d)) {
@@ -1135,6 +1146,37 @@ getPixmapPath(char *domain)
 }
 
 
+char*
+getFullPixmapPath(char *file)
+{
+    char *tmp;
+
+    if (!PixmapPath || !(tmp = wfindfile(PixmapPath, file))) {
+	int bsize = 512;
+	char *path = wmalloc(bsize);
+	
+	while (!getcwd(path, bsize)) {
+	    wfree(path);
+	    bsize += bsize/2;
+	    path = malloc(bsize);
+	}
+
+	tmp = wstrconcat(path, "/");
+	wfree(path);
+	path = wstrconcat(tmp, file);
+	wfree(tmp);
+
+	return path;
+    }
+    
+    /* the file is in the PixmapPath */
+    wfree(tmp);
+
+    return wstrdup(file);
+}
+
+
+
 void
 wAbort()
 {
@@ -1351,9 +1393,13 @@ main(int argc, char **argv)
     PixmapPath = getPixmapPath(domain);
     if (!smooth) {
 	proplist_t val;
-	
+#if 0 /* some problem with Alpha... TODO: check if its right */
 	val = PLGetDictionaryEntry(domain, 
 				   PLMakeString("SmoothWorkspaceBack"));
+#else
+	val = getValueForKey(domain, "SmoothWorkspaceBack");
+#endif
+
 	if (val && PLIsString(val) && strcasecmp(PLGetString(val), "YES")==0)
 	    smooth = True;
     }
@@ -1377,15 +1423,28 @@ main(int argc, char **argv)
     if (!obey_user && DefaultDepth(dpy, scr) <= 8)
         render_mode = RDitheredRendering;
 
-    rattr.flags = RC_RenderMode | RC_ColorsPerChannel | RC_DefaultVisual;
+    rattr.flags = RC_RenderMode | RC_ColorsPerChannel
+	| RC_StandardColormap;
     rattr.render_mode = render_mode;
     rattr.colors_per_channel = cpc;
+    rattr.standard_colormap_mode = RCreateStdColormap;
 
     rc = RCreateContext(dpy, scr, &rattr);
 
+    if (!rc) {
+        rattr.standard_colormap_mode = RIgnoreStdColormap;
+        rc = RCreateContext(dpy, scr, &rattr);
+    }
+    
+    if (!rc) {
+	wfatal("could not initialize wrlib: %s",
+	       RMessageForError(RErrorCode));
+	exit(1);
+    }
+
     if (helperMode) {
 	/* lower priority, so that it wont use all the CPU */
-	nice(1000);
+	nice(15);
 
 	helperLoop(rc);
     } else {
@@ -1393,7 +1452,10 @@ main(int argc, char **argv)
 	char buffer[4098];
 
 	if (!texture) {
-	    sprintf(buffer, "(%s, \"%s\", %s)", style, image_name, back_color);
+	    char *image_path = getFullPixmapPath(image_name);
+	    
+	    sprintf(buffer, "(%s, \"%s\", %s)", style, image_path, back_color);
+	    wfree(image_path);
 	    texture = (char*)buffer;
 	}
 
@@ -1409,7 +1471,7 @@ main(int argc, char **argv)
 	    changeTexture(tex);
 	else {
 	    /* always update domain */
-	    changeTextureForWorkspace(domain, texture, workspace-1);
+	    changeTextureForWorkspace(domain, texture, workspace);
 	}
     }
 
